/*
 *  Copyright (C) 1998,1999  Jesper Pedersen <jews@imada.ou.dk>
 *  This code is released under GNU GPL version 2 or later
 */

import com.sun.java.swing.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.border.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class AudioTracksFrame extends JPanel {
    protected BurnITFrame parent;
    protected JTable      tableView;
    protected JScrollPane scrollpane;
    protected JScrollPane tableAggregate;
    protected JPanel      borderPane;
    protected JPanel      borderPane2;
    protected JPanel      info;
    protected JLabel      totaltracks;
    protected JLabel      totaltime;
    protected JLabel      totalplaying;

    protected Vector      names;
    protected Vector      tableData; 

    protected JProgressBar bar;

    protected CDSize      cdsize;

    protected Utils       utils;
    protected Settings    settings;

    public AudioTracksFrame(BurnITFrame p, Utils utl, Settings set) {
        this.setLayout(new BorderLayout());

	utils = utl;
	parent = p;
	settings = set;

	borderPane = new JPanel();
	borderPane.setBorder(new EmptyBorder(2,2,2,2));
	borderPane.setLayout(new BorderLayout());
	
	borderPane2 = new JPanel();
	borderPane2.setBorder(new MatteBorder(1,1,1,1,Color.gray));
	borderPane2.setLayout(new BorderLayout());

	info = new JPanel();
	info.setBorder(new EmptyBorder(2,2,2,2));
	info.setLayout(new GridLayout(1,3));

	totaltracks = new JLabel();
	totaltracks.setForeground(Color.black);
	totaltime = new JLabel();
	totaltime.setForeground(Color.black);
	totalplaying = new JLabel();
	totalplaying.setForeground(Color.black);

	info.add(totaltracks);
	info.add(totaltime);
	info.add(totalplaying);

	bar = new JProgressBar();
	bar.setMaximum(800);

        borderPane2.add(info, BorderLayout.CENTER);
        borderPane.add(borderPane2, BorderLayout.CENTER);

        tableAggregate = createTable();
        this.add(borderPane, BorderLayout.NORTH);
        this.add(tableAggregate, BorderLayout.CENTER);
	this.add(bar,BorderLayout.SOUTH);

	setTotalTracks();
	setTotalTime();
	setTotalPlaying();
    }


    private JScrollPane createTable() {
      tableData = new Vector();

      names = new Vector();
      names.addElement(new String("Track"));
      names.addElement(new String("Length"));
      names.addElement(new String("Type"));
      
      TableModel dataModel = new AbstractTableModel() {
	public int getColumnCount() { 
	  return names.size();
	}
	public int getRowCount() {
	  return tableData.size();
	}
	public Object getValueAt(int row, int col) {
	  Vector tmp; 
	  tmp = (Vector)tableData.elementAt(row);
	  return tmp.elementAt(col);
	}
	public String getColumnName(int column) {
	  return (String)names.elementAt(column);
	}
	public Class getColumnClass(int c) {
	  return getValueAt(0, c).getClass();
	}
	public boolean isCellEditable(int row, int col) {
	  return false;
	}
	public void setValueAt(Object aValue, int row, int column) { 
	  Vector tmp;
	  tmp = (Vector)tableData.elementAt(row);
	  tmp.setElementAt(aValue,column);
	  tableData.setElementAt(tmp,row);
	}
      };

      MouseListener ml = new MouseAdapter() {
	public void mouseClicked(MouseEvent e) {
	  if (e.getClickCount() == 3) {
	    Vector v;
	    String s,tmp;
	    v = (Vector)tableData.elementAt(tableView.getSelectedRow());
	    s = (String)v.elementAt(0);
	    if (utils.isAudioFile(s)) {
	      Vector track = new Vector();
	      Vector tckno = new Vector();
	      
	      tmp = new String("File ");
	      tmp = tmp.concat(s);
	      tckno.addElement(tmp);
	      track.addElement(tckno);
	      track.addElement(s);

	      CDPlayer cdp = new CDPlayer(settings,utils,track);
	    }
	  }
	}
      };

      // Create the table
      tableView = new JTable(dataModel);
      
      tableView.setRowHeight(20);
      tableView.setShowHorizontalLines(false);
      tableView.setShowVerticalLines(false);
      tableView.getTableHeader().setReorderingAllowed(false);
      tableView.setSelectionMode(0);
      tableView.addMouseListener(ml);
      
      scrollpane = new JScrollPane(tableView);
      return scrollpane;
    }

    public void setTotalTracks() {
      String tmp;
      int i;

      tmp = new String("Total Tracks: ");
      i = tableView.getRowCount();
      if ((parent.getState() == BurnITFrame.isMixedModeCD)||
	  (parent.getState() == BurnITFrame.isMixedModeCD_wImage)) {
	i++;
      }
      tmp = tmp.concat(new Integer(i).toString());

      totaltracks.setText(tmp);
      repaint();
    }

    public void setTotalTime() {
      String tmp;
      float value;
      float sec_f;
      int sec,min;

      if (cdsize != null) {
	value = cdsize.getTotalSize();
	
	sec_f = value * (1024*1024) / 176300;
	
	sec = new Float(sec_f).intValue();
	min = 0;
	while (sec >= 60) {
	  min++;
	  sec = sec - 60;
	}
      } else {
	sec = 0;
	min = 0;
      }
      tmp = new String("Total Time: ");
      tmp = tmp.concat(new Integer(min).toString());
      tmp = tmp.concat(":");
      if (sec < 10) {
	tmp = tmp.concat("0");
      }
      tmp = tmp.concat(new Integer(sec).toString());
      totaltime.setText(tmp);
      repaint();
    }

    public void setTotalPlaying() {
      Vector v;
      int i,min,sec;
      String tmp,min_s,sec_s;

      min = 0;
      sec = 0;
      for (i=0;i<tableData.size();i++) {
	v = (Vector)tableData.elementAt(i);
	tmp = (String)v.elementAt(1);
	if (tmp.compareTo("") != 0) {
	  min_s = tmp.substring(0,tmp.indexOf(':'));
	  sec_s = tmp.substring(tmp.indexOf(':')+1);
	  min = min + new Integer(min_s).intValue();
	  sec = sec + new Integer(sec_s).intValue();
	}
      }
      while (sec >= 60) {
	min++;
	sec = sec - 60;
      }

      tmp = new String("Playing Time: ");
      tmp = tmp.concat(new Integer(min).toString());
      tmp = tmp.concat(":");
      if (sec < 10) {
	tmp = tmp.concat("0");
      }
      tmp = tmp.concat(new Integer(sec).toString());
      totalplaying.setText(tmp);
      repaint();
    }

    public void addPath(String s) {
      Vector newEntry;
      String track,length,type,res;
      int size,sec,min;
      float cal_sec;

      size = 0;
      sec = 0;
      min = 0;
      
      newEntry = new Vector();
      track = new String(s);
      res = new String(utils.getFileSize(s));
      size = new Integer(res).intValue();
      if (s.toUpperCase().endsWith(".MP3")) {
	size = size * 11;
      }
      cal_sec = size / 176300;
      sec = new Float(cal_sec).intValue();
      while (sec >= 60) {
	min++;
	sec = sec - 60;
      }

      length = new String();
      length = length.concat(new Integer(min).toString());
      length = length.concat(":");
      if (sec < 10) {
	length = length.concat("0");
      }
      length = length.concat(new Integer(sec).toString());

      if (s.toUpperCase().endsWith(".MP3")) {
	type = new String("MP3 File");
      } else if (s.toUpperCase().endsWith(".CDR")) {
	type = new String("CDR File");
      } else {
	type = new String("WAV File");
      }
      newEntry.addElement(track);
      newEntry.addElement(length);
      newEntry.addElement(type);

      tableData.addElement(newEntry);
      tableView.repaint();

      updateTable();
    }

    private void updateTable() {
      Vector v;
      int i,min,sec;
      String tmp,min_s,sec_s;
      float value,f;

      value = 0;
      f = 0;
      for (i=0;i<tableData.size();i++) {
	v = (Vector)tableData.elementAt(i);
	tmp = (String)v.elementAt(1);
	if (tmp.compareTo("") != 0) {
	  min_s = tmp.substring(0,tmp.indexOf(':'));
	  sec_s = tmp.substring(tmp.indexOf(':')+1);
	  min = new Integer(min_s).intValue();
	  sec = new Integer(sec_s).intValue();
	  sec = sec + min * 60;
	  f = f + sec;
	}
      }
      sec = tableData.size() - 1;  // 2 seconds between tracks
      sec = sec * 2;
      f = f + sec;

      value = (f * 176300) / (1024*1024);
      cdsize.setAudioTracksSize(value);
    }

    public void updateValues() {
      String tmp;
      int i;
      float value;

      value = cdsize.getTotalSize();
      tmp = new String(new Float(value).toString());
      tmp = tmp.substring(0,tmp.indexOf('.')+2);
      setTotalTracks();
      setTotalTime();
      setTotalPlaying();

      i = new Float(value).intValue();
      if (i <= 800) {
	bar.setValue(i);
      } else {
	bar.setValue(800);
      }
      if (i <= 640) {
	bar.setForeground(Color.green);
      } else if ((i > 640)&&(i <= 700)) {
	bar.setForeground(Color.yellow);
      } else {
	bar.setForeground(Color.red);
      }
    }

    public void deleteElement() {
      if ((tableData.size() > 0)&&(tableView.getSelectedRow() != -1)) {
	tableData.removeElementAt(tableView.getSelectedRow());
	tableView.repaint();
	updateTable();
      }
    }

    public void deleteAll() {
      if (tableData.size() > 0) {
	tableData.removeAllElements();
	tableView.repaint();
	updateTable();
      }
    }

    public void moveUp() {  
      int select;
      Vector v1,v2;

      select = tableView.getSelectedRow();
      
      if (select > 0) {
	v1 = (Vector)tableData.elementAt(select);
	v2 = (Vector)tableData.elementAt(select-1);
	tableData.setElementAt(v2,select);
	tableData.setElementAt(v1,select-1);
	tableView.repaint();
      }
    }

    public void moveDown() {  
      int select;
      Vector v1,v2;

      select = tableView.getSelectedRow();

      if (select < tableData.size()-1) {
	v1 = (Vector)tableData.elementAt(select);
	v2 = (Vector)tableData.elementAt(select+1);
	tableData.setElementAt(v2,select);
	tableData.setElementAt(v1,select+1);
	tableView.repaint();
      }
    }

    // Refresh screen
    public void refresh() {
      tableView.repaint();
      updateValues();
    }

    public void setCDSizeObject(CDSize cd) {
      cdsize = cd;
    }

    public ExecuteCmdsThread getConvertionThread(Supervisor s) {
      Vector v,result;
      int i;
      String cmd,tmp;

      result = new Vector();
      for (i=0;i<tableData.size();i++) {
	v = (Vector)tableData.elementAt(i);
	tmp = (String)v.elementAt(2);
	if (tmp.compareTo("MP3 File") == 0) {
	  cmd = new String(settings.getDecoder());
	  cmd = cmd.concat(" ");
	  tmp = (String)v.elementAt(0);
	  cmd = cmd.concat(tmp);
	  cmd = cmd.concat(" ");
	  cmd = cmd.concat(settings.getTmp());
	  cmd = cmd.concat("/");

	  StringTokenizer st = new StringTokenizer(tmp,"/");
	  while (st.hasMoreElements()) {
	    tmp = st.nextToken();
	  }
	  tmp = tmp.substring(0,tmp.length()-4);
	  cmd = cmd.concat(tmp);
	  cmd = cmd.concat(".cdr");
	  
	  result.addElement(cmd);
	}
      }
      if (result.size() > 0) {
	String[] cmds = new String[result.size()];
	for (i=0;i<result.size();i++) {
	  cmds[i] = (String)result.elementAt(i);
	}
	ExecuteCmdsThread th = new ExecuteCmdsThread(cmds,1,utils.getNrOfCPUs(),s);
	return th;
      }
      return null;
    }

    public boolean legal() {
      Vector v;
      String tmp;
      boolean cdr,wav,mp3;
      
      cdr = false;
      wav = false;
      mp3 = false;
      for (int i=0;i<tableData.size();i++) {
	v = (Vector)tableData.elementAt(i);
	tmp = (String)v.elementAt(2);
	if (tmp.compareTo("MP3 File") == 0) {
	  mp3 = true;
	} else if (tmp.compareTo("CDR File") == 0) {
	  cdr = true;
	} else if (tmp.compareTo("WAV File") == 0) {
	  wav = true;
	}
      }
      if ((mp3)&&(cdr)&&(wav == false)) {
	return true;
      } else if ((mp3)&&(cdr == false)&&(wav == false)) {
	return true;
      } else if ((mp3 == false)&&(cdr)&&(wav == false)) {
	return true;
      } else if ((mp3 == false)&&(cdr == false)&&(wav)) {
	return true;
      }
      return false;
    }

    protected boolean allWav() {
      Vector v;
      String tmp;
      boolean result = true;

      for (int i=0;i<tableData.size();i++) {
	v = (Vector)tableData.elementAt(i);
	tmp = (String)v.elementAt(2);
	if (tmp.compareTo("MP3 File") == 0) {
	  result = false;
	} else if (tmp.compareTo("CDR File") == 0) {
	  result = false;
	}
      }
      return result;
    }

    public Vector getTracks() {
      int i;
      Vector v,t;
      Vector result = new Vector();

      for (i=0;i<tableData.size();i++) {
	t = new Vector();
	v = (Vector)tableData.elementAt(i);
	t.addElement(new Integer(i+1).toString());
	t.addElement(v.elementAt(0));
	result.addElement(t);
      }
      return result;
    }

    public String getBurnString() {
      String s = new String(""); 
      Vector v;
      int i;
      String tmp;

      if (allWav() == false) {
	for (i=0;i<tableData.size();i++) {
	  v = (Vector)tableData.elementAt(i);
	  tmp = (String)v.elementAt(2);
	  if (tmp.compareTo("MP3 File") == 0) {
	    tmp = (String)v.elementAt(0);
	    
	    StringTokenizer st = new StringTokenizer(tmp,"/");
	    while (st.hasMoreElements()) {
	      tmp = st.nextToken();
	    }
	    tmp = tmp.substring(0,tmp.length()-4);
	    tmp = tmp.concat(".cdr");
	    
	    s = s.concat(settings.getTmp());
	    s = s.concat("/");
	    s = s.concat(tmp);
	    s = s.concat(" ");
	  } else if (tmp.compareTo("CDR File") == 0) {
	    tmp = (String)v.elementAt(0);
	    s = s.concat(tmp);
	    s = s.concat(" ");
	  }
	}
      } else {
	for (i=0;i<tableData.size();i++) {
	  v = (Vector)tableData.elementAt(i);
	  tmp = (String)v.elementAt(0);
	  s = s.concat(tmp);
	  s = s.concat(" ");
	}
      }

      return s;
    }

}
