/*
 *  Copyright (C) 1998,1999  Jesper Pedersen <jews@imada.ou.dk>
 *  This code is released under GNU GPL version 2 or later
 */

import com.sun.java.swing.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.border.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class BootFileFrame extends JPanel implements ItemListener {
  protected JPanel     dataPanel;
  protected JPanel     panel1;
  protected JPanel     panel2;
  protected JPanel     panel3;

  protected JCheckBox  bootable;

  protected ButtonGroup  buttonGroup;
  protected JRadioButton floppy;  
  protected JRadioButton harddisk;  
  protected JTextField   path;

  protected DataTrackFrame  dataTrack;
  protected Utils           utils;
  protected Settings        settings;
  
  public BootFileFrame(DataTrackFrame dt,Utils utl,Settings set) {
    dataTrack = dt;
    utils = utl;
    settings = set;
    
    this.setLayout(new BorderLayout());

    dataPanel = new JPanel();
    dataPanel.setBorder(new EmptyBorder(5,5,5,5));
    dataPanel.setLayout(new BorderLayout());

    bootable = new JCheckBox("Bootable CD",false);
    bootable.addItemListener(this);

    dataPanel.add(bootable, BorderLayout.NORTH);

    panel1 = new JPanel();
    panel1.setBorder(new EmptyBorder(5,5,5,5));
    panel1.setLayout(new GridLayout(2,1));

    panel2 = new JPanel();
    panel2.setLayout(new FlowLayout(FlowLayout.LEFT));

    panel3 = new JPanel();
    panel3.setLayout(new FlowLayout(FlowLayout.LEFT));

    buttonGroup = new ButtonGroup();

    floppy = new JRadioButton("Use floppy image",true);
    floppy.setEnabled(false);
    buttonGroup.add(floppy);

    panel2.add(floppy, BorderLayout.NORTH);

    harddisk = new JRadioButton("Use image from harddisc",false);
    harddisk.setEnabled(false);
    buttonGroup.add(harddisk);

    path = new JTextField(30);
    
    panel3.add(harddisk);
    panel3.add(path);

    panel1.add(panel2);
    panel1.add(panel3);

    dataPanel.add(panel1, BorderLayout.CENTER);

    this.add(dataPanel, BorderLayout.NORTH);
  }
  
  // Refresh screen
  public void refresh() {
    repaint();
  }
  
  public void addPath(String s) {
    Vector v;
    String prefix;

    v = dataTrack.getDirs();
    if (v.size() > 0) {
      for (int i=0; i<v.size();i++) {
	prefix = (String)v.elementAt(i);
	if (s.startsWith(prefix) == true) {
	  s = s.substring(prefix.length()+1);
	  bootable.setSelected(true);
	  harddisk.setSelected(true);
	  path.setText(s);
	}
      }
    }
  }

  public boolean bootableCD() {
    return bootable.isSelected();
  }

  /**
     Returns false if source is floppy
     else returns true (harddisk)
  */
  public boolean whichSource() {
    if (floppy.isSelected()) {
      return false;
    }
    return true;
  }

  public String getImagePath() {
    return path.getText();
  }

  public String getImageCatalog() {
    String s;
    int i;

    s = path.getText();
    i = s.lastIndexOf("/");
    if (i != -1) {
      s = s.substring(0,i);
      s = s.concat("/boot.ctl");
    } else {
      s = new String("boot.ctl");
    }
    return s;
  }

  public void itemStateChanged(ItemEvent event) {
    if (event.getSource() == bootable) {
      if (bootable.isSelected()) {
	floppy.setEnabled(true);
	harddisk.setEnabled(true);
      } else {
	floppy.setEnabled(false);
	harddisk.setEnabled(false);
      }
    }
  }
}

