/*
 *  Copyright (C) 1998,1999  Jesper Pedersen <jews@imada.ou.dk>
 *  This code is released under GNU GPL version 2 or later
 */

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;
import java.util.*;

import com.sun.java.swing.event.*;
import com.sun.java.swing.*;


public class BurnITFrame extends JFrame {
  protected JMenuBar menuBar = new JMenuBar();
  protected JMenu menuFile;
  protected JMenuItem menuFileWizard;
  protected JMenuItem menuFileNew;
  protected JMenuItem menuFileOpen;
  protected JMenuItem menuFileSave;
  protected JMenuItem menuFileSaveAs;
  protected JMenuItem menuFileProperties;
  protected JMenuItem menuFileExit;
  protected JMenu menuCDWriter;
  protected JMenuItem menuCDWriterSelect;
  protected JMenuItem menuCDWriterDiscInfo;
  protected JMenuItem menuCDWriterEraseDisc;
  protected JMenuItem menuCDWriterEject;
  protected JMenuItem menuCDWriterWrite;
  protected JMenu menuEdit;
  protected JMenuItem menuEditPaste;
  protected JMenuItem menuEditDelete;
  protected JMenuItem menuEditDeleteAll;
  protected JMenuItem menuEditMoveUp;
  protected JMenuItem menuEditMoveDown;
  protected JMenuItem menuEditReparent;
  protected JMenuItem menuEditRefresh;
  protected JMenu menuView;
  protected JCheckBoxMenuItem menuViewToolbar;
  protected JCheckBoxMenuItem menuViewStatusbar;
  protected JMenuItem menuViewFileManager;
  protected JMenuItem menuViewCDPlayer;
  protected JMenu menuHelp;
  protected JMenuItem menuHelpHelp;
  protected JMenuItem menuHelpAbout;

  protected JButton wizardButton;

  protected JButton newButton;
  protected JButton openButton;
  protected JButton saveButton;

  protected JButton selectButton;
  protected JButton discInfoButton;
  protected JButton eraseButton;
  protected JButton ejectButton;
  protected JButton writeButton;

  protected JButton deleteButton;  
  protected JButton deleteAllButton;
  protected JButton moveUpButton;  
  protected JButton moveDownButton;  
  protected JButton refreshButton;  

  protected JButton cdPlayerButton;  
  protected JButton fileManagerButton;  

  protected JButton helpButton;  

  protected JToolBar toolbar;
  protected JTabbedPane tab;
  protected JPanel statusPanel;
  protected JLabel statusbar;
  protected JLabel device;

  protected NewCDLayout newCD;
  protected GeneralFrame general;
  protected ImageFileFrame imageFile;
  protected LoadContentsFrame loadContents;
  protected FileNamesFrame fileNames;
  protected VolumeInfoFrame volumeInfo;
  protected DataTrackFrame dataTrack;
  protected AudioTracksFrame audioTracks;
  protected MkisofsFrame mkisofs;
  protected CdrecordFrame cdrecord;
  protected Disc2DiscFrame disc2disc;
  protected BootFileFrame bootFile;

  protected CDSize cdsize;

  protected static final int isMultisessionCD = 1;
  protected static final int isCDROM = 2;
  protected static final int isAudio = 3;
  protected static final int isCDCopy = 4;
  protected static final int isMixedModeCD = 5;
  protected static final int isAdvanced = 6;
  protected static final int isCDROM_wImage = 7;
  protected static final int isMixedModeCD_wImage = 8;

  protected static final int newMode = 16;

  protected int state;

  protected Font font;

  protected Wizard wiz;
  protected FileManager fm;

  protected Resource resources;
  protected Settings settings;
  protected Utils util;

  protected boolean open;

  public BurnITFrame() {
    this.getContentPane().setLayout(new BorderLayout());
    this.setSize(new Dimension(630, 420));
    this.setTitle("BurnIT");

    font = new Font("Helvetica", Font.PLAIN, 12);
    resources = new Resource();
    settings = new Settings();
    util = new Utils(settings);

    open = false;

    state = newMode;

    wiz = null;
    fm = null;

    statusbar = new JLabel();
    statusbar.setFont(font);
    statusbar.setForeground(Color.black);
    statusbar.setText(" ");

    device = new JLabel();
    device.setFont(font);
    device.setForeground(Color.black);

    statusPanel = new JPanel();
    statusPanel.setLayout(new BorderLayout());
    statusPanel.add(statusbar,BorderLayout.WEST);
    statusPanel.add(device,BorderLayout.EAST);
    
    this.getContentPane().add("North",createMenu());
    this.getContentPane().add("South",statusPanel);

    JPanel panel = new JPanel();
    panel.setLayout(new BorderLayout());    
    panel.add("North",createToolbar());
    tab = new JTabbedPane();
    createTab();
    panel.add("Center",tab);

    setDeviceName();

    this.getContentPane().add("Center", panel);

    this.setVisible(true);

    if (util.getVersion().compareTo(settings.getConfigVersion()) != 0) {
      JOptionPane f = new JOptionPane();

      f.showMessageDialog(this,"Please choose CD-Writer and set properties", 
			  "BurnIT: Important !!!", JOptionPane.INFORMATION_MESSAGE);
    }
    if (util.checkCdrecord() == false) {
      JOptionPane f = new JOptionPane();

      f.showMessageDialog(this,"cdrecord not correctly installed", 
			  "BurnIT: Important !!!", JOptionPane.ERROR_MESSAGE);
    }

  }

  private JToolBar createToolbar() {
    toolbar = new JToolBar();

    wizardButton = new JButton(new ImageIcon("images/wizard.gif"));
    wizardButton.setToolTipText(resources.getWizardToolTip());
    wizardButton.addActionListener(new BurnITFrame_menuFileWizard_ActionAdapter(this));

    newButton = new JButton(new ImageIcon("images/new.gif"));
    newButton.setToolTipText(resources.getNewToolTip());
    newButton.addActionListener(new BurnITFrame_menuFileNew_ActionAdapter(this));

    openButton = new JButton(new ImageIcon("images/open.gif"));
    openButton.setToolTipText(resources.getOpenToolTip());
    openButton.addActionListener(new BurnITFrame_menuFileOpen_ActionAdapter(this));

    saveButton = new JButton(new ImageIcon("images/save.gif"));
    saveButton.setToolTipText(resources.getSaveToolTip());
    saveButton.addActionListener(new BurnITFrame_menuFileSave_ActionAdapter(this));

    selectButton = new JButton(new ImageIcon("images/select.gif"));
    selectButton.setToolTipText(resources.getSelectToolTip());
    selectButton.addActionListener(new BurnITFrame_menuCDWriterSelect_ActionAdapter(this));

    discInfoButton = new JButton(new ImageIcon("images/disctool.gif"));
    discInfoButton.setToolTipText(resources.getDiscInfoToolTip());
    discInfoButton.addActionListener(new BurnITFrame_menuCDWriterDiscInfo_ActionAdapter(this));

    eraseButton = new JButton(new ImageIcon("images/erase.gif"));
    eraseButton.setToolTipText(resources.getEraseToolTip());
    eraseButton.addActionListener(new BurnITFrame_menuCDWriterEraseDisc_ActionAdapter(this));

    ejectButton = new JButton(new ImageIcon("images/eject.gif"));
    ejectButton.setToolTipText(resources.getEjectToolTip());
    ejectButton.addActionListener(new BurnITFrame_menuCDWriterEject_ActionAdapter(this));

    writeButton = new JButton(new ImageIcon("images/write.gif"));
    writeButton.setToolTipText(resources.getWriteToolTip());
    writeButton.addActionListener(new BurnITFrame_menuCDWriterWrite_ActionAdapter(this));

    deleteButton = new JButton(new ImageIcon("images/delete.gif"));
    deleteButton.setToolTipText(resources.getDeleteToolTip());
    deleteButton.addActionListener(new BurnITFrame_menuEditDelete_ActionAdapter(this));

    deleteAllButton = new JButton(new ImageIcon("images/deleteall.gif"));
    deleteAllButton.setToolTipText(resources.getDeleteAllToolTip());
    deleteAllButton.addActionListener(new BurnITFrame_menuEditDeleteAll_ActionAdapter(this));

    moveUpButton = new JButton(new ImageIcon("images/moveup.gif"));
    moveUpButton.setToolTipText(resources.getMoveUpToolTip());
    moveUpButton.addActionListener(new BurnITFrame_menuEditMoveUp_ActionAdapter(this));

    moveDownButton = new JButton(new ImageIcon("images/movedown.gif"));
    moveDownButton.setToolTipText(resources.getMoveDownToolTip());
    moveDownButton.addActionListener(new BurnITFrame_menuEditMoveDown_ActionAdapter(this));

    refreshButton = new JButton(new ImageIcon("images/refresh.gif"));
    refreshButton.setToolTipText(resources.getRefreshToolTip());
    refreshButton.addActionListener(new BurnITFrame_menuEditRefresh_ActionAdapter(this));

    cdPlayerButton = new JButton(new ImageIcon("images/cdply.gif"));
    cdPlayerButton.setToolTipText(resources.getCDPlayerToolTip());
    cdPlayerButton.addActionListener(new BurnITFrame_menuViewCDPlayer_ActionAdapter(this));

    fileManagerButton = new JButton(new ImageIcon("images/filem.gif"));
    fileManagerButton.setToolTipText(resources.getFileManagerToolTip());
    fileManagerButton.addActionListener(new BurnITFrame_menuViewFileManager_ActionAdapter(this));

    helpButton = new JButton(new ImageIcon("images/help.gif"));
    helpButton.setToolTipText(resources.getHelpToolTip());
    helpButton.addActionListener(new BurnITFrame_menuHelpHelp_ActionAdapter(this));

    toolbar.add(wizardButton);
    toolbar.addSeparator();
    toolbar.add(newButton);
    toolbar.add(openButton);
    toolbar.add(saveButton);
    toolbar.addSeparator();
    toolbar.add(selectButton);
    toolbar.add(discInfoButton);
    toolbar.add(eraseButton);
    toolbar.add(ejectButton);
    toolbar.add(writeButton);
    toolbar.addSeparator();
    toolbar.add(deleteButton);
    toolbar.add(deleteAllButton);
    toolbar.add(moveUpButton);
    toolbar.add(moveDownButton);
    toolbar.add(refreshButton);
    toolbar.addSeparator();
    toolbar.add(cdPlayerButton);
    toolbar.add(fileManagerButton);
    toolbar.addSeparator();
    toolbar.add(helpButton);

    toolbar.setFloatable(false);

    return toolbar;
  }

  private void createTab() {
    tab.removeAll();

    newCD = new NewCDLayout(this);

    general = new GeneralFrame(state,settings);
    imageFile = new ImageFileFrame(settings,util);
    loadContents = new LoadContentsFrame(util);
    fileNames = new FileNamesFrame(settings);
    volumeInfo = new VolumeInfoFrame();
    dataTrack = new DataTrackFrame(util,settings);
    audioTracks = new AudioTracksFrame(this,util,settings);
    disc2disc = new Disc2DiscFrame(util,settings);
    bootFile = new BootFileFrame(dataTrack,util,settings);

    cdsize = new CDSize();

    // 
    cdsize.setDataTrackFrame(dataTrack);
    cdsize.setAudioTracksFrame(audioTracks);
    cdsize.setImageFileFrame(imageFile);

    // Register the frame with cdsize
    audioTracks.setCDSizeObject(cdsize);
    dataTrack.setCDSizeObject(cdsize);
    imageFile.setCDSizeObject(cdsize);

    cdrecord = new CdrecordFrame(this,settings);
    mkisofs = new MkisofsFrame(this,settings,util,cdrecord);

    if (state != newMode) {
      if (state == isAdvanced) {
	tab.addTab(resources.getMkisofs(),mkisofs);
	tab.addTab(resources.getCdrecord(),cdrecord);
      } else if (state == isMixedModeCD) {
	tab.addTab(resources.getGeneral(),general);
	tab.addTab(resources.getFileNames(),fileNames);
	tab.addTab(resources.getVolumeInfo(),volumeInfo);
	tab.addTab(resources.getDataTrack(),dataTrack);
	tab.addTab(resources.getAudioTracks(),audioTracks);
	tab.addTab(resources.getBootFile(),bootFile);
      } else if (state == isCDCopy) {
	tab.addTab(resources.getGeneral(),general);
	tab.addTab(resources.getDisc2Disc(),disc2disc);
      } else if (state == isAudio) {
	tab.addTab(resources.getGeneral(),general);
	tab.addTab(resources.getAudioTracks(),audioTracks);
      } else if (state == isCDROM) {
	tab.addTab(resources.getGeneral(),general);
	tab.addTab(resources.getFileNames(),fileNames);
	tab.addTab(resources.getVolumeInfo(),volumeInfo);
	tab.addTab(resources.getDataTrack(),dataTrack);
	tab.addTab(resources.getBootFile(),bootFile);
      } else if (state == isMultisessionCD) {
	tab.addTab(resources.getGeneral(),general);
	tab.addTab(resources.getLoadContents(),loadContents);
	tab.addTab(resources.getFileNames(),fileNames);
	tab.addTab(resources.getVolumeInfo(),volumeInfo);
	tab.addTab(resources.getDataTrack(),dataTrack);
	tab.addTab(resources.getBootFile(),bootFile);
      } else if (state == isCDROM_wImage) {
	tab.addTab(resources.getGeneral(),general);
	tab.addTab(resources.getImageFile(),imageFile);
      } else if (state == isMixedModeCD_wImage) {
	tab.addTab(resources.getGeneral(),general);
	tab.addTab(resources.getImageFile(),imageFile);
	tab.addTab(resources.getAudioTracks(),audioTracks);
      } else {
	// View All Mode
	tab.addTab(resources.getGeneral(),general);
	tab.addTab(resources.getImageFile(),imageFile);
	tab.addTab(resources.getLoadContents(),loadContents);
	tab.addTab(resources.getFileNames(),fileNames);
	tab.addTab(resources.getVolumeInfo(),volumeInfo);
	tab.addTab(resources.getDataTrack(),dataTrack);
	tab.addTab(resources.getAudioTracks(),audioTracks);
	tab.addTab(resources.getMkisofs(),mkisofs);
	tab.addTab(resources.getCdrecord(),cdrecord);
	tab.addTab(resources.getDisc2Disc(),disc2disc);
	tab.addTab(resources.getBootFile(),bootFile);
	tab.addTab("New",newCD);
      }
    } else {
      tab.addTab("New",newCD);
    }
  }

  private JMenuBar createMenu() {
    menuFile = new JMenu("File");
    menuFile.setMnemonic('F');

    menuFileWizard = new JMenuItem("BurnIT Wizard");
    menuFileWizard.addActionListener(new BurnITFrame_menuFileWizard_ActionAdapter(this));
    menuFileNew = new JMenuItem("New ...");
    menuFileNew.setMnemonic('N');
    menuFileNew.addActionListener(new BurnITFrame_menuFileNew_ActionAdapter(this));
    menuFileOpen = new JMenuItem("Open ...");
    menuFileOpen.setMnemonic('O');
    menuFileOpen.addActionListener(new BurnITFrame_menuFileOpen_ActionAdapter(this));
    menuFileSave = new JMenuItem("Save");
    menuFileSave.setMnemonic('S');
    menuFileSave.addActionListener(new BurnITFrame_menuFileSave_ActionAdapter(this));
    menuFileSaveAs = new JMenuItem("Save as ...");
    menuFileSaveAs.setMnemonic('a');
    menuFileSaveAs.addActionListener(new BurnITFrame_menuFileSaveAs_ActionAdapter(this));
    menuFileProperties = new JMenuItem("Properties ...");
    menuFileProperties.addActionListener(new BurnITFrame_menuFileProperties_ActionAdapter(this));
    menuFileExit = new JMenuItem("Exit");
    menuFileExit.setMnemonic('x');
    menuFileExit.addActionListener(new BurnITFrame_menuFileExit_ActionAdapter(this));

    menuCDWriter = new JMenu("CD-Writer");
    menuCDWriter.setMnemonic('C');

    menuCDWriterSelect = new JMenuItem("Select CD-Writer ...");
    menuCDWriterSelect.setMnemonic('S');
    menuCDWriterSelect.addActionListener(new BurnITFrame_menuCDWriterSelect_ActionAdapter(this));
    menuCDWriterDiscInfo = new JMenuItem("Disc Info and Tools");
    menuCDWriterDiscInfo.setMnemonic('I');
    menuCDWriterDiscInfo.addActionListener(new BurnITFrame_menuCDWriterDiscInfo_ActionAdapter(this));
    menuCDWriterEraseDisc = new JMenuItem("Erase Disc");
    menuCDWriterEraseDisc.setMnemonic('r');
    menuCDWriterEraseDisc.addActionListener(new BurnITFrame_menuCDWriterEraseDisc_ActionAdapter(this));
    menuCDWriterEject = new JMenuItem("Eject / Tray");
    menuCDWriterEject.setMnemonic('E');
    menuCDWriterEject.addActionListener(new BurnITFrame_menuCDWriterEject_ActionAdapter(this));
    menuCDWriterWrite = new JMenuItem("Write!");
    menuCDWriterWrite.setMnemonic('W');
    menuCDWriterWrite.addActionListener(new BurnITFrame_menuCDWriterWrite_ActionAdapter(this));

    menuEdit = new JMenu("Edit");
    menuEdit.setMnemonic('E');

    menuEditPaste = new JMenuItem("Paste");
    menuEditPaste.setMnemonic('P');
    menuEditPaste.addActionListener(new BurnITFrame_menuEditPaste_ActionAdapter(this));
    menuEditDelete = new JMenuItem("Delete");
    menuEditDelete.setMnemonic('D');
    menuEditDelete.addActionListener(new BurnITFrame_menuEditDelete_ActionAdapter(this));
    menuEditDeleteAll = new JMenuItem("Delete All");
    menuEditDeleteAll.setMnemonic('A');
    menuEditDeleteAll.addActionListener(new BurnITFrame_menuEditDeleteAll_ActionAdapter(this));
    menuEditMoveUp = new JMenuItem("Move Up");
    menuEditMoveUp.setMnemonic('M');
    menuEditMoveUp.addActionListener(new BurnITFrame_menuEditMoveUp_ActionAdapter(this));
    menuEditMoveDown = new JMenuItem("Move Down");
    menuEditMoveDown.setMnemonic('o');
    menuEditMoveDown.addActionListener(new BurnITFrame_menuEditMoveDown_ActionAdapter(this));
    menuEditReparent = new JMenuItem("Reparenting ...");
    menuEditReparent.addActionListener(new BurnITFrame_menuEditReparent_ActionAdapter(this));
    menuEditRefresh = new JMenuItem("Refresh Totals");
    menuEditRefresh.setMnemonic('R');
    menuEditRefresh.addActionListener(new BurnITFrame_menuEditRefresh_ActionAdapter(this));

    menuView = new JMenu("View");
    menuView.setMnemonic('V');

    menuViewToolbar = new JCheckBoxMenuItem("Toolbar",true);
    menuViewToolbar.setMnemonic('T');
    menuViewToolbar.addActionListener(new BurnITFrame_menuViewToolbar_ActionAdapter(this));
    menuViewStatusbar = new JCheckBoxMenuItem("Statusbar",true);
    menuViewStatusbar.setMnemonic('S');
    menuViewStatusbar.addActionListener(new BurnITFrame_menuViewStatusbar_ActionAdapter(this));
    menuViewCDPlayer = new JMenuItem("CD Player");
    menuViewCDPlayer.addActionListener(new BurnITFrame_menuViewCDPlayer_ActionAdapter(this));
    menuViewFileManager = new JMenuItem("File Manager");
    menuViewFileManager.addActionListener(new BurnITFrame_menuViewFileManager_ActionAdapter(this));

    menuHelp = new JMenu("Help");
    menuHelp.setMnemonic('H');
    menuHelpHelp = new JMenuItem("Help Topics");
    menuHelpHelp.addActionListener(new BurnITFrame_menuHelpHelp_ActionAdapter(this));
    menuHelpAbout = new JMenuItem("About");
    menuHelpAbout.setMnemonic('A');
    menuHelpAbout.addActionListener(new BurnITFrame_menuHelpAbout_ActionAdapter(this));

    menuFile.add(menuFileWizard);
    menuFile.add(new JSeparator());
    menuFile.add(menuFileNew);
    menuFile.add(menuFileOpen);
    menuFile.add(menuFileSave);
    menuFile.add(menuFileSaveAs);
    menuFile.addSeparator();
    menuFile.add(menuFileProperties);
    menuFile.addSeparator();
    menuFile.add(menuFileExit);

    menuCDWriter.add(menuCDWriterSelect);
    menuCDWriter.add(menuCDWriterDiscInfo);
    menuCDWriter.add(menuCDWriterEraseDisc);
    menuCDWriter.add(menuCDWriterEject);
    menuCDWriter.add(new JSeparator());
    menuCDWriter.add(menuCDWriterWrite);

    menuEdit.add(menuEditPaste);
    menuEdit.add(new JSeparator());
    menuEdit.add(menuEditDelete);
    menuEdit.add(menuEditDeleteAll);
    menuEdit.add(new JSeparator());
    menuEdit.add(menuEditMoveUp);
    menuEdit.add(menuEditMoveDown);
    menuEdit.add(menuEditReparent);
    menuEdit.add(menuEditRefresh);

    menuView.add(menuViewToolbar);
    menuView.add(menuViewStatusbar);
    menuView.add(new JSeparator());
    menuView.add(menuViewCDPlayer);
    menuView.add(menuViewFileManager);

    menuHelp.add(menuHelpHelp);
    menuHelp.add(new JSeparator());
    menuHelp.add(menuHelpAbout);

    menuBar.add(menuFile);
    menuBar.add(menuCDWriter);
    menuBar.add(menuEdit);
    menuBar.add(menuView);
    menuBar.add(menuHelp);

    return menuBar;
  }
  
  // Refresh screen
  public void refresh() {
    updateScreen();
  }

  // Update screen
  public void updateScreen() {
    if (menuViewToolbar.getState() == true) {
      toolbar.setVisible(true);
    } else {    
      toolbar.setVisible(false);
    }
    if (menuViewStatusbar.getState() == true) {
      statusbar.setVisible(true);
      device.setVisible(true);
    } else {    
      statusbar.setVisible(false);
      device.setVisible(false);
    }
    this.pack();
    this.setSize(new Dimension(630, 420));
    repaint();
  }

  // Set name for device in statusbar 
  public void setDeviceName() {
    String s = new String("");

    if (settings.getWrModel().compareTo("") != 0) {
      s = s.concat(settings.getWrModel());
      s = s.concat(" (");
      s = s.concat(settings.getWrDevNumber());
      s = s.concat(")");
    }
    device.setText(s);
    updateScreen();
  }

  public void addPath(String s) {
    /*
      GeneralFrame general;
      LoadContentsFrame loadContents;
      FileNamesFrame fileNames;
      VolumeInfoFrame volumeInfo;
      */
    if (wiz != null) {
      wiz.addPath(s);
    } else if (tab.getSelectedComponent() instanceof MkisofsFrame) {
      if ((util.isDirectory(s) == true)||(util.isISOFile(s) == true)) {
	mkisofs.addPath(s);
      }
    } else if (tab.getSelectedComponent() instanceof CdrecordFrame) {
      if (util.isISOFile(s) == true) {
	cdrecord.addPath(s);
      }
    } else if (tab.getSelectedComponent() instanceof DataTrackFrame) {
      if (util.isDirectory(s) == true) {
	dataTrack.addPath(s);
      }
    } else if (tab.getSelectedComponent() instanceof AudioTracksFrame) {
      if (util.isAudioFile(s) == true) {
	audioTracks.addPath(s);
      }
    } else if (tab.getSelectedComponent() instanceof ImageFileFrame) {
      if (util.isISOFile(s) == true) {
	imageFile.addPath(s);
      }
      
    } else if (tab.getSelectedComponent() instanceof BootFileFrame) {
      if (util.isIMGFile(s) == true) {
	bootFile.addPath(s);
      }
    }
  }

  public int getState() {
    return state;
  }

  public void setState(int layout) {
    state = layout;
    createTab();
    updateScreen();
  }

  public GeneralFrame getGeneralFrame() {
    return general;
  }

  public ImageFileFrame getImageFileFrame() {
    return imageFile;
  }

  public LoadContentsFrame getLoadContentsFrame() {
    return loadContents;
  }

  public FileNamesFrame getFileNamesFrame() {
    return fileNames;
  }

  public VolumeInfoFrame getVolumeInfoFrame() {
    return volumeInfo;
  }

  public DataTrackFrame getDataTrackFrame() {
    return dataTrack;
  }

  public AudioTracksFrame getAudioTracksFrame() {
    return audioTracks;
  }

  public Disc2DiscFrame getDisc2DiscFrame() {
    return disc2disc;
  }

  public void wizardFinished() {
    wiz = null;
  }

  //File | Wizard action performed
  public void fileWizard_actionPerformed(ActionEvent e) {
    if (wiz == null) {
      wiz = new Wizard(this,state,settings,util);
    }
    //System.out.println("fileWizard");
  }

  //File | New action performed
  public void fileNew_actionPerformed(ActionEvent e) {
    setState(newMode);
    //System.out.println("fileNew");
  }

  //File | Open action performed
  public void fileOpen_actionPerformed(ActionEvent e) {
    //System.out.println("fileOpen");
  }

  //File | Save action performed
  public void fileSave_actionPerformed(ActionEvent e) {
    if (state == isMultisessionCD) {
      //System.out.println("multisession save");
    } else if (state == isCDROM) {
      //System.out.println("cdrom save");
    } else if (state == isAudio) {
      //System.out.println("audio save");
    } else if (state == isCDCopy) {
      //System.out.println("cdcopy save");
    } else if (state == isMixedModeCD) {
      //System.out.println("mixedmode save");
    } else if (state == isAdvanced) {
      //System.out.println("advanced save");
      settings.setAdvMki(mkisofs.getAdvMki());
      settings.setAdvCdr(cdrecord.getAdvCdr());
    }
    //System.out.println("fileSave");
  }

  //File | SaveAs action performed
  public void fileSaveAs_actionPerformed(ActionEvent e) {
    //System.out.println("fileSaveAs");
  }

  //File | Properties action performed
  public void fileProperties_actionPerformed(ActionEvent e) {
    PropertiesFrame frame = new PropertiesFrame(settings,util);
    //System.out.println("fileProperties");
  }

  //File | Exit action performed
  public void fileExit_actionPerformed(ActionEvent e) {
    //System.out.println("fileExit");
    settings.saveSettings();
    System.exit(0);
  }

  //CDWriter | Select action performed
  public void cdwriterSelect_actionPerformed(ActionEvent e) {
    SelectDevice frame = new SelectDevice(this,settings,util);
    //System.out.println("cdwriterSelect");
  }

  //CDWriter | DiscInfo action performed
  public void cdwriterDiscInfo_actionPerformed(ActionEvent e) {
    DiscInfo frame = new DiscInfo(settings,util);
    //System.out.println("cdwriterDiscInfo");
  }

  //CDWriter | EraseDisc action performed
  public void cdwriterEraseDisc_actionPerformed(ActionEvent e) {
    if (settings.getType().compareTo("CD-RW") == 0) {
      EraseDisc frame = new EraseDisc(settings,util);
    }
    //System.out.println("cdwriterEraseDisc");
  }

  //CDWriter | Eject action performed
  public void cdwriterEject_actionPerformed(ActionEvent e) {
    String cmd = new String();

    cmd = cmd.concat(settings.getCdrecord());
    cmd = cmd.concat(" dev=");
    cmd = cmd.concat(settings.getWrDevNumber());
    if (open) {
      cmd = cmd.concat(" -load");
      ExecuteThread th = new ExecuteThread(cmd,false);
      open = false;
    } else {
      cmd = cmd.concat(" -eject");
      ExecuteThread th = new ExecuteThread(cmd,false);
      open = true;
    }
    //System.out.println("cdwriterEject : OK");
  }

  //CDWriter | Write action performed
  public void cdwriterWrite_actionPerformed(ActionEvent e) {
//     general = new GeneralFrame(state,settings);
//     imageFile = new ImageFileFrame(settings);
//     loadContents = new LoadContentsFrame(util);
//     fileNames = new FileNamesFrame(settings);
//     volumeInfo = new VolumeInfoFrame();
//     dataTrack = new DataTrackFrame(util,settings);
//     audioTracks = new AudioTracksFrame(this,util,settings);

    String cmdCDR = new String("");
    String cmdMKI = new String("");
    String cmdRM  = new String("");
    Supervisor sv = new Supervisor();
    boolean runnable = true;

    cmdCDR = cmdCDR.concat(settings.getCdrecord());
    cmdMKI = cmdMKI.concat(fileNames.whichISOBuilder());

    cmdMKI = cmdMKI.concat(" ");
    cmdCDR = cmdCDR.concat(" ");

    cmdRM = cmdRM.concat("rm -f ");
    cmdRM = cmdRM.concat(general.getTmp());
    cmdRM = cmdRM.concat("/image.iso");
    
    if ((cmdMKI.indexOf("mkisofs") == -1)&&(cmdMKI.indexOf("mkhybrid") == -1)) {
      String s = new String("Please set the ");
      JOptionPane f = new JOptionPane();

      if (fileNames.getBurnString().indexOf("-hfs") != -1) {
	s = s.concat("mkhybrid ");
      } else {
	s = s.concat("mkisofs ");
      }
      s = s.concat("property variable");

      f.showMessageDialog(this,s,"BurnIT: Important !!!", JOptionPane.INFORMATION_MESSAGE);
      
      runnable = false;
    }

    if (runnable) {
      if (state == isMultisessionCD) {
	boolean able = false;
	
	cmdMKI = cmdMKI.concat(fileNames.getBurnString());
	if (bootFile.bootableCD()) {
	  cmdMKI = cmdMKI.concat(" -b ");
	  if (bootFile.whichSource()) {
	    // Harddisc
	    cmdMKI = cmdMKI.concat(bootFile.getImagePath());
	    cmdMKI = cmdMKI.concat(" -c ");
	    cmdMKI = cmdMKI.concat(bootFile.getImageCatalog());
	  } else {
	    // Floppy
	    String mkdir = new String("mkdir ");
	    mkdir = mkdir.concat(settings.getTmp());
	    mkdir = mkdir.concat("/BurnIT.tmp");

	    String dd = new String("dd if=/dev/fd0 of=");
	    dd = dd.concat(settings.getTmp());
	    dd = dd.concat("/BurnIT.tmp/boot.img bs=512");

	    util.execute(mkdir);
	    util.execute(dd);

	    cmdMKI = cmdMKI.concat("boot.img");
	    cmdMKI = cmdMKI.concat(" -c ");
	    cmdMKI = cmdMKI.concat("boot.ctl");
	  }
	}
	if (loadContents.getBurnString().compareTo("") != 0) {
	  cmdMKI = cmdMKI.concat(" -C ");
	  cmdMKI = cmdMKI.concat(loadContents.getBurnString());
	  cmdMKI = cmdMKI.concat(" -M ");
	  cmdMKI = cmdMKI.concat(settings.getWrGen());
	} else {
	  able = true;
	}
	cmdMKI = cmdMKI.concat(" ");
	cmdMKI = cmdMKI.concat(volumeInfo.getBurnString());
	
	if ((general.getOnTheFly() == false)||
	    (general.getWriteISOImage() == true)) {
	  cmdMKI = cmdMKI.concat(" -o ");
	  if (general.getWriteISOImage()) {
	    cmdMKI = cmdMKI.concat(general.getWriteDest());
	  } else {
	    cmdMKI = cmdMKI.concat(general.getTmp());
	    cmdMKI = cmdMKI.concat("/image.iso");
	  }
	}
	cmdMKI = cmdMKI.concat(" ");
	cmdMKI = cmdMKI.concat(dataTrack.getBurnString());
	if ((bootFile.bootableCD() == true)&&(bootFile.whichSource() == false)) {
	  cmdMKI = cmdMKI.concat(" ");
	  cmdMKI = cmdMKI.concat(settings.getTmp());
	  cmdMKI = cmdMKI.concat("/BurnIT.tmp");
	}

	cmdCDR = cmdCDR.concat("-v dev=");
	cmdCDR = cmdCDR.concat(settings.getWrDevNumber());
	cmdCDR = cmdCDR.concat(" speed=");
	cmdCDR = cmdCDR.concat(general.getSpeed());
	cmdCDR = cmdCDR.concat(" ");
	cmdCDR = cmdCDR.concat(settings.getComment());
	cmdCDR = cmdCDR.concat(" -multi");
	cmdCDR = cmdCDR.concat(" -pad ");
	cmdCDR = cmdCDR.concat(general.getTestBefore());
	
	cmdCDR = cmdCDR.concat(" ");
	cmdCDR = cmdCDR.concat(general.getSessionType());
	cmdCDR = cmdCDR.concat(" ");
	if ((able)&&(general.getOnTheFly())) {
	  cmdCDR = cmdCDR.concat("-");
	} else {
	  cmdCDR = cmdCDR.concat(general.getTmp());
	  cmdCDR = cmdCDR.concat("/image.iso");
	}
	if (general.getWriteISOImage()) {
	  // Only write image
	  String[] mkiso = {new String(cmdMKI)};
	  ExecuteCmdsThread th = new ExecuteCmdsThread(mkiso, 1, 1, sv);
	  sv.addThread(th);

	  if ((bootFile.bootableCD() == true)&&(bootFile.whichSource() == false)) {
	    String rmcmd = new String("rm -Rf ");
	    rmcmd = rmcmd.concat(settings.getTmp());
	    rmcmd = rmcmd.concat("/BurnIT.tmp");

	    String[] rm = {rmcmd};
	    ExecuteCmdsThread th5 = new ExecuteCmdsThread(rm, 1, 1, sv);
	    sv.addThread(th5);
	  }
	  sv.go();
	} else {
	  // Full write
	  if ((able)&&(general.getOnTheFly())) {
	    // On the Fly
	    util.execute_pipe(cmdMKI,cmdCDR);  // Burn !!
	  } else {
	    // With image
	    String[] mkcmd = {new String(cmdMKI)};
	    ExecuteCmdsThread th = new ExecuteCmdsThread(mkcmd, 1, 1, sv);
	    sv.addThread(th);
	    
	    String[] cdcmd = {new String(cmdCDR)};
	    ExecuteCmdsThread th2 = new ExecuteCmdsThread(cdcmd, 1, 1, sv);
	    sv.addThread(th2);
	    
	    String[] rmcmd = {new String(cmdRM)};
	    ExecuteCmdsThread th3 = new ExecuteCmdsThread(rmcmd, 1, 1, sv);
	    sv.addThread(th3);
	    
	    if ((bootFile.bootableCD() == true)&&(bootFile.whichSource() == false)) {
	      String rmcmd2 = new String("rm -Rf ");
	      rmcmd2 = rmcmd2.concat(settings.getTmp());
	      rmcmd2 = rmcmd2.concat("/BurnIT.tmp");
	      
	      String[] rm = {rmcmd2};
	      ExecuteCmdsThread th5 = new ExecuteCmdsThread(rm, 1, 1, sv);
	      sv.addThread(th5);
	    }

	    sv.go();  // Burn !!
	  }
	}
	//System.out.println("multisession write !");
      //////////////////////////////////////////////////////////////////////
      } else if (state == isCDROM) {
	cmdMKI = cmdMKI.concat(fileNames.getBurnString());
	cmdMKI = cmdMKI.concat(" ");
	cmdMKI = cmdMKI.concat(volumeInfo.getBurnString());
	
	if (bootFile.bootableCD()) {
	  cmdMKI = cmdMKI.concat(" -b ");
	  if (bootFile.whichSource()) {
	    // Harddisc
	    cmdMKI = cmdMKI.concat(bootFile.getImagePath());
	    cmdMKI = cmdMKI.concat(" -c ");
	    cmdMKI = cmdMKI.concat(bootFile.getImageCatalog());
	  } else {
	    // Floppy
	    String mkdir = new String("mkdir ");
	    mkdir = mkdir.concat(settings.getTmp());
	    mkdir = mkdir.concat("/BurnIT.tmp");

	    String dd = new String("dd if=/dev/fd0 of=");
	    dd = dd.concat(settings.getTmp());
	    dd = dd.concat("/BurnIT.tmp/boot.img bs=512");

	    util.execute(mkdir);
	    util.execute(dd);

	    cmdMKI = cmdMKI.concat("boot.img");
	    cmdMKI = cmdMKI.concat(" -c ");
	    cmdMKI = cmdMKI.concat("boot.ctl");
	  }
	}

	if ((general.getOnTheFly() == false)||
	    (general.getWriteISOImage() == true)) {
	  cmdMKI = cmdMKI.concat(" -o ");
	  if (general.getWriteISOImage()) {
	    cmdMKI = cmdMKI.concat(general.getWriteDest());
	  } else {
	    cmdMKI = cmdMKI.concat(general.getTmp());
	    cmdMKI = cmdMKI.concat("/image.iso");
	  }
	}
	cmdMKI = cmdMKI.concat(" ");
	cmdMKI = cmdMKI.concat(dataTrack.getBurnString());
	if ((bootFile.bootableCD() == true)&&(bootFile.whichSource() == false)) {
	  cmdMKI = cmdMKI.concat(" ");
	  cmdMKI = cmdMKI.concat(settings.getTmp());
	  cmdMKI = cmdMKI.concat("/BurnIT.tmp");
	}
	
	cmdCDR = cmdCDR.concat("-v dev=");
	cmdCDR = cmdCDR.concat(settings.getWrDevNumber());
	cmdCDR = cmdCDR.concat(" speed=");
	cmdCDR = cmdCDR.concat(general.getSpeed());
	cmdCDR = cmdCDR.concat(" ");
	cmdCDR = cmdCDR.concat(settings.getComment());
	cmdCDR = cmdCDR.concat(" -pad ");
	cmdCDR = cmdCDR.concat(general.getTestBefore());
	
	cmdCDR = cmdCDR.concat(" ");
	cmdCDR = cmdCDR.concat(general.getSessionType());
	cmdCDR = cmdCDR.concat(" ");
	if (general.getOnTheFly()) {
	  cmdCDR = cmdCDR.concat("-");
	} else {
	  cmdCDR = cmdCDR.concat(general.getTmp());
	  cmdCDR = cmdCDR.concat("/image.iso");
	}
	if (general.getWriteISOImage()) {
	  // Only write image
	  String[] mkiso = {new String(cmdMKI)};
	  ExecuteCmdsThread th = new ExecuteCmdsThread(mkiso, 1, 1, sv);
	  sv.addThread(th);

	  if ((bootFile.bootableCD() == true)&&(bootFile.whichSource() == false)) {
	    String rmcmd = new String("rm -Rf ");
	    rmcmd = rmcmd.concat(settings.getTmp());
	    rmcmd = rmcmd.concat("/BurnIT.tmp");

	    String[] rm = {rmcmd};
	    ExecuteCmdsThread th5 = new ExecuteCmdsThread(rm, 1, 1, sv);
	    sv.addThread(th5);
	  }
	  sv.go();
	} else {
	  // Full write
	  if (general.getOnTheFly()) {
	    // On the Fly
	    util.execute_pipe(cmdMKI,cmdCDR);  // Burn !!
	  } else {
	    // With image
	    String[] mkcmd = {new String(cmdMKI)};
	    ExecuteCmdsThread th = new ExecuteCmdsThread(mkcmd, 1, 1, sv);
	    sv.addThread(th);
	    
	    String[] cdcmd = {new String(cmdCDR)};
	    ExecuteCmdsThread th2 = new ExecuteCmdsThread(cdcmd, 1, 1, sv);
	    sv.addThread(th2);
	    
	    String[] rmcmd = {new String(cmdRM)};
	    ExecuteCmdsThread th3 = new ExecuteCmdsThread(rmcmd, 1, 1, sv);
	    sv.addThread(th3);
	    
	    if ((bootFile.bootableCD() == true)&&(bootFile.whichSource() == false)) {
	      String rmcmd2 = new String("rm -Rf ");
	      rmcmd2 = rmcmd2.concat(settings.getTmp());
	      rmcmd2 = rmcmd2.concat("/BurnIT.tmp");
	      
	      String[] rm = {rmcmd2};
	      ExecuteCmdsThread th5 = new ExecuteCmdsThread(rm, 1, 1, sv);
	      sv.addThread(th5);
	    }

	    sv.go();  // Burn !!
	  }
	}
	//System.out.println("cdrom write !");
      //////////////////////////////////////////////////////////////////////
      } else if (state == isCDROM_wImage) {
	cmdCDR = cmdCDR.concat("-v dev=");
	cmdCDR = cmdCDR.concat(settings.getWrDevNumber());
	cmdCDR = cmdCDR.concat(" speed=");
	cmdCDR = cmdCDR.concat(general.getSpeed());
	cmdCDR = cmdCDR.concat(" ");
	cmdCDR = cmdCDR.concat(settings.getComment());
	cmdCDR = cmdCDR.concat(" -pad ");
	cmdCDR = cmdCDR.concat(general.getTestBefore());
	
	cmdCDR = cmdCDR.concat(" ");
	cmdCDR = cmdCDR.concat(general.getSessionType());
	cmdCDR = cmdCDR.concat(" ");
	
	cmdCDR = cmdCDR.concat(imageFile.getBurnString());
	
	ExecuteThread th = new ExecuteThread(cmdCDR);
	//System.out.println("cdrom with image write !");
      } else if (state == isAudio) {
	
	if (audioTracks.legal()) {
	  ExecuteCmdsThread th = audioTracks.getConvertionThread(sv);
	  if (th != null) {
	    sv.addThread(th);
	  }
	  cmdCDR = cmdCDR.concat("-v dev=");
	  cmdCDR = cmdCDR.concat(settings.getWrDevNumber());
	  cmdCDR = cmdCDR.concat(" speed=");
	  cmdCDR = cmdCDR.concat(general.getSpeed());
	  cmdCDR = cmdCDR.concat(" ");
	  cmdCDR = cmdCDR.concat(settings.getComment());
	  cmdCDR = cmdCDR.concat(" -pad ");
	  cmdCDR = cmdCDR.concat(general.getTestBefore());
	  
	  cmdCDR = cmdCDR.concat(" ");
	  cmdCDR = cmdCDR.concat(general.getSessionType());
	  cmdCDR = cmdCDR.concat(" ");
	  
	  cmdCDR = cmdCDR.concat(audioTracks.getBurnString());
	  
	  String[] cdcmd = {new String(cmdCDR)};
	  ExecuteCmdsThread th2 = new ExecuteCmdsThread(cdcmd, 1, 1, sv);
	  sv.addThread(th2);
	  
	  sv.go();  // Burn !!
	}
	//System.out.println("audio write !");
      } else if (state == isCDCopy) {
	if (disc2disc.legal()) {
	  //////////////////
	  // CDROM Copy
	  //////////////////
	  if (disc2disc.getType() == Disc2DiscFrame.isCDROMType) {
	    cmdCDR = cmdCDR.concat("-v dev=");
	    cmdCDR = cmdCDR.concat(disc2disc.getWriteDev());
	    cmdCDR = cmdCDR.concat(" speed=");
	    cmdCDR = cmdCDR.concat(general.getSpeed());
	    cmdCDR = cmdCDR.concat(" ");
	    cmdCDR = cmdCDR.concat(general.getTestBefore());
	    cmdCDR = cmdCDR.concat(" -isosize ");
	    cmdCDR = cmdCDR.concat(disc2disc.getReadDev());

	    ExecuteThread th = new ExecuteThread(cmdCDR);
	  //////////////////
	  // AUDIO Copy
	  //////////////////
	  } else if (disc2disc.getType() == Disc2DiscFrame.isAudioType) {
	    String cmd = new String(settings.getAudioRip());
	    if (settings.getRipType()) {
	      // Cdda2wav
	      cmd = cmd.concat(" -D ");
	      cmd = cmd.concat(disc2disc.getReadDevNumber());
	      cmd = cmd.concat(" -x -H -O cdr -B ");
	      cmd = cmd.concat(general.getTmp());
	    } else {
	      // Cdparanoia
	      cmd = cmd.concat(" -v -g ");
	      cmd = cmd.concat(disc2disc.getReadGen());
	      cmd = cmd.concat(" -R -B 1- ");
	      cmd = cmd.concat(general.getTmp());
	    }
	    String[] audiocmd = {new String(cmd)};
	    ExecuteCmdsThread th1 = new ExecuteCmdsThread(audiocmd, 1, 1, sv);
	    sv.addThread(th1);

	    cmd = new String(settings.getCdrecord());
	    cmd = cmd.concat(" -v dev=");
	    cmd = cmd.concat(disc2disc.getWriteDev());
	    cmd = cmd.concat(" speed=");
	    cmd = cmd.concat(general.getSpeed());
	    cmd = cmd.concat(" ");
	    cmd = cmd.concat(general.getTestBefore());
	    cmd = cmd.concat(" -audio ");
	    cmd = cmd.concat(general.getTmp());
	    if (cmd.endsWith("/") == false) {
	      cmd = cmd.concat("/");
	    }
	    if (settings.getRipType()) {
	      cmd = cmd.concat("*.cdr");
	    } else {
	      cmd = cmd.concat("*.raw");
	    }
	    String[] cdrcmd = {new String(cmd)};
	    ExecuteCmdsThread th2 = new ExecuteCmdsThread(cdrcmd, 1, 1, sv);
	    sv.addThread(th2);
	    
	    cmd = new String("rm -f ");
	    cmd = cmd.concat(general.getTmp());
	    if (cmd.endsWith("/") == false) {
	      cmd = cmd.concat("/");
	    }
	    if (settings.getRipType()) {
	      cmd = cmd.concat("*.cdr");
	    } else {
	      cmd = cmd.concat("*.raw");
	    }
	    String[] rmcmd = {new String(cmd)};
	    ExecuteCmdsThread th3 = new ExecuteCmdsThread(rmcmd, 1, 1, sv);
	    sv.addThread(th3);
	    
	    sv.go();
	  }
	}
	//System.out.println("cdcopy write !");
      } else if (state == isMixedModeCD) {
	
	if (audioTracks.legal()) {
	  cmdMKI = cmdMKI.concat(fileNames.getBurnString());
	  cmdMKI = cmdMKI.concat(" ");
	  cmdMKI = cmdMKI.concat(volumeInfo.getBurnString());
	  
	  if (bootFile.bootableCD()) {
	    cmdMKI = cmdMKI.concat(" -b ");
	    if (bootFile.whichSource()) {
	      // Harddisc
	      cmdMKI = cmdMKI.concat(bootFile.getImagePath());
	      cmdMKI = cmdMKI.concat(" -c ");
	      cmdMKI = cmdMKI.concat(bootFile.getImageCatalog());
	    } else {
	      // Floppy
	      String mkdir = new String("mkdir ");
	      mkdir = mkdir.concat(settings.getTmp());
	      mkdir = mkdir.concat("/BurnIT.tmp");
	      
	      String dd = new String("dd if=/dev/fd0 of=");
	      dd = dd.concat(settings.getTmp());
	      dd = dd.concat("/BurnIT.tmp/boot.img bs=512");
	      
	      util.execute(mkdir);
	      util.execute(dd);
	      
	      cmdMKI = cmdMKI.concat("boot.img");
	      cmdMKI = cmdMKI.concat(" -c ");
	      cmdMKI = cmdMKI.concat("boot.ctl");
	    }
	  }
	  if ((general.getOnTheFly() == false)||
	      (general.getWriteISOImage() == true)) {
	    cmdMKI = cmdMKI.concat(" -o ");
	    if (general.getWriteISOImage()) {
	      cmdMKI = cmdMKI.concat(general.getWriteDest());
	    } else {
	      cmdMKI = cmdMKI.concat(general.getTmp());
	      cmdMKI = cmdMKI.concat("/image.iso");
	    }
	  }
	  cmdMKI = cmdMKI.concat(" ");
	  cmdMKI = cmdMKI.concat(dataTrack.getBurnString());
	  if ((bootFile.bootableCD() == true)&&(bootFile.whichSource() == false)) {
	    cmdMKI = cmdMKI.concat(" ");
	    cmdMKI = cmdMKI.concat(settings.getTmp());
	    cmdMKI = cmdMKI.concat("/BurnIT.tmp");
	  }
	  
	  String[] mkcmd = {new String(cmdMKI)};
	  ExecuteCmdsThread th1 = new ExecuteCmdsThread(mkcmd, 1, 1, sv);
	  sv.addThread(th1);
	  
	  ExecuteCmdsThread th2 = audioTracks.getConvertionThread(sv);
	  if (th2 != null) {
	    sv.addThread(th2);
	  }
	  cmdCDR = cmdCDR.concat("-v dev=");
	  cmdCDR = cmdCDR.concat(settings.getWrDevNumber());
	  cmdCDR = cmdCDR.concat(" speed=");
	  cmdCDR = cmdCDR.concat(general.getSpeed());
	  cmdCDR = cmdCDR.concat(" ");
	  cmdCDR = cmdCDR.concat(settings.getComment());
	  cmdCDR = cmdCDR.concat(" -pad ");
	  cmdCDR = cmdCDR.concat(general.getTestBefore());
	  
	  cmdCDR = cmdCDR.concat(" ");
	  cmdCDR = cmdCDR.concat(general.getSessionType());
	  cmdCDR = cmdCDR.concat(" ");
	  cmdCDR = cmdCDR.concat(general.getTmp());
	  cmdCDR = cmdCDR.concat("/image.iso");
	  
	  cmdCDR = cmdCDR.concat(" -audio ");
	  cmdCDR = cmdCDR.concat(audioTracks.getBurnString());
	  
	  String[] cdcmd = {new String(cmdCDR)};
	  ExecuteCmdsThread th3 = new ExecuteCmdsThread(cdcmd, 1, 1, sv);
	  sv.addThread(th3);
	  
	  String[] rmcmd = {new String(cmdRM)};
	  ExecuteCmdsThread th4 = new ExecuteCmdsThread(rmcmd, 1, 1, sv);
	  sv.addThread(th4);
	  
	  if ((bootFile.bootableCD() == true)&&(bootFile.whichSource() == false)) {
	    String rmcmd2 = new String("rm -Rf ");
	    rmcmd2 = rmcmd2.concat(settings.getTmp());
	    rmcmd2 = rmcmd2.concat("/BurnIT.tmp");
	    
	    String[] rm = {rmcmd2};
	    ExecuteCmdsThread th5 = new ExecuteCmdsThread(rm, 1, 1, sv);
	    sv.addThread(th5);
	  }
	  
	  sv.go();  // Burn !!
	}
	//System.out.println("mixedmode write !");
      } else if (state == isMixedModeCD_wImage) {
	if (audioTracks.legal()) {
	  ExecuteCmdsThread th1 = audioTracks.getConvertionThread(sv);
	  if (th1 != null) {
	    sv.addThread(th1);
	  }
	  cmdCDR = cmdCDR.concat("-v dev=");
	  cmdCDR = cmdCDR.concat(settings.getWrDevNumber());
	  cmdCDR = cmdCDR.concat(" speed=");
	  cmdCDR = cmdCDR.concat(general.getSpeed());
	  cmdCDR = cmdCDR.concat(" ");
	  cmdCDR = cmdCDR.concat(settings.getComment());
	  cmdCDR = cmdCDR.concat(" -pad ");
	  cmdCDR = cmdCDR.concat(general.getTestBefore());
	  
	  cmdCDR = cmdCDR.concat(" ");
	  cmdCDR = cmdCDR.concat(general.getSessionType());
	  cmdCDR = cmdCDR.concat(" ");
	  
	  cmdCDR = cmdCDR.concat(imageFile.getBurnString());
	  
	  cmdCDR = cmdCDR.concat(" -audio ");
	  cmdCDR = cmdCDR.concat(audioTracks.getBurnString());
	  
	  String[] cdcmd = {new String(cmdCDR)};
	  ExecuteCmdsThread th2 = new ExecuteCmdsThread(cdcmd, 1, 1, sv);
	  sv.addThread(th2);
	  
	  sv.go();  // Burn !!
	}
	//System.out.println("mixedmode with image write !");
      } else if (state == isAdvanced) {
	if (general.getOnTheFly() == true) {
	  // Burn WITH-OUT image
	  cmdCDR = cmdCDR.concat(cdrecord.getBurnString(true));
	  
	  cmdMKI = cmdMKI.concat(mkisofs.getBurnString(true));
	  
	  util.execute_pipe(cmdMKI,cmdCDR);  // Burn !!
	} else {
	  // Burn WITH image
	  cdrecord.setImageName(mkisofs.getImageName());
	  
	  cmdMKI = cmdMKI.concat(mkisofs.getBurnString(false));
	  
	  String[] mkcmd = {new String(cmdMKI)};
	  ExecuteCmdsThread th = new ExecuteCmdsThread(mkcmd, 1, 1, sv);
	  sv.addThread(th);
	  
	  cmdCDR = cmdCDR.concat(cdrecord.getBurnString(false));
	  
	  String[] cdcmd = {new String(cmdCDR)};
	  ExecuteCmdsThread th2 = new ExecuteCmdsThread(cdcmd, 1, 1, sv);
	  sv.addThread(th2);
	  
	  sv.go();  // Burn !!
	}
	//System.out.println("advanced write !");
      }
      //System.out.println("cdwriterWrite");
    }
  }

  //Edit | Paste action performed
  public void editPaste_actionPerformed(ActionEvent e) {
    //System.out.println("editPaste");
  }

  //Edit | Delete action performed
  public void editDelete_actionPerformed(ActionEvent e) {
    if (tab.getSelectedComponent() instanceof DataTrackFrame) {
      dataTrack.deleteElement();
    } else if (tab.getSelectedComponent() instanceof AudioTracksFrame) {
      audioTracks.deleteElement();
    }
    //System.out.println("editDelete");
  }

  //Edit | DeleteAll action performed
  public void editDeleteAll_actionPerformed(ActionEvent e) {
    if (tab.getSelectedComponent() instanceof DataTrackFrame) {
      dataTrack.deleteAll();
    } else if (tab.getSelectedComponent() instanceof AudioTracksFrame) {
      audioTracks.deleteAll();
    }
    //System.out.println("editDeleteAll");
  }

  //Edit | MoveUp action performed
  public void editMoveUp_actionPerformed(ActionEvent e) {
    if (tab.getSelectedComponent() instanceof DataTrackFrame) {
      dataTrack.moveUp();
    } else if (tab.getSelectedComponent() instanceof AudioTracksFrame) {
      audioTracks.moveUp();
    }
    //System.out.println("editMoveUp");
  }

  //Edit | MoveDown action performed
  public void editMoveDown_actionPerformed(ActionEvent e) {
    if (tab.getSelectedComponent() instanceof DataTrackFrame) {
      dataTrack.moveDown();
    } else if (tab.getSelectedComponent() instanceof AudioTracksFrame) {
      audioTracks.moveDown();
    }
    //System.out.println("editMoveDown");
  }

  //Edit | Reparent action performed
  public void editReparent_actionPerformed(ActionEvent e) {
    if (tab.getSelectedComponent() instanceof DataTrackFrame) {
      dataTrack.reparent();
    }
    //System.out.println("editReparent");
  }

  //Edit | Refresh action performed
  public void editRefresh_actionPerformed(ActionEvent e) {
    general.refresh();
    imageFile.refresh();
    loadContents.refresh();
    fileNames.refresh();
    volumeInfo.refresh();
    dataTrack.refresh();
    audioTracks.refresh();
    mkisofs.refresh();
    cdrecord.refresh();
    disc2disc.refresh();
    this.refresh();
    //System.out.println("editRefresh");
  }

  //View | Toolbar action performed
  public void viewToolbar_actionPerformed(ActionEvent e) {
    //System.out.println("viewToolbar");
    updateScreen();
  }

  //View | Statusbar action performed
  public void viewStatusbar_actionPerformed(ActionEvent e) {
    //System.out.println("viewStatusbar");
    updateScreen();
  }

  //View | CDPlayer action performed
  public void viewCDPlayer_actionPerformed(ActionEvent e) {
    int i;
    String tmp;
    Vector v,t;
    Vector no = new Vector();
    Vector tracks = new Vector();

    if (tab.getSelectedComponent() instanceof AudioTracksFrame) {
      // Virtual CD play
      v = audioTracks.getTracks();

      for (i=0;i<v.size();i++) {
	t = (Vector)v.elementAt(i);
	tmp = new String("Virtual track ");
	tmp = tmp.concat((String)t.elementAt(0));
	no.addElement(tmp);
      }
      tracks.addElement(no);
      for (i=0;i<v.size();i++) {
	t = (Vector)v.elementAt(i);
	tracks.addElement(t.elementAt(1));
      }

      CDPlayer cdp = new CDPlayer(settings,util,tracks); 
    } else {
      // Real CD play
      v = util.getTracks();
      for (i=0;i<v.size();i++) {
	t = (Vector)v.elementAt(i);
	tmp = new String("Track ");
	tmp = tmp.concat((String)t.elementAt(0));
	no.addElement(tmp);
      }
      tracks.addElement(no);
      for (i=0;i<v.size();i++) {
	tracks.addElement(new String(""));
      }
      CDPlayer cdp = new CDPlayer(settings,util,tracks); 
    }
  }

  //View | FileManager action performed
  public void viewFileManager_actionPerformed(ActionEvent e) {
    if (fm == null) {
      fm = new FileManager(this,settings,util);
    } else {
      fm.setVisible(true);
    }
  }

  //Help | Help action performed
  public void helpHelp_actionPerformed(ActionEvent e) {
    //System.out.println("helpHelp");
  }

  //Help | About action performed
  public void helpAbout_actionPerformed(ActionEvent e) {
    AboutBox frame = new AboutBox(util);
    //System.out.println("helpAbout");
  }
}
