/*
 *  Copyright (C) 1998,1999  Jesper Pedersen <jews@imada.ou.dk>
 *  This code is released under GNU GPL version 2 or later
 */

import com.sun.java.swing.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.border.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

class CDPlayer extends JFrame implements ActionListener {
  protected JPanel       dataPanel;
  protected JPanel       buttonPanel;

  protected JButton      play;
  protected JButton      stop;
  protected JButton      prev;
  protected JButton      next;
  protected JButton      exit;

  protected JComboBox    list;

  protected Vector       tracks;
  protected Vector       tckno;

  protected boolean      playing;

  protected ExecuteThread th;

  protected Utils        utils;
  protected Settings     settings;

  public CDPlayer(Settings set, Utils utl, Vector v) {
    String s;

    this.setTitle("BurnIT: CD Player");

    setSize(new Dimension(350,50));

    this.getContentPane().setLayout(new BorderLayout()); 

    settings = set;
    utils = utl;
    tracks = v;
    
    playing = false;

    dataPanel = new JPanel();
    dataPanel.setLayout(new BorderLayout());

    list = new JComboBox();

    if (tracks.size() > 0) {
      tckno = (Vector)tracks.firstElement(); 
      tracks.removeElementAt(0);
      
      for (int i=0; i<tckno.size(); i++) {
	s = (String)tckno.elementAt(i);
	list.addItem(s);
      }
    } else {
      list.addItem(new String("No tracks"));
    }

    dataPanel.add(list, BorderLayout.CENTER);

    buttonPanel = new JPanel();
    buttonPanel.setLayout(new GridLayout(1,5));

    exit = new JButton("Exit");
    exit.addActionListener(this);
    buttonPanel.add(exit);

    stop = new JButton("Stop");
    stop.addActionListener(this);
    buttonPanel.add(stop);

    prev = new JButton("Prev");
    prev.addActionListener(this);
    buttonPanel.add(prev);

    play = new JButton("Play");
    play.addActionListener(this);
    buttonPanel.add(play);

    next = new JButton("Next");
    next.addActionListener(this);
    buttonPanel.add(next);

    this.getContentPane().add(dataPanel, BorderLayout.CENTER);
    this.getContentPane().add(buttonPanel, BorderLayout.SOUTH);

    this.setVisible(true);
  }

  protected void killPlayer() {
    playing = false;
    th.killProcess();
    try {
      th.join();
    } catch (Exception e) {
    }
    th = null;
  }

  public void threadFinished() {
    int index;

    if (playing) {
      playing = false;
      th = null;
      index = list.getSelectedIndex();
      if (index+1 < list.getItemCount()) {
	list.setSelectedIndex(index+1);
	play(index+1);
      }
    }
  }

  protected void play(int indx) {
    String s;
    String cmd;

    s = (String)tracks.elementAt(indx);
    if (s.compareTo("") == 0) {
      // From CD
      if (settings.getRipType()) {
	// Cdda2Wav
	cmd = new String(settings.getAudioRip());
	cmd = cmd.concat(" -D ");
	cmd = cmd.concat(settings.getRdDevNumber());
	cmd = cmd.concat(" -x -H -e -N");
	cmd = cmd.concat(" -t ");
	cmd = cmd.concat((String)tckno.elementAt(indx));
	
	th = new ExecuteThread(cmd,false,this);
	playing = true;
      } else {
	// Cdparanoia 
	
      }
    } else {
      // From file
      if (s.toUpperCase().endsWith(".CDR")) {
	if (settings.getCDRPlayer().compareTo("") != 0) {
	  cmd = new String(settings.getCDRPlayer());
	  cmd = cmd.concat(" ");
	  cmd = cmd.concat(s);
	  th = new ExecuteThread(cmd,false,this);
	  playing = true;
	}
      } else if (s.toUpperCase().endsWith(".WAV")) {
	if (settings.getWAVPlayer().compareTo("") != 0) {
	  cmd = new String(settings.getWAVPlayer());
	  cmd = cmd.concat(" ");
	  cmd = cmd.concat(s);
	  th = new ExecuteThread(cmd,false,this);
	  playing = true;
	}
      } else if (s.toUpperCase().endsWith(".MP3")) {
	if (settings.getMP3Player().compareTo("") != 0) {
	  cmd = new String(settings.getMP3Player());
	  cmd = cmd.concat(" ");
	  cmd = cmd.concat(s);
	  th = new ExecuteThread(cmd,false,this);
	  playing = true;
	}
      }
    }
    list.repaint();
  }

  public void actionPerformed(ActionEvent e) {
    int index;

    // The Play button was clicked on, handle it ...
    if (e.getSource() == play) {
      if (playing == false) {
	index = list.getSelectedIndex();
	play(index);
      }
    // The Stop button was clicked on, handle it ...
    } else if (e.getSource() == stop) {
      if (playing) {
	killPlayer();
      }
    // The Next button was clicked on, handle it ...
    } else if (e.getSource() == next) {
      index = list.getSelectedIndex();
      if (index+1 < list.getItemCount()) {
	if (playing) {
	  killPlayer();
	}
	list.setSelectedIndex(index+1);
	play(index+1);
      }
    // The Prev button was clicked on, handle it ...
    } else if (e.getSource() == prev) {
      index = list.getSelectedIndex();
      if (index-1 >= 0) {
	if (playing) {
	  killPlayer();
	}
	list.setSelectedIndex(index-1);
	play(index-1);
      }
    // The Exit button was clicked on, handle it ...
    } else if (e.getSource() == exit) {
      if (playing) {
	killPlayer();
      }
      setVisible(false);
      dispose();
    }
  }
}
