/*
 *  Copyright (C) 1998,1999  Jesper Pedersen <jews@imada.ou.dk>
 *  This code is released under GNU GPL version 2 or later
 */

import com.sun.java.swing.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.border.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;


public class DataTrackFrame extends JPanel {
    protected JTable       tableView;
    protected JScrollPane  scrollpane;
    protected JScrollPane  tableAggregate;
    protected Vector       tableData;
    protected Vector       names;
    protected JPanel       borderPane;
    protected JPanel       borderPane2;
    protected JPanel       info;
    protected JLabel       totalsize;
    protected JLabel       totaldirs;
    protected JLabel       totalfiles;
    protected JProgressBar bar;
    protected int          max;

    protected CDSize      cdsize;

    protected Utils utils;
    protected Settings set;

    public DataTrackFrame(Utils utl,Settings st) {
        this.setLayout(new BorderLayout());

	utils = utl;
	set = st;

	borderPane = new JPanel();
	borderPane.setBorder(new EmptyBorder(2,2,2,2));
	borderPane.setLayout(new BorderLayout());
	
	borderPane2 = new JPanel();
	borderPane2.setBorder(new MatteBorder(1,1,1,1,Color.gray));
	borderPane2.setLayout(new BorderLayout());

	info = new JPanel();
	info.setBorder(new EmptyBorder(2,2,2,2));
	info.setLayout(new GridLayout(1,3));

	totalsize = new JLabel();
	totalsize.setForeground(Color.black);
	setTotalSize("0.0");

	totaldirs = new JLabel();
	totaldirs.setForeground(Color.black);
	setTotalDirs();

	totalfiles = new JLabel();
	totalfiles.setForeground(Color.black);
	setTotalFiles();

	bar = new JProgressBar();
	if (set.getType().compareTo("DVD-R") != 0) {
	  // CD-R or CD-RW
	  max = 640 + 160;
	} else {
	  max = 2600 + 400;
	}
	bar.setMaximum(max);

	info.add(totalsize);
	info.add(totaldirs);
	info.add(totalfiles);

        borderPane2.add(info, BorderLayout.CENTER);
        borderPane.add(borderPane2, BorderLayout.CENTER);

        tableAggregate = createTable();
        this.add(borderPane, BorderLayout.NORTH);
        this.add(tableAggregate, BorderLayout.CENTER);
	this.add(bar,BorderLayout.SOUTH);
    }


    private JScrollPane createTable() {
      tableData = new Vector();

      names = new Vector();
      names.addElement(new String("Name"));
      names.addElement(new String("Size"));
      names.addElement(new String("Type"));
      names.addElement(new String("Modified"));
      names.addElement(new String("Destination"));
      
      TableModel tableModel = new AbstractTableModel() {
	public int getColumnCount() { 
	  return names.size();
	}
	public int getRowCount() {
	  return tableData.size();
	}
	public Object getValueAt(int row, int col) {
	  Vector tmp; 
	  tmp = (Vector)tableData.elementAt(row);
	  return tmp.elementAt(col);
	}
	public String getColumnName(int column) {
	  return (String)names.elementAt(column);
	}
	public Class getColumnClass(int c) {
	  return getValueAt(0, c).getClass();
	}
	public boolean isCellEditable(int row, int col) {
	  if (col == 4) {
	    return true;
	  }
	  return false;
	}
	public void setValueAt(Object aValue, int row, int column) { 
	  Vector tmp;
	  tmp = (Vector)tableData.elementAt(row);
	  tmp.setElementAt(aValue,column);
	  tableData.setElementAt(tmp,row);
	}
      };

      // Create the table
      tableView = new JTable(tableData,names);
      tableView.setModel(tableModel);

      //tableView.setRowHeight(20);
      tableView.setShowHorizontalLines(false);
      tableView.setShowVerticalLines(false);
      tableView.getTableHeader().setReorderingAllowed(false);
      tableView.setSelectionMode(0);
      
      scrollpane = new JScrollPane(tableView);
      return scrollpane;
    }

    public void setTotalSize(String s) {
      String tmp;

      tmp = new String("Total Size: ");
      tmp = tmp.concat(s);
      tmp = tmp.concat(" Mb");
      totalsize.setText(tmp);
    }

    public void setTotalDirs() {
      Vector v;
      String tmp;
      int i,total,current;

      total = 0;
      if (tableData != null) {
	for (i=0;i<tableData.size();i++) {
	  v = (Vector)tableData.elementAt(i);
	  tmp = (String)v.elementAt(0);
	  current = utils.getNumOfDirs(tmp);
	  total = total + current;
	}
      }
      tmp = new String("Total Dirs: ");
      tmp = tmp.concat(new Integer(total).toString());
      totaldirs.setText(tmp);
    }

    public void setTotalFiles() {
      Vector v;
      String tmp;
      int i,total,current;

      total = 0;
      if (tableData != null) {
	for (i=0;i<tableData.size();i++) {
	  v = (Vector)tableData.elementAt(i);
	  tmp = (String)v.elementAt(0);
	  current = utils.getNumOfFiles(tmp);
	  total = total + current;
	}
      }
      tmp = new String("Total Files: ");
      tmp = tmp.concat(new Integer(total).toString());
      totalfiles.setText(tmp);
    }

    public void addPath(String s) {
      Vector newEntry;
      String name,size,type,modified,dest;

      newEntry = new Vector();
      name = new String(s);
      size = new String(utils.getDirSize(s));
      type = new String("Directory");
      modified = new String(utils.getModifiedDate(s));
      dest = new String("/");

      newEntry.addElement(name);
      newEntry.addElement(size);
      newEntry.addElement(type);
      newEntry.addElement(modified);
      newEntry.addElement(dest);

      tableData.addElement(newEntry);
      tableView.repaint();

      updateTable();
    }

    private void updateTable() {
      Vector v;
      String tmp;
      int i;
      float value,f;

      value = 0;
      for (i=0;i<tableData.size();i++) {
	v = (Vector)tableData.elementAt(i);
	tmp = (String)v.elementAt(1);
	if (tmp.compareTo("") != 0) {
	  f = Float.valueOf(tmp).floatValue();
	  value = value + f;
	}
      }
      cdsize.setDataTrackSize(value);
    }

    public void updateValues() {
      String tmp;
      int i;
      float value;

      value = cdsize.getTotalSize();
      tmp = new String(new Float(value).toString());
      tmp = tmp.substring(0,tmp.indexOf('.')+2);
      setTotalSize(tmp);
      setTotalFiles();
      setTotalDirs();

      i = new Float(value).intValue();
      if (i <= max) {
	bar.setValue(i);
      } else {
	bar.setValue(max);
      }
      if (max == 800) {
	if (i <= 640) {
	  bar.setForeground(Color.green);
	} else if ((i > 640)&&(i <= 700)) {
	  bar.setForeground(Color.yellow);
	} else {
	  bar.setForeground(Color.red);
	}
      } else {
	if (i <= 2600) {
	  bar.setForeground(Color.green);
	} else {
	  bar.setForeground(Color.red);
	}
      }
    }

    public Vector getDirs() {
      Vector result = new Vector();
      Vector v;
      String s;

      for (int i=0;i<tableData.size();i++) {
	v = (Vector)tableData.elementAt(i);
	s = (String)v.elementAt(0);
	result.addElement(s);
      }
      return result;
    }

    public void deleteElement() {
      if ((tableData.size() > 0)&&(tableView.getSelectedRow() != -1)) {
	tableData.removeElementAt(tableView.getSelectedRow());
	tableView.repaint();
	updateTable();
      }
    }

    public void deleteAll() {
      if (tableData.size() > 0) {
	tableData.removeAllElements();
	tableView.repaint();
	updateTable();
      }
    }

    public void moveUp() {  
      int select;
      Vector v1,v2;

      select = tableView.getSelectedRow();
      
      if (select > 0) {
	v1 = (Vector)tableData.elementAt(select);
	v2 = (Vector)tableData.elementAt(select-1);
	tableData.setElementAt(v2,select);
	tableData.setElementAt(v1,select-1);
	tableView.repaint();
      }
    }

    public void moveDown() {  
      int select;
      Vector v1,v2;

      select = tableView.getSelectedRow();

      if (select < tableData.size()-1) {
	v1 = (Vector)tableData.elementAt(select);
	v2 = (Vector)tableData.elementAt(select+1);
	tableData.setElementAt(v2,select);
	tableData.setElementAt(v1,select+1);
	tableView.repaint();
      }
    }

    // Refresh screen
    public void refresh() {

    }

    public void setCDSizeObject(CDSize cd) {
      cdsize = cd;
    }

    public void reparent() {
      boolean ret;
      int select;

      select = tableView.getSelectedRow();

      if (select != -1) {
	try {
	  ret = tableView.editCellAt(select,4);
	} catch (IllegalArgumentException e) {
	}
      }
    }

    public String getBurnString() {
      String burn_dir = new String("");
      Vector v;
      String source_dir,dest_dir;

      for (int i=0;i<tableData.size();i++) {
	v = (Vector)tableData.elementAt(i);
	source_dir = (String)v.elementAt(0);
	dest_dir = (String)v.elementAt(4);
	if (dest_dir.compareTo("/") != 0) {
	  burn_dir = burn_dir.concat(dest_dir);
	  if (dest_dir.endsWith("/") == false) {
	    burn_dir = burn_dir.concat("/");
	  }
	  burn_dir = burn_dir.concat("=");
	}
	burn_dir = burn_dir.concat(source_dir);
	burn_dir = burn_dir.concat(" ");
      }

      return burn_dir;
    }
}
