/*
 *  Copyright (C) 1998,1999  Jesper Pedersen <jews@imada.ou.dk>
 *  This code is released under GNU GPL version 2 or later
 */

import com.sun.java.swing.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.border.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class Disc2DiscFrame extends JPanel implements ItemListener {
  public static final int isCDROMType = 0;
  public static final int isAudioType = 1;
  public static final int isERRORType = 2;

  protected JPanel     dataPanel;

  protected JPanel     panel1;
  protected JPanel     panel2;
  protected JPanel     panel3;
    
  protected JLabel     rdLabel;
  protected JComboBox  rdDev;
  protected JPanel     rdPanel;    

  protected JLabel     wrLabel;
  protected JComboBox  wrDev;
  protected JPanel     wrPanel;    
  
  protected JLabel     typeLabel;
  protected JComboBox  type;
  protected JPanel     typePanel;    
  
  protected Vector     devices;

  protected Utils      utils;
  protected Settings   settings;
  
  public Disc2DiscFrame(Utils utl,Settings set) {
    Vector elm;
    String tmp;
    int i;

    utils = utl;
    settings = set;
    
    this.setLayout(new BorderLayout());

    dataPanel = new JPanel();
    dataPanel.setLayout(new GridLayout(3,1));

    panel1 = new JPanel();
    panel1.setLayout(new BorderLayout());
    panel1.setBorder(new EmptyBorder(5,5,5,5));

    panel2 = new JPanel();
    panel2.setLayout(new BorderLayout());
    panel2.setBorder(new EmptyBorder(5,5,5,5));

    panel3 = new JPanel();
    panel3.setLayout(new BorderLayout());
    panel3.setBorder(new EmptyBorder(5,5,5,5));

    rdLabel = new JLabel("Source device");
    rdLabel.setForeground(Color.black);
    rdLabel.setBackground(Color.gray);
    panel1.add(rdLabel, BorderLayout.NORTH);

    wrLabel = new JLabel("Destination device");
    wrLabel.setForeground(Color.black);
    wrLabel.setBackground(Color.gray);
    panel2.add(wrLabel, BorderLayout.NORTH);

    typeLabel = new JLabel("CD Type");
    typeLabel.setForeground(Color.black);
    typeLabel.setBackground(Color.gray);
    panel3.add(typeLabel, BorderLayout.NORTH);

    rdDev = new JComboBox();
    wrDev = new JComboBox();
    devices = utils.getSCSIDevices();
    for (i=0; i<devices.size();i++) {
      elm = (Vector)devices.elementAt(i);
      tmp = (String)elm.elementAt(2);
      rdDev.addItem(tmp);
      wrDev.addItem(tmp);
    }
    rdDev.setSelectedItem(settings.getRdModel());
    wrDev.setSelectedItem(settings.getWrModel());

    rdPanel = new JPanel();
    rdPanel.setLayout(new BorderLayout());

    rdPanel.add(rdDev, BorderLayout.WEST);

    wrPanel = new JPanel();
    wrPanel.setLayout(new BorderLayout());

    wrPanel.add(wrDev, BorderLayout.WEST);

    panel1.add(rdPanel, BorderLayout.SOUTH);
    panel2.add(wrPanel, BorderLayout.SOUTH);

    type = new JComboBox();
    type.addItem(new String("CD-ROM"));
    type.addItem(new String("AUDIO"));

    typePanel = new JPanel();
    typePanel.setLayout(new BorderLayout());

    typePanel.add(type, BorderLayout.WEST);
    panel3.add(typePanel, BorderLayout.SOUTH);

    dataPanel.add(panel1);
    dataPanel.add(panel2);
    dataPanel.add(panel3);

    this.add(dataPanel, BorderLayout.NORTH);
  }
  
  // Refresh screen
  public void refresh() {
    repaint();
  }
  
  public boolean legal() {
    if (rdDev.getSelectedIndex() != wrDev.getSelectedIndex()) {
      return true;
    }
    return false;
  }

  public int getType() {
    String s;

    s = (String)type.getSelectedItem();
    if (s.compareTo("CD-ROM") == 0) {
      return isCDROMType;
    } else if (s.compareTo("AUDIO") == 0) { 
      return isAudioType;
    }
    return isERRORType;
  }

  public String getWriteDev() {
    String ret = new String("");
    Vector elm;

    elm = (Vector)devices.elementAt(wrDev.getSelectedIndex());
    ret = (String)elm.elementAt(0);

    return ret;
  }

  public String getReadDev() {
    String ret = new String("/dev/sr");
    Integer num;
    String nums;

    num = new Integer(rdDev.getSelectedIndex());
    nums = num.toString();
    ret = ret.concat(nums);
    return ret;
  }

  public String getReadDevNumber() {
    String ret = new String("");
    Vector elm;

    elm = (Vector)devices.elementAt(rdDev.getSelectedIndex());
    ret = (String)elm.elementAt(0);

    return ret;
  }

  public String getReadGen() {
    String ret = new String("/dev/sg");
    Integer num;
    String nums;

    num = new Integer(rdDev.getSelectedIndex());
    nums = num.toString();
    ret = ret.concat(nums);

    return ret;
  }

  public void setSource(int i) {
    rdDev.setSelectedIndex(i);
  }

  public int getSource() {
    return rdDev.getSelectedIndex();
  }

  public void setDest(int i) {
    wrDev.setSelectedIndex(i);
  }

  public int getDest() {
    return wrDev.getSelectedIndex();
  }

  public void setCDType(int i) {
    type.setSelectedIndex(i);
  }

  public int getCDType() {
    return type.getSelectedIndex();
  }

  public void itemStateChanged(ItemEvent event) {
    // If a JComboBox was clicked on, repaint !
    if (event.getSource() instanceof JComboBox) {
      repaint();
    }
  }
}

