/*
 *  Copyright (C) 1998,1999  Jesper Pedersen <jews@imada.ou.dk>
 *  This code is released under GNU GPL version 2 or later
 */

import com.sun.java.swing.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.border.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

class DiscInfo extends JFrame implements ActionListener {
  protected JPanel       dataPane;
  protected JPanel       trackPane;
  protected JPanel       trackPaneTop;
  protected JPanel       trackPaneTopLeft;
  protected JPanel       trackPaneTopRight;
  protected JPanel       trackPaneBot;
  protected JPanel       encodePane;
  protected JPanel       encodePane1;
  protected JPanel       encodePane2;
  protected JPanel       panel1;
  protected JPanel       panel2;
  protected JPanel       panel3;
  protected JPanel       panel4;
  protected JPanel       panel5;

  protected JButton      select;
  protected JButton      refresh;
  protected JButton      tray;

  protected JLabel       destLabel;
  protected JTextField   dest;

  protected JPanel       buttonPane;
  protected JPanel       buttonPane1;
  protected JPanel       buttonPane2;
  protected JButton      run;
  protected JButton      play;
  protected JButton      cancel;

  protected JTabbedPane  tab;
  protected JTable       trackTable;
  protected JScrollPane  scrollTrackPane;

  protected Vector       tableData;
  protected Vector       names;

  protected JRadioButton k64;
  protected JRadioButton k112;
  protected JRadioButton k128;
  protected JRadioButton k256;
  protected JRadioButton kother;
  protected ButtonGroup  buttonGroup;
  protected JTextField   rate;

  protected JCheckBox    Delete;
  protected JCheckBox    MPEG;
  protected JCheckBox    CDR;
  
  protected boolean      open;

  protected Utils        utils;
  protected Settings     settings;

  public DiscInfo(Settings set, Utils utl) {
    this.setTitle("BurnIT: Disc Info and Tools");

    setSize(new Dimension(500,310));

    this.getContentPane().setLayout(new BorderLayout()); 

    settings = set;
    utils = utl;

    open = false;

    dataPane = new JPanel();
    dataPane.setLayout(new BorderLayout());

    trackPane = new JPanel();
    trackPane.setLayout(new BorderLayout());

    trackPaneTop = new JPanel();
    trackPaneTop.setLayout(new GridLayout(1,2));

    trackPaneTopLeft = new JPanel();
    trackPaneTopLeft.setLayout(new FlowLayout(FlowLayout.LEFT));

    select = new JButton("  Select All  ");
    select.addActionListener(this);
    trackPaneTopLeft.add(select);

    trackPaneTopRight = new JPanel();
    trackPaneTopRight.setLayout(new FlowLayout(FlowLayout.RIGHT));

    refresh = new JButton("Refresh");
    refresh.addActionListener(this);
    trackPaneTopRight.add(refresh);

    tray = new JButton("Tray");
    tray.addActionListener(this);
    trackPaneTopRight.add(tray);

    trackPaneBot = new JPanel();
    trackPaneBot.setLayout(new FlowLayout(FlowLayout.LEFT));

    destLabel = new JLabel("Destination:");
    destLabel.setForeground(Color.black);
    trackPaneBot.add(destLabel);

    dest = new JTextField(40);
    dest.setText(settings.getTmp());
    trackPaneBot.add(dest);

    names = new Vector();
    names.addElement(new String("Track"));
    names.addElement(new String("Start"));
    names.addElement(new String("Length"));
    names.addElement(new String("Type"));

    TableModel tableModel = new AbstractTableModel() {
      public int getColumnCount() { 
	return names.size();
      }
      public int getRowCount() {
	return tableData.size();
      }
      public Object getValueAt(int row, int col) {
	Vector tmp; 
	tmp = (Vector)tableData.elementAt(row);
	return tmp.elementAt(col);
      }
      public String getColumnName(int column) {
	return (String)names.elementAt(column);
      }
      public Class getColumnClass(int c) {
	return getValueAt(0, c).getClass();
      }
      public boolean isCellEditable(int row, int col) {
	return false;
      }
      public void setValueAt(Object aValue, int row, int column) { 
	Vector tmp;
	tmp = (Vector)tableData.elementAt(row);
	tmp.setElementAt(aValue,column);
	tableData.setElementAt(tmp,row);
      }
    };
    tableData = new Vector();

    trackTable = new JTable(tableData,names);
    trackTable.setModel(tableModel);
    trackTable.getTableHeader().setReorderingAllowed(false);
    trackTable.setSelectionMode(2);

    scrollTrackPane = new JScrollPane(trackTable);
 
    trackPaneTop.add(trackPaneTopLeft);
    trackPaneTop.add(trackPaneTopRight);

    trackPane.add(trackPaneTop, BorderLayout.NORTH);
    trackPane.add(scrollTrackPane, BorderLayout.CENTER);
    trackPane.add(trackPaneBot, BorderLayout.SOUTH);
    
    encodePane = new JPanel();
    encodePane.setLayout(new BorderLayout());
    encodePane.setBorder(new EmptyBorder(5,5,5,5));

    encodePane1 = new JPanel();
    encodePane1.setLayout(new BorderLayout());

    encodePane2 = new JPanel();
    encodePane2.setLayout(new GridLayout(5,1));

    panel1 = new JPanel();
    panel1.setLayout(new FlowLayout(FlowLayout.LEFT));

    panel2 = new JPanel();
    panel2.setLayout(new FlowLayout(FlowLayout.LEFT));

    panel3 = new JPanel();
    panel3.setLayout(new FlowLayout(FlowLayout.LEFT));

    panel4 = new JPanel();
    panel4.setLayout(new FlowLayout(FlowLayout.LEFT));

    panel5 = new JPanel();
    panel5.setLayout(new FlowLayout(FlowLayout.LEFT));

    Delete = new JCheckBox("Delete files");
    encodePane1.add(Delete,BorderLayout.NORTH);
    CDR = new JCheckBox("Rip as CDR file",true);
    encodePane1.add(CDR,BorderLayout.CENTER);
    MPEG = new JCheckBox("MPEG Encoding");
    encodePane1.add(MPEG,BorderLayout.SOUTH);

    buttonGroup = new ButtonGroup();
    k64 = new JRadioButton(" 64k");
    buttonGroup.add(k64);
    k112 = new JRadioButton("112k");
    buttonGroup.add(k112);
    k128 = new JRadioButton("128k",true);
    buttonGroup.add(k128);
    k256 = new JRadioButton("256k");
    buttonGroup.add(k256);
    kother = new JRadioButton("Other");
    buttonGroup.add(kother);

    rate = new JTextField(30);

    panel1.add(k64);

    panel2.add(k112);

    panel3.add(k128);

    panel4.add(k256);

    panel5.add(kother);
    panel5.add(rate);

    encodePane2.add(panel1);
    encodePane2.add(panel2);
    encodePane2.add(panel3);
    encodePane2.add(panel4);
    encodePane2.add(panel5);

    encodePane.add(encodePane1,BorderLayout.NORTH);
    encodePane.add(encodePane2,BorderLayout.CENTER);
    
    tab = new JTabbedPane();

    tab.addTab("Tracks",trackPane);
    tab.addTab("Encode",encodePane);

    dataPane.add(tab, BorderLayout.CENTER);

    buttonPane = new JPanel();
    buttonPane.setLayout(new GridLayout(1,2));
    buttonPane.setBorder(new EmptyBorder(5,5,5,5));

    buttonPane1 = new JPanel();
    buttonPane1.setLayout(new FlowLayout(FlowLayout.LEFT));

    buttonPane2 = new JPanel();
    buttonPane2.setLayout(new FlowLayout(FlowLayout.RIGHT));

    cancel = new JButton("Cancel");
    cancel.addActionListener(this);
    buttonPane1.add(cancel);

    play = new JButton("Play");
    play.addActionListener(this);
    buttonPane2.add(play);

    run = new JButton("Run");
    run.addActionListener(this);
    buttonPane2.add(run);

    buttonPane.add(buttonPane1);
    buttonPane.add(buttonPane2);

    this.getContentPane().add(dataPane, BorderLayout.CENTER);
    this.getContentPane().add(buttonPane, BorderLayout.SOUTH);

    this.setVisible(true);
    getTracks();
  }

  protected void getTracks() {
    if (trackTable.getRowCount() > 0) {
      trackTable.removeRowSelectionInterval(0,trackTable.getRowCount());
    }
    tableData = utils.getTracks();
    trackTable.repaint();
  }


  public void actionPerformed(ActionEvent e) {
    // The Run button was clicked on, handle it ...
    if (e.getSource() == run) {
      int i,j,total,index;
      String s,enc,cmd;
      String[] cmds;
      Supervisor sv = new Supervisor();

      // Buggy JTable.getSelectedRows()
      total = 0;
      for (i=0;i<trackTable.getRowCount();i++) {
	if (trackTable.isRowSelected(i)) {
	  total++;
	}
      }

      cmds = new String[total];

      index=0;
      for (i=0; i<trackTable.getRowCount(); i++) {
	if (trackTable.isRowSelected(i)) {
	  // Read the file
	  cmd = new String(settings.getAudioRip());
	  if (settings.getRipType()) {
	    // Cdda2wav
	    cmd = cmd.concat(" -D ");
	    cmd = cmd.concat(settings.getRdDevNumber());
	    cmd = cmd.concat(" -S ");
	    cmd = cmd.concat(settings.getRdRead());
	    cmd = cmd.concat(" -x -H -O ");
	    if (CDR.isSelected()) {
	      cmd = cmd.concat("cdr");
	    } else {
	      cmd = cmd.concat("wav");
	    }
	    cmd = cmd.concat(" -t ");
	  } else {
	    // Cdparanoia
	    cmd = cmd.concat(" -v -g ");
	    cmd = cmd.concat(settings.getRdGen());
	    if (CDR.isSelected()) {
	      cmd = cmd.concat(" -R ");
	    } else {
	      cmd = cmd.concat(" -r ");
	    }
	  }
	  s = (String)trackTable.getValueAt(i,0);
	  cmd = cmd.concat(s);
	  if (s.charAt(0) == ' ') {
	    s = s.replace(' ','0');
	  }
	  cmd = cmd.concat(" ");
	  cmd = cmd.concat(dest.getText());
	  cmd = cmd.concat("/");
	  cmd = cmd.concat(s);
	  if (CDR.isSelected()) {
	    cmd = cmd.concat(".cdr");
	  } else {
	    cmd = cmd.concat(".wav");
	  }
	  
	  cmds[index] = cmd;
	  index++;
	}
      }
      ExecuteCmdsThread th = new ExecuteCmdsThread(cmds, 1, 1, sv);
      sv.addThread(th);
      
      if (MPEG.isSelected()) {
	if (Delete.isSelected()) {
 	  cmds = new String[2*total];
	} else {
 	  cmds = new String[total];
	}
	index = 0;
	for (i=0; i<trackTable.getRowCount(); i++) {
	  if (trackTable.isRowSelected(i)) {
	    // Encode the file
	    cmd = new String(settings.getEncoder());
	    
	    if (settings.getEncoderType()) {
	      // Encode type encoder (options first)
	      cmd = cmd.concat(" ");
	      if (k64.isSelected()) {
		cmd = cmd.concat("64");
	      } else if (k112.isSelected()) {
		cmd = cmd.concat("112");
	      } else if (k128.isSelected()) {
		cmd = cmd.concat("128");
	      } else if (k256.isSelected()) {
		cmd = cmd.concat("256");
	      } else if (kother.isSelected()) {
		if (rate.getText().compareTo("") != 0) {
		  cmd = cmd.concat(rate.getText());
		} else {
		  cmd = cmd.concat("128");
		}
	      }
	      s = (String)trackTable.getValueAt(i,0);
	      if (s.charAt(0) == ' ') {
		s = s.replace(' ','0');
	      }
	      cmd = cmd.concat(" ");
	      cmd = cmd.concat(dest.getText());
	      cmd = cmd.concat("/");
	      cmd = cmd.concat(s);
	      if (CDR.isSelected()) {
		cmd = cmd.concat(".cdr");
	      } else {
		cmd = cmd.concat(".wav");
	      }
	      cmd = cmd.concat(" ");
	      cmd = cmd.concat(dest.getText());
	      cmd = cmd.concat("/");
	      cmd = cmd.concat(s);
	      cmd = cmd.concat(".mp3");
	    } else {
	      // l3enc type encoder (options last)
	      j = cmd.indexOf(" ",0);
	      enc = new String(cmd);
	      enc = enc.substring(j);
	      cmd = cmd.substring(0,j);
	      s = (String)trackTable.getValueAt(i,0);
	      if (s.charAt(0) == ' ') {
		s = s.replace(' ','0');
	      }
	      cmd = cmd.concat(" ");
	      cmd = cmd.concat(dest.getText());
	      cmd = cmd.concat("/");
	      cmd = cmd.concat(s);
	      if (CDR.isSelected()) {
		cmd = cmd.concat(".cdr");
	      } else {
		cmd = cmd.concat(".wav");
	      }
	      cmd = cmd.concat(" ");
	      cmd = cmd.concat(dest.getText());
	      cmd = cmd.concat("/");
	      cmd = cmd.concat(s);
	      cmd = cmd.concat(".mp3");
	      cmd = cmd.concat(enc);
	      cmd = cmd.concat(" ");
	      if (k64.isSelected()) {
		cmd = cmd.concat("64");
	      } else if (k112.isSelected()) {
		cmd = cmd.concat("112");
	      } else if (k128.isSelected()) {
		cmd = cmd.concat("128");
	      } else if (k256.isSelected()) {
		cmd = cmd.concat("256");
	      } else if (kother.isSelected()) {
		if (rate.getText().compareTo("") != 0) {
		  cmd = cmd.concat(rate.getText());
		} else {
		  cmd = cmd.concat("128");
		}
	      }
	    }
	    cmds[index] = cmd;
	    index++;

	    if (Delete.isSelected()) {
	      cmd = new String("rm -f ");
	      cmd = cmd.concat(dest.getText());
	      if (cmd.endsWith("/") == false) {
		cmd = cmd.concat("/");
	      }
	      cmd = cmd.concat(s);
	      if (CDR.isSelected()) {
		cmd = cmd.concat(".cdr");
	      } else {
		cmd = cmd.concat(".wav");
	      }
	      
	      cmds[index] = cmd;
	      index++;
	    }
	  }
	}
 	int number;
 	if (Delete.isSelected()) {
 	  number = 2;
 	} else {
 	  number = 1;
 	}
 	ExecuteCmdsThread th2 = new ExecuteCmdsThread(cmds, number, utils.getNrOfCPUs(), sv);
 	sv.addThread(th2);
      }
      sv.go();

    // The Play button was clicked on, handle it ...
    } else if (e.getSource() == play) {
      String s;
      int i;
      Vector tracks = new Vector();
      Vector number = new Vector();

      for (i=0;i<trackTable.getRowCount();i++) {
	if (trackTable.isRowSelected(i)) {
	  s = new String("Track ");
	  s = s.concat((String)trackTable.getValueAt(i,0));
	  number.addElement(s);
	}
      }
      tracks.addElement(number);
      for (i=0;i<trackTable.getRowCount();i++) {
	if (trackTable.isRowSelected(i)) {
	  tracks.addElement(new String(""));
	}
      }

      CDPlayer cdp = new CDPlayer(settings,utils,tracks); 
    // The Tray button was clicked on, handle it ...
    } else if (e.getSource() == tray) {
      String cmd = new String(settings.getCdrecord());
      cmd = cmd.concat(" dev=");
      cmd = cmd.concat(settings.getRdDevNumber());
      if (open) {
	cmd = cmd.concat(" -load");
	utils.execute(cmd,false);
	getTracks();
	open = false;
      } else {
	cmd = cmd.concat(" -eject");
	ExecuteThread th = new ExecuteThread(cmd,false);
	open = true;
      }
    // The Refresh button was clicked on, handle it ...
    } else if (e.getSource() == refresh) {
      open = false;
      getTracks();
    // The Select button was clicked on, handle it ...
    } else if (e.getSource() == select) {
      if (select.getText().compareTo("  Select All  ") == 0) {
	trackTable.setRowSelectionInterval(0,trackTable.getRowCount()); 
	select.setText(" Deselect All ");
      } else {
	trackTable.clearSelection();
	select.setText("  Select All  ");
      }
    // The Cancel button was clicked on, handle it ...
    } else if (e.getSource() == cancel) {
      setVisible(false);
      dispose();
    }
  }
}
