/*
 *  Copyright (C) 1998,1999  Jesper Pedersen <jews@imada.ou.dk>
 *  This code is released under GNU GPL version 2 or later
 */

import com.sun.java.swing.tree.*;
import com.sun.java.swing.*;
import java.lang.*;
import java.awt.*;

public class FileManager_TCR implements TreeCellRenderer {
  protected Icon closedIcon;
  protected Icon leafIcon;
  protected Icon openIcon;
  protected Icon cdrIcon;
  protected Icon wavIcon;
  protected Icon mp3Icon;
  protected Icon isoIcon;
  protected Icon imgIcon;
  protected Color textSelectionColor;
  protected Color textNonSelectionColor;
  protected Color backgroundSelectionColor;
  protected Color backgroundNonSelectionColor;
  protected Color borderSelectionColor;
  protected JLabel label;
  protected Font textSelectionFont;
  protected Font textNonSelectionFont;
  

  public FileManager_TCR() {
    label = new JLabel();
    label.setOpaque(true);
    Font f=label.getFont();
    textSelectionFont=new Font(f.getName(),f.getStyle()|Font.ITALIC,f.getSize());
    textNonSelectionFont=f;
    
    setLeafIcon(new ImageIcon("images/dir_w.gif"));
    setClosedIcon(new ImageIcon("images/dir_y.gif"));
    setOpenIcon(new ImageIcon("images/dir_y.gif"));
    
    setCDRIcon(new ImageIcon("images/cdr.gif"));
    setWAVIcon(new ImageIcon("images/wav.gif"));
    setMP3Icon(new ImageIcon("images/mp3.gif"));
    setISOIcon(new ImageIcon("images/iso.gif"));
    setIMGIcon(new ImageIcon("images/img.gif"));

    setTextSelectionColor(UIManager.getColor("Tree.textSelectionColor"));
    setTextNonSelectionColor(UIManager.getColor("Tree.textNonSelectionColor"));
    setBackgroundSelectionColor(UIManager.getColor("Tree.selectionBackground"));
    setBackgroundNonSelectionColor(UIManager.getColor("Tree.backgroundNonSelectionColor"));
    setBorderSelectionColor(UIManager.getColor("Tree.borderSelectionColor"));
  }
  
  /**
   * Returns the default open icon
   */
  public Icon getDefaultOpenIcon() {
    return openIcon;
  }
  
  /**
   * Returns the default close icon
   */
  public Icon getDefaultClosedIcon() {
    return closedIcon;
  }
  
  /**
   * Returns the default leaf icon
   */
  public Icon getDefaultLeafIcon() {
    return leafIcon;
  }
  
  /**
   * Sets the open icon
   */
  public void setOpenIcon(Icon newIcon) {
    openIcon = newIcon;
  }
  
  /**
   * Returns the open icon 
   */
  public Icon getOpenIcon() {
    return openIcon;
  }
  
  /**
   * Sets the closed icon
   */
  public void setClosedIcon(Icon newIcon) {
    closedIcon = newIcon;
  }
  
  /**
   * Returns the closed icon
   */
  public Icon getClosedIcon() {
    return closedIcon;
  }
  
  /**
   * Sets the leaf icon
   */
  public void setLeafIcon(Icon newIcon) {
    leafIcon = newIcon;
  }
  
  /**
   * Returns the leaf icon 
   */
  public Icon getLeafIcon() {
    return leafIcon;
  }
  
  /**
   * Sets the icon used to represent CDR nodes.
   */
  public void setCDRIcon(Icon newIcon) {
    cdrIcon = newIcon;
  }
  
  /**
   * Returns the icon used to represent CDR nodes.
   */
  public Icon getCDRIcon() {
    return cdrIcon;
  }
  
  /**
   * Sets the icon used to represent WAV nodes.
   */
  public void setWAVIcon(Icon newIcon) {
    wavIcon = newIcon;
  }
  
  /**
   * Returns the icon used to represent WAV nodes.
   */
  public Icon getWAVIcon() {
    return wavIcon;
  }
  
  /**
   * Sets the icon used to represent MP3 nodes.
   */
  public void setMP3Icon(Icon newIcon) {
    mp3Icon = newIcon;
  }
  
  /**
   * Returns the icon used to represent MP3 nodes.
   */
  public Icon getMP3Icon() {
    return mp3Icon;
  }
  
  /**
   * Sets the icon used to represent ISO nodes.
   */
  public void setISOIcon(Icon newIcon) {
    isoIcon = newIcon;
  }
  
  /**
   * Returns the icon used to represent ISO nodes.
   */
  public Icon getISOIcon() {
    return isoIcon;
  }
  
  /**
   * Sets the icon used to represent IMG nodes.
   */
  public void setIMGIcon(Icon newIcon) {
    imgIcon = newIcon;
  }
  
  /**
   * Returns the icon used to represent IMG nodes.
   */
  public Icon getIMGIcon() {
    return imgIcon;
  }
  
  /**
   * Sets selected text color
   */
  public void setTextSelectionColor(Color newColor) {
    textSelectionColor = newColor;
  }
  
  /**
   * Returns selected text color
   */
  public Color getTextSelectionColor() {
    return textSelectionColor;
  }
  
  /**
   * Sets non-selected text color
   */
  public void setTextNonSelectionColor(Color newColor) {
    textNonSelectionColor = newColor;
  }
  
  /**
   * Returns non-selected text color
   */
  public Color getTextNonSelectionColor() {
    return textNonSelectionColor;
  }
  
  /**
   * Sets selected background color
   */
  public void setBackgroundSelectionColor(Color newColor) {
    backgroundSelectionColor = newColor;
  }
  
  /**
   * Returns selected background color
   */
  public Color getBackgroundSelectionColor() {
    return backgroundSelectionColor;
  }
  
  /**
   * Sets non-selected background color
   */
  public void setBackgroundNonSelectionColor(Color newColor) {
    backgroundNonSelectionColor = newColor;
  }
  
  /**
   * Returns non-selected background color
   */
  public Color getBackgroundNonSelectionColor() {
    return backgroundNonSelectionColor;
  }
  
  /**
   * Sets selected border color
   */
  public void setBorderSelectionColor(Color newColor) {
    borderSelectionColor = newColor;
  }
  
  /**
   * Returns selected border color
   */
  public Color getBorderSelectionColor() {
    return borderSelectionColor;
  }

  public Component getTreeCellRendererComponent(JTree tree,
						Object value, boolean selected, 
						boolean expanded, boolean leaf, int row,
						boolean hasFocus)  {
    DefaultMutableTreeNode node=(DefaultMutableTreeNode)value;
    
    if (selected) {
      label.setForeground(getTextSelectionColor());
      label.setBackground(getBackgroundSelectionColor());
      label.setFont(textSelectionFont);
    } else {
      label.setForeground(getTextNonSelectionColor());
      label.setBackground(getBackgroundNonSelectionColor());
      label.setFont(textNonSelectionFont);
    }
    label.setText(value.toString());
    label.setToolTipText(value.toString());
    
    if (node instanceof CDR) {
      label.setIcon(cdrIcon);
    } else if (node instanceof WAV) {
      label.setIcon(wavIcon);
    } else if (node instanceof MP3) {
      label.setIcon(mp3Icon);
    } else if (node instanceof ISO) {
      label.setIcon(isoIcon);
    } else if (node instanceof IMG) {
      label.setIcon(imgIcon);
    } else {
      if (leaf) {
	label.setIcon(getLeafIcon());
      } else if (expanded) {
	label.setIcon(getOpenIcon());
      } else {
	label.setIcon(getClosedIcon());
      }
    }
    return label;
  }
}
