/*
 *  Copyright (C) 1998,1999  Jesper Pedersen <jews@imada.ou.dk>
 *  This code is released under GNU GPL version 2 or later
 */

import com.sun.java.swing.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.border.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class FileNames extends JPanel implements ItemListener {
    protected JPanel        dataPane;
    protected JToggleButton iso;
    protected JToggleButton win;
    protected JToggleButton joliet;
    protected JToggleButton mac;
    protected JToggleButton rockridge;
    protected JToggleButton realrockridge;

    protected JLabel        isoLabel;
    protected JLabel        winLabel;
    protected JLabel        jolietLabel;
    protected JLabel        macLabel;
    protected JLabel        rockridgeLabel;
    protected JLabel        realrockridgeLabel;

    protected ButtonGroup   buttonGroup;

    protected boolean       multi;

    protected Settings      settings;

    public FileNames(Settings set, boolean m) {
        this.setLayout(new BorderLayout());
	this.setBorder(new MatteBorder(1,1,1,1,Color.gray));

	settings = set;
	multi = m;
	
	dataPane = new JPanel();
	dataPane.setBorder(new EmptyBorder(5,5,5,5));
	dataPane.setLayout(new GridLayout(6,2));

	if (multi == false) {
	  buttonGroup = new ButtonGroup();
	  iso = new JRadioButton("ISO-9660");
	  buttonGroup.add(iso);
	  win = new JRadioButton("Windows 9x/NT");
	  buttonGroup.add(win);
	  joliet = new JRadioButton("Joliet");
	  buttonGroup.add(joliet);
	  mac = new JRadioButton("Mac");
	  buttonGroup.add(mac);
	  rockridge = new JRadioButton("Rock-Ridge",true);
	  buttonGroup.add(rockridge);
	  realrockridge = new JRadioButton("Real Rock-Ridge");
	  buttonGroup.add(realrockridge);
	} else {
	  iso = new JCheckBox("ISO-9660");
	  iso.addItemListener(this);
	  win = new JCheckBox("Windows 9x/NT");
	  win.addItemListener(this);
	  joliet = new JCheckBox("Joliet");
	  joliet.addItemListener(this);
	  mac = new JCheckBox("Mac");
	  mac.addItemListener(this);
	  rockridge = new JCheckBox("Rock-Ridge");
	  rockridge.addItemListener(this);
	  realrockridge = new JCheckBox("Real Rock-Ridge");
	  realrockridge.addItemListener(this);
	}

	setFileSystem();

	isoLabel = new JLabel("Standard ISO-9660");
	isoLabel.setForeground(Color.black);
	winLabel = new JLabel("Long Filenames (32 characters)");
	winLabel.setForeground(Color.black);
	jolietLabel = new JLabel("Joliet (64 characters Uni-code)");
	jolietLabel.setForeground(Color.black);
	macLabel = new JLabel("HFS hybrid / ISO-9660");
	macLabel.setForeground(Color.black);
	rockridgeLabel = new JLabel("Rockridge with no users");
	rockridgeLabel.setForeground(Color.black);
	realrockridgeLabel = new JLabel("Rockridge with users");
	realrockridgeLabel.setForeground(Color.black);

	dataPane.add(iso);
	dataPane.add(isoLabel);
	dataPane.add(win);
	dataPane.add(winLabel);
	dataPane.add(joliet);
	dataPane.add(jolietLabel);
	dataPane.add(mac);
	dataPane.add(macLabel);
	dataPane.add(rockridge);
	dataPane.add(rockridgeLabel);
	dataPane.add(realrockridge);
	dataPane.add(realrockridgeLabel);

        this.add(dataPane, BorderLayout.CENTER);
    }

    public String getFileSystem() {
      String r = new String("");

      if (multi == false) {
	if (iso.isSelected() == true) {
	  r = new String("ISO-9660");
	} else if (win.isSelected() == true) {
	  r = new String("Windows 9x/NT");
	} else if (joliet.isSelected() == true) {
	  r = new String("Joliet");
	} else if (mac.isSelected() == true) {
	  r = new String("Mac");
	} else if (rockridge.isSelected() == true) {
	  r = new String("Rock-Ridge");
	} else {
	  r = new String("Real Rock-Ridge");
	}
      } else {
	if (iso.isSelected() == true) {
	  r = new String("ISO-9660");
	  r = r.concat("|");
	}
	if (win.isSelected() == true) {
	  r = r.concat("Windows 9x/NT");
	  r = r.concat("|");
	}
	if (joliet.isSelected() == true) {
	  r = r.concat("Joliet");
	  r = r.concat("|");
	}
	if (mac.isSelected() == true) {
	  r = r.concat("Mac");
	  r = r.concat("|");
	}
	if (rockridge.isSelected() == true) {
	  r = r.concat("Rock-Ridge");
	  r = r.concat("|");
	}
	if (realrockridge.isSelected() == true) { 
	  r = r.concat("Real Rock-Ridge");
	}
      }
      return r;
    }

    public void setFileSystem() {
      String s;
      s = settings.getFileSystem();

      StringTokenizer st = new StringTokenizer(s,"|");
      while (st.hasMoreTokens()) {
	s = st.nextToken();
	setFileSystem(s);
      }
    }

    public void setFileSystem(String s) {
      if (s.compareTo("ISO-9660") == 0) {
	iso.setSelected(true);
      } else if (s.compareTo("Windows 9x/NT") == 0) {
	win.setSelected(true);
      } else if (s.compareTo("Joliet") == 0) {
	joliet.setSelected(true);
      } else if (s.compareTo("Mac") == 0) {
	mac.setSelected(true);
      } else if (s.compareTo("Rock-Ridge") == 0) {
	rockridge.setSelected(true);
      } else {
	realrockridge.setSelected(true);
      }
    }

  public void itemStateChanged(ItemEvent event) {
    // If the rockridge or realrockridge was clicked on ...
    if (event.getSource() == rockridge) {
      if ((realrockridge.isSelected() == true)&&
	  (rockridge.isSelected() == true)) {
	realrockridge.setSelected(false);
	rockridge.setSelected(true);
      }
    } else
    if (event.getSource() == realrockridge) {
      if ((rockridge.isSelected() == true)&&
	  (realrockridge.isSelected() == true)) {
	rockridge.setSelected(false);
	realrockridge.setSelected(true);
      }
    }
    if ((iso.isSelected() == false)&&
	(win.isSelected() == false)&&
	(joliet.isSelected() == false)&&
	(mac.isSelected() == false)&&
	(rockridge.isSelected() == false)&&
	(realrockridge.isSelected() == false)) {
      iso.setSelected(true);
    }
  }

  public String whichISOBuilder() {
    String result = new String("");

    if (mac.isSelected() == false) {
      result = new String(settings.getMkisofs());
    } else {
      result = new String(settings.getMkhybrid());
    }
    return result;
  }


  public String getBurnString() {
    Hashtable h = new Hashtable();
    String options = new String("");
    String a_opt = new String("-a");
    String D_opt = new String("-D");
    String J_opt = new String("-J");
    String l_opt = new String("-l");
    String L_opt = new String("-L");
    String r_opt = new String("-r");
    String R_opt = new String("-R");
    String v_opt = new String("-v");

    String hfs_opt = new String("-hfs");

    h.put(a_opt,new String());
    h.put(v_opt,new String());
    if (iso.isSelected()) {

    }
    if (win.isSelected()) {
      h.put(D_opt,new String());
      h.put(l_opt,new String());
    }
    if (joliet.isSelected()) {
      h.put(D_opt,new String());
      h.put(l_opt,new String());
      h.put(J_opt,new String());
    }
    if (mac.isSelected()) {
      h.put(hfs_opt,new String());
    }
    if (rockridge.isSelected()) {
      h.put(l_opt,new String());
      h.put(L_opt,new String());
      h.put(r_opt,new String());
    }
    if (realrockridge.isSelected()) { 
      h.put(l_opt,new String());
      h.put(L_opt,new String());
      h.put(R_opt,new String());
    }

    options = options.concat("-a");
    if (h.containsKey(D_opt)) {
      options = options.concat(" -D");
    }
    if (h.containsKey(J_opt)) {
      options = options.concat(" -J");
    }
    if (h.containsKey(l_opt)) {
      options = options.concat(" -l");
    }
    if (h.containsKey(L_opt)) {
      options = options.concat(" -L");
    }
    if (h.containsKey(r_opt)) {
      options = options.concat(" -r");
    }
    if (h.containsKey(R_opt)) {
      options = options.concat(" -R");
    }
    if (h.containsKey(hfs_opt)) {
      options = options.concat(" -hfs");
    }
    options = options.concat(" -v");

    return options;
  }
}
