/*
 *  Copyright (C) 1998,1999  Jesper Pedersen <jews@imada.ou.dk>
 *  This code is released under GNU GPL version 2 or later
 */

import com.sun.java.swing.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.border.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

class PropertiesFrame extends JFrame implements ActionListener, ItemListener  {
  protected JTabbedPane  tab;
  
  protected JPanel       dataPane1;
  protected JPanel       pathPane;
  protected JPanel       dataPane2;
  protected JPanel       sourcePane;
  protected JPanel       dataPane3;
  protected JPanel       devPane;
  protected JPanel       dataPane4;
  protected JPanel       prgPane;

  protected JLabel       mntLabel;
  protected JTextField   mnt;
  protected JLabel       tmpLabel;
  protected JTextField   tmp;

  protected JLabel       sourceLabel;
  protected JTextField   source;
  protected JPanel       filepanel;
  protected JLabel       filenamesLabel;
  protected FileNames    filenames;
  protected JCheckBox    multifs;
  protected JPanel       adv;
  protected JPanel       adv_panel1;
  protected JPanel       adv_panel2;
  
  protected JLabel       rdDevLabel;
  protected JComboBox    rdDev;
  protected JLabel       rdSpeedLabel;
  protected RdComboBox   rdSpeed;
  protected Vector       devices;

  protected JLabel       mkisofsLabel;
  protected JTextField   mkisofs;
  protected JLabel       cdrecordLabel;
  protected JTextField   cdrecord;
  protected JLabel       mkhybridLabel;
  protected JTextField   mkhybrid;
  protected JLabel       audioripLabel;
  protected JTextField   audiorip;
  protected JLabel       mp3playerLabel;
  protected JTextField   mp3player;
  protected JLabel       wavplayerLabel;
  protected JTextField   wavplayer;
  protected JLabel       cdrplayerLabel;
  protected JTextField   cdrplayer;
  protected JPanel       encoderPanel;
  protected JLabel       encoderLabel;
  protected JTextField   encoder;
  protected JCheckBox    encodertype;
  protected JLabel       decoderLabel;
  protected JTextField   decoder;

  protected JPanel       buttonPane;
  protected JButton      ok;
  protected JButton      cancel;

  protected Settings     settings;
  protected Utils        utils;

  public PropertiesFrame(Settings set, Utils utl) {
    String s;

    this.setTitle("BurnIT: Properties");

    updateWindow();

    this.getContentPane().setLayout(new BorderLayout()); 

    settings = set;
    utils = utl;

    dataPane1 = new JPanel();
    dataPane1.setBorder(new EmptyBorder(5,5,5,5));
    dataPane1.setLayout(new BorderLayout());

    pathPane = new JPanel();
    pathPane.setLayout(new GridLayout(2,2));
    pathPane.setBorder(new EmptyBorder(5,5,5,5));

    mntLabel = new JLabel("Image Mount point");
    mntLabel.setForeground(Color.black);
    
    mnt = new JTextField(30);
    mnt.setText(settings.getMnt());

    tmpLabel = new JLabel("Temp Directory");
    tmpLabel.setForeground(Color.black);
    
    tmp = new JTextField(30);
    tmp.setText(settings.getTmp());

    pathPane.add(mntLabel);
    pathPane.add(mnt);
    pathPane.add(tmpLabel);
    pathPane.add(tmp);

    dataPane1.add(pathPane, BorderLayout.NORTH);

    dataPane2 = new JPanel();
    dataPane2.setBorder(new EmptyBorder(5,5,5,5));
    dataPane2.setLayout(new BorderLayout());

    filepanel = new JPanel();
    filepanel.setLayout(new BorderLayout());
    filepanel.setBorder(new EmptyBorder(5,5,5,5));

    filenamesLabel = new JLabel("Default filesystem");
    filenamesLabel.setForeground(Color.black);

    filenames = new FileNames(settings,settings.getAdvancedUser());

    filepanel.add(filenamesLabel, BorderLayout.NORTH);
    filepanel.add(filenames, BorderLayout.CENTER);

    sourcePane = new JPanel();
    sourcePane.setLayout(new FlowLayout(FlowLayout.LEFT));

    sourceLabel = new JLabel("Default source path");
    sourceLabel.setForeground(Color.black);
    
    source = new JTextField(30);
    source.setText(settings.getDflSourcePath());
 
    sourcePane.add(sourceLabel);
    sourcePane.add(source);

    adv = new JPanel();
    adv.setBorder(new EmptyBorder(5,5,5,5));
    adv.setLayout(new FlowLayout(FlowLayout.LEFT));

    multifs = new JCheckBox("Enable multi filesystems",settings.getAdvancedUser());
    multifs.setForeground(Color.black);
    multifs.addItemListener(this);

    adv.add(multifs);

    dataPane2.add(sourcePane, BorderLayout.NORTH);
    dataPane2.add(filepanel, BorderLayout.CENTER);
    dataPane2.add(adv, BorderLayout.SOUTH);

    dataPane3 = new JPanel();
    dataPane3.setBorder(new EmptyBorder(5,5,5,5));
    dataPane3.setLayout(new BorderLayout());

    devPane = new JPanel();
    devPane.setLayout(new GridLayout(2,2));
    devPane.setBorder(new EmptyBorder(5,5,5,5));

    rdDevLabel = new JLabel("Read Device");
    rdDevLabel.setForeground(Color.black);
    
    rdDev = new JComboBox();
    Vector elm;
    String tmp;
    devices = utils.getSCSIDevices();
    for (int i=0; i<devices.size();i++) {
      elm = (Vector)devices.elementAt(i);
      tmp = (String)elm.elementAt(2);
      rdDev.addItem(tmp);
    }
    rdDev.setSelectedItem(settings.getRdModel());

    rdSpeedLabel = new JLabel("Read Speed");
    rdSpeedLabel.setForeground(Color.black);
    
    rdSpeed = new RdComboBox();
    rdSpeed.setSelectedItem(settings.getRdRead()); 

    devPane.add(rdDevLabel);
    devPane.add(rdDev);
    devPane.add(rdSpeedLabel);
    devPane.add(rdSpeed);

    dataPane3.add(devPane, BorderLayout.NORTH);

    dataPane4 = new JPanel();
    dataPane4.setBorder(new EmptyBorder(5,5,5,5));
    dataPane4.setLayout(new BorderLayout());

    prgPane = new JPanel();
    prgPane.setLayout(new GridLayout(9,2));
    prgPane.setBorder(new EmptyBorder(5,5,5,5));

    mkisofsLabel = new JLabel("mksiofs");
    mkisofsLabel.setForeground(Color.black);
    
    mkisofs = new JTextField(30);
    mkisofs.setText(settings.getMkisofs());

    cdrecordLabel = new JLabel("cdrecord");
    cdrecordLabel.setForeground(Color.black);
    
    cdrecord = new JTextField(30);
    cdrecord.setText(settings.getCdrecord());

    mkhybridLabel = new JLabel("mkhybrid");
    mkhybridLabel.setForeground(Color.black);
    
    mkhybrid = new JTextField(30);
    mkhybrid.setText(settings.getMkhybrid());

    audioripLabel = new JLabel("Audio ripper");
    audioripLabel.setForeground(Color.black);
    
    audiorip = new JTextField(30);
    audiorip.setText(settings.getAudioRip());

    mp3playerLabel = new JLabel("MP3 Player");
    mp3playerLabel.setForeground(Color.black);
    
    mp3player = new JTextField(30);
    mp3player.setText(settings.getMP3Player());

    wavplayerLabel = new JLabel("WAV Player");
    wavplayerLabel.setForeground(Color.black);
    
    wavplayer = new JTextField(30);
    wavplayer.setText(settings.getWAVPlayer());

    cdrplayerLabel = new JLabel("CDR Player");
    cdrplayerLabel.setForeground(Color.black);
    
    cdrplayer = new JTextField(30);
    cdrplayer.setText(settings.getCDRPlayer());

    encoderPanel = new JPanel();
    encoderPanel.setLayout(new BorderLayout());

    encoderLabel = new JLabel("Encoder");
    encoderLabel.setForeground(Color.black);
    
    encoder = new JTextField(30);
    encoder.setText(settings.getEncoder());

    encodertype = new JCheckBox("",settings.getEncoderType());

    encoderPanel.add(encodertype, BorderLayout.EAST);
    encoderPanel.add(encoder, BorderLayout.CENTER);

    decoderLabel = new JLabel("Decoder");
    decoderLabel.setForeground(Color.black);
    
    decoder = new JTextField(30);
    decoder.setText(settings.getDecoder());

    prgPane.add(mkisofsLabel);
    prgPane.add(mkisofs);
    prgPane.add(cdrecordLabel);
    prgPane.add(cdrecord);
    prgPane.add(mkhybridLabel);
    prgPane.add(mkhybrid);
    prgPane.add(audioripLabel);
    prgPane.add(audiorip);
    prgPane.add(cdrplayerLabel);
    prgPane.add(cdrplayer);
    prgPane.add(wavplayerLabel);
    prgPane.add(wavplayer);
    prgPane.add(mp3playerLabel);
    prgPane.add(mp3player);
    prgPane.add(encoderLabel);
    prgPane.add(encoderPanel);
    prgPane.add(decoderLabel);
    prgPane.add(decoder);

    dataPane4.add(prgPane, BorderLayout.NORTH);

    tab = new JTabbedPane();

    tab.addTab("Default",dataPane2);
    tab.addTab("Devices",dataPane3);
    tab.addTab("Paths",dataPane1);
    tab.addTab("Programs",dataPane4);

    buttonPane = new JPanel();
    buttonPane.setLayout(new BorderLayout());
    buttonPane.setBorder(new EmptyBorder(5,5,5,5));

    ok = new JButton("OK");
    ok.addActionListener(this);
    buttonPane.add(ok, BorderLayout.EAST);

    cancel = new JButton("Cancel");
    cancel.addActionListener(this);
    buttonPane.add(cancel, BorderLayout.WEST);

    this.getContentPane().add(tab, BorderLayout.CENTER);
    this.getContentPane().add(buttonPane, BorderLayout.SOUTH);

    this.setVisible(true);
    updateWindow();
  }

  private void updateWindow() {
    this.pack();
    this.setSize(new Dimension(500,330));
    repaint();
  }

  private void saveSettings() {
    Vector device;
    String s;
    int i;

    settings.setConfigVersion(utils.getVersion());
    settings.setMnt(mnt.getText());
    settings.setTmp(tmp.getText());
    settings.setDflSourcePath(source.getText());
    settings.setFileSystem(filenames.getFileSystem());
    if (multifs.isSelected() == true) {
      settings.setAdvancedUser(true);
    } else {
      settings.setAdvancedUser(false);
    }
    i = rdDev.getSelectedIndex();
    device = (Vector)devices.elementAt(i);
    s = (String)device.elementAt(0);
    settings.setRdDevNumber(s);
    s = (String)device.elementAt(2);
    settings.setRdModel(s);
    s = new String("/dev/scd");
    s = s.concat(new Integer(i).toString());
    settings.setRdDev(s);
    s = new String("/dev/sr");
    s = s.concat(new Integer(i).toString());
    settings.setRdGen(s);
    s = (String)rdSpeed.getSelectedItem();
    settings.setRdRead(s);
    settings.setMkisofs(mkisofs.getText());
    settings.setCdrecord(cdrecord.getText());
    settings.setMkhybrid(mkhybrid.getText());
    s = audiorip.getText();
    settings.setAudioRip(s);
    if (s.indexOf("cdparanoia") == -1) {
      settings.setRipType(true);
    } else {
      settings.setRipType(false);
    }
    settings.setCDRPlayer(cdrplayer.getText());
    settings.setWAVPlayer(wavplayer.getText());
    settings.setMP3Player(mp3player.getText());
    settings.setEncoder(encoder.getText());
    settings.setEncoderType(encodertype.isSelected());
    settings.setDecoder(decoder.getText());
  }

  public void itemStateChanged(ItemEvent event) {
    // If the MULTIFS was clicked on, handle it ...
    if (event.getSource() == multifs) {
      if (checkSetup()) {
	saveSettings();
	PropertiesFrame f = new PropertiesFrame(settings,utils);
	this.setVisible(false);
	this.dispose();
      }
    }
  }

  private boolean checkSetup() {
    String s = new String("");
    boolean result = true;

    if (rdDev.getSelectedIndex() == -1) {
	result = false;
	s = new String("Please choose read device");
    }

    if (result == false) {
      JOptionPane f = new JOptionPane();
      f.showMessageDialog(this,s,"BurnIT: Error", JOptionPane.ERROR_MESSAGE);
    }

    return result;
  }

  public void actionPerformed(ActionEvent e) {
    // The OK button was clicked on, handle it ...
    if (e.getSource() == ok) {
      if (checkSetup()) {
	// Save the settings
	saveSettings();
	
	setVisible(false);
	dispose();
      }
    // The CANCEL button was clicked on, handle it ...
    } else if (e.getSource() == cancel) {
      setVisible(false);
      dispose();
    }
  }
}
