/*
 *  Copyright (C) 1998,1999  Jesper Pedersen <jews@imada.ou.dk>
 *  This code is released under GNU GPL version 2 or later
 */

import com.sun.java.swing.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.border.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

class Wizard extends JFrame implements ActionListener {
  protected BurnITFrame  parent;

  protected JPanel       borderPane;
  protected JPanel       borderPane2;
  protected JPanel       dataPane;
  protected JPanel       buttonPane;
  protected JPanel       buttonPane1;
  protected JPanel       buttonPane2;
 
  protected GeneralFrame      pgeneral;
  protected ImageFileFrame    pimageFile;
  protected LoadContentsFrame ploadContents;
  protected FileNamesFrame    pfileNames;
  protected VolumeInfoFrame   pvolumeInfo;
  protected DataTrackFrame    pdataTrack;
  protected AudioTracksFrame  paudioTracks;
  protected Disc2DiscFrame    pdisc2disc;

  protected JButton      next;
  protected JButton      prev;
  protected JButton      cancel;

  protected Font         fontLarge;
  protected Font         fontMedium;
  protected Font         fontNormal;

  protected int          layout;

  protected static final int WelcomeMenu = 1;
  protected static final int FilesystemMenu = 2 ;
  protected static final int DataMenu = 31;
  protected static final int AudioMenu = 32;
  protected static final int VolInfoMenu = 4;
  protected static final int FinalMenu = 5;
  protected static final int MultiMenu = 6;
  protected static final int ImagefileMenu = 7;
  protected static final int Disc2DiscMenu = 8;
  protected int state = WelcomeMenu;

  protected JPanel       panel1;
  protected JPanel       panel2;
  protected JPanel       panel3;
  protected JPanel       panel4;
  protected JPanel       panel5;
  protected JPanel       panel6;
  protected JPanel       panel7;
  protected JPanel       panel8;

  // Window 1
  protected JLabel       welcome;
  protected JLabel       wizard;
  protected JLabel       type;

  // Window 2
  protected JLabel       fileLabel;
  protected FileNames    filesystem;

  // Window 3.1
  protected JButton      add;
  protected JButton      remove;
  protected JList        sourceListData;
  protected JScrollPane  sourcePane;
  protected Vector       data;
  protected JTextField   input;

  // Window 3.2
  protected JList        sourceListAudio;
  protected Vector       audio;

  // Window 4
  protected JLabel       volIDLabel;
  protected JTextField   volID;
  protected JLabel       volPRELabel;
  protected JTextField   volPRE;
  protected JLabel       volPUBLabel;
  protected JTextField   volPUB;
  protected JLabel       appIDLabel;
  protected JTextField   appID;

  // Window 5
  protected JLabel       speedLabel;
  protected JLabel       onTheFlyLabel;
  protected JLabel       testBeforeLabel;
  protected WrComboBox   speed;
  protected JCheckBox    onTheFly;
  protected JCheckBox    testBefore;

  // Window 6
  protected JLabel       multiLabel;
  protected JLabel       resultLabel;
  protected JButton      multi;
  protected String       magic;

  // Window 7
  protected JLabel       imageFileLabel;
  protected JTextField   imageFile;

  // Window 8
  protected Disc2DiscFrame disc2disc;

  // Settings and utilities
  protected Utils        utils;
  protected Settings     settings;

  public Wizard(BurnITFrame p, int l, Settings set, Utils utl) {
    String s;

    this.setTitle("BurnIT: Wizard");

    parent = p;
    layout = l;
    settings = set;
    utils = utl;

    pgeneral = parent.getGeneralFrame();
    pimageFile = parent.getImageFileFrame();
    ploadContents = parent.getLoadContentsFrame();
    pfileNames = parent.getFileNamesFrame();
    pvolumeInfo = parent.getVolumeInfoFrame();
    pdataTrack = parent.getDataTrackFrame();
    paudioTracks = parent.getAudioTracksFrame();
    pdisc2disc = parent.getDisc2DiscFrame();

    fontLarge  = new Font("Helvetica", Font.BOLD, 18);
    fontMedium = new Font("Helvetica", Font.BOLD, 16);
    fontNormal = new Font("Helvetica", Font.BOLD, 14);

    buttonPane = new JPanel();
    buttonPane.setLayout(new GridLayout(1,2));

    buttonPane1 = new JPanel();
    buttonPane1.setLayout(new FlowLayout(FlowLayout.LEFT));

    buttonPane2 = new JPanel();
    buttonPane2.setLayout(new FlowLayout(FlowLayout.RIGHT));
    
    next = new JButton("Next >");
    next.addActionListener(this);
    next.setVisible(false);

    prev = new JButton("< Prev");
    prev.addActionListener(this);
    prev.setVisible(false);

    cancel = new JButton("Cancel");
    cancel.addActionListener(this);

    buttonPane1.add(cancel);

    buttonPane2.add(prev);
    buttonPane2.add(next);

    buttonPane.add(buttonPane1);
    buttonPane.add(buttonPane2);

    // 1
    welcome = new JLabel("Welcome to the BurnIT Wizard !");
    welcome.setForeground(Color.black);
    welcome.setFont(fontMedium);
    
    wizard = new JLabel("The Wizard will now help you to create a");
    wizard.setForeground(Color.black);
    wizard.setFont(fontNormal);
    
    type = new JLabel();
    type.setForeground(Color.black);
    type.setFont(fontLarge);
      
    s = this.getTitle();
    
    if (layout == BurnITFrame.isMultisessionCD) {
      type.setText("Multisession CD");
      s = s.concat(" (Multisession CD)");
    } else if (layout == BurnITFrame.isCDROM) {
      type.setText("CD-ROM");
      s = s.concat(" (CD-ROM)");
    } else if (layout == BurnITFrame.isAudio) {
      type.setText("Audio CD");
      s = s.concat(" (Audio CD)");
    } else if (layout == BurnITFrame.isCDCopy) {
      type.setText("Disc-2-Disc Copy");
      s = s.concat(" (Disc-2-Disc)");
    } else if (layout == BurnITFrame.isMixedModeCD) {
      type.setText("Mixed Mode CD");
      s = s.concat(" (Mixed Mode CD)");
    } else if (layout == BurnITFrame.isCDROM_wImage) {
      type.setText("CD-ROM from a image");
      s = s.concat(" (CD-ROM from image)");
    } else if (layout == BurnITFrame.isMixedModeCD_wImage) {
      type.setText("Mixed Mode CD from a image");
      s = s.concat(" (Mixed Mode CD from image)");
    } else {
      type.setText("No Wizard for this mode");
    }
    this.setTitle(s);
      
    // 2
    fileLabel = new JLabel("Choose filesystem");
    fileLabel.setForeground(Color.black);
    
    filesystem = new FileNames(settings,false);
      
    // 3
    add = new JButton("Add");
    add.addActionListener(this);
    
    remove = new JButton("Remove");
    remove.addActionListener(this);
    
    input = new JTextField(30);
    
    sourceListData = new JList();
    sourceListAudio = new JList();

    data = new Vector();
    audio = new Vector();

    // 4
    volIDLabel = new JLabel("Volume ID");
    volIDLabel.setForeground(Color.black);
    
    volID = new JTextField(20);
    
    volPRELabel = new JLabel("Volume Preparer");
    volPRELabel.setForeground(Color.black);
    
    volPRE = new JTextField(20);
    
    volPUBLabel = new JLabel("Volume Publisher");
    volPUBLabel.setForeground(Color.black);
    
    volPUB = new JTextField(20);

    appIDLabel = new JLabel("Application ID");
    appIDLabel.setForeground(Color.black);
    
    appID = new JTextField(20);

    // 5
    speedLabel = new JLabel("Write speed");
    speedLabel.setForeground(Color.black);
    speedLabel.setFont(fontNormal);
    
    speed = new WrComboBox();
    
    onTheFlyLabel = new JLabel("Write on the Fly");
    onTheFlyLabel.setForeground(Color.black);
    onTheFlyLabel.setFont(fontNormal);

    onTheFly = new JCheckBox();
    
    if (layout == BurnITFrame.isCDCopy) {
      onTheFlyLabel.setEnabled(false);
      onTheFly.setEnabled(false);
    }
    
    testBeforeLabel = new JLabel("Test before write");
    testBeforeLabel.setForeground(Color.black);
    testBeforeLabel.setFont(fontNormal);
    
    testBefore = new JCheckBox();
    
    // 6
    multiLabel = new JLabel("Multi session information");
    multiLabel.setForeground(Color.black);
    
    resultLabel = new JLabel();
    resultLabel.setForeground(Color.black);
    
    multi = new JButton("Get msinfo");
    multi.addActionListener(this);

    magic = new String("");

    // 7
    imageFileLabel = new JLabel("Choose image file");
    imageFileLabel.setForeground(Color.black);
    
    imageFile = new JTextField(20);

    // 8
    disc2disc = new Disc2DiscFrame(utils,settings);
    
    updateScreen();

    createWindow(state);

    this.setVisible(true);
  }


  private void updateScreen() {
    this.pack();
    this.setSize(new Dimension(500, 300));
    repaint();
  }

  private void createWindow(int st) {
    this.getContentPane().removeAll();
    this.getContentPane().setLayout(new BorderLayout()); 

    borderPane = new JPanel();
    borderPane.setLayout(new BorderLayout());
    borderPane.setBorder(new EmptyBorder(5,5,5,5));

    borderPane2 = new JPanel();
    borderPane2.setLayout(new BorderLayout());
    borderPane2.setBorder(new MatteBorder(1,1,1,1,Color.gray));

    dataPane = new JPanel();
    dataPane.setBorder(new EmptyBorder(5,5,5,5));

    ////////////////////////////////////////////////////////////
    //  Welcome Menu
    ////////////////////////////////////////////////////////////
    if (st == WelcomeMenu) {
      prev.setVisible(false);

      dataPane.setLayout(new GridLayout(3,1));
      
      panel1 = new JPanel();
      panel1.setLayout(new FlowLayout(FlowLayout.CENTER));
      
      panel2 = new JPanel();
      panel2.setLayout(new FlowLayout(FlowLayout.CENTER));
      
      panel3 = new JPanel();
      panel3.setLayout(new FlowLayout(FlowLayout.CENTER));
      
      panel1.add(welcome);
      if ((layout != BurnITFrame.isAdvanced)&&(layout != BurnITFrame.newMode)) {
	panel2.add(wizard);
	next.setVisible(true);
      }
      panel3.add(type);
      
      dataPane.add(panel1);
      dataPane.add(panel2);
      dataPane.add(panel3);
    ////////////////////////////////////////////////////////////
    //  Filesystem Menu
    ////////////////////////////////////////////////////////////
    } else if (st == FilesystemMenu) {
      prev.setVisible(true);

      dataPane.setLayout(new BorderLayout());

      panel1 = new JPanel();
      panel1.setLayout(new FlowLayout(FlowLayout.LEFT));

      panel1.add(fileLabel);

      panel2 = new JPanel();
      panel2.setLayout(new FlowLayout(FlowLayout.LEFT));

      panel2.add(filesystem);

      dataPane.add(panel1, BorderLayout.NORTH);
      dataPane.add(panel2, BorderLayout.CENTER);
    ////////////////////////////////////////////////////////////
    //  Data Menu
    ////////////////////////////////////////////////////////////
    } else if (st == DataMenu) {
      dataPane.setLayout(new BorderLayout());

      panel1 = new JPanel();
      panel1.setLayout(new BorderLayout());

      panel2 = new JPanel();
      panel2.setLayout(new BorderLayout());

      panel3 = new JPanel();
      panel3.setLayout(new BorderLayout());

      sourceListData.setListData(data);

      sourcePane = new JScrollPane(sourceListData);

      panel2.add(add, BorderLayout.WEST);
      panel2.add(input, BorderLayout.CENTER);
      
      panel3.add(remove, BorderLayout.NORTH);

      panel1.add(panel3, BorderLayout.WEST);
      panel1.add(sourcePane, BorderLayout.CENTER);

      dataPane.add(panel1, BorderLayout.NORTH);
      dataPane.add(panel2, BorderLayout.SOUTH);
    ////////////////////////////////////////////////////////////
    //  Audio Menu
    ////////////////////////////////////////////////////////////
    } else if (st == AudioMenu) {
      next.setText("Next >");
      prev.setVisible(true);

      dataPane.setLayout(new BorderLayout());

      panel1 = new JPanel();
      panel1.setLayout(new BorderLayout());

      panel2 = new JPanel();
      panel2.setLayout(new BorderLayout());

      panel3 = new JPanel();
      panel3.setLayout(new BorderLayout());

      sourceListAudio.setListData(audio);

      sourcePane = new JScrollPane(sourceListAudio);

      panel2.add(add, BorderLayout.WEST);
      panel2.add(input, BorderLayout.CENTER);
      
      panel3.add(remove, BorderLayout.NORTH);

      panel1.add(panel3, BorderLayout.WEST);
      panel1.add(sourcePane, BorderLayout.CENTER);

      dataPane.add(panel1, BorderLayout.NORTH);
      dataPane.add(panel2, BorderLayout.SOUTH);
    ////////////////////////////////////////////////////////////
    //  Volume Information Menu
    ////////////////////////////////////////////////////////////
    } else if (st == VolInfoMenu) {
      next.setText("Next >");

      dataPane.setLayout(new GridLayout(2,2));

      panel1 = new JPanel();
      panel1.setLayout(new BorderLayout());

      panel2 = new JPanel();
      panel2.setLayout(new BorderLayout());

      panel3 = new JPanel();
      panel3.setLayout(new BorderLayout());

      panel4 = new JPanel();
      panel4.setLayout(new BorderLayout());

      panel5 = new JPanel();
      panel5.setLayout(new BorderLayout());

      panel6 = new JPanel();
      panel6.setLayout(new BorderLayout());

      panel7 = new JPanel();
      panel7.setLayout(new BorderLayout());

      panel8 = new JPanel();
      panel8.setLayout(new BorderLayout());

      panel1.add(volIDLabel, BorderLayout.NORTH);
      panel1.add(volID, BorderLayout.SOUTH);
      
      panel2.add(volPRELabel, BorderLayout.NORTH);
      panel2.add(volPRE, BorderLayout.SOUTH);
      
      panel3.add(volPUBLabel, BorderLayout.NORTH);
      panel3.add(volPUB, BorderLayout.SOUTH);

      panel4.add(appIDLabel, BorderLayout.NORTH);
      panel4.add(appID, BorderLayout.SOUTH);

      panel5.add(panel1, BorderLayout.NORTH);
      panel6.add(panel2, BorderLayout.NORTH);
      panel7.add(panel3, BorderLayout.NORTH);
      panel8.add(panel4, BorderLayout.NORTH);

      dataPane.add(panel5);
      dataPane.add(panel6);
      dataPane.add(panel7);
      dataPane.add(panel8);
    ////////////////////////////////////////////////////////////
    //  Final Menu
    ////////////////////////////////////////////////////////////
    } else if (st == FinalMenu) {
      next.setText("  Ok  ");

      dataPane.setLayout(new BorderLayout());

      panel1 = new JPanel();
      panel1.setLayout(new GridLayout(3,1));

      panel2 = new JPanel();
      panel2.setLayout(new GridLayout(1,2));

      panel3 = new JPanel();
      panel3.setLayout(new GridLayout(1,2));

      panel4 = new JPanel();
      panel4.setLayout(new GridLayout(1,2));

      panel2.add(speedLabel);
      panel2.add(speed);

      panel3.add(onTheFlyLabel);
      panel3.add(onTheFly);

      panel4.add(testBeforeLabel);
      panel4.add(testBefore);

      panel1.add(panel2);
      panel1.add(panel3);
      panel1.add(panel4);

      dataPane.add(panel1, BorderLayout.NORTH);
    ////////////////////////////////////////////////////////////
    //  Multisession Menu
    ////////////////////////////////////////////////////////////
    } else if (st == MultiMenu) {
      prev.setVisible(true);

      dataPane.setLayout(new BorderLayout());

      panel1 = new JPanel();
      panel1.setLayout(new BorderLayout());

      panel2 = new JPanel();
      panel2.setLayout(new BorderLayout());

      if (magic.compareTo("") == 0) {
	resultLabel.setText("Blank CD");
      } else {
	resultLabel.setText("Multisession CD found");
      }

      panel2.add(multi, BorderLayout.WEST);
      panel2.add(resultLabel, BorderLayout.CENTER);

      panel1.add(multiLabel, BorderLayout.NORTH);
      panel1.add(panel2, BorderLayout.SOUTH);

      dataPane.add(panel1, BorderLayout.NORTH);
    ////////////////////////////////////////////////////////////
    //  Imagefile Menu
    ////////////////////////////////////////////////////////////
    } else if (st == ImagefileMenu) {
      next.setText("Next >");
      prev.setVisible(true);

      dataPane.setLayout(new BorderLayout());

      panel1 = new JPanel();
      panel1.setLayout(new BorderLayout());

      panel1.add(imageFileLabel, BorderLayout.NORTH);
      panel1.add(imageFile, BorderLayout.SOUTH);

      dataPane.add(panel1, BorderLayout.NORTH);
    ////////////////////////////////////////////////////////////
    //  Disc2Disc Menu
    ////////////////////////////////////////////////////////////
    } else if (st == Disc2DiscMenu) {
      next.setText("Next >");
      prev.setVisible(true);

      dataPane.setLayout(new BorderLayout());
      dataPane.add(disc2disc, BorderLayout.NORTH);
    }

    borderPane2.add(dataPane, BorderLayout.CENTER);

    borderPane.add(borderPane2, BorderLayout.CENTER);
    borderPane.add(buttonPane, BorderLayout.SOUTH);

    this.getContentPane().add(borderPane, BorderLayout.CENTER);
    updateScreen();
  }

  public void addPath(String s) {
    if (state == DataMenu) {
      input.setText(s);
    } else if (state == AudioMenu) {
      input.setText(s);
    } else if (state == ImagefileMenu) {
      imageFile.setText(s);
    }
  }

  private void makeUpdate() {
    String s;

    pgeneral.setOnTheFly(onTheFly.isSelected());
    pgeneral.setTestBefore(testBefore.isSelected());
    s = (String)speed.getSelectedItem();
    pgeneral.setSpeed(s);

    if (layout == BurnITFrame.isMultisessionCD) {
      // Multisession info
      ploadContents.setMultisession(magic);

      // Filesystem 
      pfileNames.setFileSystem(filesystem.getFileSystem());

      // Volume Info
      pvolumeInfo.setVolumeId(volID.getText());
      pvolumeInfo.setVolumePreparer(volPRE.getText());
      pvolumeInfo.setVolumePublisher(volPUB.getText());
      pvolumeInfo.setApplicationId(appID.getText());

      // Data track
      while (data.isEmpty() == false) {
	s = (String)data.firstElement();
	data.removeElementAt(0);
	pdataTrack.addPath(s);
      }
    } else if (layout == BurnITFrame.isCDROM) {
      // Filesystem 
      pfileNames.setFileSystem(filesystem.getFileSystem());

      // Volume Info
      pvolumeInfo.setVolumeId(volID.getText());
      pvolumeInfo.setVolumePreparer(volPRE.getText());
      pvolumeInfo.setVolumePublisher(volPUB.getText());
      pvolumeInfo.setApplicationId(appID.getText());
      
      // Data track
      while (data.isEmpty() == false) {
	s = (String)data.firstElement();
	data.removeElementAt(0);
	pdataTrack.addPath(s);
      }
    } else if (layout == BurnITFrame.isAudio) {
      // Audio track
      while (audio.isEmpty() == false) {
	s = (String)audio.firstElement();
	audio.removeElementAt(0);
	paudioTracks.addPath(s);
      }
    } else if (layout == BurnITFrame.isCDCopy) {
      // Disc-2-Disc info
      pdisc2disc.setSource(disc2disc.getSource()); 
      pdisc2disc.setDest(disc2disc.getDest()); 
      pdisc2disc.setCDType(disc2disc.getCDType()); 

    } else if (layout == BurnITFrame.isMixedModeCD) {
      // Filesystem 
      pfileNames.setFileSystem(filesystem.getFileSystem());

      // Volume Info
      pvolumeInfo.setVolumeId(volID.getText());
      pvolumeInfo.setVolumePreparer(volPRE.getText());
      pvolumeInfo.setVolumePublisher(volPUB.getText());
      pvolumeInfo.setApplicationId(appID.getText());
      
      // Data track
      while (data.isEmpty() == false) {
	s = (String)data.firstElement();
	data.removeElementAt(0);
	pdataTrack.addPath(s);
      }

      // Audio track
      while (audio.isEmpty() == false) {
	s = (String)audio.firstElement();
	audio.removeElementAt(0);
	paudioTracks.addPath(s);
      }
    } else if (layout == BurnITFrame.isCDROM_wImage) {
      // Image file
      pimageFile.addPath(imageFile.getText());

    } else if (layout == BurnITFrame.isMixedModeCD_wImage) {

      // Image file
      pimageFile.addPath(imageFile.getText());

      // Audio track
      while (audio.isEmpty() == false) {
	s = (String)audio.firstElement();
	audio.removeElementAt(0);
	paudioTracks.addPath(s);
      }
    }

    parent.wizardFinished();
    this.setVisible(false);
    this.dispose();
  }

  public void actionPerformed(ActionEvent e) {
    // The Next button was clicked on, handle it ...
    if (e.getSource() == next) {
      if (layout == BurnITFrame.isCDROM) {
	if (state == WelcomeMenu) {
	  state = FilesystemMenu;
	} else if (state == FilesystemMenu) {
	  state = DataMenu;
	} else if (state == DataMenu) {
	  state = VolInfoMenu;
	} else if (state == VolInfoMenu) {
	  state = FinalMenu;
	} else if (state == FinalMenu) {
	  makeUpdate();
	}
      } else if (layout == BurnITFrame.isCDROM_wImage) {
	if (state == WelcomeMenu) {
	  state = ImagefileMenu;
	} else if (state == FinalMenu) {
	  makeUpdate();
	} else if (state == ImagefileMenu) {
	  state = FinalMenu;
	}
      } else if (layout == BurnITFrame.isAudio) {
	if (state == WelcomeMenu) {
	  state = AudioMenu;
	} else if (state == AudioMenu) {
	  state = FinalMenu;
	} else if (state == FinalMenu) {
	  makeUpdate();
	}
      } else if (layout == BurnITFrame.isMixedModeCD) {
	if (state == WelcomeMenu) {
	  state = FilesystemMenu;
	} else if (state == FilesystemMenu) {
	  state = DataMenu;
	} else if (state == DataMenu) {
	  state = AudioMenu;
	} else if (state == AudioMenu) {
	  state = VolInfoMenu;
	} else if (state == VolInfoMenu) {
	  state = FinalMenu;
	} else if (state == FinalMenu) {
	  makeUpdate();
	}
      } else if (layout == BurnITFrame.isMixedModeCD_wImage) {
	if (state == WelcomeMenu) {
	  state = ImagefileMenu;
	} else if (state == AudioMenu) {
	  state = FinalMenu;
	} else if (state == FinalMenu) {
	  makeUpdate();
	} else if (state == ImagefileMenu) {
	  state = AudioMenu;
	}
      } else if (layout == BurnITFrame.isMultisessionCD) {
	if (state == WelcomeMenu) {
	  state = MultiMenu;
	} else if (state == FilesystemMenu) {
	  state = DataMenu;
	} else if (state == DataMenu) {
	  state = VolInfoMenu;
	} else if (state == VolInfoMenu) {
	  state = FinalMenu;
	} else if (state == FinalMenu) {
	  makeUpdate();
	} else if (state == MultiMenu) {
	  state = DataMenu;
	}
      } else if (layout == BurnITFrame.isCDCopy) {
	if (state == WelcomeMenu) {
	  state = Disc2DiscMenu;
	} else if (state == Disc2DiscMenu) {
	  state = FinalMenu;
	} else if (state == FinalMenu) {
	  makeUpdate();
	}
      }
      createWindow(state);
    // The Prev button was clicked on, handle it ...
    } else if (e.getSource() == prev) {
      if (layout == BurnITFrame.isCDROM) {
	if (state == WelcomeMenu) {

	} else if (state == FilesystemMenu) {
	  state = WelcomeMenu;
	} else if (state == DataMenu) {
	  state = FilesystemMenu;
	} else if (state == VolInfoMenu) {
	  state = DataMenu;
	} else if (state == FinalMenu) {
	  state = VolInfoMenu;
	}
      } else if (layout == BurnITFrame.isCDROM_wImage) {
	if (state == WelcomeMenu) {

	} else if (state == FinalMenu) {
	  state = ImagefileMenu;
	} else if (state == ImagefileMenu) {
	  state = WelcomeMenu;
	}
      } else if (layout == BurnITFrame.isAudio) {
	if (state == WelcomeMenu) {

	} else if (state == AudioMenu) {
	  state = WelcomeMenu;
	} else if (state == FinalMenu) {
	  state = AudioMenu;
	}
      } else if (layout == BurnITFrame.isMixedModeCD) {
	if (state == WelcomeMenu) {

	} else if (state == FilesystemMenu) {
	  state = WelcomeMenu;
	} else if (state == DataMenu) {
	  state = FilesystemMenu;
	} else if (state == AudioMenu) {
	  state = DataMenu;
	} else if (state == VolInfoMenu) {
	  state = AudioMenu;
	} else if (state == FinalMenu) {
	  state = VolInfoMenu;
	}
      } else if (layout == BurnITFrame.isMixedModeCD_wImage) {
	if (state == WelcomeMenu) {

	} else if (state == AudioMenu) {
	  state = ImagefileMenu;
	} else if (state == FinalMenu) {
	  state = AudioMenu;
	} else if (state == ImagefileMenu) {
	  state = WelcomeMenu;
	}
      } else if (layout == BurnITFrame.isMultisessionCD) {
	if (state == WelcomeMenu) {

	} else if (state == FilesystemMenu) {
	  state = MultiMenu;
	} else if (state == DataMenu) {
	  state = FilesystemMenu;
	} else if (state == VolInfoMenu) {
	  state = DataMenu;
	} else if (state == FinalMenu) {
	  state = VolInfoMenu;
	} else if (state == MultiMenu) {
	  state = WelcomeMenu;
	}
      } else if (layout == BurnITFrame.isCDCopy) {
	if (state == Disc2DiscMenu) {
	  state = WelcomeMenu;
	} else if (state == FinalMenu) {
	  state = Disc2DiscMenu;
	}
      }
      createWindow(state);
    // The Cancel button was clicked on, handle it ...
    } else if (e.getSource() == cancel) {
      parent.wizardFinished();
      this.setVisible(false);
      this.dispose();
    // The Add button was clicked on, handle it ...
    } else if (e.getSource() == add) {
      if (state == DataMenu) {
	data.addElement(new String(input.getText()));
      } else if (state == AudioMenu) {
	audio.addElement(new String(input.getText()));
      }
      createWindow(state);
      updateScreen();
    // The Remove button was clicked on, handle it ...
    } else if (e.getSource() == remove) {
      if (state == DataMenu) {
	if (sourceListData.getSelectedIndex() >= 0) {
	  data.removeElementAt(sourceListData.getSelectedIndex());
	}
      } else if (state == AudioMenu) {
	if (sourceListAudio.getSelectedIndex() >= 0) {
	  audio.removeElementAt(sourceListAudio.getSelectedIndex());
	}
      }
      createWindow(state);
      updateScreen();
    // The Multi button was clicked on, handle it ...
    } else if (e.getSource() == multi) {
      magic = utils.getMagicParameters();
      createWindow(state);
      updateScreen();
    }
  }

}
