#!/bin/bash

#   FILE: burn_image.sh -- Burns a filesystem image into a CD-R
# AUTHOR: W. Michael Petullo <burn_baby_burn@flyn.org>
#   DATE: 20 SEP 1998
#
# Copyright (c) 1999 W. Michael Petullo <burn_baby_burn@flyn.org>
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

. /etc/burn_baby_burn/bbb_functions

IMAGE=$1
MEDIA_TYPE=$2
ERR_DEST=$3

USAGE="Usage: ${0##*/} filename cdr|cdrw log|stderr"

if [ "$#" != 3 ]; then
	usage "$USAGE"
elif [ $MEDIA_TYPE != cdr -a $MEDIA_TYPE != cdrw ]; then
	usage "$USAGE"
elif ! configured; then
	print_message $ERR_DEST "No config found in /etc or ~"
	exit 1
fi

# This is a lock file of sorts.  Other CPU intensive scripts should check 
# for its existence before running.  Starving cdrecord of CPU cycles can 
# result in a corrupted CD.
touch /tmp/be_nice
# /usr/local/bin/saf_t_kill_xscreensaver.sh

if too_much_data $IMAGE; then
	print_message $ERR_DEST "WARNING: filesystem too large"
	rm -f /tmp/be_nice
	exit 7
fi

if [ $MEDIA_TYPE = cdr ]; then
        SPEED=$CDR_SPEED
elif [ $MEDIA_TYPE = cdrw ]; then
	SPEED=$CDRW_SPEED
fi

if [ $MEDIA_TYPE = cdrw ]; then
	erase_cdrw.sh $ERR_DEST
fi

touch /tmp/be_nice

cdrecord -v speed=$SPEED dev=$CDRECORD_DEV $IMAGE

# /usr/local/bin/reincarnate_xscreensaver.sh
rm -f /tmp/be_nice
