#!/bin/bash

#   FILE: copy_iso9660.sh -- Makes a 1-1 copy of an ISO9660 CD-ROM.
# AUTHOR: W. Michael Petullo <burn_baby_burn@flyn.org>
#   DATE: 17 FEB 1999
#
# Copyright (c) 1999 W. Michael Petullo <burn_baby_burn@flyn.org>
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

. /etc/burn_baby_burn/bbb_functions

USAGE="Usage: ${0##*/} cdr|cdrw \n
\n\
Copies the current ISO9660 disc in the CD-ROM drive. \n
\n
cdr causes the media to be burned at a faster speed. \n
cdrw causes the media to be erased and burned at a slower speed."

if [ $# != 1 ]; then
	usage "$USAGE"
elif ! configured; then
	print_message stderr "No config found in /etc or ~"
	exit 1
fi

IMAGE_TMP=`mktemp /tmp/image.XXXXXX` \
        || ( print_message stderr "${0##*/}: Can\'t make tmp file"; exit 5 )
MNT_PT_TMP=`mktemp -u /tmp/mnt_pt.XXXXXX` \
        || ( print_message stderr "${0##*/}: Can\'t make tmp dir"; exit 5 )
dd if=$CD_DEV of=$IMAGE_TMP bs=1c
echo Please insert the DESTINATION disc.
swap --wait=0 --no-mount
echo -n Are you SURE the DESTINATION disc is now in the drive? [yn]
read ANS
if [ $ANS = y ]; then
	/usr/bin/burn_image.sh $IMAGE_TMP $1 stderr
else
	rm -f $IMAGE_TMP
	exit 10
fi

mkdir $MNT_PT_TMP
if ! disc_mounts; then
	print_message stderr "WARNING: Could not mount $CD_DEV after burn"
	rm -f $IMAGE_TMP
	rmdir $MNT_PT_TMP
	exit 3
elif ! mount -t iso9660 -o loop=/dev/loop1 $IMAGE_TMP $MNT_PT_TMP; then
	print_message stderr "WARNING: Unable to loopback mount $IMAGE_TMP"
	rm -f $IMAGE_TMP
	rmdir $MNT_PT_TMP
	exit 10
elif ! verify_burn.sh $MNT_PT_TMP stderr; then
	print_message stderr "WARNING: Filesystem corrupt"
	umount $MNT_PT_TMP
	rm -f $IMAGE_TMP
	rmdir $MNT_PT_TMP
	exit 3
fi

umount $MNT_PT_TMP
rm -f $IMAGE_TMP
rmdir $MNT_PT_TMP
print_message stderr Copy seems okay.
exit 0
