#!/bin/bash

#   FILE: erase_cdrw.sh -- Erases a CD-RW.
# AUTHOR: W. Michael Petullo <burn_baby_burn@flyn.org>
#   DATE: 20 SEP 1998
#
# Copyright (c) 1999 W. Michael Petullo <burn_baby_burn@flyn.org>
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

. /etc/burn_baby_burn/bbb_functions

# This is a lock file of sorts.  Other CPU intensive scripts should check 
# for its existence before running.  Starving cdrecord of CPU cycles can 
# result in a corrupted CD.
touch /tmp/be_nice
# /usr/local/bin/saf_t_kill_xscreensaver.sh

ERR_DEST=$1
USAGE="Usage: ${0##*/} log|stderr \n
\n
log causes error messages to be logged using the logger command. \n
stderr causes error messages to be printed to stderr."

if [ $# -lt 1 ]; then
	usage "$USAGE"
elif [ $ERR_DEST != log -a $ERR_DEST != stderr ]; then
	usage "$USAGE"
elif ! configured; then
	print_message $ERR_DEST "No config found in /etc or ~"
	exit 1
fi

if ! cdrecord dev=$CDRECORD_DEV speed=$CDRW_SPEED blank=$BLANK_TYPE 2> /dev/null; then
	print_message $ERR_DEST "WARNING: Error blanking disc"
	rm /tmp/be_nice
	exit 9
fi

# /usr/local/bin/reincarnate_xscreensaver.sh
rm /tmp/be_nice

exit 0
