/*
 * 
 * 	fryit - file cdrom.h
 * 	Copyright Thomas Maier-Komor, 1999
 * 	
 */

#ifndef CDROM_H
#define CDROM_H

#include <qlist.h>

#include <pthread.h>
#include <sys/types.h>
#ifdef LINUXOS
#include <linux/cdrom.h>
#endif

#include "device.h"
#include "utils.h"

class File;
class Track;
class TrackSelection;
class Error;
class Process;

class CDROM : public Device {
	Q_OBJECT
	public:
	CDROM(const char *,const char *,const char *,const char *,const char *,
		int,int,int,int,int,const char *,const char *) throw(Error);
	~CDROM() {};

	void getTOC(QList <Track> &) throw(Error);
	int getCDtype() throw(Error);
	int getDataSize() throw(Error);
	inline int getReadSpeed() const;
	inline void setReadSpeed(int);
	bool mediaChanged() throw(Error);
	void readDataTrack(const TrackSelection *,const char *filename);
	Process *readAudio(const TrackSelection *);
	void readToProcess(int,Process *);
	void msinfo(int &, int &) throw(Error);
	
	public slots:
	void playorresume();
	void play(int,int = 0);
	void pause();
	void resume();
	void stop();

	public slots:
	void eject();
	void cancelAction();
	void processFinished();

	private:
	void *arg[5];
	QString targetfile;
	Process *pr;
	bool readingAudio;
	int rSpeed;
	bool cancelled;
	
	friend void *readDataToProcess(void **);
	friend void *readData(void **);
};

//
// inline functions
//

inline int CDROM::getReadSpeed() const {
	return rSpeed;
}

inline void CDROM::setReadSpeed(int s) {
	rSpeed = s;
}

#endif
