/*
 * 
 * 	fryit - file device.cc
 * 	Copyright Thomas Maier-Komor, 1999
 * 	
 */


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "device.h"
#include "utils.h"

#include "globals.h"

#include <cassert>

Device::Device(const char *v, const char *m, const char *a, const char *t,
	const char *r, int h, int b, int i, int l, int ar, const char *bd,
	const char *gd) :
	QObject(),
	Adapter(a),
	Vendor(v),
	Model(m),
	Type(t),
	Rev(r),
	gDev(gd),
	bDev(bd),
	Host(h),
	Bus(b),
	Id(i),
	Lun(l),
	ansirev(ar),
	locked(false),
	type(generic),
	thrFinished(true),
	thrProgress(0),
	st(idle) {
	
	Name = Vendor + " " + Model;
	Name = Name.simplifyWhiteSpace();
	if (0 == strncmp(Adapter,"scsi",4))
		scsi = true;
	else
		scsi = false;
	devStr.sprintf("%i,%i,%i", Host, Bus * 16 + Id, Lun);
}

void Device::setStatus(status s) {
	st = s;
	emit isOperating(s);
	#ifdef DODEBUG
	switch (s) {
		case idle: 
			debugmsg("Device \"%s\" is now idle",name()); break;
		case reading: 
			debugmsg("Device \"%s\" is now reading",name()); break;
		case writing: 
			debugmsg("Device \"%s\" is now writing",name()); break;
		case playing: 
			debugmsg("Device \"%s\" is now playing",name()); break;
		case paused: 
			debugmsg("Device \"%s\" is now paused",name()); break;
		case other: 
			debugmsg("Device \"%s\" is now doing other stuff",name()); break;
		default:
			assert(0);
	}
	#endif
}

bool Device::isFinished() const {
	if (thrFinished)
		pthread_join(activeThr,0);
	return thrFinished;
}

#include "device.moc"
