/*
 * 
 * 	fryit - file device.h
 * 	Copyright Thomas Maier-Komor, 1999
 * 	
 */

#ifndef DEVICE_H
#define DEVICE_H

#include <qobject.h>
#include <qstring.h>

#include <pthread.h>

class Error;

enum status {idle, playing, paused, reading, writing, other};

enum deviceType {generic, cdrom, recorder, harddisk};

class Device : public QObject {
	Q_OBJECT
	public:
	Device(const char *,const char *,const char *,const char *,
		const char *,int,int,int,int,int,const char *,const char *);
	~Device() {};
	
	inline const char *name() const;
	inline const char *adapter() const;
	inline const char *vendor() const;
	inline const char *model() const;
	inline const char *blockDev() const;
	inline const char *genDev() const;
	inline int host() const;
	inline int bus() const;
	inline int id() const;
	inline int lun() const;
	inline bool isSCSI() const;
	inline status getStatus() const;
	inline int getProgress() const;
	inline deviceType getType() const;
	bool isFinished() const;
	
	protected:
	QString Name,Adapter,Vendor,Model,Type,Rev,gDev,bDev,devStr;
	int Host,Bus,Id,Lun,ansirev;
	bool locked,scsi;
	deviceType type;
	pthread_t activeThr;
	bool thrFinished;
	int thrProgress;
	
	void setStatus(status);
	
	private:
	status st;
	
	signals:
	void isOperating(status);
	void errorOccured(Error);

	friend void threadTerminated(void *);
};

/*
 * inline functions
 */
 
inline const char *Device::name() const {
	return Name;
}

inline const char *Device::adapter() const {
	return Adapter;
}

inline const char *Device::vendor() const {
	return Vendor;
}

inline const char *Device::model() const {
	return Model;
}

inline const char *Device::blockDev() const {
	return bDev;
}

inline const char *Device::genDev() const {
	return gDev;
}

inline int Device::host() const {
	return Host;
}

inline int Device::bus() const {
	return Bus;
}

inline int Device::id() const {
	return Id;
}

inline int Device::lun() const {
	return Lun;
}

inline bool Device::isSCSI() const {
	return scsi;
}

inline status Device::getStatus() const {
	return st;
}

inline deviceType Device::getType() const {
	return type;
}

inline int Device::getProgress() const {
	return thrProgress;
}

#endif
