/*
 * 
 * 	fryit - file dialogs.cc
 * 	Copyright Thomas Maier-Komor, 1999
 * 	
 */


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "cdrom.h"
#include "device.h"
#include "dialogs.h"
#include "globals.h"
#include "lowlevel.h"
#include "recorder.h"
#include "track.h"
#include "widgets.h"
#include "license.inc"

#include <cassert>
#include <cstdio>
#include <cstdlib>
#include <cerrno>
#include <csignal>

#include <qdir.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qheader.h>
#include <qmessagebox.h>
#include <qregexp.h>
#include <qstrlist.h>
#include <qtabbar.h>
#include <qtextstream.h>



ConfigDialog::ConfigDialog(QWidget *p) : QTabDialog(p) {
	setCaption("Configuration");
	setCancelButton();
	setDefaultButton("Reset");
	setOkButton();
	
	QFont f("Helvetica",12);
	tabBar()->setFont(f);
	
	pathPage = new QVBox(this);
	CHECK_PTR(pathPage);
	pathPage->setMargin(10);
	
	cdrecordBox = new QHBox(pathPage);
	CHECK_PTR(cdrecordBox);
	cdrecordBox->setMargin(3);
	mkisofsBox = new QHBox(pathPage);
	CHECK_PTR(mkisofsBox);
	mkisofsBox->setMargin(3);
	cdda2wavBox = new QHBox(pathPage);
	CHECK_PTR(cdda2wavBox);
	cdda2wavBox->setMargin(3);
	playcdrBox = new QHBox(pathPage);
	CHECK_PTR(playcdrBox);
	playcdrBox->setMargin(3);
	cdrecordLbl = new QLabel("cdrecord: ",cdrecordBox);
	CHECK_PTR(cdrecordLbl);
	mkisofsLbl = new QLabel("mkisofs: ",mkisofsBox);
	CHECK_PTR(mkisofsLbl);
	cdda2wavLbl = new QLabel("cdda2wav: ",cdda2wavBox);
	CHECK_PTR(cdda2wavLbl);
	playcdrLbl = new QLabel("playcdr: ",playcdrBox);
	CHECK_PTR(playcdrLbl);
	cdrecordPath = new QPushButton(cdrecordBox);
	CHECK_PTR(cdrecordPath);
	mkisofsPath = new QPushButton(mkisofsBox);
	CHECK_PTR(mkisofsPath);
	cdda2wavPath = new QPushButton(cdda2wavBox);
	CHECK_PTR(cdda2wavPath);
	playcdrPath = new QPushButton(playcdrBox);
	CHECK_PTR(playcdrPath);

	datadirBox = new QHBox(pathPage);
	CHECK_PTR(datadirBox);
	datadirBox->setMargin(3);
	datadirLbl = new QLabel("Directory for track files: ",datadirBox);
	CHECK_PTR(datadirLbl);
	datadirPath = new QPushButton(datadirBox);
	CHECK_PTR(datadirPath);
	
	addTab(pathPage,"Pathes and Directories");
	
	connect(this,SIGNAL(cancelButtonPressed()),SLOT(cancel()));
	connect(this,SIGNAL(defaultButtonPressed()),SLOT(getConfig()));
	connect(this,SIGNAL(applyButtonPressed()),SLOT(setConfig()));
	
	connect(cdrecordPath,SIGNAL(clicked()),SLOT(setCdrecord()));
	connect(mkisofsPath,SIGNAL(clicked()),SLOT(setMkisofs()));
	connect(cdda2wavPath,SIGNAL(clicked()),SLOT(setCdda2wav()));
	connect(playcdrPath,SIGNAL(clicked()),SLOT(setPlaycdr()));
	connect(datadirPath,SIGNAL(clicked()),SLOT(setDatadir()));
	
	resize(400,300);
}

void ConfigDialog::show() {
	getConfig();
	QDialog::show();
}

void ConfigDialog::getConfig() {
	cdrecordPath->setText(Cdrecord);
	cdda2wavPath->setText(Cdda2wav);
	mkisofsPath->setText(Mkisofs);
	playcdrPath->setText(Playcdr);
	datadirPath->setText(DataDir);
}

void ConfigDialog::setConfig() {
	LowLevel::setDataDir(datadirPath->text());
	LowLevel::setCdrecord(cdrecordPath->text());
	LowLevel::setCdda2wav(cdda2wavPath->text());
	LowLevel::setMkisofs(mkisofsPath->text());
	LowLevel::setPlaycdr(playcdrPath->text());
}

void ConfigDialog::cancel() {
	hide();
}

void ConfigDialog::setCdrecord() {
	if (Superuser) {
		QString tmp = QFileDialog::getOpenFileName("/","cdrecord",this);
		if (!tmp.isNull())
			cdrecordPath->setText(tmp);
	} else
		QMessageBox::information(this,"Not allowed!",
					 "For security reasons this value\n"
					 "can only be changed by the superuser.",
					 "OK");
}

void ConfigDialog::setMkisofs() {
	if (Superuser) {
		QString tmp = QFileDialog::getOpenFileName("/","mkisofs",this);
		if (!tmp.isNull())
			mkisofsPath->setText(tmp);
	} else
		QMessageBox::information(this,"Not allowed!",
					 "For security reasons this value\n"
					 "can only be changed by the superuser.",
					 "OK");
}

void ConfigDialog::setCdda2wav() {
	if (Superuser) {
		QString tmp = QFileDialog::getOpenFileName("/","cdda2wav",this);
		if (!tmp.isNull())
			cdda2wavPath->setText(tmp);
	} else
		QMessageBox::information(this,"Not allowed!",
					 "For security reasons this value\n"
					 "can only be changed by the superuser.",
					 "OK");
}

void ConfigDialog::setPlaycdr() {
	if (Superuser) {
		QString tmp = QFileDialog::getOpenFileName("/","playcdr",this);
		if (!tmp.isNull())
			playcdrPath->setText(tmp);
	} else
		QMessageBox::information(this,"Not allowed!",
					 "For security reasons this value\n"
					 "can only be changed by the superuser.",
					 "OK");
}

void ConfigDialog::setDatadir() {
	QString tmp = QFileDialog::getExistingDirectory(datadirPath->text(),this);
	if (!tmp.isNull())
		datadirPath->setText(tmp);
}

BurnDialog::BurnDialog(const QString &cap) : 
	QDialog(0,0,false) {
	
	setCaption(cap);
	hlayout = new QHBoxLayout(this);
	CHECK_PTR(hlayout);
	vlayout = new QVBoxLayout();
	CHECK_PTR(vlayout);
	hlayout->addStretch(1);
	hlayout->addLayout(vlayout,8);
	hlayout->addStretch(1);
	QFont f("Helvetica",12);
	infoLbl = new QLabel("starting recorder...",this);
	CHECK_PTR(infoLbl);
	infoLbl->setFont(f);
	vlayout->addWidget(infoLbl,2);
	fifoLbl = new QLabel("FIFO fill:",this);
	CHECK_PTR(fifoLbl);
	fifoLbl->setFont(f);
	vlayout->addWidget(fifoLbl,2);
	fifo = new QProgressBar(100,this);
	CHECK_PTR(fifo);
	vlayout->addWidget(fifo,3);
	progLbl = new QLabel("Percent completed:",this);
	CHECK_PTR(progLbl);
	progLbl->setFont(f);
	vlayout->addWidget(progLbl,2);
	prog = new QProgressBar(this);
	CHECK_PTR(prog);
	vlayout->addWidget(prog,3);
	vlayout->addStretch(2);
	cancelBtn = new QPushButton("Cancel",this);
	CHECK_PTR(cancelBtn);
	vlayout->addWidget(cancelBtn,3);
	vlayout->addStretch(1);
	setMinimumSize(250,200);
	
	connect(cancelBtn,SIGNAL(clicked()),SLOT(doCancel()));
	resize(250,200);
}

BurnDialog::~BurnDialog() {
	debugmsg("Burndialog deleted");
}

void BurnDialog::doCancel() {
	hide();
	debugmsg("BurnDialog: emit cancel()");
	emit cancel();
}

void BurnDialog::setInfo(const QString &inf) {
	infoLbl->setText(inf);
}

void BurnDialog::setProgress(int pr) {
	prog->setProgress(pr);
}

void BurnDialog::setFifo(int f) {
	fifo->reset();
	fifo->setProgress(f);
}

void BurnDialog::reset() {
	prog->reset();
	prog->setProgress(0);
}

ReadDialog::ReadDialog(const char *name, QWidget *p) :
	QProgressDialog(0,"Cancel",0,p,0,false) {
	
	bar = new QProgressBar(this);
	CHECK_PTR(bar);
	setBar(bar);
	bar->setTotalSteps(100);
	QString str;
	str.sprintf("reading Track \"%s\" ...",name);
	setLabelText(str);
	QFont f("Helvetica",12);
	lbl = new QLabel(str,this);
	CHECK_PTR(lbl);
	lbl->setFont(f);
	lbl->resize(lbl->sizeHint());
	setLabel(lbl);
}

void ReadDialog::outMsg(const char *) {
}

void ReadDialog::errMsg(const char *str) {
	int prog = 0;
	QString msg = str;
	QRegExp reg("[0-9]+/\\s*[0-9]+/\\s*[0-9]+/\\s*[0-9]+\\s*[0-9]+%");
	int at = msg.find(reg);
	if (-1 != at) {
		msg = msg.mid(at);
		reg = "[0-9]+%";
		msg = msg.mid(msg.find(reg));
		sscanf(msg,"%i%%",&prog);
		bar->setProgress(prog);
	}
}

ReadDataDialog::ReadDataDialog(CDROM *cdr, TrackSelection *tr, QWidget *p = 0) :
	QProgressDialog(0,"Cancel",0,p,0,false),
	dev(cdr),
	error(false),
	t(),track(tr) {
	
	bar = new QProgressBar(this);
	CHECK_PTR(bar);
	setBar(bar);
	bar->setTotalSteps(100);
	QFont f("Helvetica",12);
	QString str;
	str.sprintf("reading Track \"%s\" ...",tr->getName());
	setLabelText(str);
	lbl = new QLabel(str,this);
	CHECK_PTR(lbl);
	lbl->setFont(f);
	lbl->resize(lbl->sizeHint());
	setLabel(lbl);
	
	if (!tr->getTrackID())
		tr->setTrackID(LowLevel::newTrackID(audio == tr->getTrackType()));
	t.changeInterval(500);
	connect(&t,SIGNAL(timeout()),SLOT(update()));
	connect(this,SIGNAL(cancelled()),SLOT(cancel()));
	QString fn = DataDir;
	fn += tr->filename();
	fn += ".cdr";
	dev->readDataTrack(tr,fn);
}

void ReadDataDialog::update() {
	bar->setProgress(dev->getProgress());
	if (dev->isFinished()) {
		hide();
		emit finished();
	}
}

void ReadDataDialog::cancel() {
	debugmsg("cancelling reading of track...");
	t.stop();
	dev->cancelAction();
}

/*
CDPlayer::CDPlayer() :
	QDialog(),
	t() {
	
	disp = new QLCDNumber(5,this);
	CHECK_PTR(disp);
	disp->move(30,30);
	track = new QSpinBox(this);
	CHECK_PTR(track);
	track->move(180,30);
	devSel = new QComboBox(this);
	CHECK_PTR(devSel);
	devSel->move(180,50);
	time = new QSlider(QSlider::Horizontal,this);
	CHECK_PTR(time);
	time->setTickmarks(QSlider::Below);
	time->move(30,50);
	
	play = new QPushButton("Play",this);
	CHECK_PTR(play);
	play->setGeometry(20,100,60,30);
	stop = new QPushButton("Stop",this);
	CHECK_PTR(stop);
	stop->setGeometry(110,100,60,30);
	pause = new QPushButton("Pause",this);
	CHECK_PTR(pause);
	pause->setGeometry(210,100,60,30);
	
	connect(devSel,SIGNAL(activated(const QString &)),SLOT(changeDevice(const QString &)));
	
	resize(300,135);
}

void CDPlayer::configure() {
	for (Device *d = CdromList.first(); d; d = CdromList.next()) {
		debugmsg("adding %s",d->name());
		devSel->insertItem(d->name());
	}
	for (Device *d = RecorderList.first(); d; d = RecorderList.next())
		devSel->insertItem(d->name());
	dev = CdromList.first();
	devSel->setCurrentItem(0);
}

void CDPlayer::doPlay() {
	try {
		dev->play(1);
	}
	catch (Error e) {
		QMessageBox::critical(0,"Error",e);
	}

}

void CDPlayer::doStop() {
	try {
		dev->stop();
	}
	catch (Error e) {
		QMessageBox::critical(0,"Error",e);
	}
}

void CDPlayer::doResume() {
	try {
		dev->resume();
	}
	catch (Error e) {
		QMessageBox::critical(0,"Error",e);
	}
}

void CDPlayer::doPause() {
	try {
		dev->pause();
	}
	catch (Error e) {
		QMessageBox::critical(0,"Error",e);
	}
}

void CDPlayer::show() {
	tracks.clear();
	dev->getTOC(tracks);
	QDialog::show();
}

void CDPlayer::changeDevice(const QString &devName) {
	for (CDROM *d = CdromList.first(); d; d = CdromList.next())
		if (devName == d->name()) {
			dev = d;
			break;
		}
	for (CDROM *d = RecorderList.first(); d; d = RecorderList.next())
		if (devName == d->name()) {
			dev = d;
			break;
		}
	dev->getTOC(tracks);
	for (Track *t = tracks.first(); t; t = tracks.next())
		if (audio != t->getTrackType())
			tracks.remove(t);
	if (tracks.count())
		track->setRange(1,tracks.count());
	else
		track->setRange(0,0);
}
*/

CopyInfo::CopyInfo() : QDialog() {
	setCaption("Fryit - License");
	view = new QMultiLineEdit(this);
	CHECK_PTR(view);
	view->setReadOnly(true);
	view->setText(GNUlicense);
	closeBtn = new QPushButton("Close",this);
	CHECK_PTR(closeBtn);
	connect(closeBtn,SIGNAL(clicked()),SLOT(close()));
	
	resize(600,450);
}

QSize CopyInfo::sizeHint() const {
	QSize s;
	s.setWidth(max(closeBtn->sizeHint().width() * 3, view->sizeHint().width()));
	s.setHeight(max(300,closeBtn->sizeHint().height() * 2 + view->sizeHint().height()));
	return s;
}

void CopyInfo::resizeEvent(QResizeEvent *) {
	view->setGeometry(0,0,width(),height() - closeBtn->sizeHint().height() * 2);
	closeBtn->setGeometry(width() - 1.5 * closeBtn->sizeHint().width(),
		height() - 1.5 * closeBtn->sizeHint().height(),
		closeBtn->sizeHint().width(),
		closeBtn->sizeHint().height());
}

void CopyInfo::close() {
	done(0);
}

DeviceConfigDialog::DeviceConfigDialog(Recorder *d) : 
	QDialog(),
	rec(d) {
	
	QString cap = d->name();
	cap.prepend("Configuration of ");
	setCaption(cap);
	
	QFont f("Helvetica",12);
	
	drvLbl = new QLabel("Driver: ",this);
	CHECK_PTR(drvLbl);
	drvLbl->setFont(f);
	drv = new QComboBox(this);
	CHECK_PTR(drv);
	drv->insertStringList(DriverDescr);
	writeSpLbl = new QLabel("Writer speed: ",this);
	CHECK_PTR(writeSpLbl);
	writeSpLbl->setFont(f);
	writeSp = new QComboBox(this);
	CHECK_PTR(writeSp);
	writeSp->insertItem("default");
	writeSp->insertItem("1");
	writeSp->insertItem("2");
	writeSp->insertItem("4");
	writeSp->insertItem("6");
	writeSp->insertItem("8");
	writeSp->insertItem("10");
	writeSp->insertItem("12");
	fifoLbl = new QLabel("FIFO size in kB:",this);
	CHECK_PTR(fifoLbl);
	fifoLbl->setFont(f);
	fifo = new QSpinBox(512,32 * 1024,256,this);
	CHECK_PTR(fifo);
	
	ok = new QPushButton("Ok",this);
	CHECK_PTR(ok);
	cancel = new QPushButton("Cancel",this);
	CHECK_PTR(cancel);
	defaults = new QPushButton("Defaults",this);
	CHECK_PTR(defaults);
	connect(ok,SIGNAL(clicked()),SLOT(accept()));
	connect(ok,SIGNAL(clicked()),SLOT(setConfig()));
	connect(cancel,SIGNAL(clicked()),SLOT(reject()));
	connect(defaults,SIGNAL(clicked()),SLOT(setDefaults()));
	
	simulate = new QCheckBox("Simulate",this);
	CHECK_PTR(simulate);
	eject = new QCheckBox("Eject when done",this);
	CHECK_PTR(eject);
	multi = new QCheckBox("Create multisession tocs",this);
	CHECK_PTR(multi);

	buttonLayout = new QHBoxLayout(0,0,15);
	CHECK_PTR(buttonLayout);
	buttonLayout->addStretch(10);
	buttonLayout->addWidget(defaults);
	buttonLayout->addWidget(ok);
	buttonLayout->addWidget(cancel);
	buttonLayout->addStretch(1);
	
	layout = new QGridLayout(this,6,4,5,3);
	CHECK_PTR(layout);
	layout->addWidget(drvLbl,0,0);
	layout->addMultiCellWidget(drv,0,0,1,3);
	layout->addWidget(fifoLbl,1,0);
	layout->addWidget(fifo,1,1);
	layout->addWidget(writeSpLbl,2,0);
	layout->addWidget(writeSp,2,1);
	layout->addMultiCellWidget(eject,1,1,2,3);
	layout->addMultiCellWidget(multi,2,2,2,3);
	layout->addMultiCellWidget(simulate,3,3,2,3);
	layout->addMultiCell(buttonLayout,5,5,0,3);
	
	resize(230,150);
}

void DeviceConfigDialog::setDefaults() {
	drv->setCurrentItem(0);
	writeSp->setCurrentItem(0);
	fifo->setValue(4096);
	multi->setChecked(false);
	eject->setChecked(true);
	simulate->setChecked(true);
}

void DeviceConfigDialog::setConfig() {
	rec->setDriver(drv->currentItem());
	int ind = writeSp->currentItem();
	rec->setWriteSpeed((ind <= 2) ? ind : (ind - 1) * 2);
	rec->setFifo(fifo->value());
	rec->setMultisession(multi->isChecked());
	rec->setEject(eject->isChecked());
	rec->setSimulate(simulate->isChecked());
}

void DeviceConfigDialog::getConfig() {
	drv->setCurrentItem(rec->getDriver());
	int sp = rec->getWriteSpeed();
	writeSp->setCurrentItem((sp <= 2) ? sp : sp / 2 + 1);
	fifo->setValue(rec->getFifo());
	multi->setChecked(rec->getMultisession());
	eject->setChecked(rec->getEject());
	simulate->setChecked(rec->getSimulate());
}

void DeviceConfigDialog::show() {
	getConfig();
	QDialog::show();
}

#include "dialogs.moc"
