/*
 * 
 * 	fryit - file dialogs.h
 * 	Copyright Thomas Maier-Komor, 1999
 * 
 */

#ifndef DIALOGS_H
#define DIALOGS_H

#include <pthread.h>

#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qdatetime.h>
#include <qgroupbox.h>
#include <qhbox.h>
#include <qheader.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlcdnumber.h>
#include <qlistbox.h>
#include <qlistview.h>
#include <qmultilinedit.h>
#include <qprogressdialog.h>
#include <qscrollview.h>
#include <qslider.h>
#include <qspinbox.h>
#include <qtabdialog.h>
#include <qtimer.h>
#include <qvbox.h>

class Track;
class Device;
class TrackSelection;
class Process;
class CDROM;
class Recorder;

class ConfigDialog : public QTabDialog {
	Q_OBJECT
	
	public:
	ConfigDialog(QWidget * = 0);
	~ConfigDialog() {};
	virtual void show();
	
	private:
	QVBox *pathPage;
	QHBox *cdrecordBox,*mkisofsBox,*cdda2wavBox,*playcdrBox,*datadirBox;
	QLabel *cdrecordLbl,*mkisofsLbl,*cdda2wavLbl,*playcdrLbl,*datadirLbl;
	QPushButton *cdrecordPath,*mkisofsPath,*cdda2wavPath,*playcdrPath,*datadirPath;
	QCheckBox *cdrecordOk,*mkisofsOk,*cdda2wavOk,*datadirOk;
	
	private slots:
	void getConfig();
	void setConfig();
	void cancel();
	void setCdrecord();
	void setMkisofs();
	void setCdda2wav();
	void setPlaycdr();
	void setDatadir();
};

class DeviceConfigDialog : public QDialog {
	Q_OBJECT
	public:
	DeviceConfigDialog(Recorder *d);
	~DeviceConfigDialog() {};
	void show();
	
	private:
	Recorder *rec;
	QLabel *drvLbl,*fifoLbl,*writeSpLbl;
	QComboBox *drv,*writeSp;
	QSpinBox *fifo;
	QPushButton *ok,*cancel,*defaults;
	QGridLayout *layout;
	QHBoxLayout *buttonLayout;
	QCheckBox *simulate,*eject,*multi;
	
	private slots:
	void setDefaults();
	void setConfig();
	void getConfig();
};

class BurnDialog : public QDialog {
	Q_OBJECT
	public:
	BurnDialog(const QString &);
	~BurnDialog();
	
	
	public slots:
	void setInfo(const QString &);
	void setProgress(int);
	void setFifo(int);
	void reset();
	
	private:
	QHBoxLayout *hlayout;
	QVBoxLayout *vlayout;
	QLabel *infoLbl,*fifoLbl,*progLbl;
	QProgressBar *fifo,*prog;
	QPushButton *cancelBtn;
	
	private slots:
	void doCancel();
	
	signals:
	void cancel();
};

class ReadDialog : public QProgressDialog {
	Q_OBJECT
	public:
	ReadDialog(const char *,QWidget * = 0);
	~ReadDialog() {};
	
	public slots:
	void outMsg(const char *);
	void errMsg(const char *);
	
	private:
	QProgressBar *bar;
	QLabel *lbl;
};

class ReadDataDialog : public QProgressDialog {
	Q_OBJECT
	public:
	ReadDataDialog(CDROM *,TrackSelection *,QWidget * = 0);
	~ReadDataDialog() {};
	
	private:
	CDROM *dev;
	bool error;
	QTimer t;
	QProgressBar *bar;
	QLabel *lbl;
	void *args[3];
	TrackSelection *track;

	private slots:
	void update();
	void cancel();
	
	signals:
	void finished();
};

/*
class CDPlayer : public QDialog {
	Q_OBJECT
	public:
	CDPlayer();
	~CDPlayer() {};
	void configure();
	void show();
	
	private slots:
	void doPlay();
	void doPause();
	void doResume();
	void doStop();
	void changeDevice(const QString &);
	
	private:
	CDROM *dev;
	QComboBox *devSel;
	QPushButton *play,*stop,*pause;
	QSlider *time;
	QSpinBox *track;
	QLCDNumber *disp;
	QTimer t;
	QList <Track> tracks;
};
*/

class CopyInfo : public QDialog {
	Q_OBJECT
	public:
	CopyInfo();
	~CopyInfo() {};
	
	public slots:
	QSize sizeHint() const;
	void resizeEvent(QResizeEvent *);
	
	private slots:
	void close();
	
	private:
	QPushButton *closeBtn;
	QMultiLineEdit *view;
};

#endif
