/*
 * 
 * 	fryit - file globals.cc
 * 	Copyright Thomas Maier-Komor, 1999
 * 	
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "globals.h"

#include <cstdio>

#ifdef NAMESPACES
namespace LowLevel {
	extern bool
		superuser;
	extern int
		rSpeed, wSpeed, fifosize, wBus, wID, wLUN, rBus, rID, rLUN,
		majVer, minVer, patVer;
	extern QStringList
		drivers,driverDescr;
	extern QList <CDROM>
		cdromList;
	extern QList <Recorder>
		recorderList;
	extern uid_t
		userID;
	extern gid_t
		groupID;
};
#else
class LowLevel {
	public:
	static bool
		superuser;
	static int
		rSpeed, wSpeed, fifosize, wBus, wID, wLUN, rBus, rID, rLUN,
		majVer, minVer, patVer;
	static QStringList
		drivers,driverDescr;
	static QList <CDROM>
		cdromList;
	static QList <Recorder>
		recorderList;
	static uid_t
		userID;
	static gid_t
		groupID;
};
#endif

const char
	*DataDir = 0, *Cdda2wav = 0, *Cdrecord = 0, *Mkisofs = 0, *Playcdr = 0;

const bool
	&Superuser = LowLevel::superuser;
	
const int
	&MajVer = LowLevel::majVer,
	&MinVer = LowLevel::minVer,
	&PatVer = LowLevel::patVer;
	
const QStringList
	&Driver = LowLevel::drivers,
	&DriverDescr = LowLevel::driverDescr;

const uid_t
	&UserID = LowLevel::userID;
	
const gid_t
	&GroupID = LowLevel::groupID;

QList <CDROM>
	&CdromList = LowLevel::cdromList;
	
QList <Recorder>
	&RecorderList = LowLevel::recorderList;


int verboseLvl = 5;

void infomsg(const char *msg, ...) {
	if (verboseLvl < 4)
		return;
	va_list val;
	va_start(val,msg);
	if (mainWin) {
		char str[4096];
		vsprintf(str,msg,val);
		mainWin->writeLog(str);
#ifdef DODEBUG
		if (verboseLvl == 5) {
			cout.vform(msg,val);
			cout << "\n";
		}
	} 
	else {
		cout.vform(msg,val);
		cout << "\n";
#endif
	}
	va_end(val);
}

void warningmsg(const char *msg, ...) {
	if (verboseLvl < 3)
		return;
	va_list val;
	va_start(val,msg);
	if (mainWin) {
		char str[4096] = "warning: ";
		vsprintf(&str[9],msg,val);
		mainWin->writeLog(str);
#ifdef DODEBUG
		if (verboseLvl == 5) {
			cout.vform(msg,val);
			cout << "\n";
		}
	} else {
		cout << "warning: ";
		cout.vform(msg,val);
		cout << "\n";
#endif
	}
	va_end(val);
}

void errormsg(const char *msg, ...) {
	if (verboseLvl < 2)
		return;
	va_list val;
	va_start(val,msg);
	if (mainWin) {
		char str[4096] = "error: ";
		vsprintf(&str[7],msg,val);
		mainWin->writeLog(str);
#ifdef DODEBUG
		if (verboseLvl == 5) {
			cout.vform(msg,val);
			cout << "\n";
		}
	} else {
		cout << "error: ";
		cout.vform(msg,val);
		cout << "\n";
#endif
	}
	va_end(val);
}

void fatalmsg(const char *msg, ...) {
	if (verboseLvl < 1)
		return;
	va_list val;
	va_start(val,msg);
	if (mainWin) {
		char str[4096] = "fatal error: ";	
		vsprintf(&str[13],msg,val);
		mainWin->writeLog(str);
	}
#ifdef DODEBUG
	cout << "fatal error: ";
	cout.vform(msg,val);
	cout << "\n";
#endif
	va_end(val);
}

