/*
 * 
 * 	fryit - file globals.h
 * 	Copyright Thomas Maier-Komor, 1999
 * 	
 */

#ifndef GLOBALS_H
#define GLOBALS_H

#include "config.h"

#include <cstdarg>
#include <fstream.h>
#include <sys/types.h>

#ifndef CDROM_H
class CDROM;
#endif

#ifndef RECORDER_H
class Recorder;
#endif

#ifndef GUI_H
#include <qapp.h>
#include <qtextstream.h>
class MainWindow : public QWidget {
	public:
	MainWindow(QApplication *);
	void writeLog(const char *);
	void quit();
	void unimplemented();
	void updateView();
	void postError(const QString &);
	void show();
	void configure();
	void displayLog();
	void writeConfig(QTextStream &);
};

extern MainWindow *mainWin;
#endif

extern int verboseLvl;

#ifdef DODEBUG
inline void debugmsg(const char *msg, ...) {
	if (verboseLvl < 5)
		return;
	va_list val;
	
	va_start(val,msg);
	cerr.vform(msg,val);
	cerr << "\n";
	va_end(val);
#else
inline void debugmsg(const char *, ...) {
#endif
}

class Error {
	public:
	Error(const QString &s) {
		debugmsg("new Error(\"%s\")",s.operator const char *());
		msg = s;
	};
	Error(const char *s) {msg = s;};
	~Error() {};
	
	operator const char *() { return msg; };
	operator QString() { return msg; };
	
	private:
	QString msg;
};



void infomsg(const char *, ...);
void warningmsg(const char *, ...);
void errormsg(const char *, ...);
void fatalmsg(const char *, ...);

inline int min(int a, int b) {
	return (a <= b) ? a : b;
}

inline int max(int a, int b) {
	return (a >= b) ? a : b;
}


extern const char 
	*DataDir, *Mkisofs, *Cdda2wav, *Playcdr, *Cdrecord;

extern const bool 
	&Superuser,
	&Multisession,
	&Simulate,
	&Ejectcd;

extern const QStringList
	&Driver,&DriverDescr;
	
extern QList <CDROM>
	&CdromList;
	
extern QList <Recorder>
	&RecorderList;
	
extern const uid_t
	&UserID;

extern const gid_t
	&GroupID;

#endif
