/*
 * 
 * 	fryit - file gui.h
 * 	Copyright Thomas Maier-Komor, 1999
 * 	
 */

#ifndef GUI_H
#define GUI_H

#include <qapplication.h>
#include <qkeycode.h>
#include <qlistview.h>
#include <qmainwindow.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <qmultilinedit.h>
#include <qpopupmenu.h>
#include <qsplitter.h>
#include <qstrlist.h>
#include <qtextstream.h>
#include <qtimer.h>

#include <pthread.h>

#include "cdrom.h"

class ConfigDialog;
class CopyInfo;
class Process;
class Recorder;
class Track;
class TrackItem;
class MakeIsofsDialog;

class ReadSelectionDialog;
class WriteSelectionDialog;
class DeviceConfigDialog;

// forward definitions
class CdromMenu;
class RecorderMenu;

class MainWindow : public QMainWindow {
	Q_OBJECT

	public:
	MainWindow(QApplication *);
	~MainWindow() {};
	void writeLog(const char *);
	void configure();
	
	public slots:
	void updateView();
	void quit();
	void unimplemented();
	void postError(const QString &);
	void displayLog();
	
	private:
	QApplication *qapp;
	QPopupMenu file,devices,actions,options,infos;
	QSplitter *splitter;
	QMultiLineEdit *log;
	QListView *diskTrackBox;
	QList <TrackItem> diskTracks;
	QList <CdromMenu> readerMenus;
	QList <RecorderMenu> writerMenus;
	QStrList errors;
	QTimer errT;
	pthread_mutex_t errorMut;
	
	ConfigDialog *confDlg;
	CopyInfo *copyinfo;
	MakeIsofsDialog *mkisofsdlg;
//	CDPlayer cdplayer;
	
	Process *player,*copyPr;
	
	int dispLog;

	private slots:
	void showCopyDialog();
	void showConfig();
	void safeConfig();
	void version();
	void aboutQT();
	void resizeEvent(QResizeEvent *);
	void deleteTrack();
	void play();
	void pause();
	void resume();
	void stop();
	void playerStopped(int);
	void issueErrors();
	void showIsofsDialog();
};

extern MainWindow *mainWin;

class CdromMenu : public QPopupMenu {
	Q_OBJECT
	public:
	CdromMenu(CDROM *);
	~CdromMenu() {};
	
	inline Device *device() const;
	
	protected slots:
	void enable(status);
	
	protected:
	CDROM *cdr;
	ReadSelectionDialog *rsd;
	int play,pause,stop;
};

class RecorderMenu : public CdromMenu {
	Q_OBJECT
	public:
	RecorderMenu(Recorder *);
	~RecorderMenu() {};
	
	private:
	Recorder *rec;
	WriteSelectionDialog *wsd;
	DeviceConfigDialog *dcd;
};

/*
 * inline functions
 */

inline Device *CdromMenu::device() const {
	return cdr;
}

#endif
