/*
 * 
 * 	fryit - file lowlevel.h
 * 	Copyright Thomas Maier-Komor, 1999
 * 	
 */

#ifndef LOWLEVEL_H
#define LOWLEVEL_H

#include <pthread.h>

#include <qstring.h>
#include <qlist.h>
#include <qlistview.h>

#include "globals.h"

class Device;
class Track;
class Process;
class Error;
class TrackSelection;

#ifdef NAMESPACES
namespace LowLevel {

	int init();
	int newTrackID(bool audio = true);
	void writeTrackInfo(const TrackSelection *);
	void writeConfig();
	void deleteTrack(const Track *);
	void getDiscTracks(QList <Track> &);
	void cleanDatadir();
	Process *playDiskTrack(const Track *,int = 0);
	
	void setDataDir(const char *);
	void setMkisofs(const char *);
	void setCdrecord(const char *);
	void setPlaycdr(const char *);
	void setCdda2wav(const char *);
	Process *copyCD(pthread_t *) throw(Error);
	Process *copyCD_pipe();

#else

class LowLevel {
	private:
	LowLevel() {};
	
	public:
	~LowLevel() {};
	
	static int init();
	static int newTrackID(bool audio = true);
	static void writeTrackInfo(const TrackSelection *);
	static void writeConfig();
	static void deleteTrack(const Track *);
	static void getDiscTracks(QList <Track> &);
	static void cleanDatadir();
	static Process *playDiskTrack(const Track *,int = 0);
	
	static void setDataDir(const char *);
	static void setMkisofs(const char *);
	static void setCdrecord(const char *);
	static void setPlaycdr(const char *);
	static void setCdda2wav(const char *);
	static Process *copyCD(pthread_t *) throw(Error);
	static Process *copyCD_pipe();
	
	private:
	static int
		majVer, minVer, patVer;
	static bool
		superuser;
	static QString
		cdrecord, cdda2wav, mkisofs, playcdr, dataDir;
	static QList <Device> 
		deviceList;
	static QList <CDROM>
		cdromList;
	static QList <Recorder>
		recorderList;
	static QStringList
		drivers,driverDescr;
	static pthread_mutex_t
		idMut;
	static QValueList <int>
		audioIDs,dataIDs;
	static uid_t
		userID;
	static gid_t
		groupID;

	static void scanSCSIDevices();
	static int checkRecorder(int,int,int);
	static void setupDriverList();
	static void initPathes();
	static void setupLists();
	static void setupIDlists();
	static void setRecorder(const QString &,int,int,int,int,int,bool,bool,bool);
	static Device *getDevice(QString &dev, int bus, int id, int lun);
	static bool boolvalue(const QString &);
	static void setOption(const QString &, const QString &);
	static void setRootOption(const QString &, const QString &);
	static void readGlobalConfig();
	static void readLocalConfig();
#endif
};


#endif
