/*
 * 
 * 	fryit - file main.cc
 * 	Copyright Thomas Maier-Komor, 1999
 * 	
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "dialogs.h"
#include "gui.h"

#include "globals.h"

#include <cstdio>
#include <csignal>
#include <exception>

#include <qapplication.h>
#include <qmessagebox.h>


#ifdef NAMESPACES
namespace LowLevel {
#else
class LowLevel {
	public:
	static 
#endif
	void init();
};

extern int verboseLvl;

MainWindow *mainWin = NULL;
bool running = false;


RETSIGTYPE sighandler(int num) {
	switch (num) {
		case SIGPIPE:
			debugmsg("catched signal SIGPIPE");
			break;
		case SIGTERM:
			debugmsg("catched signal SIGTERM");
			if (mainWin)
				mainWin->quit();
			infomsg("Catched term signal.");
			break;
		case SIGINT:
			debugmsg("catched signal SIGINT");
			break;
		default:
			errormsg("catched unexpected signale %i",num);
	}
}

void fryit_terminate() {
	cerr << "An internal error occured... fryit terminates.\n";
	if (running)
		QMessageBox::critical(mainWin,"fryit terminates",
			      "An internal error occured.\n"
			      "Fryit is now terminating...",
			      "Sorry");
	abort();
}

void fryit_unexpected() {
	cerr << "Uncatched exception! This is a bug. Please report this.\n";
	if (running)
		QMessageBox::critical(mainWin,"Uncatched exception",
			      "There was an unexpected exception thrown.\n"
			      "This is a bug! Please report this.",
			      "OK");
	abort();
}

int main(int argc, char **argv) {
	QApplication qApp(argc,argv);
	
	set_terminate(fryit_terminate);
	set_unexpected(fryit_unexpected);
	signal(SIGPIPE,sighandler);
	signal(SIGTERM,sighandler);
	signal(SIGINT,sighandler);
	mainWin = new MainWindow(&qApp);
	qApp.setMainWidget(mainWin);
	mainWin->show();
	LowLevel::init();
	mainWin->configure();
	running = true;
	return qApp.exec();
}
