/*
 * 
 * 	fryit - file master.cc
 * 	Copyright Thomas Maier-Komor, 1999
 * 	
 */

#include <qfile.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qprogressbar.h>
#include <qregexp.h>
#include <qtextstream.h>

#include <cerrno>
#include <math.h>
#include <unistd.h>

#include "cdrom.h"
#include "lowlevel.h"
#include "master.h"
#include "recorder.h"
#include "track.h"
#include "utils.h"
#include "globals.h"


MakeIsofsDialog::MakeIsofsDialog() :
	QTabDialog(),
	pr(0) {
	
	setCancelButton();
	setCaption("Master Track");
	
	volumeArea = new QGrid(2,QGrid::Horizontal,this);
	CHECK_PTR(volumeArea);
	volumeArea->setMargin(10);
	volumeArea->setSpacing(5);
	optionArea = new QVBox(this);
	CHECK_PTR(optionArea);
	optionArea->setMargin(10);
	optionArea->setSpacing(5);
	
	skipbackups = new QCheckBox("Skip backup files",optionArea);
	CHECK_PTR(skipbackups);
	rockridge = new QCheckBox("Generate Rock Ridge infos with uid,gid = 0",
		optionArea);
	CHECK_PTR(rockridge);
	translationtable = new QCheckBox("Create TRANS.TBL files",optionArea);
	CHECK_PTR(translationtable);
	joilet = new QCheckBox("Generate Joilet extensions",optionArea);
	CHECK_PTR(joilet);
	longnames = new QCheckBox("Allow full 32 character filenames",optionArea);
	CHECK_PTR(longnames);
	followlinks = new QCheckBox("Follow symbolic links",optionArea);
	CHECK_PTR(followlinks);
	allowperiod = new QCheckBox("Allow \".\" at beginning of filename",optionArea);
	CHECK_PTR(allowperiod);
	
	nameLbl = new QLabel("Track name: ",volumeArea);
	CHECK_PTR(nameLbl);
	name = new QLineEdit(volumeArea);
	CHECK_PTR(name);
	preparerLbl = new QLabel("Volume Preparer: ",volumeArea);
	CHECK_PTR(preparerLbl);
	preparer = new QLineEdit(volumeArea);
	CHECK_PTR(preparer);
	publisherLbl = new QLabel("Volume Publisher: ",volumeArea);
	CHECK_PTR(publisherLbl);
	publisher = new QLineEdit(volumeArea);
	CHECK_PTR(publisher);
	volumeidLbl = new QLabel("Volume ID: ",volumeArea);
	CHECK_PTR(volumeidLbl);
	volumeid = new QLineEdit(volumeArea);
	CHECK_PTR(volumeid);
	applicationLbl = new QLabel("Application: ",volumeArea);
	CHECK_PTR(applicationLbl);
	application = new QLineEdit(volumeArea);
	CHECK_PTR(application);
	excludeLbl = new QLabel("Exclude Files: ",volumeArea);
	CHECK_PTR(excludeLbl);
	exclude = new QLineEdit(volumeArea);
	CHECK_PTR(exclude);
	directoryLbl = new QLabel("Select directory",volumeArea);
	dirselect = new QPushButton("/",volumeArea);
	CHECK_PTR(dirselect);
	
	sessionArea = new QVBox(this);
	CHECK_PTR(sessionArea);
	sessionArea->setMargin(10);
	sessionArea->setSpacing(5);
	appendsession = new QCheckBox("Append session",sessionArea);
	CHECK_PTR(appendsession);
	new QLabel("Device with CD containing previous session:",sessionArea);
	device = new QComboBox(sessionArea);
	CHECK_PTR(device);
	
	addTab(volumeArea,"Volume");
	addTab(optionArea,"Options");
	addTab(sessionArea,"Sessions");
	
	connect(dirselect,SIGNAL(clicked()),SLOT(selectDir()));
	connect(this,SIGNAL(applyButtonPressed()),SLOT(start()));
}

void MakeIsofsDialog::show() {
	device->clear();
	for (Device *d = CdromList.first(); d; d = CdromList.next())
		if (idle == d->getStatus())
			device->insertItem(d->name());
	for (Device *d = RecorderList.first(); d; d = RecorderList.next())
		if (idle == d->getStatus())
			device->insertItem(d->name());
	QTabDialog::show();
}

void MakeIsofsDialog::selectDir() {
	QString tmp = QFileDialog::getExistingDirectory(dirselect->text());
	if (!tmp.isEmpty())
		dirselect->setText(tmp);
}

void MakeIsofsDialog::start() {
	QString params;
	QString prev;
	if (name->text().isEmpty()) {
		QMessageBox::information(
			0,
			"Retry",
			"The field \"Track name\" is mandatory\n"
			"as it is used to identify the track!","OK");
		show();
		return;
	}
	Track *tr = new Track(0,0,0,0,data1,name->text(),LowLevel::newTrackID(false));
	const char *argv[32];
	argv[0] = Mkisofs;
	argv[1] = "-o";
	QString fn = tr->filename();
	fn += ".cdr";
	argv[2] = fn;
	int a = 3;
	if (!skipbackups->isChecked())
		argv[a++] = "-a";
	if (rockridge->isChecked())
		argv[a++] = "-r";
	if (translationtable->isChecked())
		argv[a++] = "-T";
	if (joilet->isChecked())
		argv[a++] = "-J";
	if (longnames->isChecked())
		argv[a++] = "-l";
	if (followlinks->isChecked())
		argv[a++] = "-f";
	if (allowperiod->isChecked())
		argv[a++] = "-L";
	if (!preparer->text().isEmpty()) {
		argv[a++] = "-p";
		argv[a++] = preparer->text();
	}
	if (!publisher->text().isEmpty()) {
		argv[a++] = "-P";
		argv[a++] = publisher->text();
	}
	if (!volumeid->text().isEmpty()) {
		argv[a++] = "-V";
		argv[a++] = volumeid->text();
	}
	if (!exclude->text().isEmpty()) {
		argv[a++] = "-m";
		argv[a++] = exclude->text();
	}
	if (appendsession->isChecked()) {
		CDROM *cdr;
		for (cdr = CdromList.first(); cdr; cdr = CdromList.next())
			if (device->currentText() == cdr->name())
				break;
		if (!cdr)
			for (cdr = RecorderList.first(); cdr; cdr = RecorderList.next())
				if (device->currentText() == cdr->name())
					break;
		int start = 0,next = 0;
		try {
			cdr->msinfo(start,next);
		} catch (Error e) {
			mainWin->postError(e);
			return;
		}
		params.sprintf("-C%i,%i",start,next);
		argv[a++] = params;
		prev.sprintf("-M%s",cdr->blockDev());
		argv[a++] = prev;
	}
	argv[a++] = dirselect->text();
	argv[a++] = 0;
	setuid(UserID);
	setgid(GroupID);
	MakeIsofsProgress *prog = new MakeIsofsProgress(new Process(argv),tr);
	setuid(0);
	setgid(0);
	prog->show();
}

MakeIsofsProgress::MakeIsofsProgress(Process *p, Track *t) :
	QProgressDialog(),
	pr(p), track(t), writeinfo(true) {
	
	setLabel(new QLabel("creating iso9660 data track...",this));
	setBar(new QProgressBar(this));
	setProgress(0);
	connect(pr,SIGNAL(err(const char *)),SLOT(update(const char *)));
	connect(pr,SIGNAL(quit(int)),SLOT(finish()));
	connect(this,SIGNAL(cancelled()),SLOT(cancel()));
}

void MakeIsofsProgress::update(const char *msg) {
	QString m = msg;
	QRegExp reg("[0-9]+.?[0-9]*% done, estimate");
	int at;
	if (-1 < (at = m.find(reg))) {
		float prog = 0;
		sscanf(m.mid(at),"%f%% done, ",&prog);
		setProgress((int) rint(prog));
	}
}

void MakeIsofsProgress::finish() {
	hide();
	delete pr;
	if (writeinfo) {
		QString tmp = track->filename();
		QFile cdr(tmp + ".cdr");
		QFile inf(tmp + ".inf");
		inf.open(IO_WriteOnly);
		QTextStream os(&inf);
		os << "Name: " << track->getName() << "\n";
		os << "Length: " << track->getLength() << "\n";
		os << "mode : 1\n";	// data1 track
		os << "length: " << cdr.size() << "\n";
	}
	delete track;
	mainWin->updateView();
	delete this;
}

void MakeIsofsProgress::cancel() {
	writeinfo = false;
	pr->terminate();
	QString tmp = track->filename();
	if (QFile::remove(tmp + ".cdr"))
		infomsg("successfully removed file \"%s.cdr\"",track->filename());
	else
		errormsg("error removing file \"%s.cdr\": %s",
			track->filename(),strerror(errno));
}

#include "master.moc"
