/*
 * 
 * 	fryit - file recorder.cc
 * 	Copyright Thomas Maier-Komor, 1999
 * 	
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "cdrom.h"
#include "recorder.h"
#include "track.h"
#include "utils.h"
#include "globals.h"

Recorder::Recorder(const char *v, const char *m, const char *a, const char *t,
	const char *r, int h, int b, int i, int l, int ar, const char *bd, 
	const char *gd)
	: CDROM(v,m,a,t,r,h,b,i,l,ar,bd,gd),
	drv(0),wSpeed(0),fifo(4096),multi(false),eject(true),simul(true) {
	
	type = recorder;
}

Process *Recorder::writeTracks(QList <Track> &tr) {
	const char *argv[32];
	QString sp, fs;
	int a = 4;
	bool data = true;	// previous track was a data track
	
	if (idle != getStatus()) {
		errormsg("%s: device is not idle",name());
		return 0;
	}
	setStatus(writing);
	argv[0] = Cdrecord;
	argv[1] = "dev=";
	argv[2] = devStr;
	argv[3] = "-v";
	if (wSpeed) {
		sp.sprintf("speed=%i",wSpeed);
		argv[a++] = sp;
	}
	if (fifo) {
		fs.sprintf("fs=%ik",fifo);
		argv[a++] = fs;
	}
	if (eject)
		argv[a++] = "-eject";
	if (simul)
		argv[a++] = "-dummy";
	if (multi)
		argv[a++] = "-multi";
	if (0 != drv)
		argv[a++] = Driver[drv];
	QList <QString> files;
	for (Track *t = tr.first(); t; t = tr.next()) {
		debugmsg("writeTracks(): adding track %s file %s.cdr",t->getName(),t->filename());
		if (audio == t->getTrackType()) {
			if (data) {
				data = false;
				argv[a++] = "-audio";
			}
		} else if (data1 == t->getTrackType()) {
			if (!data) {
				data =  true;
				argv[a++] = "-data";
			}
		} else {
			errormsg("LowLevel::write(): unsupported data mode %i",
				t->getTrackType());
			return 0;
		}
		QString *str = new QString(DataDir);
		*str += t->filename();
		*str += ".cdr";
		files.append(str);
		argv[a++] = *str;
	}
	argv[a] = 0;
	a = -1;
	pr = new Process(argv);
	connect(pr,SIGNAL(quit(int)),SLOT(processFinished()));
	return pr;
}

Process *Recorder::writeThroughStdin(int l) {
	if (idle != getStatus()) {
		errormsg("%s: device is not idle",name());
		return 0;
	}
	setStatus(writing);

	const char *argv[32];
	int a = 4;
	QString sp,f,len;
	argv[0] = Cdrecord;
	argv[1] = "-v";
	argv[2] = "dev=";
	argv[3] = devStr;
	if (wSpeed) {
		sp.sprintf("speed=%i",wSpeed);
		argv[a++] = sp;
	}
	if (0 != drv)
		argv[a++] = Driver[drv];
	if (eject)
		argv[a++] = "-eject";
	if (simul)
		argv[a++] = "-dummy";
	if (multi)
		argv[a++] = "-multi";
	if (fifo) {
		f.sprintf("fs=%ik",fifo);
		argv[a++] = f;
	}
	len.sprintf("tsize=%i",l);
	argv[a++] = len;
	argv[a++] = "-data";
	argv[a++] = "-";
	argv[a++] = 0;
	pr = new Process(argv);
	connect(pr,SIGNAL(quit(int)),SLOT(processFinished()));
	return pr;
}

Process *Recorder::fixCD() {		// prfen...
	if (idle != getStatus()) {
		errormsg("%s: device not idle",name());
		return 0;
	}
	setStatus(writing);
	const char *argv[] = {Cdrecord,"-fix","dev=",devStr,0};
	pr = new Process(argv);
	connect(pr,SIGNAL(quit(int)),SLOT(processFinished()));
	return pr;
}

void Recorder::setDriver(int d) {
	debugmsg("%s: driver set to %s",name(),DriverDescr[d].operator const char *());
	drv = d;
}

void Recorder::setWriteSpeed(int s) {
	debugmsg("%s: speed set to %i",name(),s);
	wSpeed = s;
}

void Recorder::setFifo(int f) {
	debugmsg("%s: fifo size set to %i",name(),f);
	fifo = f;
}

void Recorder::cancelWriting() {
	debugmsg("%s: cancelWriting()",name());
	if (writing == getStatus())
		pr->terminate();
}

void Recorder::writingFinished() {
	debugmsg("%s: writingFinished()",name());
	if (pr) {
		delete pr;
		pr = 0;
	}
	setStatus(idle);
}

void Recorder::setMultisession(bool m = true) {
	#ifdef DODEBUG
	if (m)
		debugmsg("\"%s\": multisession set to true",name());
	else 
		debugmsg("\"%s\": multisession set to false",name());
	#endif
	multi = m;
}

void Recorder::setEject(bool e = true) {
	#ifdef DODEBUG
	if (e)
		debugmsg("\"%s\": eject set to true",name());
	else 
		debugmsg("\"%s\": eject set to false",name());
	#endif
	eject = e;
}

void Recorder::setSimulate(bool s = true) {
	#ifdef DODEBUG
	if (s)
		debugmsg("\"%s\": simulate set to true",name());
	else 
		debugmsg("\"%s\": simulate set to false",name());
	#endif
	simul = s;
}

#include "recorder.moc"
