/*
 * 
 * 	fryit - file selections.h
 * 	Copyright Thomas Maier-Komor, 1999
 * 
 */

#ifndef SELECTIONS_H
#define SELECTIONS_H

#include <qhbox.h>
#include <qheader.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlist.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qscrollview.h>
#include <qstring.h>
#include <qtabdialog.h>
#include <qvbox.h>


class CDROM;
class TrackSelection;
class Process;
class ReadDataDialog;
class ReadDialog;
class Recorder;
class Track;

class ReadSelectionDialog : public QTabDialog {
	Q_OBJECT
	public:
	ReadSelectionDialog(CDROM *);
	~ReadSelectionDialog() {};
	
	public slots:
	void showDialog();
	
	private:
	CDROM *dev;
	QHBox *area;
	QHeader *header;
	QScrollView *trackBox;
	ReadDialog *rdlg;
	ReadDataDialog *ddlg;
	Process *pr;
	QList <TrackSelection> tracks;
	bool first;
	
	private slots:
	void rescanCD();
	void resizeEvent(QResizeEvent *);
	void readTrack();
	void checkTrackNames();
	void selectAll();
	void doCancel();
};

class WriteSelectionDialog : public QTabDialog {
	Q_OBJECT
	public:
	WriteSelectionDialog(Recorder *);
	~WriteSelectionDialog() {};
	
	public slots:
	void updateList();
	void show();
	
	private:
	Recorder *dev;
	QFrame *area;
	QVBoxLayout *diskLay,*writeLay;
	QHBoxLayout *mainLay,*diskBtn,*writeBtn;
	QLabel *diskLabel,*writeLabel;
	QListView *diskList,*writeList;
	QPushButton *addBtn,*remBtn,*delBtn,*upBtn,*dwnBtn;
	QList <Track> diskTracks,burnTracks;
	BurnDialog burnDlg;
	Process *pr;
	int min,sec,hun,total,written;
	
	private slots:
	void addTrack();
	void remTrack();
	void upTrack();
	void downTrack();
	void delTrack();
	void start();
	void selectAll();
	void calcLength();
	void outMsg(const char *);
	void errMsg(const char *);
	void finish();
	void doCancel();
};

class CopySelectionDialog : public QDialog {
	Q_OBJECT
	public:
	CopySelectionDialog();
	~CopySelectionDialog() {};
	
	public slots:
	virtual void show();

	private:
	QGridLayout *area;
	QHBoxLayout *buttonArea;
	QComboBox *reader,*writer;
	QLabel *readerLbl,*writerLbl;
	QPushButton *ok,*cancel;
	Process *pr;
	BurnDialog *burnDlg;
	CDROM *cdr;
	Recorder *rec;
	int size;
	
	private slots:
	void start();
	void outMsg(const char *);
	void errMsg(const char *);
	void finish();
	void doCancel();
};

#endif
