/*
 * 
 * 	fryit - file track.cc
 * 	Copyright Thomas Maier-Komor, 1999
 * 	
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "globals.h"
#include "track.h"

#include <sys/types.h>
#include <math.h>

#include <qmessagebox.h>


Track::Track(int n, int m, int s, int h, int a, int c,
	trackType mo, const char *Name = 0, int id = 0) :
	number(n),
	adr(a),
	control(c),
	name(Name),
	mode(mo),
	length((int) rint((float)(m * 6000 + s * 100 + h) / 100 * 75)),
	trackID(id),
	multisession(false) {

	debugmsg("new Track(n = %i, m = %i, s = %i, h = %i, a = %i, c = %i, mo = %i, Name = %s, id = %i",
		n,m,s,h,a,c,mo,Name,id);
	if (audio == mo)
		length *= 2352;
	else
		length *= 2048;
	init(Name);
}

Track::Track(int n, int l, int a, int c, trackType mo, const char *Name = 0,
	 int id = 0) :
	number(n),
	adr(a),
	control(c),
	name(Name),
	mode(mo),
	length(l),
	trackID(id),
	multisession(false) {
	
	debugmsg("new Track(n = %i, l = %i, a = %i, c = %i, mo = %i, Name = %s, id = %i",
		n,l,a,c,mo,Name,id);
	init(Name);
}

void Track::init(const char *Name) {	
	if (audio == mode) {
		file.sprintf("audio%i",trackID);
		if (!Name)
			name.sprintf("Audio-Track %i",number);
	} else if (data1 == mode) {
		file.sprintf("data%i",trackID);
		if (!Name)
			name.sprintf("Data-Track %i",number);
	} else {
		infomsg("unexpected track mode %i - handling like data1");
		file.sprintf("data%i",trackID);
		if (!Name)
			name.sprintf("Data-Track %i",number);
		debugmsg("new Track(n = %i, l = %i, a = %i, c = %i, mo = %i, Name = %s, id = %i",
			number,length,adr,control,mode,Name,trackID);
	}
}

Track::~Track() {
	debugmsg("Track \"%s\" deleted",getName());
}

void Track::setName(const char *Name) {
	name = Name;
}

const char *Track::getName() const {
	return name;
}

void Track::setTrackID(int id) {
	debugmsg("track id set to %i",id);
	trackID = id;
	if (audio == mode)
		file.sprintf("audio%i",trackID);
	else
		file.sprintf("data%i",trackID);
}

void Track::getTime(int &m,int &s,int &h) const {
	int tmp = length / ((audio == mode) ? 2352 : 2048);
	tmp = (int) rint((float)tmp / 75 * 100);
	m = tmp / 6000;
	tmp %= 6000;
	s = tmp / 100;
	tmp %= 100;
	h = tmp;
}

