/*
 * 
 * 	fryit - file track.h
 * 	Copyright Thomas Maier-Komor, 1999
 * 	
 */

#ifndef TRACK_H
#define TRACK_H

#include <qstring.h>

extern void debugmsg(const char *, ...);

class CDROM;
class TimeBox;

enum trackType {audio,data1,data2,xa1,xa2,cdi,unknown};

class Track {
	public:
	Track(int,int,int,int,int,int,trackType,const char * = 0,int = 0);
	Track(int,int,int,int,trackType,const char * = 0,int = 0);
	~Track();
	
	void setName(const char *);
	const char *getName() const;
	void setTrackID(int);
	inline void getOptions(int &,int &,trackType &) const;
	inline void setFilename(const char *);
	void getStart(int &,int &,int &) const;
	void getEnd(int &,int &,int &) const;
	
	inline void setTime(int,int,int);
	void getTime(int &,int &,int &) const;
	inline void setLength(int);
	inline int getLength() const;
	inline const char *filename() const;
	inline trackType getTrackType() const;
	inline int getNumber() const;
	inline int getTrackID() const;
	inline bool isMultiSession() const;
	inline void setMultiSession(bool = false);
	
	protected:
	int number,adr,control,smin,ssec,shun,emin,esec,ehun;
	QString name,file;
	trackType mode;
	int length,trackID;
	bool multisession;
	
	private:
	void init(const char *);
};

//
// inline functions
//

inline void Track::getOptions(int &a,int &c,trackType &m) const {
	a = adr;
	c = control;
	m = mode;
}

inline void Track::setLength(int l) {
	length = l;
}

inline int Track::getLength() const {
	return length;
}

inline int Track::getTrackID() const {
	return trackID;
}

inline const char *Track::filename() const {
	return file;
}

inline trackType Track::getTrackType() const {
	return mode;
}

inline int Track::getNumber() const {
	return number;
}

inline bool Track::isMultiSession() const {
	return multisession;
}

void Track::setMultiSession(bool m = false) {
	#ifdef DEBUG
	if (m)
		debugmsg("multisession set to true");
	else 
		debugmsg("multisession set to false");
	#endif
	multisession = m;
}

void Track::setFilename(const char *f) {
	file = f;
}

void Track::setTime(int m, int s, int h) {
	debugmsg("Track \"%s\": time set to %i:%02i.%02i",getName(),m,s,h);
	length = (m * 6000 + s * 100 + h) / 100 * 75 * 2352;
}

#endif
