/*
 * 
 * 	fryit - file utils.cc
 * 	Copyright Thomas Maier-Komor, 1999
 * 	
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <cassert>
#include <cerrno>
#include <csignal>
#include <cstdio>
#include <fcntl.h>
#include <malloc.h>
#include <sys/fsuid.h>
#include <sys/wait.h>
#include <unistd.h>

#include <qmessagebox.h>

#include "globals.h"
#include "utils.h"

int readProc(QString &out, QString &err, const char *argv[]) {
	int errp[2],outp[2],pid;
	
	err = "";
	out = "";
	if (pipe(errp)) {
		fatalmsg("error creating pipe: %s",strerror(errno));
		return -1;
	}
	if (pipe(outp)) {
		fatalmsg("error creating pipe: %s",strerror(errno));
		return -1;
	}

	if ((pid = fork()) == 0) {	// child
		char *a[32];
		int c;
		for (c = 0; argv[c]; c++) {
			a[c] = (char *) malloc(strlen(argv[c]) + 1);
			strcpy(a[c],argv[c]);
			if (c == 32) {
				fatalmsg("more than 32 parameter in call to readproc");
				return -1;
			}
		}
		a[c] = 0;

		#ifdef DODEBUG
		QString s0,s1;
		s0.sprintf("reading Process {\"%s\"",a[0]);
		for (int c = 1; a[c]; c++) {
			s1.sprintf(",\"%s\"",a[c]);
			s0 = s0 + s1;
		}
		s0.append("}");
		debugmsg("%s",s0.operator const char *());
		#endif
	
		if (outp[1] != STDOUT_FILENO)
			dup2(outp[1],STDOUT_FILENO);
		if (errp[1] != STDERR_FILENO)
			dup2(errp[1],STDERR_FILENO);
		close(outp[0]);
		close(outp[1]);
		close(errp[0]);
		close(errp[1]);
		execvp(a[0],a);
		cerr << "exec error: " << strerror(errno) << "\n";	// just in case
		exit(-1);
	}
	if (pid < 0) {
		fatalmsg("error forking: %s",strerror(errno));
		return -1;
	}

	// father
	close(outp[1]);		// close ends for writing
	close(errp[1]);
	char ostr[1024],estr[1024];
	int cnt = 0;
	int ret;
	{
		if ((cnt = (read(outp[0],ostr,1023 * sizeof(char)))) >= 0) {
			ostr[cnt] = '\0';
			out += ostr;
		} else
			errormsg("readProc(): error reading stdout: %s",strerror(errno));
		if ((cnt = read(errp[0],estr,1023 * sizeof(char))) >= 0) {
			estr[cnt] = '\0';
			err += estr;
		} else
			errormsg("readProc(): error reading stderr: %s",strerror(errno));
	} while (!waitpid(pid,&ret,WNOHANG));
	debugmsg("readProc(): child process exited - return value: %i",ret);

	int tot;
	do {
		tot = 0;
		if ((cnt = (read(outp[0],ostr,1023 * sizeof(char)))) >= 0) {
			ostr[cnt] = '\0';
			out += ostr;
			tot += cnt;
		} else
			errormsg("readProc(): error reading stdout: %s",strerror(errno));
		if ((cnt = read(errp[0],estr,1023 * sizeof(char))) >= 0) {
			estr[cnt] = '\0';
			err += estr;
			tot += cnt;
		} else
			errormsg("readProc(): error reading stderr: %s",strerror(errno));
	} while (tot);
	close(errp[0]);
	close(outp[0]);
	return ret;
}

Process::Process(const char *argv[]) : QObject() {
	name = argv[0];
	if (pipe(errp))
		return;
	if (pipe(outp))
		return;
	if (pipe(inp))
		return;
	if ((childpid = fork()) == 0) {	// child
		int c;
		#ifdef DODEBUG
		QString s0,s1;
		s0.sprintf("starting Process {\"%s\"",argv[0]);
		for (c = 1; argv[c]; c++) {
			s1.sprintf(",\"%s\"",argv[c]);
			s0 = s0 + s1;
		}
		s0.append("}");
		debugmsg("%s",s0.operator const char *());
		#endif
		if (inp[0] != STDIN_FILENO)
			dup2(inp[0],STDIN_FILENO);
		if (outp[1] != STDOUT_FILENO)
			dup2(outp[1],STDOUT_FILENO);
		if (errp[1] != STDERR_FILENO)
			dup2(errp[1],STDERR_FILENO);
		close(outp[0]);
		close(outp[1]);
		close(errp[0]);
		close(errp[1]);
		close(inp[0]);
		close(inp[1]);
		char *a[32];
		for (c = 0; argv[c]; c++) {
			a[c] = (char *) malloc(strlen(argv[c]) + 1);
			strcpy(a[c],argv[c]);
			if (c == 32) {
				fprintf(stderr,"more than 32 parameter in call to readproc");
				exit(-1);
			}
		}
		a[c] = 0;
		execvp(a[0],a);
		exit(-1);	// just in case
	}

	// father
	close(outp[1]);		// close ends for writing
	close(errp[1]);
	close(inp[0]);
	int fl = fcntl(outp[0],F_GETFL,0);
	fl |= O_NONBLOCK;
	fcntl(outp[0],F_SETFL,fl);
	fl = fcntl(errp[0],F_GETFL,0);
	fl |= O_NONBLOCK;
	fcntl(errp[0],F_SETFL,fl);
	connect(&scanner,SIGNAL(timeout()),this,SLOT(manageChild()));
	scanner.start(50);
	running = true;
}

Process::~Process() {
	close(inp[1]);
	close(outp[0]);
	close(errp[0]);
}

void Process::manageChild() {
	char ostr[1024], estr[1024];
	int cnt;
		
	if ((cnt = (read(outp[0],ostr,1023 * sizeof(char)))) > 0) {
		ostr[cnt] = '\0';
		emit out(ostr);
	}
	if ((cnt = read(errp[0],estr,1023 * sizeof(char))) > 0) {
		estr[cnt] = '\0';
		emit err(estr);
	}
	if (waitpid(childpid,&retValue,WNOHANG) > 0) {
		debugmsg("process exited with return value %i",retValue);
		emit quit(retValue);
		running = false;
		if (retValue < 0)
			warningmsg("%s exited with code %i",
				name.operator const char *(),
				retValue);
		scanner.stop();
	}
}

int Process::getExitCode(int &r, bool wait = true) {
	if (!running) {
		r = retValue;
		return 0;
	}
	switch(waitpid(childpid,&r,wait ? 0 : WNOHANG)) {
		case 0:
			return 0;
		case -1:
			errormsg("error in wait(): %s",strerror(errno));
			return -1;
		default:
			return childpid;
	}
	return 0;
}

void Process::terminate() {
	debugmsg("sending term signal to process %i",childpid);
	::kill(childpid,SIGTERM);
}

void Process::kill() {
	debugmsg("sending kill signal to process %i",childpid);
	::kill(childpid,SIGKILL);
}

void Process::stop() {
	::kill(childpid,SIGSTOP);
	scanner.stop();
}

void Process::cont() {
	::kill(childpid,SIGCONT);
	scanner.start(100);
}

void Process::interrupt() {
	debugmsg("sending interrupt signal to process %i",childpid);
	::kill(childpid,SIGINT);
}

void Process::cleanup() {
	if (running)
		terminate();
	delete this;
}

void Process::input(const void *in,int length) throw(Error) {
	if (write(inp[1],in,length) < 0) {
		QString err;
		err = "error writing data to process: \n";
		err += strerror(errno);
		throw Error(err);
	}
}

void Process::endinput() {
	close(inp[1]);
}

File::File(const char *name, int mode, bool s = false) throw(Error) :
	closed(false),
	super(s) {

	if (super)
		setfsuid(0);
	fd = open(name,mode);
	if (super)
		setfsuid(UserID);
	if (fd == -1) {
		QString err;
		err.sprintf("could not open file %s: %s",name,strerror(errno));
		throw Error(err);
	}
	closed = false;
}

File::File(const char *name, int mode, int acc) throw(Error) {
	fd = open(name,mode,acc);
	if (fd == -1) {
		QString err;
		err.sprintf("could not open file %s: %s",name,strerror(errno));
		throw Error(err);
	}
	closed = false;
}

File::~File() {
	close();
}

void File::close() {
	if (!closed)
		::close(fd);
}

#include "utils.moc"
