/*
 * 
 * 	fryit - file widgets.h
 * 	Copyright Thomas Maier-Komor, 1999
 * 	
 */

#ifndef WIDGETS_H
#define WIDGETS_H

#include <cassert>

#include <qcheckbox.h>
#include <qhbox.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qspinbox.h>

#include "track.h"

class CDROM;
class TimeBox;

class TrackSelection : public QHBox, public Track {
	Q_OBJECT
	public:
	TrackSelection(CDROM *,QWidget *,const Track *);
	~TrackSelection() {};
	
	inline bool isSelected() const;
	inline void setSelected(bool = true);
	inline bool complete() const;
	inline void getStartTime(int &,int &,int &) const;
	inline void getEndTime(int &,int &,int &) const;
	void getSelectedTime(int &,int &,int &) const;
	inline int getStartOffset() const;
	inline int getEndOffset() const;
	QSize sizeHint() const;
	int getColumnSize(int) const;
	
	public slots:
	void setName(const QString &);
	
	private:
	CDROM *cdrom;
	QLineEdit *nameEdit;
	QCheckBox *selectBox,*completeBox;
	TimeBox *sT,*eT;
	QSpinBox *sO,*eO;
	QPushButton *playBtn;
	QLabel *space;
	
	private slots:
	void setComplete(bool);
	void play();
	void checkTimes();
	void checkOffsets();
};

class TrackItem : public QListViewItem {
	public:
	TrackItem(const Track *,QListView * = 0,QListViewItem * = 0);
	~TrackItem() {};
	
	QString text(int) const;
	
	const Track *tr;	
	
	private:
	mutable QString col1;
};

class TimeBox : public QHBox {
	Q_OBJECT
	public:
	TimeBox(int,int,int,int,int,int,QWidget * = 0);
	~TimeBox() {};
	
	void setMinimum(int,int,int);
	void setMaximum(int,int,int);
	void getCurrent(int &,int &,int &) const;
	void setCurrent(int,int,int);
	void setPos(int);
	void lock(bool = true);
	QSize sizeHint() const;
	inline int pos() const;
	
	private:
	int minMin,minSec,minHun,maxMin,maxSec,maxHun,lm,ls,lh;
	QSpinBox *min,*sec,*hun;
	QLabel *l0,*l1;
	bool locked;
	
	private slots:
	void checkMin(int);
	void checkSec(int);
	void checkHun(int);
	
	signals:
	void timeChanged(int,int,int);
	void posChanged();
};

//
// inline functions
//

inline bool TrackSelection::isSelected() const {
	return selectBox->isChecked();
}

inline void TrackSelection::setSelected(bool s = true) {
	selectBox->setChecked(s);
}

inline bool TrackSelection::complete() const {
	return completeBox->isChecked();
}

inline int TrackSelection::getStartOffset() const {
	assert(sO);
	return sO->value();
}

inline int TrackSelection::getEndOffset() const {
	assert(eO);
	return eO->value();
}

inline void TrackSelection::getStartTime(int &m, int &s, int &h) const {
	assert(sT);
	sT->getCurrent(m,s,h);
}

inline void TrackSelection::getEndTime(int &m, int &s, int &h) const {
	assert(sT);
	eT->getCurrent(m,s,h);
}

inline int TimeBox::pos() const {
	return min->value() * 6000 + sec->value() * 100 + hun->value();
}

#endif
