#include "cdtree.h"

/*** KDE ***/
#ifdef KDE2
#include <klocale.h>
#include <qapplication.h>
#else
#include <kapp.h>
#endif

/*** C++ ***/
#include <dirent.h>     // fr die Verzeichnisoperationen
#include <fstream.h>

/*** KisoCD ***/
#include "tools.h"
#include "version.h"

#ifdef USEWAITCURS
extern unsigned char cursArt;
#endif

int CDTree::getVerz(unsigned int index, string &SrcPath, string &DstPath)
{
  if (index >= VerzListe.size()) return(-1);
  SrcPath = VerzListe[index].SrcPath;
  DstPath = VerzListe[index].DstPath;
  return(0);
}

int CDTree::getVerzNum() 
{
  return(VerzListe.size());
}

int CDTree::Append(string SrcPath, string DstPath, bool FollowSymLink)
{
  // true, wenn es eine Datei ist:
  bool aFile = (SrcPath[SrcPath.size()-1] != '/') && (SrcPath != "dummydir");

  if (!aFile && SrcPath != "dummydir") 
    {
      // erster Teil: schauen, ob es das src-Dir im System gibt
      DIR* Verzeichnis;
      
      Verzeichnis = opendir(SrcPath.c_str());
      if (Verzeichnis == NULL) 
	{
	  QString tmp = i18n("\nThis directory does not seem to exist on your system!");
	  tmp = SrcPath.c_str() + tmp;
	  errorMessage(tmp);
	  return(-2);
	}
      closedir(Verzeichnis);
    }

  // zweiter Teil: wenn es in der Liste noch nicht vorhanden ist, anhngen!
  CDVerz tmp;

  tmp.SrcPath = SrcPath;
  tmp.DstPath = DstPath;
  for (unsigned int Cnt = 0; Cnt < VerzListe.size(); Cnt++) 
    {
      if (SrcPath == VerzListe[Cnt].SrcPath && SrcPath != "dummydir")
	{
	  QString schonGeta = i18n("You have already included this file/directory\n"
				   "but I'll add it anyway...\n"
				   "The CDs size given in the status bar may now be quite wrong,\n"
				   "so dont forget to check the real one using \n"
				   "\"Single Step|Simulate image creation\"!\n\n"
				   "Where you included it before:\n");
	  schonGeta += VerzListe[Cnt].DstPath.c_str();
	  errorMessage(schonGeta);
	}
    }
  
  // Jetzt werden gleich Daten ber dieses Verzeichnis gesammelt:
  tmp.Inhalt = new Verzeichnis;

  // Wartecursor einschalten:
#ifdef USEWAITCURS
#ifdef KDE2
  QApplication::setOverrideCursor(QApplication::waitCursor); 
#else
  QApplication::setOverrideCursor(waitCursor); 
#endif
#endif
  cursArt=1;

  if (!aFile) 
    {
#ifdef VERBOSEMESSAGES
      cerr << "Verz angehngt\n";
#endif
      if (tmp.Inhalt->Setzen(SrcPath, FollowSymLink) != 0) 
	{
	  delete tmp.Inhalt;
	  QApplication::restoreOverrideCursor(); 
	  // Fehlermeldung wird von ~->Setzen bernommen!
	  return(-4);
	}
    }
  else 
    {
#ifdef VERBOSEMESSAGES
      cerr << "Datei angehngt\n";
#endif
      if (tmp.Inhalt->SetzenD(SrcPath, DstPath, FollowSymLink) != 0) 
	{
	  delete tmp.Inhalt;
	  QApplication::restoreOverrideCursor(); 
	  QString meld = i18n("\nThere is no such file on your system or it is\na symbolic link and you told me not to follow them!");
	  meld = SrcPath.c_str() + meld;
	  errorMessage(meld);
	  return(-3);
	}
    }
  // und merken!
  VerzListe.push_back(tmp);  
  QApplication::restoreOverrideCursor(); 
#ifdef VERBOSEMESSAGES  
  cerr << "angehngt: " << SrcPath << " -> " << DstPath << "\n";
#endif
  return(0);
}

int CDTree::Remove(unsigned int index)
{
  if (index >= VerzListe.size()) return(-1);
#ifdef VERBOSEMESSAGES
  cerr << "gelscht: " << VerzListe[index].SrcPath << " -> " << VerzListe[index].DstPath << "\n";
#endif
  delete VerzListe[index].Inhalt;
  VerzListe.erase(VerzListe.begin()+index);
  return(0);  
}

int CDTree::getCDVerzTree(QListView* ktl, bool Append)
{
  ofstream  outDat;
  string    outDatPath;

  // macht es berhaupt Sinn?
  if (VerzListe.size() == 0) 
    {
      ktl->clear();
      return(-1);
    }

  outDatPath = getPersConfigDir();
  outDatPath += "/dirs.lst";

  outDat.open(outDatPath.c_str());
  if (!outDat) return(-1);

  // Wartecursor einschalten:
#ifdef USEWAITCURS
#ifdef KDE2
  QApplication::setOverrideCursor(QApplication::waitCursor); 
#else
  QApplication::setOverrideCursor(waitCursor);   
#endif
#endif
  // Gesamtgre ausrechnen:
  Gesamtgroesse = 0;
  for (unsigned int Cnt=0; Cnt < VerzListe.size(); Cnt++) 
    Gesamtgroesse += VerzListe[Cnt].Inhalt->GetSizeR();

#ifdef VERBOSEMESSAGES
  cout << "Gesamtgre gerechnet\n";
#endif
  if (!Append) 
    {
      // TreeList lschen und dann fllen!
      ktl->clear();
#ifdef VERBOSEMESSAGES
      cout << "TreeList gelscht\n";
#endif      
    }
  

  for (unsigned int Cnt=0; Cnt < VerzListe.size(); Cnt++) 
    {
#ifdef VERBOSEMESSAGES
      cout << "Ausfhrung: " << VerzListe[Cnt].SrcPath << "   " 
	   << VerzListe[Cnt].DstPath;
#endif   
      VerzListe[Cnt].Inhalt->fillTreeList(ktl,
					  VerzListe[Cnt].SrcPath, 
					  VerzListe[Cnt].DstPath,
					  Ausschluss,
					  outDat);
#ifdef VERBOSEMESSAGES
      cout << "..fertig\n";
#endif           
    }      
#ifdef VERBOSEMESSAGES
  cout << "TreeList gefllt\n";
#endif

  ktl->triggerUpdate();

  ktl->repaint();
#ifdef USEWAITCURS
  QApplication::restoreOverrideCursor(); 
#endif 
  close_ofs(outDat);

  return(0);
}

unsigned long CDTree::getCDSize()
{
  return(Gesamtgroesse);
}

void CDTree::calcSize()
{
  for (unsigned int Cnt = 0; Cnt < VerzListe.size(); Cnt++) {
    VerzListe[Cnt].Inhalt->calcSize(Ausschluss);
  }
}

void CDTree::Zeig(KStatusBar* statBar,
		    QListView* DatList, 
		    QListView* ktl, 
		    string pfad)
{
  unsigned long   size  = 0;
  unsigned long   sizeR = 0;
  string          tmp;
  
  DatList->clear();

  for (unsigned int Cnt = 0; Cnt < VerzListe.size(); Cnt++) {
    VerzListe[Cnt].Inhalt->zeigInhalt(&size, &sizeR, pfad, DatList, 
				      Ausschluss);
  }
  DatList->repaint();
  // die Gren anzeigen
  tmp = i18n("files");
  tmp += ":";		    
#ifdef KDE2
  DatList->setColumnText(0, getSizeText(i18n("files") + ":", size));  
#else
  DatList->setColumnText(0, getSizeText(tmp.c_str(), size));
#endif
  statBar->changeItem(getSizeText("", sizeR), 0);
}

int CDTree::erstellListe(string dateiPfad)
{
  ofstream           outDat;
  unsigned long int  linkCnt = 0;

  outDat.open(dateiPfad.c_str());

  outDat << "l()\n";
  outDat << "{\n";
  outDat << "  if ln -s \"$S\" \"$D\"\n";
  outDat << "    then\n";
  outDat << "      true\n";
  outDat << "    else\n";
  outDat << "     echo KisoCD: aborted, because there was an error when linking $S to $D\n";
  outDat << "     exit\n";
  outDat << "  fi\n";
  outDat << "}\n";
  outDat << "l_p()\n";
  outDat << "{\n";
  outDat << "  if ln -s \"$S\" \"$D\"\n";
  outDat << "    then\n";
  outDat << "      echo -n .\n";
  outDat << "    else\n";
  outDat << "     echo KisoCD: aborted, because there was an error when linking $S to $D\n";
  outDat << "     exit\n";
  outDat << "  fi\n";
  outDat << "}\n";
  outDat << "l_pn()\n";
  outDat << "{\n";
  outDat << "  if ln -s \"$S\" \"$D\"\n";
  outDat << "    then\n";
  outDat << "      echo .\n";
  outDat << "    else\n";
  outDat << "     echo KisoCD: aborted, because there was an error when linking $S to $D\n";
  outDat << "     exit\n";
  outDat << "  fi\n";
  outDat << "}\n\n";

  if (outDat) 
    for (unsigned int Cnt = 0; Cnt < VerzListe.size(); Cnt++) 
      VerzListe[Cnt].Inhalt->erstellListe(outDat, Ausschluss, linkCnt);    
  else return(-1);
  outDat << "echo \n";
  
  close_ofs(outDat);
  return(0);
}

int CDTree::getEx(unsigned int index, string &Muster)
{
  if (index >= Ausschluss.size()) return(-1);
  Muster = Ausschluss[index];
  return(0);
}

int CDTree::getExNum() 
{
  return(Ausschluss.size());
}

int CDTree::Exclude(string Muster)
{
  if (Muster[Muster.size()-1] == '/') {
    // erster Teil: schauen, ob es das Dir im System gibt
    DIR* Verzeichnis;
    
    Verzeichnis = opendir(Muster.c_str());
    if (Verzeichnis == NULL) {
      QString meld = i18n("\nThis directory does not seem to exist on your system!");
      meld = Muster.c_str() + meld;
      errorMessage(meld);
      return(-2);
    }
    closedir(Verzeichnis);
  }
  Ausschluss.push_back(Muster);
#ifdef VERBOSEMESSAGES
  cerr << "angehngt: " << Muster << "\n";
#endif
  return(0);
}

int CDTree::removeEx(unsigned int index)
{
  if (index >= Ausschluss.size()) return(-1);
#ifdef VERBOSEMESSAGES
  cerr << "gelscht: " << Ausschluss[index] << "\n";
#endif
  Ausschluss.erase(Ausschluss.begin()+index);
  return(0);  
}

bool CDTree::existDestDir(string testName)
{
#ifdef VERBOSEMESSAGES
  cout << "Teste auf " << testName << "\n";
#endif
  for (unsigned int Cnt=0; Cnt < VerzListe.size(); Cnt++)     
    {
      if (VerzListe[Cnt].DstPath.find(testName) == 0) return(true);
    }
    
  return(false);
}

CDTree::~CDTree()
{
  for (unsigned int Cnt = 0; Cnt < VerzListe.size(); Cnt++) {
    delete VerzListe[Cnt].Inhalt;
  }
}













