#include "copy.h"

/*** KDE ***/
#include <ksimpleconfig.h>
#ifdef KDE2
#include <klocale.h>
#else
#include <kapp.h>
#endif

/*** Qt ***/
#include <qmessagebox.h>

/*** C/C++ ***/
#include <string>

/*** KisoCD ***/
#include "subprocess.h"
#include "opt_sys.h"
#include "opt_cd.h"
#include "opt_cddoc.h"
#include "opt_ui.h"
#include "version.h"

#include "menuid.copydata"
#include "module.id"

extern int                moduleExitCode;
extern int                debuglevel;
extern DebugFile          debugFile; 
extern KSimpleConfig*     globalConf;
extern string             HeimatPfad;
extern T_Options          aktOpts;
extern T_Options          defOpts;
extern T_SysOptions       sysOpts;


Copy::Copy(QWidget *parent, const char *name, int aModuleID)
	: KCDModule(parent, name, aModuleID)
{
#ifdef VERBOSEMESSAGES
  cout << "Copy::Copy() called\n";
#endif
  /* ***************************************** */  
  menuFile = new QPopupMenu;

  menuFile->insertSeparator();
  menuFile->insertItem(i18n("E&xit"), ID_FILE_EXIT);

  /* ***************************************** */
  menuSingleStep = new QPopupMenu;

  menuSingleStep->insertItem(i18n("open/close tray"),        ID_SSTEP_OPENCLOSETRAY);
  if (globalConf->readEntry("mSStep_createImage", "enabled") == "enabled") 
    menuSingleStep->insertItem(i18n("Create Image"),         ID_SSTEP_CREATEIMAGE);
  if (globalConf->readEntry("mSStep_mountViewImage", "enabled") == "enabled") 
    menuSingleStep->insertItem(i18n("Mount and View Image"), ID_SSTEP_MOUNTVIEWIMAGE);
  if (globalConf->readEntry("mSStep_createScript", "enabled") == "enabled") 
    menuSingleStep->insertItem(i18n("Create ShellScript"),   ID_SSTEP_CREATESCRIPT);
  if (globalConf->readEntry("mSStep_burnIt", "enabled") == "enabled") 
    menuSingleStep->insertItem(i18n("Burn CD now!"),         ID_SSTEP_BURNIT);
  if (globalConf->readEntry("mSStep_delImage", "enabled") == "enabled") 
    menuSingleStep->insertItem(i18n("Delete Image"),         ID_SSTEP_DELIMAGE);
  
  /* ***************************************** */  
  getStdCDDocMenu(menuCdDoc);

  /* ***************************************** */
  getStdCDRWMenu(menuCDRW);

  /* ***************************************** */
  getStdOptionsMenu(menuOptions);

  /* ***************************************** */
  menuView = new QPopupMenu;

  if (globalConf->readEntry("mView_sub1", "enabled") == "enabled") 
    menuView->insertItem(i18n("output of subprocess:cdwriter"), ID_VIEW_SUBCD);
  if (globalConf->readEntry("mView_sub2", "enabled") == "enabled") 
    menuView->insertItem(i18n("output of subprocess:image"),    ID_VIEW_SUBIMAGE);
  menuView->setItemEnabled(ID_VIEW_SUBIMAGE, FALSE);
  menuView->setItemEnabled(ID_VIEW_SUBCD,    FALSE);

  /* ***************************************** */  
  getStdHelpMenu(menuHelp);

  /* ***************************************** */  
  menuBar = new KMenuBar(this, "menubar");
  CHECK_PTR( menuBar );
  menuBar->insertItem(i18n("&File"),         menuFile);
  menuBar->insertItem( i18n("&Single Step"), menuSingleStep);
  if (globalConf->readEntry("mCDDoc", "enabled") == "enabled")   
    menuBar->insertItem( i18n("cd&Doc"),     menuCdDoc );
  if (globalConf->readEntry("mOptions", "enabled") == "enabled") 
    menuBar->insertItem( i18n("&Options"),   menuOptions );
  if (globalConf->readEntry("mCDRW", "enabled") == "enabled") 
    menuBar->insertItem( i18n("CDR&W"),      menuCDRW );
  menuBar->insertItem(i18n("&View"),         menuView);
  menuBar->insertItem(i18n("&Help"),         menuHelp);
  
  menuBar->show();
  setMenu(menuBar);
  /* ***************************************** */  
  mainView = new CopyWidget(this, "copywidget");  
  setView(mainView);
  /* ***************************************** */  
  connect (menuBar, 
	   SIGNAL (activated (int)), 
	   SLOT(commandCallback(int)));
  connect (menuSingleStep,    
	   SIGNAL (aboutToShow()), 
	   SLOT (setupMenu()));
  /* ***************************************** */  
  connect(mainView->pb_openClose, 
	  SIGNAL(clicked()), 
	  SLOT(clickOn_openClose()));
  connect(mainView->pb_readImage, 
	  SIGNAL(clicked()), 
	  SLOT(clickOn_readImage()));
  connect(mainView->pb_burnCD, 
	  SIGNAL(clicked()), 
	  SLOT(clickOn_burnCD()));  
  /* ***************************************** */  
  mainView->update(aktOpts);
  
  /* ***************************************** */  
  myCDTools.setDevice(sysOpts.readDevice);
  myCDTools.setMountpoint(sysOpts.mountPoint);

  /* ***************************************** */  
  imageProc = new Subprocess("-");

  /* ***************************************** */  
  fMounted = false;
  /* ***************************************** */  
  setupMenu();
  /* ***************************************** */  
  myTimer = new QTimer(this);
}

Copy::~Copy()
{
#ifdef VERBOSEMESSAGES
  cout << "Copy::~Copy() called\n";
#endif
  if (fMounted) unmountImage(fMounted, sysOpts.mountPoint);
  delete imageProc;
}

void Copy::setupMenu()
{
  bool fOpenCloseEnabled;
  bool fCreateEnabled;
  bool fBurnEnabled;
  bool fMountDelImage;

  if (burnProc->isRunning() || imageProc->isRunning())
    {
      fOpenCloseEnabled = false;
      fCreateEnabled    = false;
      fBurnEnabled      = false;
      fMountDelImage    = false;
    }
  else 
    {
      bool fCreateImageAndImageExists;

      fCreateImageAndImageExists = (aktOpts.fCreateImage && existDat(sysOpts.imageFilePath));

      fOpenCloseEnabled = true;
      fCreateEnabled    = aktOpts.fCreateImage;
      fBurnEnabled      = !aktOpts.fCreateImage || fCreateImageAndImageExists;
      fMountDelImage    = fCreateImageAndImageExists;
    }

  mainView->setButtonsEnabled(fOpenCloseEnabled, fCreateEnabled, fBurnEnabled);
  menuSingleStep->setItemEnabled(ID_SSTEP_OPENCLOSETRAY,  fOpenCloseEnabled);
  menuSingleStep->setItemEnabled(ID_SSTEP_CREATEIMAGE,    fCreateEnabled);
  menuSingleStep->setItemEnabled(ID_SSTEP_MOUNTVIEWIMAGE, fMountDelImage);
  menuSingleStep->setItemEnabled(ID_SSTEP_CREATESCRIPT,   fBurnEnabled);
  menuSingleStep->setItemEnabled(ID_SSTEP_BURNIT,         fBurnEnabled);      
  menuSingleStep->setItemEnabled(ID_SSTEP_DELIMAGE,       fMountDelImage);      
}

void Copy::clickOn_readImage()
{
#ifdef VERBOSEMESSAGES
  cout << "void Copy::clickOn_readImage()\n";
#endif
  commandCallback(ID_SSTEP_CREATEIMAGE);
}

void Copy::clickOn_openClose()
{
#ifdef VERBOSEMESSAGES
  cout << "void Copy::clickOn_openClose()\n";
#endif
  commandCallback(ID_SSTEP_OPENCLOSETRAY);
}

void Copy::clickOn_burnCD()
{
#ifdef VERBOSEMESSAGES
  cout << "void Copy::clickOn_burnCD()\n";
#endif
  commandCallback(ID_SSTEP_BURNIT);
}

void Copy::updateCreateImageStatus()
{
  struct stat attribs;

  stat(sysOpts.imageFilePath.c_str(), &attribs);

  imageProc->setStatusLine(getSizeText("", attribs.st_size));
}

void Copy::updateBurnStatus(QString line)
{
  int      timeleft;
  QString  tmp;
  
  timeleft = progress.tellRemaining(line);
  tmp.setNum(timeleft);
  tmp = line + i18n("\nFinished in ") + tmp + i18n(" minutes.");
  burnProc->setStatusLine(tmp);
}

int Copy::createImageScript()
{
  string scriptPfad = HeimatPfad;

  if (debuglevel & 1 != 0)
    {
      debugFile.newSection("function entered:");
      debugFile.output("int Edit::createImageScript(...)"); 
    }

  scriptPfad += "/cdScript.sh";  
  ofstream outDat;
  
  outDat.open(scriptPfad.c_str());
  if (!outDat) 
    { 
      errorMessage(i18n("Could not create file!\n")); 
      return(-1);
    }
  else 
    {
      outDat << "#! /bin/sh\n\n";
      outDat << "# " << ProggieName << " " << ProggieVersion << "\n";

      // Mglichkeiten:
      // cp /dev/cdrom /tmp/imagefile
      // cat /dev/cdrom > /tmp/imagefile
      // dd if=/dev/cdrom of=/tmp/imagefile      
      outDat << "dd "
	     << "if=" << sysOpts.readDevice << " "
	     << "of=" << sysOpts.imageFilePath << "\n";
      
      close_ofs(outDat);
      chmod(scriptPfad.c_str(), (S_IXGRP | S_IXOTH | S_IRWXU | S_IRGRP | S_IROTH));
    }  
  
  return(0);  
}

int Copy::createFinalScript(bool allowShutdown)
{
  string scriptPfad = HeimatPfad;

  if (debuglevel & 1 != 0)
    {
      debugFile.newSection("function entered:");
      debugFile.output("int Edit::createFinalScript(...)"); 
    }

  scriptPfad += "/cdScript.sh";  
  ofstream outDat;
  
  outDat.open(scriptPfad.c_str());
  if (!outDat) 
    { 
      errorMessage(i18n("Could not create file!\n")); 
      return(-1);
    }
  else 
    {
      outDat << "#! /bin/sh\n\n";
      outDat << "# " << ProggieName << " " << ProggieVersion << "\n";

      outDat << BURNCMD;
      if (aktOpts.fDummyWrite) outDat << BC_DUMMY;
      if (aktOpts.fPadTrack)   outDat << BC_PAD;
      if (sysOpts.fEject)      outDat << BC_EJECT;	  
      outDat << BC_DEV << getDevID(sysOpts.writeDevice) << " ";
      
      if (aktOpts.fCreateImage) 
	outDat << BC_SPEED << sysOpts.nWritingSpeedImg
	       << " '" << sysOpts.imageFilePath << "'"
	       << "\n";
      else 
	outDat << BC_SPEED << sysOpts.nWritingSpeedOTF << " "
	       << BC_ISOSIZE
	       << sysOpts.readDevice
	       << "\n";
      
      if (allowShutdown && sysOpts.fShutdown) outDat << "shutdown -h now\n";
      close_ofs(outDat);
      chmod(scriptPfad.c_str(), (S_IXGRP | S_IXOTH | S_IRWXU | S_IRGRP | S_IROTH));
    }  
  
  return(0);  
}

void Copy::imageFinished(KProcess* proc)
{
  if (debuglevel & 1 != 0) 
    {
      debugFile.newSection("function entered:");
      debugFile.output("void Copy::imageFinished(KProcess *proc)");
    }
  
  if (proc == imageProc->getProc())
    {      
      disconnect(imageProc, SIGNAL (finished(KProcess *)), 
		 this,      SLOT(imageFinished(KProcess *)));
            
      if (debuglevel & 1 != 0) 
	{
	  debugFile.newSection("imagecreation");
	  for (unsigned int cnt = 0; cnt < imageProc->getLineCnt(); cnt++)
	    debugFile.output(imageProc->getLine(cnt));	  
	}
      
      imageProc->show();
      imageProc->setLabel(i18n("Subprocess has terminated."));
      if (sysOpts.fProcFinMsg)
	infoMessage(i18n("Finished creation of the imagefile.\n"
			 "Please take a look at the output."));      
      imageProc->setProcFinished();

      myTimer->stop();
      disconnect(myTimer, SIGNAL(timeout()),
		 this,    SLOT(updateCreateImageStatus()));

      if (fMounted)
	{
	  // das Image remounten:
	  infoMessage(i18n("As your image has now been changed, it makes sense to remount it.\n"
			   "Please make sure no process uses the currently mounted image and\n"
			   "press OK."));
	  
	  unmountImage(fMounted, sysOpts.mountPoint);
	  mountViewImage(fMounted, sysOpts);
	}      
    }
#ifdef VERBOSEMESSAGES
  else 
    cout << "void Copy::imageFinished(KProcess *proc) ohne gltigen Parameter\n";
#endif
  setupMenu();
}

void Copy::burnFinished(KProcess* proc)
{
  if (debuglevel & 1 != 0) 
    {
      debugFile.newSection("function entered:");
      debugFile.output("void Copy::burnFinished(KProcess *proc)");
    }
  
  if (proc == burnProc->getProc())
    {      
      disconnect(burnProc, SIGNAL (finished(KProcess *)), 
		 this,     SLOT(burnFinished(KProcess *)));
      disconnect(burnProc, SIGNAL(REMatched(QString)),
		 this,     SLOT(updateBurnStatus(QString)));
            
      if (debuglevel & 1 != 0) 
	{
	  debugFile.newSection("process which used the burner");
	  for (unsigned int cnt = 0; cnt < burnProc->getLineCnt(); cnt++)
	    debugFile.output(burnProc->getLine(cnt));	  
	}
      
      burnProc->show();
      burnProc->setLabel(i18n("Subprocess has terminated."));
      if (burnProc->containsString("cdrecord: No disk / Wrong disk!")
	  || burnProc->containsString("cdrecord: No disc / Wrong disc!"))
	errorMessage(i18n("There is no disc/a wrong disc!\n"));
      else 	
	{
	  if (sysOpts.fProcFinMsg)	    
	    infoMessage(i18n("Burning of the CD is finished.\n"
			     "Please take a look at the output."));
	}
      
      burnProc->setProcFinished();
    }
#ifdef VERBOSEMESSAGES
  else 
    cout << "void Copy::burnFinished(KProcess *proc) ohne gltigen Parameter\n";
#endif
  setupMenu();
}

bool Copy::preExit()
{
  bool tmp = true;
  
  if (burnProc->isRunning() || imageProc->isRunning())
    if (QMessageBox::warning(this, i18n("Running subprocesses"), 
			     i18n("There is at least one subprocess still running." 
				  "You will kill it if you exit now."),
			     i18n("Exit"), i18n("Cancel"), 
			     0, 1, 1)) tmp = false;
  return(tmp);
}


void Copy::commandCallback(int id_)
{
  bool cdOptsChanged = false;
  
  switch(id_) 
    {
    case ID_FILE_EXIT: 
      if (preExit())
	{
	  moduleExitCode = ID_MODULE_EXIT;
	  delete(this);
	}
      break;      
      /* ***************************************** */  
    case ID_SSTEP_OPENCLOSETRAY:
      myCDTools.toggleOpen();   
      break;
    case ID_SSTEP_CREATEIMAGE:
      if (burnProc->isRunning() || imageProc->isRunning())
	errorMessage(i18n("process already running!"));
      else
	{
	  if (!createImageScript())
	    {
	      string  pfad = HeimatPfad;
	      QString infT;
	  
	      pfad += "/cdScript.sh"; 

	      infT = i18n("This window just shows you the output of a the subprocess\n"
			  "creating the image. You can safely close it.");
	      
	      imageProc->newProc(infT, pfad);
	      connect(imageProc, SIGNAL(finished(KProcess*)),
		      this,      SLOT(imageFinished(KProcess*)));
	      
	      if (!imageProc->startProc(1, pfad, sysOpts.fProcWinOnStart))
		{
		  menuView->setItemEnabled(ID_VIEW_SUBIMAGE, true);		      
		  setupMenu();
		  connect(myTimer, SIGNAL(timeout()),
			  this,    SLOT(updateCreateImageStatus()));
		  myTimer->start(1000, false);
		};			  	      
	    }
	}
      break;
    case ID_SSTEP_MOUNTVIEWIMAGE:
      // Vorhandensein der Datei testen:
      if (!existDat(sysOpts.imageFilePath.c_str())) errorMessage(i18n("image does not exist!"));
      else
	{ 
	  debugFile.newFile();
	  mountViewImage(fMounted, sysOpts);
	}
      break;
    case ID_SSTEP_CREATESCRIPT:
      createFinalScript(true);
      break;
    case ID_SSTEP_BURNIT:
      if (burnProc->isRunning() || imageProc->isRunning())
	errorMessage(i18n("process already running!"));
      else
	{
	  int rueckfrage;
	  
	  rueckfrage = preBurningBox(sysOpts.fShutdown);	
	  
	  if (rueckfrage) 
	    {
	      string pfad = HeimatPfad;
	      
	      pfad += "/cdScript.sh"; 
	      if (createFinalScript(rueckfrage == 1) == 0)
		{
		  QString infT;
		  if (aktOpts.fDummyWrite)
		    infT = i18n("This window just shows you the output of the process currently\n"
				"making the simulation write of the cd. You can safely close it.");
		  else
		    infT = i18n("This window just shows you the output of the process currently\n"
				"burning the cd. You can safely close it.");
		  
		  burnProc->newProc(infT, pfad);
		  connect(burnProc, SIGNAL(finished(KProcess*)),
			  this,     SLOT(burnFinished(KProcess*)));
		  
		  if (!burnProc->startProc(1, pfad, sysOpts.fProcWinOnStart))
		    {
		      menuView->setItemEnabled(ID_VIEW_SUBCD, true);		      
		      setupMenu();
		      
		      burnProc->setAutoStatusLine("Track [0123456789]*:[ 0123456789]*of[ 0123456789]*MB written .*",
						  true);
		      progress.cdrecordStarted();
		      connect(burnProc, SIGNAL(REMatched(QString)),
			      this,     SLOT(updateBurnStatus(QString)));		      
		    }
		}
	    }
	}
      break;
    case ID_SSTEP_DELIMAGE:
      if (unlink(sysOpts.imageFilePath.c_str()) != 0) 
	errorMessage(i18n("Error deleting image file:\n"
			   "  Does it exist?\n"
			   "  Do I have the permissions to delete it?\n"
			   "  Is the image mounted?"));
      break;

      /* ***************************************** */  
    case ID_OPTIONS_SYSTEM: 
      {
	Opt_sys tmpDiag(this, "name", globalConf, &sysOpts, !burnProc->isRunning());
	tmpDiag.exec();
	myCDTools.setDevice(sysOpts.readDevice);
	myCDTools.setMountpoint(sysOpts.mountPoint);
      } 
      break;
    case ID_OPTIONS_CDDOC:
      {
	Opt_cddoc tmpDiag(this, "name", globalConf, &sysOpts);
	
	tmpDiag.exec();
      } 
      break;
    case ID_OPTIONS_UI:
      {
	Opt_ui tmpDiag(this, "name", globalConf, &sysOpts);
	
	tmpDiag.exec();
      } 
      break;
    case ID_OPTIONS_CD: 
      {
	Opt_cd tmpDiag(this, "name", globalConf, &defOpts, &aktOpts, 
		       &cdOptsChanged, &moduleExitCode);
	
	tmpDiag.exec();
      }
      setupMenu();
      mainView->update(aktOpts);      
      if (moduleExitCode != moduleID) 
	if (preExit()) delete(this);
      break;  
      /* ***************************************** */  
    case ID_VIEW_SUBIMAGE:
      imageProc->show();
      break;
    case ID_VIEW_SUBCD:
      burnProc->show();
      break;
      
      /* ***************************************** */  
    default:
      if (!stdCDRWCallback(id_, menuView, ID_VIEW_SUBCD))
	if (!stdCDDocCallback(id_))
	  stdHelpCallback(id_);
    }
}



CopyWidget::CopyWidget(QWidget* parent, const char* name) : QWidget (parent, name)
{
#ifdef VERBOSEMESSAGES
  cout << "CopyWidget::CopyWidget() called\n";
#endif
  gb_insertSource = new QGroupBox(this, "");
  gb_insertSource->setGeometry(10, 10, 350, 65);
  gb_insertSource->setTitle(i18n("first step: insert source"));
  pb_openClose = new QPushButton(gb_insertSource, "pb1");
  pb_openClose->setGeometry(10, 20, 330, 30);
  pb_openClose->setText(i18n("open/close tray"));
  
  gb_readImage = new QGroupBox(this, "");
  gb_readImage->setGeometry(10, 90, 350, 65);
  gb_readImage->setTitle(i18n("second step: create image"));
  pb_readImage = new QPushButton(gb_readImage, "pb2");
  pb_readImage->setGeometry(10, 20, 330, 30);
  pb_readImage->setText(i18n("Create Image"));
  
  gb_burnCD = new QGroupBox(this, "");
  gb_burnCD->setGeometry(10, 170, 350, 65);
  gb_burnCD->setTitle(i18n("last step: burn CD"));
  pb_burnCD    = new QPushButton(gb_burnCD, "pb3");
  pb_burnCD   ->setGeometry(10, 20, 330, 30);
  pb_burnCD   ->setText(i18n("Burn CD now!"));
}

void CopyWidget::setButtonsEnabled(bool openClose, 
				   bool read,
				   bool burn)
{
  pb_openClose->setEnabled(openClose);
  pb_readImage->setEnabled(read);
  pb_burnCD->setEnabled(burn);
}

void CopyWidget::update(T_Options &currOpts)
{
  // je nach on-the-fly/image den zweiten Schritt nicht darstellen
  if (currOpts.fCreateImage)
    {
      gb_readImage->show();
      gb_burnCD->setGeometry(10, 170, 350, 65);
    }
  else
    {
      gb_readImage->hide();
      gb_burnCD->setGeometry(10, 90, 350, 65);
    }
}

#include "copy.moc"


















