#include "dndlist.h"
#include <klocale.h>

dndList::dndList(QWidget * parent, const char * name, bool fMS, bool fTDM) 
        : QListView(parent, name)
{
  fMultiSelect = fMS;
  fTrackDragMove = fTDM;
  setMultiSelection(fMultiSelect);  
  setAcceptDrops(true);
  connect(this, SIGNAL(rightButtonPressed(QListViewItem *, const QPoint &, int)), 
	  SLOT(contextMenu(QListViewItem *, const QPoint &, int)));  
}

void dndList::dragEnterEvent(QDragEnterEvent * event)
{
  if (fTrackDragMove) setMultiSelection(false);
  event->accept(QUriDrag::canDecode(event));
}

void dndList::dragLeaveEvent(QDragLeaveEvent * event)
{
  if (fTrackDragMove) setMultiSelection(fMultiSelect);
}

void dndList::dropEvent(QDropEvent * event)
{
  QStrList      list;
  QString       entry;
  unsigned int  cnt;
  string        dropList = "";
  int           dropType;
  int           anzahl = 0;

  if (fTrackDragMove) setMultiSelection(fMultiSelect);
  
  if (QUriDrag::decode(event, list))
    {
#ifdef VERBOSEMESSAGES    
      cout << "drop:\n";
#endif      
      for(cnt=0; cnt< list.count(); cnt++)
        {
	  entry = (list.at(cnt));
          if (entry.find("file:", 0, false) == 0) entry.remove(0, 5);
	  
          if (entry.find("mailto:", 0, false) == 0) 
	    {
#ifdef VERBOSEMESSAGES    	    
	      cout << "   - mailto\n";
#endif      	      
	    }
	  else 
            if (entry.find("http:", 0, false) == 0)
	      {
#ifdef VERBOSEMESSAGES    	      
	        cout << "   - http:\n";
#endif      		
	      }
	    else 
	      if (entry.find("ftp:", 0, false) == 0)
	        {
#ifdef VERBOSEMESSAGES    		
		  cout << "   - ftp:\n";
#endif      		  
		}
	      else
	        {
		  dropType = getUriType(entry);
#ifdef VERBOSEMESSAGES    		
		  cout << "   " <<  dropType << " " <<  entry << "\n";
#endif      		  
                  if (dropType != -1) 
		    {
		      dropType &= ~128;
		      if (dropType == 1) 
		        {
		          dropList += "file:";
			  dropList += entry.latin1();
			  dropList += "/\n";
			}
		      else 
		        {
			  dropList += "file:";
			  dropList += entry.latin1();
			  dropList += "\n";
			}
		      anzahl++;
		    }
		}
        }
    }        
  if (anzahl == 1) dropList.replace(dropList.length()-1, 1, "");
  if (dropList != "") emit drop(dropList);    
#ifdef VERBOSEMESSAGES
  cout << "drop:" << "*" << dropList << "*\n";
#endif  
}

void dndList::dragMoveEvent(QDragMoveEvent *dme)
{
  QPoint position;
  dme->accept();
  if (fTrackDragMove)
    {        
      position = dme->pos();
      setSelected(itemAt(position), true);
    }
}

void dndList::contextMenu(QListViewItem *item, const QPoint &point, int col)
{
  int         choiceContext;
  QPopupMenu  *pm;

  pm = new QPopupMenu(this);
  pm->insertItem(i18n("New directory.."), 0, 0);       // 0
  choiceContext = pm->exec(QCursor::pos());
  delete pm;

  switch (choiceContext) {
  case 0: emit newDir();
    break;
  default: {}
  }
}




