#include "switches.h"
#include "edit.h"

#include "menuid.createdata"
#include "module.id"

/*** KDE ***/
#include <kfiledialog.h>
#include <kurl.h>
#include <ksimpleconfig.h>
#ifdef KDE2
#include <klocale.h>
#endif
#include <kapp.h>

/*** Qt ***/
#include <qmessagebox.h>

/*** C++ ***/
#include <stdlib.h>   // fr getenv(..) und system(..)
#include <unistd.h>   // fr chdir(...)
#include "sys/stat.h" // fr die Dateiattribute
#include "sys/types.h"
#include "ctype.h"

/*** KisoCD ***/


extern int                moduleExitCode;
extern int                debuglevel;
extern DebugFile          debugFile; 
extern KSimpleConfig*     globalConf;
extern string             HeimatPfad;
extern T_Options          aktOpts;
extern T_Options          defOpts;
extern T_SysOptions       sysOpts;

Edit::Edit(QWidget *parent, const char *name, int aModuleID)
  : KCDModule(parent, name, aModuleID)
{
  // Das Men erstellen:
  createMenu();
  
  createStatusBar();

  // Das Fenster fllen:
  createMainView();
#ifndef KDE2    
  DDDirektaufnahme = new KDNDDropZone(DatList, DndURL);
#endif
  /* ***************************************** */      
  NOTsaved    = false;
  ProjectOpen = false;
  aktVerz     = ""; 
  aktDateiName = "";  
  
  fMounted = false;
  brennGesamtStatus = 0;
  
  /* ***************************************** */    
  imageSub = new Subprocess(i18n("This window just shows you the output of a subprocess currently\n"
				       "running. You can safely close it."));
  myFileView = new SimpleKFVDialog;
  
  /* ***************************************** */    
  connect (menuBar,    SIGNAL (activated (int)), SLOT (commandCallback (int)));
  connect (menuFile,    SIGNAL (aboutToShow()), SLOT (setupFileMenu()));
  connect (menuCdDoc,    SIGNAL (aboutToShow()), SLOT (setupCDDocMenu()));
  connect (burnSingleStep,    SIGNAL (aboutToShow()), SLOT (setupSStepMenu()));
  
  connect (ktlist, SIGNAL (currentChanged (QListViewItem*)), 
	   SLOT (changedDirCallback (QListViewItem*)));
#ifdef KDE2
  connect(ktlist,     SIGNAL(drop(string)), SLOT(dropAktion(string)));
  connect(DatList,    SIGNAL(drop(string)), SLOT(dropAktion(string)));
  connect(ktlist,     SIGNAL(newDir()), SLOT(machNeuVerz()));
  connect(DatList,    SIGNAL(newDir()), SLOT(machNeuVerz()));
#else
  connect (DDDirektaufnahme, 
	   SIGNAL (dropAction(KDNDDropZone *)), 
	   SLOT (DDDirektCB(KDNDDropZone *)));
#endif
  connect(myFileView, SIGNAL(fileDrop(string)), SLOT(dropAktion(string)));
    
  /* ***************************************** */    
  checkPrerequisites();
#ifdef VERBOSEMESSAGES
  cerr << "Edit::Init() done\n";
#endif
}

int Edit::createMenu()
{
  /* ***************************************** */
  menuFile = new QPopupMenu;
  
  if (globalConf->readEntry("mFile_new", "enabled") == "enabled") 
    menuFile->insertItem(i18n("New"),                ID_FILE_NEW);
  if (globalConf->readEntry("mFile_open", "enabled") == "enabled") 
    menuFile->insertItem(i18n("Open.."),             ID_FILE_OPEN);
  menuFile->insertSeparator();
  if (globalConf->readEntry("mFile_save", "enabled") == "enabled") 
    menuFile->insertItem(i18n("Save"),               ID_FILE_SAVE);
  if (globalConf->readEntry("mFile_saveAs", "enabled") == "enabled") 
    menuFile->insertItem(i18n("Save as.."),          ID_FILE_SAVEAS);
  menuFile->insertSeparator();
  if (globalConf->readEntry("mFile_BurnCD", "enabled") == "enabled") 
    menuFile->insertItem(i18n("Burn CD"),          ID_FILE_BURNCD);  
  menuFile->insertSeparator();
  menuFile->insertItem(i18n("E&xit"),              ID_FILE_EXIT);  

  /* ***************************************** */
  burnSingleStep = new QPopupMenu;

  if (globalConf->readEntry("mSStep_simulateImageCreation", "enabled") == "enabled") 
    burnSingleStep->insertItem(i18n("Simulate image creation"), ID_SSTEP_TESTIMAGE);
  if (globalConf->readEntry("mSStep_createImage", "enabled") == "enabled") 
    burnSingleStep->insertItem(i18n("Create Image"),            ID_SSTEP_CREATEIMAGE);
  if (globalConf->readEntry("mSStep_mountViewImage", "enabled") == "enabled") 
    burnSingleStep->insertItem(i18n("Mount and View Image"),    ID_SSTEP_MOUNTVIEWIMAGE);
  if (globalConf->readEntry("mSStep_createScript", "enabled") == "enabled") 
    burnSingleStep->insertItem(i18n("Create ShellScript"),      ID_SSTEP_CREATESCRIPT);
  if (globalConf->readEntry("mSStep_burnIt", "enabled") == "enabled") 
    burnSingleStep->insertItem(i18n("Burn CD now!"),            ID_SSTEP_BURNIT);
  if (globalConf->readEntry("mSStep_delImage", "enabled") == "enabled") 
    burnSingleStep->insertItem(i18n("Delete Image"),            ID_SSTEP_DELIMAGE);

  /* ***************************************** */  
  getStdHelpMenu(menuHelp);

  /* ***************************************** */  
  getStdCDDocMenu(menuCdDoc);
  
  /* ***************************************** */
  getStdCDRWMenu(menuCDRW);
  
  /* ***************************************** */
  getStdOptionsMenu(menuOptions);
  
  /* ***************************************** */
  menuView = new QPopupMenu;

  if (globalConf->readEntry("mView_sub1", "enabled") == "enabled") 
    menuView->insertItem(i18n("output of subprocess:cdwriter"),         ID_VIEW_SUBCD);
  if (globalConf->readEntry("mView_sub2", "enabled") == "enabled") 
    menuView->insertItem(i18n("output of subprocess:image"),         ID_VIEW_SUBIMAGE);
  menuView->insertSeparator();
  if (globalConf->readEntry("mView_simplefm", "enabled") == "enabled") 
    menuView->insertItem(i18n("simple filemanager"),         ID_VIEW_SIMPLEFM);
  if (globalConf->readEntry("mView_kfm", "enabled") == "enabled") 
    menuView->insertItem(i18n("kFileManager"),         ID_VIEW_KFM);
  menuView->setItemEnabled(ID_VIEW_SUBIMAGE, FALSE);
  menuView->setItemEnabled(ID_VIEW_SUBCD,    FALSE);

  /* ***************************************** */    
  menuVirtcd = new QPopupMenu;

  menuVirtcd->insertItem(i18n("Edit.."),          ID_VIRTCD_EDIT);
  menuVirtcd->insertItem(i18n("New directory.."), ID_VIRTCD_ADDDIR);
  menuVirtcd->setItemEnabled(ID_VIRTCD_EDIT,    FALSE);
  menuVirtcd->setItemEnabled(ID_VIRTCD_ADDDIR,  TRUE);

  /* ***************************************** */    
  menuBar = new KMenuBar( this );
  CHECK_PTR( menuBar );

  menuBar->insertItem( i18n("&File"),    menuFile );
  if (globalConf->readEntry("mSStep", "enabled") == "enabled")   
    menuBar->insertItem( i18n("&Single Step"), burnSingleStep);
  menuBar->insertItem( i18n("Virtual &CD"),    menuVirtcd );  
  if (globalConf->readEntry("mCDDoc", "enabled") == "enabled")   
    menuBar->insertItem( i18n("cd&Doc"),   menuCdDoc );
  if (globalConf->readEntry("mOptions", "enabled") == "enabled") 
    menuBar->insertItem( i18n("&Options"), menuOptions );
  if (globalConf->readEntry("mCDRW", "enabled") == "enabled") 
    menuBar->insertItem( i18n("CDR&W"), menuCDRW );
  if (globalConf->readEntry("mView", "enabled") == "enabled") 
    menuBar->insertItem( i18n("&View"), menuView );
  menuBar->insertSeparator();
  menuBar->insertItem( i18n("&Help"),    menuHelp );
  
  menuBar->show();
  setMenu(menuBar);
  
  return(0);
}

int Edit::createStatusBar()
{
  statusBar = new KStatusBar(this);
  statusBar->insertItem(i18n("Welcome to KisoCD!"), 0);
  statusBar->insertItem("99999 MBytes", 1);
  VoelleGefuehl = new KProgress(0, 100, 0, KProgress::Horizontal, statusBar);
#ifdef KDE2  
  statusBar->addWidget((QWidget *)VoelleGefuehl, 300, 2);
#else
  statusBar->insertWidget((QWidget *)VoelleGefuehl, 300, 2);
#endif
  setStatusBar(statusBar);
  QToolTip::add(statusBar, i18n("statusbar: shows size of selected dir and subdirs, size of entire CD"));
  statusBar->changeItem("", 1);  
  return(0);
}

int Edit::createMainView()
{
  string tmp;
#ifdef KDE2
  Panner = new QSplitter(this);
  // Die TreeList erzeugen:
  ktlist = new dndList(Panner, "CD-Vorschau Verzeichnisse", false, true);
  ktlist->addColumn(i18n("directories"));
  ktlist->setColumnWidth(0, 150);
  ktlist->setRootIsDecorated(true);
  ktlist->setMinimumSize(100, 100);
  QToolTip::add(ktlist, i18n("shows directory tree of your CD. Drop things here!"));
  // die ListBox(Dateien im gewhlten Verzeichnis) erzeugen
  DatList = new dndList(Panner, "CD-Vorschau Dateien", true, false);  
  DatList->addColumn(i18n("files"));
  DatList->addColumn(i18n("size"));   
  DatList->addColumn(i18n("from"));  
  DatList->setColumnWidth(0, 187);
  DatList->setColumnWidth(1, 80);
  DatList->setColumnWidth(2, 100);
  DatList->setMinimumSize(200, 100);  
  QToolTip::add(DatList, i18n("shows files in the selected dir. Drop things here!"));
  setView(Panner);
#else
  Panner = new KNewPanner(this, 0, KNewPanner::Vertical, 
			   KNewPanner::Percent, 30);
  setView(Panner);
  // Die TreeList erzeugen:
  ktlist = new QListView(Panner, "CD-Vorschau Verzeichnisse");
  ktlist->addColumn(i18n("directories"));
  ktlist->setRootIsDecorated(true);
  QToolTip::add(ktlist, i18n("shows directory tree of your CD"));
  // die ListBox(Dateien im gewhlten Verzeichnis) erzeugen
  DatList = new QListView(Panner, "CD-Vorschau Dateien");  
  DatList->addColumn(i18n("files"));
  DatList->addColumn(i18n("size"));   
  DatList->addColumn(i18n("from"));  
  DatList->setColumnWidth(0, 187);
  DatList->setColumnWidth(1, 80);
  DatList->setColumnWidth(2, 147);
  QToolTip::add(DatList, i18n("shows files in the selected dir. Drop things here!"));
  Panner->activate(ktlist, DatList);
#endif

  return(0);
}

int Edit::saveToFile(const char* filename)
{
  ofstream   outDat;
  string     SrcPath,
             DstPath;  
  
  outDat.open(filename);
  if (!outDat)
    {
      errorMessage(i18n("Unable to open file for saving!\n"));
#ifdef VERBOSEMESSAGES
      cerr << filename << "\n";
#endif
      return(-1);
    }
  outDat << "# " << ProggieName << " " << ProggieVersion << "\n";
  outDat << "# Projectfile Type 3\n";
  for (int Cnt = 0; Cnt < 9; Cnt++) 
    {
      if (aktOpts.switches[Cnt]) outDat << " 1\n"; 
      else outDat << " 0\n"; 
    }
  
  outDat << aktOpts.IDdisc      << "\n";
  outDat << aktOpts.IDvolume    << "\n";
  outDat << aktOpts.IDpreparer  << "\n";
  outDat << aktOpts.IDpublisher << "\n";
  
  if (aktOpts.fCreateImage) outDat << " 1\n";
  else outDat << " 0\n";
  if (aktOpts.fDummyWrite) outDat << " 1\n"; 
  else outDat << " 0\n"; 
  if (aktOpts.fbootCD) outDat << " 1\n"; 
  else outDat << " 0\n"; 
  
  outDat << aktOpts.bootImagePath << "\n";
  
  if (aktOpts.fMulti) outDat << " 1\n"; 
  else outDat << " 0\n"; 
  if (aktOpts.fMultiFirstSess) outDat << " 1\n"; 
  else outDat << " 0\n"; 
  if (aktOpts.fMultiLastSess) outDat << " 1\n"; 
  else outDat << " 0\n"; 

  // new option at end
  if (aktOpts.fPadTrack) outDat << " 1\n"; 
  else outDat << " 0\n"; 

  outDat << "[\n";
  
  // Gibt es MeineCD an dieser Stelle schon ?
  
  if (ProjectOpen) 
    {  
      for (int Cnt = 0; Cnt < MeineCD->getExNum(); Cnt++) 
	{
	  MeineCD->getEx(Cnt, SrcPath);
	  outDat << SrcPath << "\n";
	}
    }
  outDat << "[\n";
  
  if (ProjectOpen)
    {
      for (int Cnt = 0; Cnt < MeineCD->getVerzNum(); Cnt++) 
	{
	  MeineCD->getVerz(Cnt, SrcPath, DstPath);
	  DstPath.replace(0, 3, "");
	  outDat << DstPath << "=" << SrcPath << "\n";
	}
    }
  outDat << "[\n";
  // das latex-File ans Ende kopieren
  {
    ifstream  latexFile;
    string    latexFilePfad = HeimatPfad;
    char      c;
    
    latexFilePfad += "/cdDoc.tex";
    latexFile.open(latexFilePfad.c_str());
    while ((c = latexFile.get()) != EOF) outDat << c;
    
    latexFile.close();
  }
  close_ofs(outDat);  
  
  NOTsaved = false;
  texFileSavedWatch.lookNow(HeimatPfad + "/cdDoc.tex");
  return(0);
}

int Edit::loadFromFile(const char* filename)
  // Die Nummerierung der sections in der Eingabedatei:
  //   0. Konfig-Daten
  //   1. Ausschlsse (Verzeichnis, Datei, Muster)
  //   2. einfache Zuordnungen (Zielverz=Quellverz, Zielverz=Quellverz+Datei) 
  //   3. die LaTeX-Datei
  // Trennzeichen zwischen den sections: ffnende eckige Klammer
{
  ifstream      inDat;
  ofstream      latexFile;
  string        latexFilePath;
  char          Zeile[500];
  int           section = 0;
  string        Zeile2;
  unsigned int  TrennPos = 0;
  bool          fileType1 = false;
  bool          fileType3 = false;
  
  latexFilePath = HeimatPfad + "/cdDoc.tex";
  latexFile.open(latexFilePath.c_str());
  if (!latexFile) 
    {
      errorMessage(i18n("Unable to create cdDoc.tex!\n"));
      return(-1);
    }
  
  inDat.open(filename);
  if (!inDat) 
    {
      errorMessage(i18n("Unable to open file!\n"));
      return(-1);
    }
   else 
     {
       // prfen, ob es die Datei einer alten Version ist;
       // wenn es so ist, mit der alten Methode einlesen, dabei
       // allerdings nur die Dinge einlesen, die auch nach dem 
       // neuen Dateityp Auswirkungen htten.
       inDat.getline(Zeile, 499);
       inDat.getline(Zeile, 499); 
       if (Zeile[1] == '0' || Zeile[1] == '1') 
	 {
	   fileType1 = true;
	   inDat.close();
	   inDat.open(filename);
	   infoMessage(i18n("Loading file in old format and saving in new one..."));
	 }
       else
         {
           if (Zeile[19] == '3') fileType3 = true;
         }
     }
  
  if (ProjectOpen) delete MeineCD;   
  
  MeineCD = new CDTree;

  // wenn es ein Project File vor Version 3 ist, ist die PadTrack-Option noch nicht vorhanden.
  // Damit sich das Programm dann wie gewohnt verhlt, wird PadTrack standardmssig abgeschaltet.
  aktOpts.fPadTrack = false;

  if (fileType1)
    {
      aktOpts.fMulti           = false;
      aktOpts.fMultiFirstSess  = false;
      aktOpts.fMultiLastSess   = false;

      while (!inDat.eof()) 
	{
	  inDat.getline(Zeile, 499);
	  if (Zeile[0] == '[') section++; 
	  else 
	    {
	      if ((strlen(Zeile) != 0) || (section == 3)) 
		{
		  switch (section) {
		  case 0:
		    if (TrennPos <= 8) aktOpts.switches[TrennPos] = (Zeile[1] == '1');	     
		    else 
		      {
			switch (TrennPos) {
			case  9: aktOpts.IDdisc        = Zeile;             break;
			case 10: aktOpts.IDvolume      = Zeile;             break;
			case 11: aktOpts.IDpreparer    = Zeile;             break;
			case 12: aktOpts.IDpublisher   = Zeile;             break;
			case 13: aktOpts.fCreateImage  = (Zeile[1] == '1'); break;
			case 14:                                            break;
			case 15:                                            break;
			case 16:                                            break;
			case 17: aktOpts.fDummyWrite   = (Zeile[1] == '1'); break;  
			case 18: aktOpts.fbootCD       = (Zeile[1] == '1'); break;
			case 19: aktOpts.bootImagePath = Zeile;             break;
			case 20: break; // hier steht nur die Versionsnummer
			}
		      }
		    TrennPos++;
		    break;
		  case 1:
		    Zeile2 = Zeile;
		    MeineCD->Exclude(Zeile2);
		    break;
		  case 2:
		    Zeile2 = Zeile;
		    TrennPos = Zeile2.find("=");
		    if (TrennPos != string::npos) 
		      {
			MeineCD->Append(Zeile2.substr(TrennPos+1, Zeile2.size()-TrennPos-1), 
					"/CD" + Zeile2.substr(0, TrennPos), 
					aktOpts.switches[0]);
		      }
		    break;
		  case 3:
		    latexFile << Zeile << "\n";
		    break;	  
		  }
		}
	    }
	}               
    }
  else
    {
      while (!inDat.eof()) 
	{
	  inDat.getline(Zeile, 499);
	  if (Zeile[0] == '[') section++; 
	  else 
	    {
	      if ((strlen(Zeile) != 0) || (section == 3) || 
	          (section == 0 && 
		    (TrennPos == 9 || TrennPos == 10 || TrennPos == 11 || TrennPos == 12 || TrennPos == 16))) 
		{
		  switch (section) {
		  case 0:
		    if (TrennPos <= 8) aktOpts.switches[TrennPos] = (Zeile[1] == '1');
		    else 
		      {
			switch (TrennPos) {
			case  9: aktOpts.IDdisc        = Zeile;             break;
			case 10: aktOpts.IDvolume      = Zeile;             break;
			case 11: aktOpts.IDpreparer    = Zeile;             break;
			case 12: aktOpts.IDpublisher   = Zeile;             break;
			case 13: aktOpts.fCreateImage  = (Zeile[1] == '1'); break;
			case 14: aktOpts.fDummyWrite   = (Zeile[1] == '1'); break;  
			case 15: aktOpts.fbootCD       = (Zeile[1] == '1'); break;
			case 16: aktOpts.bootImagePath = Zeile;             break;
			case 17: aktOpts.fMulti          = (Zeile[1] == '1'); break;
			case 18: aktOpts.fMultiFirstSess = (Zeile[1] == '1'); break;  
			case 19: aktOpts.fMultiLastSess  = (Zeile[1] == '1'); break;
                          /* file type == 3 */
			case 20: aktOpts.fPadTrack = (Zeile[1] == '1'); break;
			}
		      }
		    TrennPos++;
		    break;
		  case 1:
		    Zeile2 = Zeile;
		    MeineCD->Exclude(Zeile2);
		    break;
		  case 2:
		    Zeile2 = Zeile;
		    TrennPos = Zeile2.find("=");
		    if (TrennPos != string::npos) 
		      {
			MeineCD->Append(Zeile2.substr(TrennPos+1, Zeile2.size()-TrennPos-1), 
					"/CD" + Zeile2.substr(0, TrennPos), 
					aktOpts.switches[0]);
		      }
		    break;
		  case 3:
		    latexFile << Zeile << "\n";
		    break;	  
		  }
		}
	    }
	}    
    }
  inDat.close();
  close_ofs(latexFile);

  // falls man noch eine alte Datei hat, in der "fAllFiles" gesetzt
  // ist, wird es hier berschrieben:
  aktOpts.switches[2] = false;


#ifdef VERBOSEMESSAGES  
  cout << "Datei eingelesen\n";
#endif
  // zweiter Teil: Zielbaum erzeugen lassen und anzeigen:
  MeineCD->calcSize();
#ifdef VERBOSEMESSAGES  
  cout << "Ende calcSize\n";
#endif
  
  MeineCD->getCDVerzTree(ktlist, false);
#ifdef VERBOSEMESSAGES  
  cout << "Ende getCDVerzTree\n";
#endif
  
  setzeVoelleGefuehl(MeineCD->getCDSize());
  clearLabels();
  DatList->clear();
  ProjectOpen = true;
  menuVirtcd->setItemEnabled(ID_VIRTCD_EDIT, true);
  aktDateiName = filename;

  texFileSavedWatch.lookNow(latexFilePath);
  
  // new und save disablen
  NOTsaved = false;
  texFileWatch.flag = false;
  
  ktlist->setSelected(ktlist->firstChild(), true);
  ktlist->setOpen(ktlist->firstChild(), true);
#ifdef VERBOSEMESSAGES      
  cout << "Ende LoadFromFile\n";
#endif
    
  // berprfen von sysOpts.imageFilePath
  if (sysOpts.imageFilePath[sysOpts.imageFilePath.size()-1] == '/')
    {
      errorMessage(i18n("Please give a valid path for the imagefile!\n"
		    "(options|more)\n"
		    "I corrected it to a default which possibly makes\n"
		    "no sense but will not cause strange errors. The point\n"
		    "is that you have to give a file, not a directory!"));  
      sysOpts.imageFilePath = "/usr/local/stupid_place.raw";
    }

  // Wenn es eine Datei der alten Version war, gleich unter dem 
  // neuen Format speichern:
  if (fileType1) saveToFile(filename);
  
  return(0);
}

Edit::~Edit()
{
#ifdef VERBOSEMESSAGES
  cerr << "Edit::~Edit() called..\n";
#endif  
  if (fMounted) unmountImage(fMounted, sysOpts.mountPoint);    
  if (ProjectOpen) delete MeineCD;
  delete imageSub;
}

bool Edit::preExit()
{
  bool tmp = true;
  
  if (prozessAktiv())
    if (QMessageBox::warning(this, i18n("Running subprocesses"), 
			     i18n("There is at least one subprocess still running." 
				  "You will kill it if you exit now."),
			     i18n("Exit"), i18n("Cancel"), 
			     0, 1, 1)) tmp = false;
  if (tmp)
    if (NOTsaved) 
      if (QMessageBox::warning(this, i18n("Unsaved Changes"), 
			       i18n("You have unsaved changes, you will lose " 
				    "them if you exit now."),
			       i18n("Exit"), i18n("Cancel"), 
			       0, 1, 1)) tmp = false;
  return(tmp);
}

void Edit::commandCallback(int id_)
{
#ifdef SLOTLOG
  cerr << "Slot: commandCallback, id_ = " << id_ << "\n";
  cerr.flush();
#endif
  KShellProcess*  process = NULL;
  bool processStarted = false;
  bool tmp = true;
  
  
  switch(id_) {
  case ID_FILE_EXIT:
    moduleExitCode = ID_MODULE_EXIT;
    if (preExit()) delete(this);    
    break;
  case ID_FILE_OPEN: 
    if (NOTsaved) 
      if (QMessageBox::warning(this, i18n("Unsaved Changes"), 
			       i18n("You have unsaved changes, you will lose " 
			       "them if you open a file now."),
			       i18n("open file"), i18n("Cancel"), 
			       0, 1, 1)) tmp = false;     
    if (tmp) {    
#ifdef KDE2
      QString name= KFileDialog::getOpenURL(HeimatPfad.c_str(), "*.cd").url();   
#else
      QString name= KFileDialog::getOpenFileName(HeimatPfad.c_str(), "*.cd");   
#endif
      if (!name.isEmpty()) 
	{ 
#ifdef KDE2
	  name.replace(0, 5, "");  
#endif
	  if (fMounted) unmountImage(fMounted, sysOpts.mountPoint);
	  if (loadFromFile((const char*)(name)) == 0)
	    {
	      string newCaption = ProggieName;
	      
	      aktVerz = "";
	      newCaption += ": " + aktDateiName;
	      setCaption((const char*)newCaption.c_str());
	    }
	}      
    }
    break;
  case ID_FILE_SAVEAS:{
#ifdef KDE2
    QString name1= KFileDialog::getSaveURL(HeimatPfad.c_str(), "*.cd").url();
#else
    QString name1= KFileDialog::getSaveFileName(HeimatPfad.c_str(), "*.cd");
#endif
    if (!name1.isEmpty())       
      {	
	string name = (const char*)name1;
#ifdef KDE2
	if (name.find("file:") == 0) name.replace(0, 5, "");
#endif
	
	if ((name.length() != 0) && (name[name.length()-1] != '/')) { 
	  string newCaption = ProggieName;
	  if (name.find(".cd") != name.length()-3) { name += ".cd"; }
	  saveToFile(name.c_str()); 
	  
	  aktDateiName = name;    
	  
	  newCaption += ": " + name;
	  setCaption((const char*)newCaption.c_str());
	}
	else errorMessage(i18n("You must give a filename!"));
      }
  } break;
  case ID_FILE_SAVE:
    saveToFile(aktDateiName.c_str());
    break;
  case ID_FILE_NEW:
    if (NOTsaved) 
      if (QMessageBox::warning(this, i18n("Unsaved Changes"), 
			       i18n("You have unsaved changes, you will lose " 
				    "them if you open a new project now."),
			       i18n("new project"), i18n("Cancel"), 
			       0, 1, 1)) tmp = false;         
    if (tmp) 
      {    
	if (fMounted) unmountImage(fMounted, sysOpts.mountPoint);
	
	menuVirtcd->setItemEnabled(ID_VIRTCD_EDIT, false);
	NOTsaved    = false;
	texFileWatch.flag = false;
	texFileSavedWatch.lookNow(HeimatPfad + "/cdDoc.tex");
	// Optionen auf default-Werte setzen
	aktOpts = defOpts;
	// die Labels, listbox und treelist reinigen:
	setzeVoelleGefuehl(0);
	clearLabels();
	ktlist->clear();
	DatList->clear();           
	aktVerz = "";
	aktDateiName = "";
	// cd-objekt lschen
	if (ProjectOpen) 
	  {
	    delete MeineCD;     
	    ProjectOpen = false;
	  }
	setCaption(ProggieName);      
	makeNewLatexFile(HeimatPfad);
      }
    break;
  case ID_FILE_BURNCD:
    if (prozessAktiv()) errorMessage(i18n("process already running!"));
    else
      {
	debugFile.newFile();
	burnCD_guided();
      }
    break;
  case ID_SSTEP_CREATEIMAGE:   
    if (imageSub->isRunning()) errorMessage(i18n("process already running!"));
    else
      { 
	debugFile.newFile();
	startCreateImage();
      }
    break;
  case ID_SSTEP_TESTIMAGE: 
    if (imageSub->isRunning()) errorMessage(i18n("process already running!"));
    else
      { 
	debugFile.newFile();
	startImageSim(0);
      }
    break;
  case ID_SSTEP_MOUNTVIEWIMAGE: 
    // Vorhandensein der Datei testen:
    if (!existDat(sysOpts.imageFilePath.c_str())) errorMessage(i18n("image does not exist!"));
    else
      { 
	debugFile.newFile();
	mountViewImage(fMounted, sysOpts);
      }
    break;
  case ID_SSTEP_DELIMAGE:
    if (unlink(sysOpts.imageFilePath.c_str()) != 0) 
      errorMessage(i18n("Error deleting image file:\n"
			 "  Does it exist?\n"
			 "  Do I have the permissions to delete it?\n"
			 "  Is the image mounted?"));
    break;
  case ID_SSTEP_CREATESCRIPT:
    createFinalScript(true);
    break;
  case ID_SSTEP_BURNIT:
    if (burnProc->isRunning())
      errorMessage(i18n("process already running!"));
    else
      { 
	debugFile.newFile();
	startBrennProzess();
      }
    break;
  case ID_CDDOC_CREATE:
    if (!QMessageBox::warning(this, i18n("sure?"), 
			      i18n("This will overwrite your old file!"),
			      i18n("OK"), i18n("Abort"), 
			      0, 1, 1)) {
      /* Neue Doku erzeugen, abhngig davon, welche ste im QListView offen sind */ 
      string Pfad = HeimatPfad + "/cdDoc.tex";
      string koverPfad = HeimatPfad;
      ofstream outDat;
      ofstream koverDat;
      
      
      if (NOTsaved) koverPfad += "/cdDoc.kover";
      else 
	{
	  string tmpStr = aktDateiName;
	  while (tmpStr.find("/") != string::npos) tmpStr.replace(0, 1, "");
	  tmpStr.replace(tmpStr.length()-2, 2, "kover");
	  koverPfad += "/" + tmpStr;
	}      
      
      koverDat.open(koverPfad.c_str());
      if (!koverDat)
	{
	  QString fehlerm = i18n("Could not write kover-file: ");
	  fehlerm += koverPfad.c_str();
	  errorMessage(fehlerm);
	}
      else 
	{
	  QString meldung = i18n("kover-file saved as\n");
	  meldung += koverPfad.c_str();    
	  QMessageBox::about(0, i18n("Info"), meldung);	
	  writeKoverFileHeader(koverDat);
	}
      
      outDat.open(Pfad.c_str());
      if (!outDat) {
	QString meldung = i18n("Could not write tex-file ");
	meldung += Pfad.c_str();
	errorMessage(meldung);
      }
      else {
	QListViewItem *tmpItem;
	QListViewItem *tmpItem2;
	
	writeLatexFileHeader(outDat);	
	
	/* Jetzt den Baum abgrasen und das Zeugs schreiben...                      */
	/* mit firstChild einsteigen, dann mit firstChild/nextSibling weitermachen */
	/* Sichtbarkeit der Kinder mit isOpen() mit prfen                         */ 
	/* Anzahl der Kinder eines Items mit childCount() prfen                   */
	tmpItem = ktlist->firstChild();
	tmpItem = tmpItem->firstChild();
		
	/* ES MSSEN FR LATEX SONDERZEICHEN KONVERTIERT WERDEN!!!!!! */
	
	while (tmpItem != NULL) {
#ifdef KDE2
	  outDat <<  "          \\item[" 
		 << str2Latex(tmpItem->text(0).latin1()) << "]\n";
#else
	  outDat <<  "          \\item[" 
		 << str2Latex(tmpItem->text(0)) << "]\n";
#endif
	  koverDat << tmpItem->text(0) << ": ";
	  
	  if (tmpItem->isOpen()) {
	    // es hat Kinder und ist offen. Die Kinder einfach nur auflisten
	    tmpItem2 = tmpItem->firstChild();
	    while (tmpItem2 != NULL) {
#ifdef KDE2
              outDat << "            " 
		     << str2Latex(tmpItem2->text(0).latin1()) << ";\n";
#else
	      outDat << "            " 
		     << str2Latex(tmpItem2->text(0)) << ";\n";
#endif
	      koverDat << tmpItem2->text(0) << ", ";
	      tmpItem2 = tmpItem2->nextSibling();
	    }
	    koverDat << ">|<";
	  }
	  
	  tmpItem = tmpItem->nextSibling();
	}

	writeLatexFileFooter(outDat);
	close_ofs(outDat);
	
	writeKoverFileFooter(koverDat);
	close_ofs(koverDat);

	startLatexCompile(process, processStarted);
      }
    }
    break;
  case ID_VIEW_SUBIMAGE:
    imageSub->show();
    break;  
  case ID_VIEW_SUBCD:
    burnProc->show();
    break;  
  case ID_VIEW_SIMPLEFM:
    myFileView->show();
    break;  
  case ID_VIEW_KFM: 
    if (execKFM("/")) errorMessage(i18n("Error executing kfm!"));
    break;  
  case ID_VIRTCD_EDIT:
    editwindowClick();
    break;  
  case ID_VIRTCD_ADDDIR:
    machNeuVerz();
    break;  
  default:
    if (!stdCDRWCallback(id_, menuView, ID_VIEW_SUBCD))
      if (!stdCDDocCallback(id_))
	if (!stdHelpCallback(id_))
	  stdOptionsCallback(id_, NOTsaved);    
  }
  if (processStarted) delete process;
}

int Edit::getSessionData(long int      &ms1, 
			 long int      &ms2, 
			 string        &sessPfad)
{
  int      exitCode = 0;
  string   tmp; 
  string   tmpPfad;
  bool     infoFehler;
  
  infoMessage(i18n("Please insert the cd you want to add the session to into your cd-writer\n"
	      "and press OK when you're finished."));
  
  tmp = getDevID(sysOpts.writeDevice);
  tmp = "cdrecord -msinfo dev=" + tmp + " &> ";
  tmpPfad = getNrFile("/tmp/", "kisocd");
  tmp += tmpPfad;

  system(tmp.c_str());

  tmp = dateiInhalt(tmpPfad);
  
  unlink(tmpPfad.c_str());
  infoFehler = (tmp.find("cdrecord: No disk / Wrong disk") != string::npos ||
		tmp.find("cdrecord: No disc / Wrong disc") != string::npos ||
		tmp.find("cdrecord: Cannot read session offset") != string::npos ||
		tmp.find("cdrecord: Cannot read first writable address") != string::npos ||
		tmp.find("cdrecord: ") != string::npos);

  {
    // Die Werte herauslesen
    string tmpStr2;
    string tmpStr3 = tmp;
    
    while (isspace(tmpStr3[tmpStr3.length()-1])) tmpStr3.replace(tmpStr3.length()-1, 1, "");
    while (tmpStr3.find(" ") != string::npos) tmpStr3.replace(0, 1, "");

    tmpStr2 = tmpStr3.substr(0, tmpStr3.find(","));
    ms1 = atoi(tmpStr2.c_str());
    tmpStr3.replace(0, tmpStr3.find(",")+1, "");
    ms2 = atoi(tmpStr3.c_str());
  }

  if (infoFehler)
    {
      // MAYBE there was an error
      tmp += "\n-------------------------------------\n"
	"Decide if it was just a warning of if an error happened. If there are two numbers\n"
	"delimited by a \",\" at the end of the above output\n";
      tmp += "(I guess \"";
      tmp += itoStr(ms1, ' ', 1);
      tmp += ",";
      tmp += itoStr(ms2, ' ', 1);
      tmp += "\"), everything may be alright.";
      
      if (debuglevel & 1 != 0)
	{
	  debugFile.newSection("error on cdrecord -msinfo ?");
	  debugFile.output(tmp);
	}

      if (!QMessageBox::warning(this, i18n("Not everything was alright:"), 
				tmp.c_str(),
				i18n("Continue"), i18n("Cancel"), 
				0, 1, 1)) 
	infoFehler = false;
      else 
	exitCode = -1;
    }


  if (!infoFehler && !exitCode)
    {      
      // die CD mounten und ein Verzeichnis fr die nchste Session suchen
      // dazu muss das Image unmounted werden, was evtl. gerade gemounted ist!
      if (fMounted) 
	{
	  infoMessage(i18n("Please make sure no process uses the currently mounted image and\n"
		      "press OK."));
	  unmountImage(fMounted, sysOpts.mountPoint);
	}
      if (!fMounted)
	{
	  string tmpBef;
	  string tmpPfad;
	
	  exitCode = -1;  
	  tmpBef = "mount ";
	  tmpBef += sysOpts.writeDeviceSFile;
	  tmpBef += " ";
	  tmpBef += sysOpts.mountPoint;
	  tmpBef += " &> "; 
	  tmpPfad = getNrFile("/tmp/", "kisocd");
	  tmpBef += tmpPfad;
	  if (system(tmpBef.c_str()) != 0) 
	    {
	      QString fehler = i18n("Error when mounting! I'll just show you mount's output:\n");
	      
	      fehler += dateiInhalt(tmpPfad).c_str();	 
	      errorMessage(fehler); 
	      exitCode = -1;
	    }
	  else 
	    {
	      fMounted = true;
	      exitCode = 0;
	      sessPfad = getNrPath(sysOpts.mountPoint + "/", "sess");
	      if (fMounted) unmountImage(fMounted, sysOpts.mountPoint);
	    }
	  unlink(tmpPfad.c_str());	      
	}
    }
  
  return(exitCode);
}

void Edit::changedDirCallback(QListViewItem* newItem)
{
#ifdef SLOTLOG
  cerr << "Slot: changedDirCallback\n";
  cerr.flush();
#endif
  
  aktVerz = holPfadInhalt(newItem);
  MeineCD->Zeig(statusBar, DatList, ktlist, aktVerz);
}

#ifndef KDE2
void Edit::DDDirektCB(KDNDDropZone* _dropzone)
{
#ifdef SLOTLOG
  cerr << "Slot: DDDirectCB\n";
  cerr.flush();
#endif
  
  string dropped;
  QString droppedQStr;
  
  droppedQStr = _dropzone->getData();
  KURL::decodeURL(droppedQStr);
  dropped = droppedQStr;
  dropAktion(dropped);
}
#endif

void Edit::dropAktion(string dropped)
{
  if (dropped.find("file:") == string::npos) 
    {
      errorMessage(i18n("You can only drop local files and dirs!\n"));
      if (dropped.find("http:") != string::npos || dropped.find("ftp:") != string::npos)
	downloadFile(dropped);
    }
  else 
    {
      string  rechteSeite;
      
      if (aktVerz == "") rechteSeite = "/CD/"; 
      else rechteSeite = aktVerz; 
      
      // testen, ob mehrere Elemente gedropped werden, die erfahren dann eine 
      // Spezialbehandlung
      if (dropped.find("\nfile:") != string::npos)
	{
	  // mehrere Elemente:
	  int choiceDND;
	  
	  if (sysOpts.fSimpleDND) choiceDND = 0;
	  else 
	    {
	      QPopupMenu  *pm;
	      pm = new QPopupMenu;
	      pm->insertItem(i18n("just copy"), 0, 0); // 0
	      pm->insertItem(i18n("copy to new dir"), 1, 1); // 1
	      choiceDND = pm->exec(QCursor::pos());
	      delete pm;
	    }
	  
	  switch (choiceDND) {
	  case 1:
	    {
	      rechteSeite.replace(0, 3, "");
	      
	      NeuVerzDiag tmpDiag(this, "name", 
				  "~#0123456789_-.abcdefghijklmnopqrstuvwxyz '()[]{}ABCDEFGHIJKLMNOPQRSTUVWXYZ/", 
				  i18n("Please give a directory under which the new files/dirs should appear on the cd, for example /user/pics/ or /files/."), 
				  rechteSeite);
	      rechteSeite = "/CD" + rechteSeite;
	      
	      string ziel;
	      
	      if (tmpDiag.exec() == QDialog::Accepted) 
		{
#ifdef KDE2		
                  ziel = tmpDiag.tmpQLineEdit->text().latin1();
#else		  
		  ziel = tmpDiag.tmpQLineEdit->text();
#endif		  
		  ziel = rmSlashes(ziel);
		  if (ziel == "") errorMessage(i18n("You gave an empty string!\n"));		
		  else 
		    {
		      if (ziel.length() > 0)
			if (ziel[ziel.size()-1] != '/') ziel += '/';		    
		      rechteSeite = "/CD/" + ziel;
#ifdef VERBOSEMESSAGES
		      cerr << "    " << rechteSeite << "\n";
#endif
		    }
		}
	      // neues Dir einlesen:
	      else break;
	      // ansonsten kein break, denn es muss ja noch ausgefhrt werden!
	    }
	  case 0:
	    // kein Dir zu setzen, es wird nach "rechteSeite" kopiert
	    // in einer Schleife alle Elemente einfgen:
	    // wenn unter den Elementen eins dabei ist, was nicht "file:" ist, 
	    // fllt das hier gar nicht auf! also auch keine Benachrichtigung darber...
	    // ist mir aber egal...
	    if (dropped.find("file:") == 0) dropped.replace(0, 5, "");
	    else dropped.replace(0, dropped.find("\nfile:")+6, "");
#ifdef VERBOSEMESSAGES
	    cout << dropped << "\n";
#endif
	    while (dropped.length() > 0) 
	      {
		string teilstring;
		string wohin;
		
		
		if (dropped.find("\n") == string::npos) teilstring = dropped;
		else teilstring = dropped.substr(0, dropped.find("\n"));
		
		wohin = rechteSeite;
		
		if (teilstring[teilstring.size()-1] == '/') 
		  {
		    string teilstring2 = teilstring;
		    
		    teilstring2.replace(teilstring2.size()-1, 1, "");
		    while (teilstring2.find("/") != string::npos)
		      {
			teilstring2.replace(0, 1, "");
		      }
		    wohin += teilstring2 + "/";
		  }
#ifdef VERBOSEMESSAGES
		cout << teilstring << "->" << wohin << "\n";
#endif
		
		addedDrop(teilstring, wohin);
		
		if (dropped.find("\nfile:") == string::npos) dropped = "";
		else dropped.replace(0, dropped.find("\nfile:")+6, "");
	      }
	    
	    
	    break;
	  default:
	    // gar nichts machen
	    break;
	  } 
	}
      else 
	{
	  // es wird nur ein Element gedropped:
	  int choiceDND;
	  
	  dropped.replace(0, 5, "");
	  
	  if (sysOpts.fSimpleDND) 
	    {
	      if (dropped[dropped.size()-1] == '/') choiceDND = 2;
	      else choiceDND = 0;
	    }
	  else 
	    {
	      // Noch nachfragen, wie der Abwurf erfolgen soll:
	      QPopupMenu  *pm;
	      
	      pm = new QPopupMenu;
	      
	      if (dropped[dropped.size()-1] == '/') 
		  pm->insertItem(i18n("only the files"), 0, 0);             // 0
	      else 
		  pm->insertItem(i18n("only the file"), 0, 0);              // 0
	      if (dropped[dropped.size()-1] == '/') 
		  pm->insertItem(i18n("in new directory"), 1, 1);           // 1
	      else 
		  pm->insertItem(i18n("in new directory/rename"), 1, 1);    // 1
	      if (dropped[dropped.size()-1] == '/') 
		  pm->insertItem(i18n("with directory"), 2, 2);             // 2    
	      choiceDND = pm->exec(QCursor::pos());
	      delete pm;
	    }
	  
	  switch (choiceDND) {
	  case 0: 
	    addedDrop(dropped, rechteSeite);
	    break;
	  case 1:
	    {
	      string hilfeText;
	      if (dropped[dropped.size()-1] == '/') 
		hilfeText =  i18n("Please give a directory under which the new file(s) should appear on the cd, for example /user/pics/ or /files/.");	      
	      else 
		hilfeText = i18n("Please give a directory under which the new file should appear on the cd, for example /user/pics/ or /files/. If you don't close your input with a slash, the last item will be the new filename.");
	      
	      
	      rechteSeite.replace(0, 3, "");
	      
	      NeuVerzDiag tmpDiag(this, "name", 
				  "~#0123456789_-.abcdefghijklmnopqrstuvwxyz '()[]{}ABCDEFGHIJKLMNOPQRSTUVWXYZ/", 
				  hilfeText.c_str(), 
				  rechteSeite);
	      
	      rechteSeite = "/CD" + rechteSeite;
	      string ziel;
	      
	      if (tmpDiag.exec() == QDialog::Accepted) 
		{
#ifdef KDE2		
		  ziel = tmpDiag.tmpQLineEdit->text().latin1();
#else
                  ziel = tmpDiag.tmpQLineEdit->text();		  
#endif		  
		  ziel = rmSlashes(ziel);
		  if (ziel == "") errorMessage(i18n("You gave an empty string!\n"));
		  else 
		    {
		      if (ziel.length() > 0)
			if ((ziel[ziel.size()-1] != '/') && 
			    (dropped[dropped.size()-1] == '/')) ziel += '/';		  
		      ziel = "/CD/" + ziel;
#ifdef VERBOSEMESSAGES
		      cerr << "    " << ziel << "\n";
#endif
		      addedDrop(dropped, ziel);
		    }
		}
	    }
	    break;
	  case 2:
	    {
	      string Ziel = dropped;
	      
	      Ziel.replace(Ziel.size()-1, 1, "");
	      while (Ziel.find("/") != string::npos) 
		Ziel.replace(0, 1, "");	      
	      Ziel = rechteSeite+Ziel+"/";
	      addedDrop(dropped, Ziel);
	    }
	    break;
	  
	  }
	}
    }
}

void Edit::addedDrop(string src, string dst)
  /*
    hier kommen z.B. solche Parameter an (/home/jens/download/ aus dem 
    System nach /download/ auf der CD):
    src: /home/jens/download/
    dst: /CD/download/
  */
{
  bool fFirstDrop = true;
#ifdef VERBOSEMESSAGES    
  cout << "void Edit::addedDrop(string src, string dst);\n";
  cout << "  src: " << src << "\n";
  cout << "  dst: " << dst << "\n";
#endif  
  if (ProjectOpen) fFirstDrop = (MeineCD->getVerzNum() == 0);
  
  if (!ProjectOpen) MeineCD = new CDTree;
  if (MeineCD->Append(src, dst, aktOpts.switches[0]) == 0) 
    {
      // zweiter Teil: Zielbaum erzeugen lassen und anzeigen:
      MeineCD->calcSize();
      MeineCD->getCDVerzTree(ktlist, ProjectOpen);
      setzeVoelleGefuehl(MeineCD->getCDSize());
      clearLabels();
      if (ProjectOpen) MeineCD->Zeig(statusBar, DatList, ktlist, aktVerz);
      else DatList->clear();
    
      // wenn dies der erste Drop war, sollte die Auswahl auf "CD" gesetzt werden, 
      // damit man gleich im rechten Fenster sieht, was im Root ist.
      if (fFirstDrop)
	{
	  ktlist->setSelected(ktlist->firstChild(), true);
	  ktlist->setOpen(ktlist->firstChild(), true);
	}
      ProjectOpen = true;
      menuVirtcd->setItemEnabled(ID_VIRTCD_EDIT, true);
      NOTsaved    = true;
    }
}

void Edit::clearLabels()
{
  statusBar->changeItem("", 0);
  DatList->setColumnText(0, i18n("files"));
}

void Edit::setzeVoelleGefuehl(unsigned long WieVoll)
{
  unsigned int tmpVoll = WieVoll;
  
  currentSize = WieVoll;

  // Was dazuaddieren, wenn "bootable cd" gewhlt ist:
  // Soll man die geschtzte Grsse eines Boot-Images dazuaddieren, 
  // wenn bootable gewhlt ist, aber das image nicht existiert?
  // Mache ich mal lieber nicht...
  if (aktOpts.fbootCD)
    {
      struct stat fAttr;
      
      tmpVoll += 2048; // fr boot.catalogue
      if (!stat(aktOpts.bootImagePath.c_str(), &fAttr)) tmpVoll += fAttr.st_size;      
    }
  
  statusBar->changeItem(getSizeText("", tmpVoll), 1);
  
  tmpVoll /= int(1024*1024*6.5); // hier steht 6.5 als Faktor fr eine Standard-650MB-CD!!!!
  
  if (tmpVoll > 100)
    {
      VoelleGefuehl->setBarColor(red);
      tmpVoll = 100;
      VoelleGefuehl->setTextEnabled(false);
    }
  else
    {
      VoelleGefuehl->setBarColor(green);
      VoelleGefuehl->setTextEnabled(true);
    }
  VoelleGefuehl->setValue(tmpVoll);
}

void Edit::editwindowClick()
{
#ifdef SLOTLOG
  cerr << "Slot: editwindowClick\n";
  cerr.flush();
#endif
  
  if (ProjectOpen) 
    {
      CDEditData tmpDiag(this, "einName", MeineCD);
      
      tmpDiag.exec();
      // folgende Ausgabe bewirkte, dass es nach DialogBeendigung kein "segmentation fault" 
      // mehr gab!
#ifdef VERBOSEMESSAGES
      cerr << "Dialog beendet\n";
#endif
      // Baum muss neu angezeigt werden, wenn was verndert wurde:
      if (tmpDiag.fEditiert)
	{
	  MeineCD->calcSize();
	  MeineCD->getCDVerzTree(ktlist, false);
	  setzeVoelleGefuehl(MeineCD->getCDSize());
	  clearLabels();
	  DatList->clear();
	  NOTsaved    = true;
	  // "CD" selektieren, ist einfach freundlicher fr den User
	  if (MeineCD->getVerzNum() != 0) ktlist->setSelected(ktlist->firstChild(), true);
	  // aktVerz muss gesetzt werden, damit das nchste hinzufgen nicht in die Hose
	  // geht:
	  if ((MeineCD->getVerzNum() == 0) || (ktlist->currentItem() == NULL)) {
	    aktVerz = "";
	    if (MeineCD->getVerzNum() == 0) setzeVoelleGefuehl(0);
	  }
	  else aktVerz = holPfadInhalt(ktlist->currentItem());	    
	  ktlist->setOpen(ktlist->firstChild(), true);
	}
    }
}

void Edit::writeMkisofsCommand(ofstream &outDat, 
			       bool     fImageFile,
			       bool     test,
			       string   bootDir,
			       long int ms1,
			       long int ms2)
{
  if (debuglevel & 1 != 0)
    {
      debugFile.newSection("function entered:");
      debugFile.output("void Edit::writeMkisofsCommand(...)"); 
    }


  outDat << "mkisofs ";
  if (test)                outDat << "--print-size -q "; 
  if (aktOpts.switches[0] || needMirrorDir()) outDat << "-f ";
  if (aktOpts.switches[1]) outDat << "-L ";
  if (aktOpts.switches[2]) outDat << "-a ";
  if (aktOpts.switches[3]) outDat << "-l ";
  if (aktOpts.switches[4]) outDat << "-J ";
  if (aktOpts.switches[5]) outDat << "-T ";
  if (aktOpts.switches[6]) outDat << "-v ";
  if (aktOpts.switches[7]) outDat << "-r "; // flags werden gendert
  if (aktOpts.switches[8]) outDat << "-R ";
  
  if (aktOpts.fMulti && !aktOpts.fMultiFirstSess)
    outDat << "-C " << ms1 << "," << ms2 << " -M " << getDevID(sysOpts.writeDevice) << " ";
  
  if (aktOpts.fbootCD)
    {
      // bootDir zurechtschneiden:
      while (bootDir.find("/") != string::npos) bootDir.replace(0, 1, "");
      
      outDat << "-b " << bootDir << "/boot.image ";
      outDat << "-c " << bootDir << "/boot.cat ";
    }
  
  if (aktOpts.IDdisc.length() != 0) 
    outDat << "-A " << backslashifySpecial(aktOpts.IDdisc)      << " ";
  if (aktOpts.IDpublisher.length() != 0) 
    outDat << "-P " << backslashifySpecial(aktOpts.IDpublisher) << " ";
  if (aktOpts.IDpreparer.length() != 0) 
    outDat << "-p " << backslashifySpecial(aktOpts.IDpreparer)  << " ";
  if (aktOpts.IDvolume.length() != 0) 
    outDat << "-V " << backslashifySpecial(aktOpts.IDvolume)    << " ";

  if (fImageFile) outDat << "-o " << backslashifySpecial(sysOpts.imageFilePath) << " ";
  
  string Pfad1, Pfad2;
    
  if (!needMirrorDir())
    {
      // Die Ausschlsse angeben:
      for (int Cnt = 0; Cnt < MeineCD->getExNum(); Cnt++) 
	{
	  MeineCD->getEx(Cnt, Pfad1);      
	  if (Pfad1[Pfad1.size()-1] == '/') 
	    {
	      outDat << "-x ";
	      Pfad1.replace(Pfad1.size()-1, 1, "");
	    }
	  else outDat << "-m ";	  
	  outDat << backslashifySpecial(Pfad1) << " ";
	}
    }

  if (aktOpts.fbootCD && !needMirrorDir())
    {
      MeineCD->getVerz(0, Pfad1, Pfad2);
      Pfad2.replace(0, 3, "");
      outDat << backslashifySpecial(Pfad1) << " ";
    }
  else   
    {
      if (needMirrorDir()) outDat << "$V ";		  	
      else      
	for (int Cnt = 0; Cnt < MeineCD->getVerzNum(); Cnt++) 
	  {
	    MeineCD->getVerz(Cnt, Pfad1, Pfad2);
	    if (Pfad1 != "dummydir")
	      {
		Pfad2.replace(0, 3, "");
		outDat << backslashifySpecial(Pfad2) << "=" 
		       << backslashifySpecial(Pfad1) << " ";
	      }
	  }
    }
}

void Edit::startBrennProzess()
{
  int rueckfrage;
  
  if (debuglevel & 1 != 0)
    {
      debugFile.newSection("function entered:");
      debugFile.output("void Edit::startBrennProzess(...)"); 
    }

  rueckfrage = preBurningBox(sysOpts.fShutdown);	
  
  if (rueckfrage) 
    {
      string pfad = HeimatPfad;
      
      pfad += "/cdScript.sh"; 
      if (createFinalScript(rueckfrage == 1, false) == 0)
	{
	  QString infT;
	  if (aktOpts.fDummyWrite)
	    infT = i18n("This window just shows you the output of the process currently\n"
			"making the simulation write of the cd. You can safely close it.");
	  else
	    infT = i18n("This window just shows you the output of the process currently\n"
			"burning the cd. You can safely close it.");
	  
	  
	  if (!startSubProc(burnProc, infT, ID_VIEW_SUBCD, pfad, 3))
	    {
	      brennGesamtStatus = 0;
	      if (aktOpts.fCreateImage)
		{
		  burnProc->setAutoStatusLine("Track [0123456789]*:[ 0123456789]*of[ 0123456789]*MB written .*",
					      true);
		  progress.cdrecordStarted();
		}
	      else
		{
		  burnProc->setAutoStatusLine("Track [0123456789]*:[ 0123456789]*MB written .*",
					      true);
		  progress.cdrecordStarted(currentSize/(1024*1024));
		}
	      connect(burnProc, SIGNAL(REMatched(QString)),
		      this,     SLOT(updateBurnStatus(QString)));		      	      
	    }
	}
    }
}

void Edit::startImageSim(int bGS)
{
  long int ms1, ms2;
  int      exitCode = 0;
  string   sessPfad = "";  

  if (debuglevel & 1 != 0)
    {
      debugFile.newSection("function entered:");
      debugFile.output("int Edit::startImageSim(...)"); 
    }

  
  if (aktOpts.fMulti && !aktOpts.fMultiFirstSess)
    {
      exitCode = getSessionData(ms1, ms2, sessPfad);
      sessOffset = ms2;
    }
  else sessOffset = 0;
  string pfad = createImageScript(true, ms1, ms2, sessPfad);
  
  if (pfad != "" && exitCode == 0)
    {
      if (!startSubProc(imageSub,
			i18n("This window just shows you the output of the subprocess\n"
			     "simulating the creation of the image. You can safely close it."),
			ID_VIEW_SUBIMAGE, pfad, 2))
	brennGesamtStatus = bGS; 
    } 
  else
    reenableMenuEntries();
}

int Edit::startCreateImage()
{
  long int ms1, ms2;
  int      exitCode = 0;
  string   sessPfad = "";  

  if (debuglevel & 1 != 0)
    {
      debugFile.newSection("function entered:");
      debugFile.output("int Edit::startCreateImage(...)"); 
    }

  
  if (aktOpts.fMulti && !aktOpts.fMultiFirstSess) exitCode = getSessionData(ms1, ms2, sessPfad);
  
  string pfad = createImageScript(false, ms1, ms2, sessPfad);
  
  if (pfad != "" && exitCode == 0)
    {
      if (startSubProc(imageSub, 
		       i18n("This window just shows you the output of a the subprocess\n"
			    "creating the image. You can safely close it."),
		       ID_VIEW_SUBIMAGE, pfad, 1) == 1) 
	exitCode = 1;
      else 
	brennGesamtStatus = 0;
    }
  return(exitCode);
}

void Edit::machNeuVerz()
{
  string  startVerz;
  
  if (aktVerz == "") startVerz = "/"; 
  else 
    {
      startVerz = aktVerz; 
      startVerz.replace(0, 3, "");
    }
  
  NeuVerzDiag tmpDiag(this, "name", 
		      "~#0123456789_-.abcdefghijklmnopqrstuvwxyz '()[]{}ABCDEFGHIJKLMNOPQRSTUVWXYZ/", 
		      i18n("Please give the path of the new directory!"), startVerz);
  string ziel;
  
  if (tmpDiag.exec() == QDialog::Accepted) 
    {
#ifdef KDE2    
      ziel = tmpDiag.tmpQLineEdit->text().latin1();
#else
      ziel = tmpDiag.tmpQLineEdit->text();      
#endif      
      ziel = rmSlashes(ziel);
      if (ziel == "") errorMessage(i18n("We already have a root-directory!\n"));		
      else 
	{
	  if (ziel.length() > 0)
	    if (ziel[ziel.size()-1] != '/') ziel += '/';	  
	  ziel = "/CD/" + ziel;
#ifdef VERBOSEMESSAGES
	  cerr << "    " << ziel << "\n";
#endif
	  addedDrop("dummydir", ziel);
	}
    }
}

int Edit::bootableActions(ofstream &outDat, string &bootDir)
  // bootDir wird zurckgegeben, damit es nach Brennen/Imageerzeugung wieder gelscht
  // werden kann.
{
  // wenn mehrere Dateien/Pfade eingebunden sind oder nur einer, aber nicht
  // als root auf der CD, muss man dies ndern. In diesem Fall nicht null, sondern 1
  // zurckliefern.
  string Pfad1, Pfad2;
  int    retCode = 0;
  

  if (debuglevel & 1 != 0)
    {
      debugFile.newSection("function entered:");
      debugFile.output("int Edit::bootableActions(...)"); 
    }

  MeineCD->getVerz(0, Pfad1, Pfad2);
    
  if (needMirrorDir())
    {
      int nrCnt = 0;
      
      retCode = 1;
      
      do
	{
	  nrCnt++;
	  Pfad1 = "/CD/boot" + itoStr(nrCnt, '0', 3) + "/";
	}
      while (MeineCD->existDestDir(Pfad1) && (nrCnt < 1000));
      
      if (nrCnt == 1000) Pfad1 = "";
      else Pfad1 = "${V}/boot" + itoStr(nrCnt, '0', 3);
      bootDir = Pfad1;
    }
  else bootDir = getNrPath(Pfad1, "boot");
  
  // Befehle im Script, die das Verzeichnis anlegen und das Image reinkopieren.
  outDat << "# Anlegen des Boot-Verzeichnisses:\n";
  outDat << "if mkdir " << bootDir << ";\n";
  outDat << "then\n";
  outDat << "  echo \"KisoCD: created bootDir\";\n";
  outDat << "else\n";
  outDat << "  echo \"KisoCD: error creating bootDir\"\n";
  outDat << "  exit;\n";
  outDat << "fi\n\n";
  
  outDat << "if cp " << aktOpts.bootImagePath << " " << bootDir << "/boot.image;\n";
  outDat << "then\n";
  outDat << "  echo \"KisoCD: copied bootimage\";\n";
  outDat << "else\n";
  outDat << "  rmdir " << bootDir << "\n";
  outDat << "  echo \"KisoCD: error copying bootimage\"\n";
  outDat << "  exit;\n";
  outDat << "fi\n";
  outDat << "# Anlegen des Boot-Verzeichnisses beendet.\n\n";  
  
  return(retCode);
}

string Edit::createImageScript(bool     test,
			       long int ms1,
			       long int ms2, 
			       string   sessPfad)
{
  string bootDir;
  string scriptPfad = HeimatPfad; 
  scriptPfad += "/cdScript.sh";
  
  ofstream outDat;


  if (debuglevel & 1 != 0)
    {
      debugFile.newSection("function entered:");
      debugFile.output("int Edit::createImageScript(...)"); 
    }

  
  outDat.open(scriptPfad.c_str());
  if (!outDat) 
    { 
      errorMessage(i18n("Could not create file!\n")); 
      return("");
    }
  else 
    {
      outDat << "#! /bin/sh\n\n";
      outDat << "# " << ProggieName << " " << ProggieVersion << "\n";
      if (!test) outDat << "rm -f " << backslashifySpecial(sysOpts.imageFilePath) << "\n";
      
      if (needMirrorDir()) 
	{
	  string linkFile;
	  createMirrDir(outDat, sessPfad, sysOpts, linkFile, HeimatPfad);
	  MeineCD->erstellListe(linkFile);
	}
      // Besondere Infos und Massnahmen, wenn bootable gewhlt ist:
      if (aktOpts.fbootCD) bootableActions(outDat, bootDir);
      // image erzeugen
      writeMkisofsCommand(outDat, true, test, bootDir, ms1, ms2);
      outDat << "\n";
      
      if (aktOpts.fbootCD) 
	{
	  // das Boot-Zeug wieder lschen
	  outDat << "rm -f " << bootDir << "/boot.image\n";
	  outDat << "rm -f " << bootDir << "/boot.cat\n";
	  outDat << "rmdir " << bootDir << "\n";
	}
      if (needMirrorDir()) deleteMirrDir(outDat);
      // fertig
      close_ofs(outDat);
      chmod(scriptPfad.c_str(), (S_IXGRP | S_IXOTH | S_IRWXU | S_IRGRP | S_IROTH));
      return(scriptPfad);    
    }
}

int Edit::createFinalScript(bool allowShutdown,
			    bool delImage = false)
{
  string scriptPfad = HeimatPfad;

  if (debuglevel & 1 != 0)
    {
      debugFile.newSection("function entered:");
      debugFile.output("int Edit::createFinalScript(...)"); 
    }

  scriptPfad += "/cdScript.sh";  
  ofstream outDat;
  
  outDat.open(scriptPfad.c_str());
  if (!outDat) 
    { 
      errorMessage(i18n("Could not create file!\n")); 
      return(-1);
    }
  else 
    {
      outDat << "#! /bin/sh\n\n";
      outDat << "# " << ProggieName << " " << ProggieVersion << "\n";
      
      if (aktOpts.fCreateImage) 
	{
	  // das Image sollte schon vorhanden sein, es muss nur noch gebrannt werden
	  writeCdrecordCmd(outDat, aktOpts, sysOpts);

	  if (delImage) outDat << "rm " << backslashifySpecial(sysOpts.imageFilePath) << "\n";
	}
      else 
	{
	  if (needMirrorDir()) 
	    {
	      string linkFile;
	      createMirrDir(outDat, "", sysOpts, linkFile, HeimatPfad);
	      MeineCD->erstellListe(linkFile);
	    }
	  
	  // Besondere Infos und Massnahmen, wenn bootable gewhlt ist:
	  string bootDir;
	  
	  if (aktOpts.fbootCD) bootableActions(outDat, bootDir);
	  // image erzeugen und durch eine pipe zu cdrecord schicken
	  writeMkisofsCommand(outDat, false, false, bootDir, 0, 0);      
	  outDat << "| ";

	  writeCdrecordCmd(outDat, aktOpts, sysOpts);
	  
	  if (aktOpts.fbootCD) 
	    {
	      // das Boot-Zeug wieder lschen
	      outDat << "rm -f " << bootDir << "/boot.image\n";
	      outDat << "rm -f " << bootDir << "/boot.cat\n";
	      outDat << "rmdir " << bootDir << "\n";
	    }
	  if (needMirrorDir()) deleteMirrDir(outDat);	
	}

      if (allowShutdown && sysOpts.fShutdown) outDat << "shutdown -h now\n";      
      close_ofs(outDat);
      chmod(scriptPfad.c_str(), (S_IXGRP | S_IXOTH | S_IRWXU | S_IRGRP | S_IROTH));
    }  
  
  return(0);
}

void Edit::burnCD_guided(int someVar = 0)
{
  if (debuglevel & 1 != 0)
    {
      debugFile.newSection("function entered:");
      debugFile.output("void Edit::burnCD_guided(" 
		       + itoStr(someVar, ' ', 1) + ")");
      debugFile.output("  brennGesamtStatus = " + itoStr(brennGesamtStatus, ' ', 1));
    }

  switch (brennGesamtStatus) {
  case 0:
    // Viele Menpunkte disablen, whrend dieser Gesamtvorgang luft
    disableMenuEntries();
    startImageSim(1);
    break;
  case 1:
    {
      Subprocess *zwischenergebnis;
      
      zwischenergebnis = 
	new Subprocess(i18n("Hit 'Continue' if everything looks alright and you want to\n"
				  "go on burning the CD. Especially check if the contents will\n"
				  "fit on your CD."), 
			     this, "noname", true, (someVar == 0));
      for (unsigned int cnt = 0; cnt < imageSub->getLineCnt(); cnt++)
	zwischenergebnis->addLine(imageSub->getLine(cnt));	  
      zwischenergebnis->showBottom();
      
      imageSub->hide();
      
      if (zwischenergebnis->exec() == 1)
	{
	  // weiter im Text!
	  if (aktOpts.fCreateImage) 
	    // image erstellen:
	    {
	      if (startCreateImage() == 0) brennGesamtStatus = 2;
	      else
		{
		  reenableMenuEntries();
		  brennGesamtStatus = 0;
		  unlink(sysOpts.imageFilePath.c_str());
		}
	    }
	  else
	    // on-the-fly brennen:
	    {
	      startBrennProzess();
	      brennGesamtStatus = 0;
	      reenableMenuEntries();
	    }
	}
      else 
	{
	  reenableMenuEntries();
	  brennGesamtStatus = 0;
	}
      
      delete zwischenergebnis;
    }
    break;
  case 2:
    {
      Subprocess *zwischenergebnis;
      
      zwischenergebnis = 
	new Subprocess(i18n("Hit 'Continue' if everything looks alright and you want to\n"
				  "go on burning the CD."), 
			     this, "noname", true, false);
      for (unsigned int cnt = 0; cnt < imageSub->getLineCnt(); cnt++)
	zwischenergebnis->addLine(imageSub->getLine(cnt));	  
      zwischenergebnis->showBottom();

      imageSub->hide();
      
      if (zwischenergebnis->exec() == 1) startBrennProzess();
      else unlink(sysOpts.imageFilePath.c_str());		  
      brennGesamtStatus = 0;
      
      reenableMenuEntries();
      
      delete zwischenergebnis;	      
    }
    break;
  }
}

void Edit::untersucheMkisofsAusgabe(unsigned long &size)
{  
  string           Ausgabe;

  if (debuglevel & 1 != 0) 
    {
      debugFile.newSection("function entered:");
      debugFile.output("void Edit::untersucheMkisofsAusgabe()");
    }

  size = 0;
  
  Ausgabe = imageSub->getLatestLineWith("Total extents scheduled to be written = ", 0);
  if (Ausgabe.length() != 0)
    {
      Ausgabe.replace(0, 39, "");
      size = atoi(Ausgabe.c_str());
    }

  imageSub->addLine("#########################################################");
  if (size == 0) 
    {
      imageSub->addLine(i18n("Is your filesystem empty or"));
      imageSub->addLine(i18n("am I not able to read mkisofs' output?")); 
    }
  else 
    {
      
      imageSub->addLine(i18n("According to mkisofs, your CD will contain"));
      Ausgabe = getSizeText("", (size+sessOffset)*2048);
      imageSub->addLine(Ausgabe.c_str());
      if (aktOpts.fMulti && !aktOpts.fMultiFirstSess)
	{
	  imageSub->addLine(i18n("The size of the current session is"));
	  Ausgabe = getSizeText("", size*2048);
	  imageSub->addLine(Ausgabe.c_str());
	}
      imageSub->addLine(i18n("Check the difference to what KisoCD told you!"));
    }
  
  imageSub->addLine("#########################################################");
  imageSub->showBottom();

  if (debuglevel & 1 != 0) 
    {
      debugFile.newSection("imagecreation/simulation");
      for (unsigned int cnt = 0; cnt < imageSub->getLineCnt(); cnt++)
	  debugFile.output(imageSub->getLine(cnt));	  
    }
  
  imageSub->show();
  imageSub->setLabel(i18n("Simulation finished.\n"
			  "Please take a look at the last lines..."));
}

void Edit::updateBurnStatus(QString line)
{
  int      timeleft;
  QString  tmp;
  
  timeleft = progress.tellRemaining(line);
  tmp.setNum(timeleft);
  tmp = line + i18n("\nFinished in ") + tmp + i18n(" minutes.");
  burnProc->setStatusLine(tmp);
}

void Edit::prozessEnde(KProcess *proc)
{
  if (debuglevel & 1 != 0) 
    {
      debugFile.newSection("function entered:");
      debugFile.output("void Edit::prozessEnde(KProcess *proc)");
    }
  
  if (proc == burnProc->getProc() || proc == imageSub->getProc())
    {      
      if (proc == burnProc->getProc()) 
	{
	  // es war ein "burnProc"
	  disconnect(burnProc, SIGNAL (finished(KProcess *)), 
		     this, SLOT(prozessEnde(KProcess *)));
	  disconnect(burnProc, SIGNAL(REMatched(QString)),
		     this,     SLOT(updateBurnStatus(QString)));

          if (debuglevel & 1 != 0) 
	    {
	      debugFile.newSection("process which used the burner");
	      for (unsigned int cnt = 0; cnt < burnProc->getLineCnt(); cnt++)
		debugFile.output(burnProc->getLine(cnt));	  
	    }
      
	  // es war ein normaler Brennprozess
	  burnProc->show();
	  burnProc->setLabel(i18n("Subprocess has terminated."));
	  if (burnProc->containsString("cdrecord: No disk / Wrong disk!")
	      || burnProc->containsString("cdrecord: No disc / Wrong disc!"))
	    errorMessage(i18n("There is no disc/a wrong disc!\n"));
	  else 		
	    {
	      if (sysOpts.fProcFinMsg)		
		infoMessage(i18n("Burning of the CD is finished.\n"
				 "Please take a look at the output."));
	    }
	  
	  burnProc->setProcFinished();
	}
      else 
	{
	  unsigned long size = 0;

	  // es war ein "imageSub"
	  
	  disconnect(imageSub, SIGNAL (finished(KProcess *)), 
		     this, SLOT(prozessEnde(KProcess *)));

          if (debuglevel & 1 != 0) 
	    {
	      debugFile.newSection("imagecreation/simulation");
	      for (unsigned int cnt = 0; cnt < imageSub->getLineCnt(); cnt++)
		debugFile.output(imageSub->getLine(cnt));	  
	    }

	  if (imageSub->getID() == 2)
	    untersucheMkisofsAusgabe(size);
	  else
	    {
	      imageSub->show();
	      imageSub->setLabel(i18n("Subprocess has terminated."));
	      if (brennGesamtStatus == 0 && sysOpts.fProcFinMsg) 
		infoMessage(i18n("Finished creation of the imagefile\n"
				 "or its simulation.\n"
				 "Please take a look at the output."));
	      if (fMounted)
		{
		  // das Image remounten:
		  infoMessage(i18n("As your image has now been changed, it makes sense to remount it.\n"
				   "Please make sure no process uses the currently mounted image and\n"
				   "press OK."));
		  
		  unmountImage(fMounted, sysOpts.mountPoint);
		  mountViewImage(fMounted, sysOpts);
		}
	    }
	  
	  /***************************/
	  imageSub->setProcFinished();	

	  if (brennGesamtStatus != 0) burnCD_guided(size);
	}      
    }
#ifdef VERBOSEMESSAGES
  else 
    cout << "void Edit::prozessEnde(KProcess *proc) ohne gltigen Parameter\n";
#endif
}

bool Edit::prozessAktiv()
{
  return(imageSub->isRunning() || burnProc->isRunning());
}

void Edit::setupFileMenu()
{
  bool tmp = false;
  
  if (aktDateiName != "") 
    menuFile->setItemEnabled(ID_FILE_SAVE, NOTsaved);
  else 
    menuFile->setItemEnabled(ID_FILE_SAVE, false);
  if (texFileSavedWatch.hasChanged()) 
    menuFile->setItemEnabled(ID_FILE_SAVE, true);

  menuFile->setItemEnabled(ID_FILE_SAVEAS, true);

  if (ProjectOpen) tmp = (MeineCD->getVerzNum() != 0); 
  menuFile->setItemEnabled(ID_FILE_BURNCD, tmp);

  if (brennGesamtStatus != 0)
    {
      menuFile->setItemEnabled(ID_FILE_NEW, false);
      menuFile->setItemEnabled(ID_FILE_OPEN, false);
      menuFile->setItemEnabled(ID_FILE_BURNCD, false);
    }
  else
    menuFile->setItemEnabled(ID_FILE_NEW, true);
}

void Edit::setupCDDocMenu()
{
  bool tmp = false;
  
  if (ProjectOpen) { tmp = (MeineCD->getVerzNum() != 0); }
  
  menuCdDoc->setItemEnabled(ID_CDDOC_CREATE,     tmp);
}

void Edit::setupSStepMenu()
{
  bool tmp = false;
  
  if (ProjectOpen) { tmp = (MeineCD->getVerzNum() != 0); }
  
  burnSingleStep->setItemEnabled(ID_SSTEP_TESTIMAGE,    tmp);
  burnSingleStep->setItemEnabled(ID_SSTEP_CREATEIMAGE,  (tmp && aktOpts.fCreateImage));
  burnSingleStep->setItemEnabled(ID_SSTEP_CREATESCRIPT, tmp);
  burnSingleStep->setItemEnabled(ID_SSTEP_BURNIT,       tmp);

  if (aktOpts.fCreateImage)
    {
      tmp = existDat(sysOpts.imageFilePath);
      if (imageSub->isRunning()) tmp = false;

      burnSingleStep->setItemEnabled(ID_SSTEP_CREATESCRIPT,   tmp);	  
      burnSingleStep->setItemEnabled(ID_SSTEP_BURNIT,         tmp);	  
      burnSingleStep->setItemEnabled(ID_SSTEP_DELIMAGE,       tmp);

      if (aktOpts.fMulti && !aktOpts.fMultiFirstSess) tmp = false;
      burnSingleStep->setItemEnabled(ID_SSTEP_MOUNTVIEWIMAGE, tmp);
    }
  else 
    {
      burnSingleStep->setItemEnabled(ID_SSTEP_MOUNTVIEWIMAGE, false);
      burnSingleStep->setItemEnabled(ID_SSTEP_DELIMAGE,       false);
    }

  if (burnProc->isRunning()) burnSingleStep->setItemEnabled(ID_SSTEP_BURNIT,       false);
  if (burnProc->isRunning() && aktOpts.fMulti)
    {
      burnSingleStep->setItemEnabled(ID_SSTEP_TESTIMAGE,    false);
      burnSingleStep->setItemEnabled(ID_SSTEP_CREATEIMAGE,  false);
    }

  if (brennGesamtStatus != 0)
    {
      burnSingleStep->setItemEnabled(ID_SSTEP_TESTIMAGE, false);
      burnSingleStep->setItemEnabled(ID_SSTEP_CREATEIMAGE, false);
      burnSingleStep->setItemEnabled(ID_SSTEP_CREATESCRIPT, false);
      burnSingleStep->setItemEnabled(ID_SSTEP_BURNIT, false);
      burnSingleStep->setItemEnabled(ID_SSTEP_DELIMAGE, false);
    }
}

bool Edit::needMirrorDir()
{
  bool    tmp = false;
  string  Pfad1, Pfad2;
  
  MeineCD->getVerz(0, Pfad1, Pfad2);
  if (aktOpts.fbootCD && 
      ((MeineCD->getVerzNum() != 1) || ((MeineCD->getVerzNum() == 1) && (Pfad2 != "/CD/"))))
    tmp = true;
    
  // folgendes muss noch verfeinert werden: nur wenn die Verzeichnisstruktur nicht 
  // das gewnscht Aussehen hat, muss die Sache angewandt werden....
  if (aktOpts.fMulti && !aktOpts.fMultiFirstSess) tmp = true;
  
  return(tmp);
}

void Edit::disableMenuEntries()
{
  menuOptions->setItemEnabled(ID_OPTIONS_CD, false);
  menuOptions->setItemEnabled(ID_OPTIONS_SYSTEM, false);

  menuCDRW->setItemEnabled(ID_CDRW_ERASEALL, false);
  menuCDRW->setItemEnabled(ID_CDRW_ERASEFAST, false);
  menuCDRW->setItemEnabled(ID_CDRW_ERASELASTSESS, false);

  menuVirtcd->setItemEnabled(ID_VIRTCD_EDIT, false);
  menuVirtcd->setItemEnabled(ID_VIRTCD_ADDDIR, false);
}

void Edit::reenableMenuEntries()
{
  menuFile->setItemEnabled(ID_FILE_OPEN, true);

  menuOptions->setItemEnabled(ID_OPTIONS_CD, true);
  menuOptions->setItemEnabled(ID_OPTIONS_SYSTEM, true);

  menuCDRW->setItemEnabled(ID_CDRW_ERASEALL, true);
  menuCDRW->setItemEnabled(ID_CDRW_ERASEFAST, true);
  menuCDRW->setItemEnabled(ID_CDRW_ERASELASTSESS, true);

  menuVirtcd->setItemEnabled(ID_VIRTCD_EDIT, true);
  menuVirtcd->setItemEnabled(ID_VIRTCD_ADDDIR, true);
}

int Edit::startSubProc(Subprocess*   &subProc, 
		       QString       shortInfoText, 
		       int           menuID, 
		       string        pfad, 
		       int           ID)
// Belegung von ID:
//      bei imageSub
//         1 bei image-erzeugung
//         2 bei Simulation
//      bei burnProc
//         3 bei brennen
{
  int exitcode;

  if (debuglevel & 1 != 0) 
    {
      debugFile.newSection("function entered:");
      debugFile.output( "int Edit::startSubProc(Subprocess*   &subProc, "
		        "QString shortInfoText, int menuID, string pfad, ");
      debugFile.output(("                       int ID = " + itoStr(ID, ' ', 1)) + ")");
    }
  subProc->newProc(shortInfoText, pfad);  

  connect(subProc, SIGNAL (finished(KProcess *)), 
	  this, SLOT(prozessEnde(KProcess *)));

  exitcode = subProc->startProc(ID, pfad, sysOpts.fProcWinOnStart);
  if (!exitcode)
    menuView->setItemEnabled(menuID, true);

  return(exitcode);
}

#include "edit_ext.cpp"

#include "edit.moc"













