// -*- c++ -*-

#ifndef EDIT_H
#define EDIT_H

/*** KisoCD ***/
#include "switches.h"
#include "tools.h"
#include "neuverzdiag.h"
#include "cdeditdata.h"
#include "kcdmodule.h"
#include "cdtree.h"
#include "types.h"
#include "subprocess.h"
#include "simplekfvdialog.h"
#include "version.h"
#include "edit2.h"

/*** Qt ***/
#include <qlistview.h>
#include <qtooltip.h>

/*** KDE ***/
#include <kprocess.h>
#include <kmenubar.h>
#ifdef KDE2
#include <qsplitter.h>
#include "dndlist.h"
#else
#include <knewpanner.h>
#include <drag.h>
#endif
#include <kprogress.h>

/*** C++ ***/
#include <fstream.h>
#include <iostream.h>
#include <string>



class Edit : public KCDModule
{
  Q_OBJECT
    
 public: 
  Edit(QWidget *parent = 0, const char *name = 0, int aModuleID = 0);
  ~Edit();
  
 public slots:
  void commandCallback(int id_);
  void changedDirCallback(QListViewItem *); 
#ifndef KDE2  
  void DDDirektCB(KDNDDropZone* _dropzone);
#endif
  void editwindowClick();
  void dropAktion(string);
  void setupFileMenu();
  void setupSStepMenu();
  void setupCDDocMenu();
  void prozessEnde(KProcess *proc);
  void machNeuVerz();
  void updateBurnStatus(QString line);
 private:
  unsigned int       currentSize;
  long int           sessOffset;
  Subprocess*        imageSub;
  RemainingTimeTeller progress;
  bool               NOTsaved;
  bool               ProjectOpen;
  string             aktDateiName;  // Projektdatei!
  string             aktVerz;       // das im rechten Fenster angezeigte
  FileWatch          texFileSavedWatch;
  bool               fMounted;      // ist das aktuelle Image gemountet? 
  int                brennGesamtStatus;

  SimpleKFVDialog*   myFileView;

  // Child widgets

  QPopupMenu         *menuView;
  QPopupMenu         *menuVirtcd;
  QPopupMenu         *burnSingleStep;
  
  KStatusBar         *statusBar;
  QListView          *ktlist;
  QListView          *DatList;
#ifdef KDE2  
//  QSplitter*         Panner;
#else
  KDNDDropZone       *DDDirektaufnahme;
  KNewPanner         *Panner;
#endif
  KProgress          *VoelleGefuehl;
  /********************************/
  int checkPrerequisites();
  virtual bool preExit();
  /********************************/
  void untersucheMkisofsAusgabe(unsigned long &size);
  int startSubProc(Subprocess*   &subProc, 
		   QString       shortInfoText, 		   
		   int           menuID, 
		   string        pfad, 
		   int           ID);
  int createStatusBar();
  int createMenu();
  int createMainView();
  int startCreateImage();
  void startImageSim(int);
  void startBrennProzess();
  void burnCD_guided(int someVar = 0);
  int saveToFile(const char* filename);
  int loadFromFile(const char* filename);
  void addedDrop(string src, 
		 string dst);
  void setzeVoelleGefuehl(unsigned long WieVoll);
  void writeMkisofsCommand(ofstream &outDat, 
			   bool     fImageFile, 
			   bool     test, 
			   string   bootDir, 
			   long int ms1, 
			   long int ms2);
  int createFinalScript(bool allowShutdown,
			bool delImage = false);
  string createImageScript(bool     test, 
			   long int ms1, 
			   long int ms2,
			   string   sessPfad);
  int getSessionData(long int &ms1, 
		     long int &ms2, 
		     string &sessPfad);
  int bootableActions(ofstream &outDat, 
		      string &bootDir);
  bool prozessAktiv();
  bool needMirrorDir();
  void clearLabels();
  
  void disableMenuEntries();
  void reenableMenuEntries();
public:
  CDTree           *MeineCD;
};

#endif // EDIT_H





















