#include "switches.h"
#include "kcdmodule.h"

/*** KDE ***/
#include <klocale.h>
#include <kapp.h>

/*** Qt ***/
#include <qmessagebox.h>

/*** C/C++ ***/

/*** KisoCD ***/
#include "version.h"
#include "opt_sys.h"
#include "opt_cd.h"
#include "opt_cddoc.h"
#include "opt_ui.h"
#include "quickstartdialog.h"

#include "menuid.std"
#include "module.id"

extern int                moduleExitCode;
extern int                debuglevel;
extern DebugFile          debugFile; 
extern KSimpleConfig*     globalConf;
extern string             HeimatPfad;
extern T_Options          aktOpts;
extern T_Options          defOpts;
extern T_SysOptions       sysOpts;
extern QuickstartDialog*  myQuickHelp;


KCDModule::KCDModule(QWidget *parent, const char *name, int aModuleID)
	: KTMainWindow(name)
{
  moduleID = aModuleID;
  burnProc = new Subprocess("-");
  texFileWatch.flag = false;
}

KCDModule::~KCDModule()
{
#ifdef VERBOSEMESSAGES
  cout << "KCDModule::~KCDModule() called\n";
#endif
  delete burnProc;
}

void KCDModule::getStdCDRWMenu(QPopupMenu* &menu)
{
  /* ***************************************** */
  menu = new QPopupMenu;

  if (globalConf->readEntry("mCDRW_eraseAll", "enabled") == "enabled") 
    menu->insertItem(i18n("erase all"),          ID_CDRW_ERASEALL);
  if (globalConf->readEntry("mCDRW_eraseFast", "enabled") == "enabled") 
    menu->insertItem(i18n("fast erase"),         ID_CDRW_ERASEFAST);
  if (globalConf->readEntry("mCDRW_eraseLastSession", "enabled") == "enabled") 
    menu->insertItem(i18n("erase last session"), ID_CDRW_ERASELASTSESS);
}

void KCDModule::getStdCDDocMenu(QPopupMenu* &menu)
{
  /* ***************************************** */  
  menu = new QPopupMenu;

  if (globalConf->readEntry("mCDDoc_edit", "enabled") == "enabled") 
    menu->insertItem(i18n("Edit"),    ID_CDDOC_EDIT);
  if (globalConf->readEntry("mCDDoc_compile", "enabled") == "enabled") 
    menu->insertItem(i18n("Compile"), ID_CDDOC_COMPILE);
  if (globalConf->readEntry("mCDDoc_view", "enabled") == "enabled") 
    menu->insertItem(i18n("View"),    ID_CDDOC_VIEW);
  if (globalConf->readEntry("mCDDoc_print", "enabled") == "enabled") 
    menu->insertItem(i18n("Print"),   ID_CDDOC_PRINT);
  menu->insertSeparator();
  if (globalConf->readEntry("mCDDoc_create", "enabled") == "enabled") 
    menu->insertItem(i18n("Create from current view"), ID_CDDOC_CREATE);
  menu->setItemEnabled(ID_CDDOC_CREATE, FALSE);
}

void KCDModule::getStdOptionsMenu(QPopupMenu* &menu)
{
  /* ***************************************** */
  menu = new QPopupMenu;

  if (globalConf->readEntry("mOptions_system", "enabled") == "enabled") 
    menu->insertItem(i18n("system"),         ID_OPTIONS_SYSTEM);
  if (globalConf->readEntry("mOptions_cd", "enabled") == "enabled") 
    menu->insertItem(i18n("cd"),             ID_OPTIONS_CD);
  if (globalConf->readEntry("mOptions_cdDoc", "enabled") == "enabled") 
    menu->insertItem(i18n("cdDoc"),          ID_OPTIONS_CDDOC);
  if (globalConf->readEntry("mOptions_ui", "enabled") == "enabled") 
    menu->insertItem(i18n("User Interface"), ID_OPTIONS_UI);
}

void KCDModule::getStdHelpMenu(QPopupMenu* &menu)
{
  /* ***************************************** */  
  menu = new QPopupMenu;

  menu->insertItem(i18n("Help"),            ID_HELP_HELP);
  menu->insertItem(i18n("quick start"),     ID_HELP_QUICKSTART);
  menu->insertSeparator();
  menu->insertItem(i18n(AboutProggie),      ID_HELP_ABOUT);  
}

bool KCDModule::stdCDRWCallback(int         id,
				QPopupMenu* aMenu,
				int         aMenuID)
{
  bool retCode = true;
  
  switch (id)
    {
    case ID_CDRW_ERASEALL:
      debugFile.newFile();
      blankRW(id, true, aMenu, aMenuID);
      break;
    case ID_CDRW_ERASEFAST:
      debugFile.newFile();
      blankRW(id, true, aMenu, aMenuID);
      break;
    case ID_CDRW_ERASELASTSESS:
      debugFile.newFile();
      blankRW(id, true, aMenu, aMenuID);
      break;  
    default: retCode = false;
    }
  return(retCode);
}

void KCDModule::startLatexCompile(KShellProcess* &process, 
				  bool           &processStarted, 
				  bool           wait = false)
{
  ofstream compDat;                 
  string Befehl = HeimatPfad + "/compileScript.sh";    

  compDat.open(Befehl.c_str());
  compDat << "#! /bin/sh\n\n" << "cd " << HeimatPfad.c_str() << "\n" 
	  << sysOpts.latexCmd << " cdDoc.tex\necho \"\npress <return> to continue\"\nread\n";
  close_ofs(compDat);
  chmod(Befehl.c_str(), (S_IXGRP | S_IXOTH | S_IRWXU | S_IRGRP | S_IROTH));    
  process = new KShellProcess;
  (*process) << "xterm" << "-T" << "\"KisoCD: latex output\"" << "-e" << Befehl.c_str();
  if (wait) process->start(KShellProcess::Block);      
  else process->start(KShellProcess::DontCare);      
  processStarted = true;
  texFileWatch.lookNow(HeimatPfad + "/cdDoc.tex");
  texFileWatch.flag = true;
}

bool KCDModule::stdCDDocCallback(int id)
{
  bool            retCode = true;
  KShellProcess*  process = NULL;
  bool            processStarted = false;
 
  switch (id)
    {
    case ID_CDDOC_EDIT: 
      {
	string Befehl = HeimatPfad + "/cdDoc.tex";
	
	process = new KShellProcess;
	(*process) << sysOpts.editorCmd.c_str() << Befehl.c_str();
	process->start(KShellProcess::DontCare);
	processStarted = true;    
      } 
      break;  
    case ID_CDDOC_COMPILE: 
      startLatexCompile(process, processStarted);
      break;  
    case ID_CDDOC_VIEW: 
      {
	if (texFileWatch.flag == false || texFileWatch.hasChanged())
	  startLatexCompile(process, processStarted, true);
	
	string Befehl = HeimatPfad + "/cdDoc.dvi";
	
	process = new KShellProcess;
	(*process) << sysOpts.viewCmd.c_str() << Befehl.c_str();
	process->start(KShellProcess::DontCare);
	processStarted = true;        
      } 
      break;  
    case ID_CDDOC_PRINT: 
      {
	if (texFileWatch.flag == false || texFileWatch.hasChanged())
	  startLatexCompile(process, processStarted, true);
	
	string Befehl = "cdDoc.dvi";   
	chdir(HeimatPfad.c_str());
	
	process = new KShellProcess;
	(*process) << sysOpts.printCmd.c_str() << Befehl.c_str();
	process->start(KShellProcess::DontCare);
	processStarted = true;        
      } 
      break;      
    default: retCode = false;
    }
  if (processStarted) delete process;
  return(retCode);
}

bool KCDModule::stdOptionsCallback(int  id,
				   bool &optsChanged)
{
  bool retCode = true;
  
  switch (id)
    {
    case ID_OPTIONS_SYSTEM: 
      {
	Opt_sys tmpDiag(this, "name", globalConf, &sysOpts, !burnProc->isRunning());
	tmpDiag.exec();
      } 
      break;
    case ID_OPTIONS_CDDOC:
      {
	Opt_cddoc tmpDiag(this, "name", globalConf, &sysOpts);
	
	tmpDiag.exec();
      } 
      break;
    case ID_OPTIONS_CD: 
      {
	Opt_cd tmpDiag(this, "name", globalConf, &defOpts, &aktOpts, 
		       &optsChanged, &moduleExitCode);
	
	tmpDiag.exec();
      }
      if (moduleExitCode != moduleID) 
	if (preExit()) delete(this);
      break;  
    case ID_OPTIONS_UI: 
      {
	Opt_ui tmpDiag(this, "name", globalConf, &sysOpts);
	
	tmpDiag.exec();
      }
      break;
    default: retCode = false;
    }
  return(retCode);
}

bool KCDModule::stdHelpCallback(int id)
{
  bool retCode = true;
  
  switch (id)
    {
    case ID_HELP_HELP: 
      showHelp();
      break;
    case ID_HELP_QUICKSTART: 
      showQuickstart();
      break;
    case ID_HELP_ABOUT:
      showAboutBox();
      break;      
    default: retCode = false;
    }
  return(retCode);
}

void KCDModule::showAboutBox()
{
  string textStr;
  textStr = ProggieName;
  textStr += " v";
  textStr += ProggieVersion;
  textStr += "\n";
  textStr += AuthorDate;
  textStr += i18n("\n\n Please report bugs, tips, .... to the above address.");
  QMessageBox::about(0, AboutProggie, textStr.c_str());
}

void KCDModule::showQuickstart()
{
  myQuickHelp->gotoFirstChapter();
  myQuickHelp->show(); 
}

void KCDModule::showHelp()
{
#ifdef KDE2
  kapp->invokeHTMLHelp("kisocdII/kisocdII.html", "KisoCD"); 
#else
  kapp->invokeHTMLHelp("kisocd/kisocd.html", "KisoCD"); 
#endif
}

int KCDModule::blankRW(int         id, 
		       bool        force,
		       QPopupMenu* aMenu,
		       int         aMenuID)
{
  int exitcode = 1;
  
  if (burnProc->isRunning())
    errorMessage(i18n("process already running!"));
  else 
    {	    
      string    pfad = HeimatPfad;
      ofstream  outDat;
      
      pfad += "/cdScript.sh"; 
      outDat.open(pfad.c_str());
      if (!outDat) errorMessage(i18n("Unable to create scriptfile!"));
      else 
	{
	  int anID = id;
	  
	  outDat << "#! /bin/sh\n\n";
	  outDat << "# " << ProggieName << " " << ProggieVersion << "\n";
	  string device = getDevID(sysOpts.writeDevice);
	  
	  outDat << "cdrecord -v ";
	  if (force) outDat << "-force ";
	  outDat << "speed=" << sysOpts.nBlankingSpeed << " ";
	  outDat << "blank=";
	  switch (id)
	    {
	    case ID_CDRW_ERASEALL: 
	      outDat << "all";
	      break;
	    case ID_CDRW_ERASEFAST: 
	      outDat << "fast";
	      break;
	    case ID_CDRW_ERASELASTSESS: 
	      outDat << "session";
	      break;
	    }
	  
	  outDat << " dev=" << device 
		 << "\n";
	  	  
	  close_ofs(outDat);
	  chmod(pfad.c_str(), (S_IXGRP | S_IXOTH | S_IRWXU | S_IRGRP | S_IROTH));

	  if (debuglevel & 1 != 0) debugFile.newFile();		  

	  burnProc->newProc(i18n("This window just shows you the output of the process "
				 "currently blanking\n"
				 "your CD. You can safely close it."), pfad);  
	  connect(burnProc, SIGNAL (finished(KProcess *)), 
		  this, SLOT(blankFinished(KProcess *)));
	  if (!burnProc->startProc(anID, pfad, sysOpts.fProcWinOnStart) && aMenuID != 0)
	    {
	      aMenu->setItemEnabled(aMenuID, true);	  
	      exitcode = 0;
	    }
	}
    }  
  return(exitcode);
}

bool KCDModule::preExit()
{
  return(true);
}

void KCDModule::closeEvent(QCloseEvent* e)
{
  e->ignore();
  moduleExitCode = ID_MODULE_EXIT;
  if (preExit()) delete(this);
}

void KCDModule::blankFinished(KProcess* proc)
{
#ifdef VERBOSEMESSAGES
  cout << "KCDModule::blankFinished(KProcess* proc) called...\n";
#endif

  if (proc == burnProc->getProc())
    {
      int action = 0;
      
      disconnect(burnProc, SIGNAL (finished(KProcess *)), 
		 this, SLOT(blankFinished(KProcess *)));
      
      if (debuglevel & 1 != 0) 
	{
	  debugFile.newSection("process which used the burner");
	  for (unsigned int cnt = 0; cnt < burnProc->getLineCnt(); cnt++)
	    debugFile.output(burnProc->getLine(cnt));	  
	}
      
      if (burnProc->containsString("cdrecord: Bad Option: -force.")) 
	action = burnProc->getID();
      else 
	{
	  burnProc->show();
	  burnProc->setLabel(i18n("Subprocess has terminated."));
	  if (burnProc->containsString("cdrecord: No disk / Wrong disk!")
	      || burnProc->containsString("cdrecord: No disc / Wrong disc!"))
	    errorMessage(i18n("There is no disc/a wrong disc!\n"));
	  else 
	    {
	      if (sysOpts.fProcFinMsg)
		infoMessage(i18n("Blanking the CDRW is finished.\n"
				 "Please take a look at the output."));
	    }
	}
      
      burnProc->setProcFinished();
      
      if (action)
	blankRW(burnProc->getID(), false, 0, 0);        
    }
#ifdef VERBOSEMESSAGES
  else
    cout << "void KCDModule::blankFinished(KProcess* proc) ohne gltigen Parameter!\n";
#endif
}

#include "kcdmodule.moc"

















