#include "options.h"
#ifdef KDE2
#include <klocale.h>
#else 
#include <kapp.h>
#endif

extern int                debuglevel;
extern DebugFile          debugFile; 


void readStringEntry(KSimpleConfig*   confDat,
		     const char*      name,
		     string           &value)
{
  if (confDat->readEntry(name, "noValue") != "noValue")
    value = confDat->readEntry(name);
}

void readNumEntry   (KSimpleConfig*   confDat,
	             const char*      name,
	             int              &value)
{
  if (confDat->readNumEntry(name, 65000) != 65000)
    value = confDat->readNumEntry(name);  
}

void readBoolEntry  (KSimpleConfig*   confDat,
		     const char*      name,
		     bool             &value)
{
  if (confDat->readEntry(name, "noValue") != "noValue")
    value = confDat->readBoolEntry(name);
}


void saveOptions(struct T_Options      &defOpts, 
		 struct T_SysOptions   &sysOpts,
		 string                HeimatPfad)
{
  KSimpleConfig  *configDat;
  string         defPfad;
  
  defPfad = HeimatPfad;
  defPfad += "/.config";
  
  configDat = new KSimpleConfig(defPfad.c_str());
  configDat->setGroup("kisocd");
  configDat->writeEntry("version", ProggieVersion);
  
  configDat->setGroup("cdDoc");
  configDat->writeEntry("latexCmd", sysOpts.latexCmd.c_str());
  configDat->writeEntry("viewCmd", sysOpts.viewCmd.c_str());
  configDat->writeEntry("printCmd", sysOpts.printCmd.c_str());
  configDat->writeEntry("editorCmd", sysOpts.editorCmd.c_str());      
  configDat->writeEntry("fAutoCompile", sysOpts.fAutoCompile);
  
  configDat->setGroup("system");
  configDat->writeEntry("mountPoint", sysOpts.mountPoint.c_str());      
  configDat->writeEntry("imageFilePath", sysOpts.imageFilePath.c_str());
  configDat->writeEntry("tmpDir", sysOpts.tmpDir.c_str());
  configDat->writeEntry("readDevice", sysOpts.readDevice.c_str());
  configDat->writeEntry("writeDevice", sysOpts.writeDevice.c_str());
  configDat->writeEntry("writeDeviceSFile", sysOpts.writeDeviceSFile.c_str());
  configDat->writeEntry("nFifoSize", sysOpts.nFifoSize);
  configDat->writeEntry("nWritingSpeedImg", sysOpts.nWritingSpeedImg);
  configDat->writeEntry("nWritingSpeedOTF", sysOpts.nWritingSpeedOTF);     
  configDat->writeEntry("nBlankingSpeed", sysOpts.nBlankingSpeed);     
  configDat->writeEntry("availSpeed", sysOpts.availSpeed.c_str());     
  
  configDat->writeEntry("fShutdown", sysOpts.fShutdown);
  configDat->writeEntry("fEject", sysOpts.fEject);    

  configDat->setGroup("UserInterface");
  configDat->writeEntry("fSimpleDND",      sysOpts.fSimpleDND);      
  configDat->writeEntry("fOptOnStartup",   sysOpts.fOptOnStartup);      
  configDat->writeEntry("fProcFinMsg",     sysOpts.fProcFinMsg);      
  configDat->writeEntry("fProcWinOnStart", sysOpts.fProcWinOnStart);      
  
  configDat->setGroup("cdDefault");
  configDat->writeEntry("IDdisc", defOpts.IDdisc.c_str());
  configDat->writeEntry("IDvolume", defOpts.IDvolume.c_str());
  configDat->writeEntry("IDpreparer", defOpts.IDpreparer.c_str());
  configDat->writeEntry("IDpublisher", defOpts.IDpublisher.c_str());
  configDat->writeEntry("bootImagePath", defOpts.bootImagePath.c_str());    
  configDat->writeEntry("fFollowSymLinks", defOpts.switches[0]);
  configDat->writeEntry("fAllowLeadingDots", defOpts.switches[1]);
  configDat->writeEntry("fAllFiles", defOpts.switches[2]);
  configDat->writeEntry("fNames32Char", defOpts.switches[3]);
  configDat->writeEntry("fJoliet", defOpts.switches[4]);
  configDat->writeEntry("fTRANSTBL", defOpts.switches[5]);
  configDat->writeEntry("fMkisofsVerbose", defOpts.switches[6]);
  configDat->writeEntry("fRRchmod", defOpts.switches[7]);
  configDat->writeEntry("fRR", defOpts.switches[8]);
  configDat->writeEntry("fMulti", defOpts.fMulti);
  configDat->writeEntry("fbootCD", defOpts.fbootCD);
  configDat->writeEntry("fCopyDataCD", defOpts.fCopyDataCD);
  configDat->writeEntry("fCreateImage", defOpts.fCreateImage);
  configDat->writeEntry("fDummyWrite", defOpts.fDummyWrite);
  configDat->writeEntry("fPadTrack", defOpts.fPadTrack);
  configDat->writeEntry("fMultiFirstSess", defOpts.fMultiFirstSess);
  configDat->writeEntry("fMultiLastSess", defOpts.fMultiLastSess);
  configDat->sync();
  delete configDat;
}

void setDefaultConfig(struct T_Options      &aktOpts, 
		      struct T_SysOptions   &sysOpts,
		      string                HeimatPfad)
{
  for (int Cnt = 0; Cnt<9; Cnt++) 
    aktOpts.switches[Cnt] = false;
  
  // Ausgeklammert, weil cdrecord ab 1.8.1 es als Fehler 
  // interpretiert:
  //aktOpts.switches[2] = true; 
  
  aktOpts.IDdisc      = "Disc-ID";
  aktOpts.IDvolume    = "Volume-ID";
  aktOpts.IDpreparer  = "Preparer-ID";
  aktOpts.IDpublisher = "Publisher-ID";
  aktOpts.fCreateImage   = true;
  aktOpts.fDummyWrite    = false;
  aktOpts.fPadTrack      = true;
  aktOpts.fbootCD        = false;
  aktOpts.bootImagePath  = "/home/none.set";
  aktOpts.fMulti         = false;
  aktOpts.fMultiFirstSess= true;
  aktOpts.fMultiLastSess = false;
  aktOpts.fCopyDataCD    = false;
  
  sysOpts.latexCmd  = "latex";
  sysOpts.viewCmd   = "xdvi";
  sysOpts.printCmd  = "dvips";
  sysOpts.editorCmd = "kedit";
  sysOpts.fAutoCompile = true;
  sysOpts.writeDevice  = "0,4,0 (Plextor CD-ROM)";
  sysOpts.writeDeviceSFile   = "/dev/scd0";
  sysOpts.readDevice         = "/dev/cdrom";
  sysOpts.mountPoint   = "/mnt";
  sysOpts.imageFilePath  = "/tmp/stupid_place.raw";
  sysOpts.tmpDir         = "/tmp/";
  sysOpts.nFifoSize      = 4;
  sysOpts.nWritingSpeedImg  = 2;
  sysOpts.nWritingSpeedOTF  = 2;
  sysOpts.nBlankingSpeed    = 2;
  sysOpts.availSpeed = "1,2,4,6,8,10,12,16,";
  
  sysOpts.fEject    = true;
  sysOpts.fShutdown = false;

  sysOpts.fSimpleDND      = false;      
  sysOpts.fOptOnStartup   = true;      
  sysOpts.fProcFinMsg     = true;
  sysOpts.fProcWinOnStart = true;      
}

int ladeVorgaben(KSimpleConfig*        globalConf,
		 string                HeimatPfad,
		 struct T_Options      &aktOpts, 
		 struct T_SysOptions   &sysOpts)
  // liefert true zurck, wenn es die Datei nicht gab oder
  // wenn sie zu einer alten Version gehrte.
{
  KSimpleConfig    *configDat;
  string           defPfad;
  int              ersterStart = 0;
  
  defPfad = HeimatPfad;
  defPfad += "/.config";
  
  configDat = new KSimpleConfig(defPfad.c_str(), true);
  configDat->setGroup("kisocd");
  
  // nur zur Sicherheit:
  setDefaultConfig(aktOpts, sysOpts, HeimatPfad);

  if (!existDat(defPfad) || configDat->readEntry("version", "none") == "none")
    {
      ersterStart = 1;
    }
  else
    {  
      configDat->setGroup("cdDoc");
      readStringEntry(configDat, "latexCmd",     sysOpts.latexCmd);
      readStringEntry(configDat, "viewCmd",      sysOpts.viewCmd);
      readStringEntry(configDat, "printCmd",     sysOpts.printCmd);
      readStringEntry(configDat, "editorCmd",    sysOpts.editorCmd);
      readBoolEntry  (configDat, "fAutoCompile", sysOpts.fAutoCompile);
      
      configDat->setGroup("system");
      readStringEntry(configDat, "mountPoint",          sysOpts.mountPoint);
      readStringEntry(configDat, "imageFilePath",       sysOpts.imageFilePath);
      readStringEntry(configDat, "tmpDir",              sysOpts.tmpDir);
      readStringEntry(configDat, "writeDevice",         sysOpts.writeDevice);
      readStringEntry(configDat, "writeDeviceSFile",    sysOpts.writeDeviceSFile);
      readStringEntry(configDat, "readDevice",          sysOpts.readDevice);
      readNumEntry   (configDat, "nFifoSize",           sysOpts.nFifoSize);
      readNumEntry   (configDat, "nWritingSpeedImg",    sysOpts.nWritingSpeedImg);
      readNumEntry   (configDat, "nWritingSpeedOTF",    sysOpts.nWritingSpeedOTF);
      readNumEntry   (configDat, "nBlankingSpeed",      sysOpts.nBlankingSpeed);
      readStringEntry(configDat, "availSpeed",          sysOpts.availSpeed);      
      readBoolEntry  (configDat, "fEject",              sysOpts.fEject);
      readBoolEntry  (configDat, "fShutdown",           sysOpts.fShutdown);
      
      configDat->setGroup("cdDefault");
      readStringEntry(configDat, "IDdisc",               aktOpts.IDdisc);
      readStringEntry(configDat, "IDvolume",             aktOpts.IDvolume);
      readStringEntry(configDat, "IDpreparer",           aktOpts.IDpreparer);
      readStringEntry(configDat, "IDpublisher",          aktOpts.IDpublisher);
      readStringEntry(configDat, "bootImagePath",        aktOpts.bootImagePath);
      readBoolEntry  (configDat, "fFollowSymLinks",      aktOpts.switches[0]);
      readBoolEntry  (configDat, "fAllowLeadingDots",    aktOpts.switches[1]);
      // umgebaut, weil cdrecord ab 1.8.1 es als Fehler deklariert:
      // readBoolEntry  (configDat, "fAllFiles",    aktOpts.switches[2]);
      aktOpts.switches[2] = false;
      readBoolEntry  (configDat, "fNames32Char",         aktOpts.switches[3]);
      readBoolEntry  (configDat, "fJoliet",              aktOpts.switches[4]);
      readBoolEntry  (configDat, "fTRANSTBL",            aktOpts.switches[5]);
      readBoolEntry  (configDat, "fMkisofsVerbose",      aktOpts.switches[6]);
      readBoolEntry  (configDat, "fRRchmod",             aktOpts.switches[7]);
      readBoolEntry  (configDat, "fRR",                  aktOpts.switches[8]);
      readBoolEntry  (configDat, "fMulti",               aktOpts.fMulti);
      readBoolEntry  (configDat, "fbootCD",              aktOpts.fbootCD);
      readBoolEntry  (configDat, "fCopyDataCD",          aktOpts.fCopyDataCD);
      readBoolEntry  (configDat, "fCreateImage",         aktOpts.fCreateImage);
      readBoolEntry  (configDat, "fDummyWrite",          aktOpts.fDummyWrite);
      readBoolEntry  (configDat, "fPadTrack",            aktOpts.fPadTrack);
      readBoolEntry  (configDat, "fMultiFirstSess",      aktOpts.fMultiFirstSess);
      readBoolEntry  (configDat, "fMultiLastSess",       aktOpts.fMultiLastSess);

      configDat->setGroup("UserInterface");
      readBoolEntry  (configDat, "fSimpleDND",       sysOpts.fSimpleDND);
      readBoolEntry  (configDat, "fOptOnStartup",    sysOpts.fOptOnStartup);
      readBoolEntry  (configDat, "fProcFinMsg",      sysOpts.fProcFinMsg);
      readBoolEntry  (configDat, "fProcWinOnStart",  sysOpts.fProcWinOnStart);      
    }
  delete configDat;

  // Jetzt die (strkeren) Vorgaben aus der globalen Datei laden

  // cdDoc
  readStringEntry(globalConf, "oCDDoc_latex",     sysOpts.latexCmd);
  readStringEntry(globalConf, "oCDDoc_view",      sysOpts.viewCmd);
  readStringEntry(globalConf, "oCDDoc_print",     sysOpts.printCmd);
  readStringEntry(globalConf, "oCDDoc_editor",    sysOpts.editorCmd);
  
  // System
  readStringEntry(globalConf, "oSys_writeDeviceSFile",    sysOpts.writeDeviceSFile);
  readStringEntry(globalConf, "oSys_writeDevice",         sysOpts.writeDevice);
  readStringEntry(globalConf, "oSys_readDevice",          sysOpts.readDevice);
  readStringEntry(globalConf, "oSys_mntpnt",              sysOpts.mountPoint);
  readStringEntry(globalConf, "oSys_availSpeed",          sysOpts.availSpeed);
  readNumEntry   (globalConf, "oSys_speedImg",            sysOpts.nWritingSpeedImg);
  readNumEntry   (globalConf, "oSys_speedOTF",            sysOpts.nWritingSpeedOTF);
  readNumEntry   (globalConf, "oSys_speedBlank",          sysOpts.nBlankingSpeed);
  readStringEntry(globalConf, "oSys_imagePath",           sysOpts.imageFilePath);
  readStringEntry(globalConf, "oSys_tmpDir",              sysOpts.tmpDir);
  readNumEntry   (globalConf, "oSys_fifoSize",            sysOpts.nFifoSize);
  readStringEntry(globalConf, "oSys_availSpeed",          sysOpts.availSpeed);
  readBoolEntry  (globalConf, "oSys_eject",               sysOpts.fEject);
  readBoolEntry  (globalConf, "oSys_shutdown",            sysOpts.fShutdown);

  // UserInterface
  readBoolEntry  (globalConf, "oUI_SimpleDND",       sysOpts.fSimpleDND);
  readBoolEntry  (globalConf, "oUI_OptOnStartup",    sysOpts.fOptOnStartup);
  readBoolEntry  (globalConf, "oUI_ProcFinMsg",      sysOpts.fProcFinMsg);
  readBoolEntry  (globalConf, "oUI_ProcWinOnStart",  sysOpts.fProcWinOnStart);      

  // cd  
  readBoolEntry  (globalConf, "oFS_followSymLnk",         aktOpts.switches[0]);
  readBoolEntry  (globalConf, "oFS_convLdngDots",         aktOpts.switches[1]);
  readBoolEntry  (globalConf, "oFS_32char",               aktOpts.switches[3]);
  readBoolEntry  (globalConf, "oFS_joliet",               aktOpts.switches[4]);
  readBoolEntry  (globalConf, "oFS_trans",                aktOpts.switches[5]);
  readBoolEntry  (globalConf, "oFS_RRchangeFlags",        aktOpts.switches[7]);
  readBoolEntry  (globalConf, "oFS_RR",                   aktOpts.switches[8]);
  readBoolEntry  (globalConf, "oBasics_createImgFile",    aktOpts.fCreateImage);
  readBoolEntry  (globalConf, "oBasics_dummyWrite",       aktOpts.fDummyWrite);
  readBoolEntry  (globalConf, "oBasics_padTrack",         aktOpts.fPadTrack);
  readBoolEntry  (globalConf, "oBasics_createBootCD",     aktOpts.fbootCD);
  readStringEntry(globalConf, "oBootCD_imgPath",          aktOpts.bootImagePath);
  readBoolEntry  (globalConf, "oBasics_createMultiCD",    aktOpts.fMulti);
  readBoolEntry  (globalConf, "oBasics_copyDataCD",       aktOpts.fCopyDataCD);


  // ein paar Sicherheitschecks
  if (sysOpts.mountPoint[0] != '/') sysOpts.mountPoint = "/mnt";
  
  // berprfen von sysOpts.imageFilePath
  if (sysOpts.imageFilePath[sysOpts.imageFilePath.size()-1] == '/')
    {
      errorMessage(i18n("Please give a valid path for the imagefile!\n"
			 "(options|more)\n"
			 "I corrected it to a default which possibly makes\n"
			 "no sense but will not cause strange errors. The point\n"
			 "is that you have to give a file, not a directory!"));  
      sysOpts.imageFilePath = "/usr/local/stupid_place.raw";
    }

  // berprfen von sysOpts.tmpDir
  if (sysOpts.tmpDir[sysOpts.tmpDir.size()-1] != '/')
    sysOpts.tmpDir += "/";

  //  sysOpts.availSpeed evtl. korrigieren
  {
    if (sysOpts.availSpeed[sysOpts.availSpeed.length()-1] != ',') sysOpts.availSpeed += ',';

    for (unsigned int cnt=0; cnt < sysOpts.availSpeed.length(); cnt++)
      {
	if (sysOpts.availSpeed.length() == 0) break;
	if (!isdigit(sysOpts.availSpeed[cnt]) && sysOpts.availSpeed[cnt] != ',')
	  sysOpts.availSpeed.replace(cnt--, 1, "");
      }
    while (sysOpts.availSpeed.length() != 0 && !isdigit(sysOpts.availSpeed[0]))
      sysOpts.availSpeed.replace(0, 1, "");
  }

  return(ersterStart);
}
















