/***************************************************************************
                          Subprocess.h  -  description                              
                             -------------------                                         
    begin                : Fri Aug 20 1999                                           
    copyright            : (C) 1999 by Jens Wilhelm Wulf                         
    email                : jens_w2@gmx.net                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef SUBPROCESSOUTPUT_H
#define SUBPROCESSOUTPUT_H

/*** C/C++ ***/
#include <sys/stat.h>
#include <stddef.h>
#include <regex.h>
#include <string>

/*** Qt ***/
#include <qwidget.h>
#include <qlistbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qdialog.h>

/*** KDE ***/
#include <kprocess.h> 

/*** KisoCD ***/
#include "switches.h"
#include "tools.h"

/**
  *@author Jens Wilhelm Wulf
  */

class Subprocess : public QDialog  {
   Q_OBJECT
public:    
   Subprocess(QString     labelInhalt, 
	      QWidget     *parent=0, 
	      const char  *name=0, 
	      bool        fModal = false,
	      bool        fehler = false);
   ~Subprocess();
   bool containsString(string);
   string getLatestLineWith(string aStr, int pos);
   bool isRunning();
   void newProc(QString, string);
   int startProc(int      aID,
		 string   pfad,
		 bool     fShowProcOnStart);
   int getID()                 { return(ID); };
   unsigned int getLineCnt()   { return(QListBox_1->count()); };
   QString getLine(int nr)     { return(QListBox_1->text(nr)); };
   void showBottom()           { QListBox_1->setBottomItem(QListBox_1->count()-1); };
   void addLine(QString line)  { QListBox_1->insertItem(line); };
   void setStatusLine(QString line);
   void setAutoStatusLine(string anAutoStatRE, bool signal) 
     { 
       emitRESignal = signal;
       autoStatusRE = anAutoStatRE; 
       regfree(autoStatusRegex);
       if (regcomp(autoStatusRegex, autoStatusRE.c_str(), 0)) 
	 errorMessage("Error when compiling a regular expression");
     };
   void setLabel(QString line) { QLabel_1->setText(line); };
   void setProcFinished()      
     { 
       disconnect(proc, SIGNAL (processExited(KProcess *)),
		  this, SLOT(procFinished(KProcess *)));
       delete proc;
       proc = NULL;       
     };
   KProcess* getProc()         { return(proc); };

signals:   
   void finished(KProcess *);
   void REMatched(QString line);
protected: 
   void initDialog();

private:
   KShellProcess*     proc;
   int                ID;

   string             str;     // akkumuliert die Zeichen, die ausgegeben werden 
                               // sollen, bis eine Zeile fertig ist.
   bool               teilAus; // ist true, wenn schon Zeichen ausgegeben wurden, 
                               // obwohl noch keine Zeile fertig war.  
   
   bool               statusLineSet;
   string             autoStatusRE;
   regex_t*           autoStatusRegex;
   bool               emitRESignal;
   int                ykoor;

   QListBox    *QListBox_1;
   QLabel      *QLabel_1;
   QLabel      *QL_status;
   QPushButton *b_continue;
   QPushButton *b_abort;
   QPushButton *b_close;

   void checkAutoStatus();
   
private slots:
   void input(KProcess *dummy, char *buffer, int buflen);
   void procFinished(KProcess *); 
};

#endif








