#include "tools.h"

/*** Qt ***/
#include <qapplication.h>
#include <qcursor.h>
#include <qmessagebox.h>
#include <qstring.h>

/*** KDE ***/
#include <ksimpleconfig.h>
#include <kprocess.h>
#ifdef KDE2
#include <klocale.h>
#include <kstddirs.h>
#else
#include <kapp.h>
#endif

/*** C++ ***/
#include <string>
#include <fstream.h> 
#include <iostream.h>
#include <unistd.h>
#include <dirent.h>

/*** KisoCD ***/
#include "version.h"


extern int                debuglevel;
extern DebugFile          debugFile; 


DebugFile::DebugFile(string dir = "", string name = "")
{
  myDir = dir;
  myBaseName = name;
  fileOpen = false;
}

DebugFile::~DebugFile()
{
  if (fileOpen) myFile.close();
}

void DebugFile::setDirName(string dir, string name)
{
  myDir = dir;
  myBaseName = name; 
}

int DebugFile::createFile()
{
  string pfad = getNrFile(myDir, myBaseName);
  myFile.open(pfad.c_str());
  fileOpen = true;
  if (!myFile) 
    {
#ifdef VERBOSEMESSAGES
      cout << "int DebugFile::createFile() open failed: " << pfad << "\n";
#endif
      return(1);
    }
  else
    { 
#ifdef VERBOSEMESSAGES
      cout << "int DebugFile::createFile() opened: " << pfad << "\n";
#endif
      return(0);
    }
}

int DebugFile::newFile()
{
  if (fileOpen) 
    {
      myFile.close();
      fileOpen = false;
    }
  return(0);
}

int DebugFile::newSection(string title = "")
{
  int    retCode = 0;
  string tmp;

  if (!fileOpen) retCode = createFile();
  if (retCode) return(retCode);
  if (title == "")
    tmp = "---------- new section ";
  else 
    tmp = "---------- new section: " + title + " ";
  while (tmp.length() < 100) tmp += "-";
  myFile << tmp << "\n";
  return(0);
}

int DebugFile::output(string aString)
{
  int retCode = 0;

  if (!fileOpen) retCode = createFile();
  if (retCode) return(retCode);
  myFile << aString << "\n";
  return(0);
}

int DebugFile::output(const char * aString)
{
  int retCode = 0;

  if (!fileOpen) retCode = createFile();
  if (retCode) return(retCode);
  myFile << aString << "\n";
  return(0);
}

int DebugFile::output(QString aString)
{
  int retCode = 0;

  if (!fileOpen) retCode = createFile();
  if (retCode) return(retCode);
  myFile << (const char*)(aString) << "\n";
  return(0);
}

int DebugFile::appendFile(string filename)
{
  ifstream  inFile;
  int       retCode = 0;
  char      c;
  
  if (!fileOpen) retCode = createFile();
  if (retCode) return(retCode);
  
  inFile.open(filename.c_str());
  if (!inFile) return(1);
  while ((c = inFile.get()) != EOF) myFile << c;
  inFile.close();
  return(0);
}

RemainingTimeTeller::RemainingTimeTeller()
{
}

void RemainingTimeTeller::cdrecordStarted(int forcedsize)
{
#ifdef VERBOSEMESSAGES
  cout << "void RemainingTimeTeller::cdrecordStarted()\n";
#endif
  starttime = time(NULL);
  forcedSize = forcedsize;
}

int RemainingTimeTeller::tellRemaining(QString line)
{
  time_t  curTime;
  double  elapsed;
  QString tmp;
  int     untilNow;
  int     whole;

  // calculate seconds since process was started
  // subtract subtract the time that cdrecord waits 
  // before starting (9 seconds)
  curTime = time(NULL);
  elapsed = difftime(curTime, starttime) - 9;
  if (elapsed <= 0.001) elapsed = 0.001;

  // extract how much was burned up to now
  tmp = line;
  if (forcedSize)
    tmp.remove(tmp.find(" MB written"), 99);
  else
    tmp.remove(tmp.find(" of"), 99);
  tmp.remove(0, tmp.length()-3);
  untilNow = tmp.toInt();

  // extract how much has to be burned
  if (forcedSize)
    whole = forcedSize;
  else
    {
      tmp = line;
      tmp.remove(tmp.find(" MB written"), 99);
      tmp.remove(0, tmp.length()-3);
      whole = tmp.toInt();
    }

  // calculate speed in MB per minute
  speed = untilNow * 60 / elapsed;

  // calculate how many minutes it takes to burn the rest and return the value
  return(int((whole-untilNow) / speed));
}

void getSysInfo(string pfad)
{
  string befehl;
  string catbefehl;
#ifdef KDE2
  string befehle[] = {"mount", "uname -mrspv", "echo $HOME", "which kisocdII", 
		      "which kisocdII.sh", "echo $KDEDIR", "echo $QTDIR", 
		      "cdrecord -scanbus", "mkisofs -v", 
		      "cat ${HOME}/.kde/share/apps/kisocdII/.config"};
#else  
  string befehle[] = {"mount", "uname -mrspv", "echo $HOME", "which kisocd", 
		      "which kisocd.sh", "echo $KDEDIR", "echo $QTDIR", 
		      "cdrecord -scanbus", "mkisofs -v", 
		      "cat ${HOME}/.kde/share/apps/kisocd/.config"};
#endif
#ifdef VERBOSEMESSAGES
  cout << "void getSysInfo(string pfad = " << pfad << ");\n";
#endif
  catbefehl = "cat " + pfad + ".tmp >> " + pfad;
  for (int cnt= 0; cnt < 10; cnt++)
    {
      befehl = befehle[cnt];
      befehl += " &> " + pfad + ".tmp";      
      system(befehl.c_str());
      system(catbefehl.c_str());
      befehl = "echo --------------------------------------------------";
      befehl += " &> " + pfad + ".tmp";      
      system(befehl.c_str());
      system(catbefehl.c_str());
    }
  catbefehl = "rm " + pfad + ".tmp";
  system(catbefehl.c_str());
}

string str2Latex(string tmpStr)
{
  string tmpStr2 = "";

  for (unsigned int Cnt = 0; Cnt < tmpStr.length(); Cnt++) 
    {
      if ((tmpStr[Cnt] == '$') || (tmpStr[Cnt] == '&') || (tmpStr[Cnt] == '%') || 
	  (tmpStr[Cnt] == '#') || (tmpStr[Cnt] == '_') || (tmpStr[Cnt] == '{') || 
	  (tmpStr[Cnt] == '}')) {
	tmpStr2+= "\\";
      }
      tmpStr2 += tmpStr[Cnt];
    }
  return(tmpStr2);
}

string holStartVerz(string altVerz)
{
  if (altVerz.length() > 2)
    if (altVerz[0] == '/')
      while (altVerz[altVerz.size()-1] != '/') altVerz.replace(altVerz.size()-1, 1, "");
    else altVerz = "";
  else altVerz = "";
  
  return(altVerz);
}

bool existDat(string DateiPfad)
{
  ifstream testDat;
//const  char     *tmp;

//  tmp = DateiPfad.c_str();
//  testDat.open(tmp);
  testDat.open((const char*)DateiPfad.c_str());
  if (!testDat) return(0); 
  else 
    { 
      testDat.close();
      return(1); 
    }
}

string dateiInhalt(string pfad)
{
  ifstream  inDat;
  char      Zeile[500];
  string    tmp = "";
  
  inDat.open(pfad.c_str());
  if (!inDat) 
    {
	QString meld;
	meld = i18n("Unable to open ");
	meld += pfad.c_str();
	errorMessage(meld);

	return(pfad);
    }
  while (!inDat.eof())
    {
      inDat.getline(Zeile, 498);
      tmp += Zeile;
      tmp += "\n";
    }        
  inDat.close();
  return(tmp);
}

QString getSizeText(const char* Beschr, unsigned long Groesse)
{
  QString tmp = "";

  if (Groesse >= 10240) {
    if (Groesse >= 10485760) {
      tmp.sprintf("%lu MByte", Groesse/1048576);
    }
    else {
      tmp.sprintf("%lu kByte", Groesse/1024);
    }
  }
  else tmp.sprintf("%lu Byte", Groesse);
  tmp.insert(0, Beschr);
  return(tmp);
}

//#ifndef KDE2
void errorMessage(const char* Inhalt)
{
  int CursorMerker = int(QApplication::overrideCursor());

  if (debuglevel & 1 != 0) 
    {
      debugFile.newSection("errormessage");
      debugFile.output(Inhalt);
    }
  cerr << Inhalt;
  if (CursorMerker != 0) {
    QApplication::restoreOverrideCursor();
  }
  QMessageBox::warning(0, i18n("KisoCD: error?!?"), Inhalt);
  if (CursorMerker != 0) {
    QApplication::setOverrideCursor(CursorMerker);
  }
}
//#endif
void errorMessage(QString Inhalt)
{
  int CursorMerker = int(QApplication::overrideCursor());

  if (debuglevel & 1 != 0) 
    {
      debugFile.newSection("errormessage");
      debugFile.output(Inhalt);
    }
#ifdef KDE2    
  cerr << Inhalt.latin1();
#else
  cerr << (const char*) Inhalt;
#endif
  if (CursorMerker != 0) {
    QApplication::restoreOverrideCursor();
  }
  QMessageBox::warning(0, i18n("KisoCD: error?!?"), Inhalt);
  if (CursorMerker != 0) {
    QApplication::setOverrideCursor(CursorMerker);
  }
}
//#ifndef KDE2
void infoMessage(const char* Inhalt)
{
  int CursorMerker = int(QApplication::overrideCursor());

  if (debuglevel & 1 != 0)
    {
      debugFile.newSection("message");
      debugFile.output(Inhalt);
    }
  cerr << Inhalt;
  if (CursorMerker != 0)
    QApplication::restoreOverrideCursor();
  QMessageBox::about(0, i18n("KisoCD: info"), Inhalt);
  if (CursorMerker != 0) 
    QApplication::setOverrideCursor(CursorMerker);
}
//#endif
void infoMessage(QString Inhalt)
{
  int CursorMerker = int(QApplication::overrideCursor());

  if (debuglevel & 1 != 0)
    {
      debugFile.newSection("message");
      debugFile.output(Inhalt);
    }
#ifdef KDE2    
  cerr << Inhalt.latin1();
#else
  cerr << (const char*)Inhalt;
#endif    
  if (CursorMerker != 0)
    QApplication::restoreOverrideCursor();
  QMessageBox::about(0, i18n("KisoCD: info"), Inhalt);
  if (CursorMerker != 0) 
    QApplication::setOverrideCursor(CursorMerker);
}

string backslashifyWhitespace(string inputStr)
{
  int Cnt = 0;
  string tmpStr = inputStr;

  while (Cnt < int(tmpStr.size()-1)) 
    {
      if (tmpStr[Cnt] == ' ') 
	tmpStr.replace(Cnt++, 1, "\\ ");
      Cnt++;
    }
  return(tmpStr);
}

string backslashifySpecial(string inputStr)
{
  int Cnt = 0;
  string tmpStr = inputStr;

  while (Cnt < int(tmpStr.size()-1)) 
    {
      switch (tmpStr[Cnt])
	{
	case '\'': tmpStr.replace(Cnt++, 1, "\\'"); break;
	case '(': tmpStr.replace(Cnt++, 1, "\\("); break;
	case ')': tmpStr.replace(Cnt++, 1, "\\)"); break;
	case ' ': tmpStr.replace(Cnt++, 1, "\\ "); break;
	}
      Cnt++;
    }
  return(tmpStr);
}

string itoStr(long int nr, char fill, unsigned char digits)
{
  string  tmp = "";
  char    digit;
  int     Cnt;
  
  if (nr == 0)
    {
      for (Cnt = 0; Cnt < digits-1; Cnt++) tmp += fill;
      tmp += '0';
    }
  else 
    {
      Cnt = 0;
      while (Cnt < digits || nr != 0)
	{
	  if (nr > 0) digit = (nr % 10)+48;
	  else digit = fill;
	  
	  tmp = digit + tmp;
	  nr /= 10;
	  Cnt++;
	}
    }
  
  return(tmp);
}


string getNrPath(string root, string name)
  // ein Verzeichnis suchen, das bei "root" beginnt und "name"###
  // als Namen hat. Es darf noch nicht existieren.
{
  int     nrCnt = 0;
  string  fullPath;
  DIR*    Verz;
  bool    frei;
  
  do 
    {
      nrCnt++;
      fullPath = root + name + itoStr(nrCnt, '0', 3);
      Verz = opendir(fullPath.c_str());
      frei = (Verz == NULL);
      if (!frei) closedir(Verz);
    }
  while (!frei && (nrCnt < 1000));
  
  
  if (nrCnt == 1000) return("");
  else return(fullPath); 
}

string getNrFile(string root, string name)
  // eine Datei suchen, das bei "root" beginnt und "name"###
  // als Namen hat. Sie darf noch nicht existieren.
{
  int     nrCnt = 0;
  string  fullPath;
  
  do 
    {
      nrCnt++;
      fullPath = root + name + itoStr(nrCnt, '0', 3);
    }
  while (existDat(fullPath) && (nrCnt < 1000));
    
  if (nrCnt == 1000) return("");
  else return(fullPath); 
}

int close_ofs(ofstream &outDat)
  // todo: prfen, ob es am Platz lag oder woran auch sonst immer...
{
  int retcode = 0;
  
  if (!outDat)
    {
      string dateiPfad;
      string befehl = "df ~ &> ";

      dateiPfad = getNrFile("/tmp/", "kisocd");              
      befehl += dateiPfad;
      if (!system(befehl.c_str())) 
	{
	  befehl = dateiInhalt(dateiPfad);      
	  if (befehl.find("100%") != string::npos)
	    errorMessage(i18n("An error occured when writing to a file.\n"
			  "Seems there is no space left in your home-directory."));
	}
      else 
	errorMessage(i18n("An error occured when writing to a file."));
      unlink(dateiPfad.c_str());
      
      retcode = 1;
    }

  outDat.close(); 
  return(retcode);
}

void dumpOptions(T_Options opts)
{
  cout << "  IDdisc: " << opts.IDdisc;
  cout << "\n  IDvolume: " << opts.IDvolume;
  cout << "\n  IDpreparer: " << opts.IDpreparer;
  cout << "\n  IDpublisher: " << opts.IDpublisher;

  if (opts.fCreateImage) cout << "\n  Createimage true";
  else cout << "\n  Createimage false";

  if (opts.fDummyWrite) cout << "\n  fDummyWrite true";
  else cout << "\n  fDummyWrite false";
  if (opts.fPadTrack) cout << "\n  fPadTrack true";
  else cout << "\n  fPadTrack false";
  if (opts.fbootCD) cout << "\n  fbootCD true";
  else cout << "\n  fbootCD false";

  cout << "\n  bootImagePath: " << opts.bootImagePath;

  if (opts.fMulti) cout << "\n  fMulti true";
  else cout << "\n  fMulti false";
  if (opts.fMultiFirstSess) cout << "\n  fMultiFirstSess true";
  else cout << "\n  fMultiFirstSess false";
  if (opts.fMultiLastSess) cout << "\n  fMultiLastSess true";
  else cout << "\n  fMultiLastSess false";
  cout << "\n";
}

string whereisExec(string name)
{
  string dateiPfad;
  string befehl = "which " + name + " &> ";

  dateiPfad = getNrFile("/tmp/", "kisocd");  
  befehl += dateiPfad;
  if (!system(befehl.c_str()))
    {
      befehl = dateiInhalt(dateiPfad);
      while (befehl[befehl.length()-1] == '\n') 
	befehl.replace(befehl.length()-1, 1, "");
    }
  else befehl = "";
  unlink(dateiPfad.c_str());
  return(befehl);
}

int execKFM(string startDir)
  // Rckgabewert:
  //   0 wenn es offensichtlich geklappt hat
  //   1 wenn was schief ging
{
  /*
  bool kfmRunning = false; 

  string tmpBef = "ps | grep \"kfm\" &> ";
  string tmpPfad = getNrFile("/tmp/", "kisocd");

  tmpBef += tmpPfad;
  if (system(tmpBef.c_str())) 
    {
      tmpBef = "error executing basic command:\n" + tmpBef;
      errorMessage(tmpBef.c_str());
      return(1);
    }
  
  tmpBef = dateiInhalt(tmpPfad);
  unlink(tmpPfad.c_str());

  while (tmpBef.find("kfm") != string::npos && kfmRunning == false)
    {
      if ((tmpBef.find("kfm") != string::npos && tmpBef.find("\"kfm\"") != string::npos &&
	   (tmpBef.find("kfm") != (tmpBef.find("\"kfm\"")+1))) ||
	  (tmpBef.find("kfm") != string::npos && tmpBef.find("\"kfm\"") == string::npos)) kfmRunning = true;
      if (tmpBef.find("\n") != string::npos)
	{
	  while (tmpBef[0] != '\n') tmpBef.replace(0, 1, "");
	  tmpBef.replace(0, 1, "");
	}
      else tmpBef = "";
    }
  
  if (kfmRunning)
    {
      cout << "kfmclient exec file:\n";
      tmpBef = "kfmclient exec file:";
      tmpBef += startDir;
      if (system(tmpBef.c_str())) return(1);
      else return(0);
    }
  else
    {
      cout << "kfm -sw exec file:\n";
      tmpBef = "kfm -sw exec file:";
      tmpBef += startDir;
      tmpBef += " &";
      if (system(tmpBef.c_str())) return(1);
      else return(0);
    }
  */
  
  string tmpBef = "kfmclient exec file:";
  tmpBef += startDir;

  return(system(tmpBef.c_str()));
}

int getPrefLang(string &lang1, string &lang2, string &lang3)
{
  string           confPfad;

  confPfad = getenv("HOME");
  confPfad += "/.kderc";

  lang1 = "en";
  lang2 = "en_UK";
  lang3 = "de";

  if (existDat(confPfad))
    {
      KSimpleConfig    *configDat;
      
      configDat = new KSimpleConfig(confPfad.c_str(), true);
      configDat->setGroup("Locale");          
      if ((confPfad = configDat->readEntry("Language", "none")) != "none")
	{      
	  cout << "Languages: " << confPfad << "\n";
	  if (confPfad.find(":") != string::npos)
	    {
	      lang1 = confPfad.substr(0, confPfad.find(":"));
	      confPfad.replace(0, confPfad.find(":")+1, "");
	      lang2 = confPfad.substr(0, confPfad.find(":"));
	      confPfad.replace(0, confPfad.find(":")+1, "");
	      lang3 = confPfad;
	    }
	}
    }
  return(0);
}

void writeLatexFileHeader(ofstream &outDat)
{
  outDat << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n";
  outDat << "%\n";
  outDat << "% autogenerated latex-file for CD-inlay and traycard.\n";
  outDat << "% Made by Jens Wilhelm Wulf for KisoCD.\n";
  outDat << "% Thanks to Beate for enhancements!\n";
  outDat << "%\n";
  outDat << "% Make your changes below (where indicated).\n";
  outDat << "%\n";
  outDat << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n";
  outDat << "\n";
  outDat << "\\documentclass[12pt, a4paper]{article}\n";
  outDat << "\\usepackage{german}\n";
  outDat << "\\usepackage[latin1]{inputenc} % fr die Umlautverwendung\n";
  outDat << "\\usepackage[T1]{fontenc}      % Erlaubt Trennung von Wrtern mit Umlauten\n";
  outDat << "\\usepackage{rotating}\n";
  outDat << "\n";
  outDat << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n";
  outDat << "%\n";
  outDat << "% Short description of the cds contents:\n";
  outDat << "%\n";
  outDat << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n";
  outDat << "\n";
  outDat << "\\def\\CD_Titel{CD 7: pictures and general documents}\n";
  outDat << "\n";
  outDat << "\\def\\Inhalt{\n";
  outDat << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n";
  outDat << "%\n";
  outDat << "% Here you can insert your latex-input:\n";
  outDat << "%\n";
  outDat << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n";
  outDat << "  \\small\n";
  outDat << "  \\begin{description}\n";  
}

void writeLatexFileFooter(ofstream &outDat)
{
  outDat << "  \\end{description}\n";
  outDat << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n";
  outDat << "%\n";
  outDat << "% There should be no need to edit below.\n";
  outDat << "%\n";
  outDat << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n";
  outDat << "}\n";
  outDat << "\n";
  outDat << "\\begin{document}\n";
  outDat << "  \\pagestyle{empty}\n";
  outDat << "  \\fontfamily{cmss}\n";
  outDat << "  \\fontsize{14}{16}\n";
  outDat << "  \\selectfont\n";
  outDat << "  \\unitlength=0.1mm\n";
  outDat << "  \\begin{picture}(1600.00,2000.00)\n";
  outDat << "    % Die Markierungen zum schneiden und knicken zeichnen:\n";
  outDat << "    \\put(0, 0){\\dashbox{3}(62, 1170)}      % linkes Ohr\n";
  outDat << "    \\put(62, 0){\\dashbox{3}(1375, 1170)}   % Mitte\n";
  outDat << "    \\put(0, 1170){\\dashbox{3}(1200, 1200)} % Mitte\n";
  outDat << "    \\put(1437, 0){\\dashbox{3}(62, 1170)}   % rechtes Ohr\n";
  outDat << "\n";
  outDat << "    \\put(0, -70){\\dashbox{3}(1170, 62)}    % Ohr zur Testansicht\n";
  outDat << "    \\put(5, -54){\\CD_Titel}\n";
  outDat << "\n";
  outDat << "    \\put(0, 2308){\\dashbox{3}(1200, 62)}   % Titel auf Vorderseite\n";
  outDat << "    \\put(5, 2324){\\CD_Titel}\n";
  outDat << "\n";
  outDat << "    \\put(62, 1108){\\dashbox{3}(1375, 62)}  % Titel auf Rckseite\n";
  outDat << "    \\put(67, 1124){\\CD_Titel}\n";
  outDat << "\n";
  outDat << "    \\put(27, 5) {\n";
  outDat << "      \\begin{rotate}{90}\n";
  outDat << "        \\CD_Titel\n";
  outDat << "      \\end{rotate}\n";
  outDat << "    }\n";
  outDat << "    \\put(1439, 1165) {\n";
  outDat << "      \\begin{rotate}{-90}\n";
  outDat << "        \\CD_Titel\n";
  outDat << "      \\end{rotate}\n";
  outDat << "    }\n";
  outDat << "    % der Text in der Mitte:\n";
  outDat << "    \\put(55, 1066){\n";
  outDat << "      \\begin{minipage}[t]{135mm}\n";
  outDat << "        \\Inhalt\n";
  outDat << "      \\end{minipage}\n";
  outDat << "      }\n";
  outDat << "    % der Text auf der Vorderseite:\n";
  outDat << "    \\put(0, 2273){\n";
  outDat << "      \\begin{minipage}[t]{115mm}\n";
  outDat << "        \\Inhalt\n";
  outDat << "      \\end{minipage}\n";
  outDat << "      }\n";
  outDat << "  \\end{picture}\n";
  outDat << "\\end{document}\n";
}

void writeKoverFileHeader(ofstream &koverDat)
{
  koverDat << "[Title]\n";
  koverDat << "Text=give me a name!\n";
  koverDat << "Font=times,32,5,iso-8859-1,50,0\n";
  koverDat << "Color=0,0,0\n";
  koverDat << "[Program]\n";
  koverDat << "Name=Kover\n";
  koverDat << "Version=0.31\n";
  koverDat << "[General]\n";
  koverDat << "BackgroundColor=255,255,255\n";
  koverDat << "Number=0\n";
  koverDat << "[Images]\n";
  koverDat << "BackgroundImageMode=1\n";
  koverDat << "BackgroundImage=\n";
  koverDat << "[Contents]\n";
  koverDat << "text=";
}

void writeKoverFileFooter(ofstream &koverDat)
{
  koverDat << "\nFont=helvetica,16,5,iso-8859-1,50,0\n";
  koverDat << "Color=0,0,0\n";
}

int writeCdrecordCmd(ofstream &outDat, T_Options aktOpts, T_SysOptions sysOpts)
{
  string device = getDevID(sysOpts.writeDevice);

  if (debuglevel & 1 != 0)
    {
      debugFile.newSection("function entered:");
      debugFile.output("int Edit::writeCdrecordCmd(...)"); 
    }

  outDat << BURNCMD;
  if (aktOpts.fDummyWrite) outDat << BC_DUMMY;
  if (aktOpts.fPadTrack)   outDat << BC_PAD;
  if (sysOpts.fEject)      outDat << BC_EJECT;
  if (aktOpts.fMulti)      outDat << BC_DATA;
  if (aktOpts.fMulti && !aktOpts.fMultiLastSess) outDat << BC_MULTI; 

  if (aktOpts.fCreateImage) 
    {
      outDat << BC_SPEED << sysOpts.nWritingSpeedImg << " ";
      outDat << BC_DEV << device << " '" << sysOpts.imageFilePath << "'";
      outDat << "\n";	  
    }
  else
    {
      outDat << BC_FS << sysOpts.nFifoSize << "m ";
      outDat << BC_SPEED << sysOpts.nWritingSpeedOTF << " ";
      outDat << BC_DEV << device << " -";
      outDat << "\n";
    }
}

string getDevID(string tmp)
{
  while (tmp.find("(") != string::npos) tmp.replace(tmp.length()-1, 1, "");  
  return(tmp);
}

string rmSlashes(string ziel)
  // entfernt fhrende Slashes und reduziert direkt aufeinanderfolgende auf einen einzigen
{
  // Fhrende '/' lschen
  if (ziel.size() != 0)
    while ((ziel[0] == '/') && (ziel.size() != 0))
      ziel.replace(0, 1, "");
  // Jetzt wre es noch schn, aufeinanderfolgende '/' zu reinigen...
  for (int Cnt = 0; Cnt < int(ziel.size()-1); Cnt++) 
    if ((ziel[Cnt] == '/') && (ziel[Cnt+1] == '/')) ziel.replace(Cnt+1, 1, "");
  
  return(ziel);
}

void mountViewImage(bool &fMounted, T_SysOptions sysOpts)
{
  if (fMounted) errorMessage(i18n("image already mounted!"));
  else 
    {
      string tmpBef;
      string tmpPfad;
      
      // mounten, je nach Erfolg fMounted setzen
      tmpBef = "mount ";
      tmpBef += sysOpts.imageFilePath;
      tmpBef += " ";
      tmpBef += sysOpts.mountPoint;
      tmpBef += " -o loop &> ";
      
      tmpPfad = getNrFile("/tmp/", "kisocd");
      tmpBef += tmpPfad;
      
      if (system(tmpBef.c_str()) != 0) 
	{
	  QString fehlerm = i18n("Error when mounting! I'll just show you mount's output:\n");
	  string fehler;
	  
	  fehlerm= fehlerm + dateiInhalt(tmpPfad).c_str();		
	  errorMessage(fehlerm); 
	  // hier setze ich fMounted nicht false, denn es kann ja sein,
	  // dass es den Fehler gerade deshalb gab, weil schon etwas
	  // gemounted ist!!!
	}
      else 
	{
	  fMounted = true;
	  if (execKFM(sysOpts.mountPoint)) errorMessage(i18n("Error executing kfm!"));
	}
      unlink(tmpPfad.c_str());
    }      	
}  
  
void unmountImage(bool &fMounted, string mountPoint)
{
  // Image unmounten, wenn es einen Fehler gibt, Meldung machen.
  // Das ganze so lange, bis image wirklich unmounted ist. 
  // Mit Hinweis auf evtl. sptere Fehler kann man auch abbrechen.
  // Hinterher fMounted entsprechend setzen.
  
  string tmpBef;
  bool fEnde = false;
  
  tmpBef = "umount " + mountPoint + " &> ";
  
  string tmpPfad = getNrFile("/tmp/", "kisocd");
  tmpBef += tmpPfad;

  while ((system(tmpBef.c_str()) != 0) && (!fEnde)) 
    {
	string fehler = dateiInhalt(tmpPfad);
	fehler += "\n";
	fehler += i18n("You can try again or cancel. The latter\n" 
		       "possibly results in further errors when trying to mount again.");
	fEnde = (QMessageBox::warning(0, i18n("error when unmounting"), 
				      fehler.c_str(),
				      i18n("try again"), i18n("Cancel"), 
				      0, 1, 1));
    }
  fMounted = fEnde;
}

#ifdef KDE2
int getUriType(QString path)
// -1  Fehler bei stat/lstat (Datei nicht vorhanden)
// 0   kein bekannter Typ
// 1   Verzeichnis
// 2   character special
// 3   block
// 4   normale Datei
// 5   fifo
// 6   socket
// Wenn es ein Symlink ist, wird nochmal 128 draufaddiert.

{
  struct stat attribs;

  if (lstat(path.latin1(), &attribs)) return(-1);
  if (S_ISDIR(attribs.st_mode)) return(1);
  if (S_ISCHR(attribs.st_mode)) return(2);  
  if (S_ISBLK(attribs.st_mode)) return(3);  
  if (S_ISREG(attribs.st_mode)) return(4);  
  if (S_ISFIFO(attribs.st_mode)) return(5);  
  if (S_ISSOCK(attribs.st_mode)) return(6);    
  if (S_ISLNK(attribs.st_mode))
    {
      if (stat(path.latin1(), &attribs)) return(-1);
      if (S_ISDIR(attribs.st_mode)) return(128+1);
      if (S_ISCHR(attribs.st_mode)) return(128+2);  
      if (S_ISBLK(attribs.st_mode)) return(128+3);  
      if (S_ISREG(attribs.st_mode)) return(128+4);  
      if (S_ISFIFO(attribs.st_mode)) return(128+5);  
      if (S_ISSOCK(attribs.st_mode)) return(128+6);        
      return(128);
    } 
  return(0);
}
#endif

string getPersConfigDir()
{
  string outDatPath;

  outDatPath = getenv("HOME");
  outDatPath += "/.kde/share/apps/";
  outDatPath += ProggieNameLC;

  return(outDatPath);
}

void makeNewLatexFile(string HeimatPfad)
{  
  // kopiert die Musterdatei fr das LatexFile von irgendwoher (muss noch festgelegt werden,
  // bisher ist das Verzeichnis vorgesehen, in dem auch die Hilfedateien stehen werden)
  // nach "HeimatPfad"/cdDoc.tex. Wird faulerweise einfach ber einen Systemaufruf gemacht.
  string htmldir;
#ifdef KDE2
  htmldir = getenv("KDEDIR");
  htmldir += "/";
  htmldir += KStandardDirs::kde_default("html");
#else
  htmldir = kapp->kde_htmldir();
#endif
  
  string befehl = "cp ";
  string tmpPfad;

  befehl += htmldir;
#ifdef KDE2
  befehl +="en/kisocdII/cdDocMuster.tex " 
#else
  befehl +="/en/kisocd/cdDocMuster.tex " 
#endif
    + HeimatPfad 
    + "/cdDoc.tex &> ";
  tmpPfad = getNrFile("/tmp/", "kisocd");
  befehl += tmpPfad;
  if (debuglevel) debugFile.output("makeNewLatexFile: " + befehl);
  
  if (system(befehl.c_str()))
    {
      befehl = HeimatPfad + "/tmp.out";
      QString befeh = i18n("Error copying latex-file:\n");
      befeh+= dateiInhalt(befehl).c_str();
      errorMessage(befeh);
    }
  unlink(tmpPfad.c_str());
}

int preBurningBox(bool fShutdown)
  // Liefert 
  //     0 bei "nicht brennen"/Abbruch, 
  //     1 bei "brennen", 
  //     2 bei "brennen aber nicht shutdown"
{
  int tmp;

  if (fShutdown)
    {
      tmp = QMessageBox::warning(0, i18n("Really sure?"), 
				 i18n("Start the writing process now and shutdown the whole system\n"
				      "when it has finished?\n"
				      "In case of shutdown you may lose data that has not been\n"
				      "saved yet!!"),
				 i18n("Yes, burn it!"), 
				 i18n("Burn it, but don't shutdown"), 
				 i18n("No, abort"), 
				 2, 2);
      switch (tmp) {
      case 0: return(1);
      case 1: return(2);
      default: return(0);
      }
    }
  else 
    {
      tmp = QMessageBox::warning(0, i18n("sure?"), 
				 i18n("Start the writing process now?"),
				 i18n("Yes, burn it!"), i18n("No, abort"), 
				 0, 1, 1);
      if (tmp == 0) return(1);
      else return(0);
    }
}











