/***************************************************************************
                          audiocd.cpp  -  description
                             -------------------
    begin                : Mon May 1 2000
    copyright            : (C) 2000 by Kai Heitkamp
    email                : koncd@kai-heitkamp.de
 ***************************************************************************/

#include <kconfig.h>
#include <kfiledialog.h>
#include <kprocess.h>
#include <kmsgbox.h>

#include "koncd.h"
#include "audiocd.h"

AudioCD::AudioCD(QWidget *parent, const char *name) : QDialog(parent,name,true){
	setCaption( i18n( "KOnCD - Audio CD" ) );
	initDialog();
	setupConnections();

	//Check for RW-Device
	KConfig *config = kapp->getConfig();
	config->setGroup( "CD-Writer" );
	if ( config->readEntry( "isRW" ) == "false" )
	{
	QComboBox_CDRWBlanking->clear();	
	QComboBox_CDRWBlanking->insertItem(i18n( "None RW !" ));
	QComboBox_CDRWBlanking->setEnabled( false );
	}
}

AudioCD::~AudioCD(){
}

// Setup Connections
void AudioCD::setupConnections(){
	connect( QPushButton_Start, SIGNAL(clicked()), SLOT(slotStart()) );
	connect( QPushButton_Quit, SIGNAL(clicked()), SLOT(slotExit()) );
	connect( QPushButton_Add, SIGNAL(clicked()), SLOT(slotAdd()) );
	connect( QPushButton_Del, SIGNAL(clicked()), SLOT(slotDelete()) );
	connect( QPushButton_DelAllTracks, SIGNAL(clicked()), SLOT(slotDeleteAllTracks()) );
}

//'Add'-Button' clicked
void AudioCD::slotAdd(){
	QString getName;
	getName = KFileDialog::getOpenFileName(0, "*.wav", this, 0);
	QListBox_Tracklist->insertItem( (const char*) getName );
}

//'Delete'-Button' clicked
void AudioCD::slotDelete(){
	QListBox_Tracklist->removeItem( QListBox_Tracklist->currentItem() );
}

//'Delete all tracks'-Button' clicked
void AudioCD::slotDeleteAllTracks(){
	QListBox_Tracklist->clear();
}

//'Start'-Button' clicked
void AudioCD::slotStart(){
	int rc;
	KConfig *config = kapp->getConfig();

	//KProcess argument
	KProcess *process = new KProcess();
	process->clearArguments();
	(*process) << "cdrecord" << "-v" << "-audio";

	//Set the CD-Writer first
	config->setGroup("CD-Writer");
	(*process) << config->readEntry("Device");

	//Set CD-Writer Options to K(*process)
	if( QCheckBox_EjectCD->isChecked() ) (*process) << "-eject";
	if( QCheckBox_Force->isChecked() ) (*process) << "-force";
	if( QCheckBox_Simulate->isChecked() ) (*process) << "-dummy";
	if( QCheckBox_padding->isChecked() ) (*process) << "-pad";
	if( QCheckBox_nofix->isChecked() ) (*process) << "-nofix";
	if( QCheckBox_ignoresize->isChecked() ) (*process) << "-ignsize";

	//Set CD-RW Options to KProcess
	if( (rc = QComboBox_CDRWBlanking -> currentItem()) )
		{
	   	if( rc == 1 ) (*process) << "blank=all";
	   	if( rc == 2 ) (*process) << "blank=fast";
	   	if( rc == 3 ) (*process) << "blank=track";
	   	if( rc == 4 ) (*process) << "blank=unreserve";
	   	if( rc == 5 ) (*process) << "blank=trtrail";
	   	if( rc == 6 ) (*process) << "blank=unclose";
	   	if( rc == 7 ) (*process) << "blank=session";
	   	}

	//Set CD-Writer Speed to KProcess)
	if( (rc = QComboBox_WriterSpeed -> currentItem()) )
		{
	   	if( rc == 0 ) (*process) << "speed=1";
	   	if( rc == 1 ) (*process) << "speed=2";
	   	if( rc == 2 ) (*process) << "speed=4";
	   	if( rc == 3 ) (*process) << "speed=6";
	   	if( rc == 4 ) (*process) << "speed=8";
	   	}

	//Add tracks to be written
	int trackcount = 0;
	int tc = QListBox_Tracklist->count();
	do {
		(*process) << QListBox_Tracklist->text( trackcount );
		trackcount++;
	}
	while ( trackcount < tc );

	//Connect the Abort-Button from ImgProOut-Dialog to Abort-Handler
	connect( procoutdlg.QPushButton_OK, SIGNAL(clicked()), this, SLOT(abortHandler()));

	//Connect the 'processExited' Signal to the 'slotProcessFinished' Slot
	connect( process, SIGNAL(processExited(KProcess *)), this, SLOT(slotBurnProcessExited(KProcess *)));

	//Connect with slotRecStderr to update KProgress (Write- / Buffer-Status) and StatusLine
	connect( process, SIGNAL(receivedStdout (KProcess *, char *, int) ), this, SLOT(slotBurnRecStderr (KProcess *, char *, int) ));
	connect( process, SIGNAL(receivedStderr (KProcess *, char *, int) ), this, SLOT(slotBurnRecStderr (KProcess *, char *, int) ));

	//Start KProcess
	if( ! process->start( KProcess::NotifyOnExit, KProcess::AllOutput ) )
		{
		KMsgBox::message( this, i18n("KOnCD - Audio CD - Error"), i18n("Couldn't start cdrecord !"), 2 );
		return;
		}

	procoutdlg.QPushButton_OK->setEnabled( false );
	if ( ! procoutdlg.isVisible() ) procoutdlg.show();

}

//KProcess outout analysis
void AudioCD::slotBurnRecStderr(KProcess *proc, char *buffer, int buflen) {
	char *c, buf[1024];
	int wrote, total, load;

	if( !buflen ) {
		return;
	}
	bzero( buf, sizeof( buf ) );
	strncpy( buf, buffer, buflen > (int) sizeof( buf ) ? sizeof(buf) - 1 : buflen );
	//Add cdrecord Messages to the Output-Window
	procoutdlg.Output->insertLine( buf );
	//Go to the End of the Output-Text
	procoutdlg.Output->setCursorPosition(procoutdlg.Output->numLines(),0,false);
	
	//Blanking a CD-RW
	if( strstr( buf, "Blanking" )) {
		procoutdlg.StatusLabel -> setText( i18n("Blanking the CDRW...") );
	}

	//Burn without Fixating
	if( strstr( buf, "Writing  time:" )) {
		procoutdlg.StatusLabel -> setText( i18n("Burn-Process complete.") );
	}

	//Fixating the CD-R
	if( strstr( buf, "Fixating..." )) {
		procoutdlg.StatusLabel -> setText( i18n("Writing TOC...") );
	}

	//	//Burn with Fixating
	if( strstr( buf, "Fixating time:" )) {
		procoutdlg.StatusLabel -> setText( i18n("Burn-Process complete.") );
   	}
	
	if( strstr( buf, "MB written" )) {
		c = index( buf, ':' );
		if( c && *c ) {
			c = strtok( c, ":\r\t " );
			if( c ) {
				wrote = atoi( c );
				c = strtok( NULL, "\r\t " );
				c = strtok( NULL, "\r\t " );
				if( c ) {
					total = atoi( c );
					c = strtok( NULL, "\r" );
					c = strstr( c, "fifo " );
					if( c ) {
						c = strtok( c + 5, "%\r\t " );
						if( c ) {
							load = atoi( c );
							procoutdlg.WriteProgress -> setValue( total ? wrote * 100 / total : 0 );
	  						procoutdlg.BufferProgress -> setBarColor( QColor( 220, 30, 30 ) );
							if( load > 40 )
  							procoutdlg.BufferProgress -> setBarColor( QColor( 200, 200, 120 ) );
							if( load > 80 )
  							procoutdlg.BufferProgress -> setBarColor( QColor( 40, 100, 60 ) );
							procoutdlg.BufferProgress -> setValue( load );

							if( ( total == wrote ) ) {
								procoutdlg.StatusLabel -> setText( i18n("Writing TOC...") );
							}
							
							if( wrote ) {
								procoutdlg.StatusLabel -> setText( i18n("Writing to CD...") );
							}

						}
					}
				}
			}
		}		
	}	

}

//KProcess finished
void AudioCD::slotBurnProcessExited(KProcess *rcproc) {
	//Check, if KProcess exited normally...
	if( rcproc->normalExit() ) {
 			if( rcproc->exitStatus() != 0 ) {
				switch ( rcproc->exitStatus() )
				{
					case 13:		procoutdlg.StatusLabel -> setText( i18n( "Error: You don't have enough permissions !" ) );
										break;

					case 19:		procoutdlg.StatusLabel -> setText( i18n( "Error: Can't open SCSI-Driver !" ) );
										break;

					case 254:	procoutdlg.StatusLabel -> setText( i18n( "Error: Corrupt CD in the Writer-Device !" ) );
										break;

					case 255:	procoutdlg.StatusLabel -> setText( i18n( "Error: No CD in the Writer-Device !" ) );
										break;
				}
				QString statusout;
				statusout.sprintf( "Exit status: %d", rcproc->exitStatus() );
 				KMsgBox::message( this, i18n( "KOnCD - Burning" ), i18n( "An Error is occurred !\n" + statusout ), 4 );
				procoutdlg.QPushButton_OK->setEnabled( true );
				return;
			}
		//If 'Leave Image' CheckBox false, delete the Imagefile
		procoutdlg.QPushButton_OK->setEnabled( true );
	}
}

//Abort-Handler
void AudioCD::abortHandler(){
	//Close dialog
	if ( procoutdlg.isVisible() ) procoutdlg.close();
}

//'Quit-Button' clicked
void AudioCD::slotExit(){
	close(true);
}
