#ifndef KRABWIDGET_H 
#define KRABWIDGET_H 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

#include "detect_kde_version.h"

#ifdef KDE2
  #include <kglobal.h>
  #include <kstddirs.h>
  #include <qmessagebox.h>
  #include <qdragobject.h>
#else
  #include <kmsgbox.h>
  #include <kfm.h>
#endif


#define MAX_MB_ON_CD    750
#define MINIMUM_FREE    2048       //Minimum of 2MB free disk space


#include <qwidget.h>
#include <qframe.h>
#include <qlistbox.h>
//#include <qstring.h>
#include <qfile.h>
#include <qtabdialog.h> 
#include <qtimer.h>
#include <qimage.h>
#include <qdir.h>

#include <qcombobox.h>
#include <qradiobutton.h>
#include <qgroupbox.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qcheckbox.h>

#include <kapp.h>
#include <kprocess.h>
#include <kslider.h>
#include <kconfig.h>
#include <ktablistbox.h>
#include <kiconloader.h>
#include <kfiledialog.h>
#include <kcolorbtn.h>

#include <cddalist.h>
#include <status_window.h>
#include <namewidget.h>

#include <cddb.h>
#include <smtp.h>
#include <smtpconfig.h>
#include <CDDBSetup.h>

struct configstruct
{
	bool	     readall,
		     testOnStartup,
		     outputfilename_capital,
		     outputfilename_replace;
	unsigned int mp3tag_version;
	QString	     outputfilename_replace_from,
		     outputfilename_replace_to,
		     scan_device;
};

struct cddbconfigstruct
{
	QString   basedatadir;
	QString   cddbbasedir;
	QStrList  cddbserverlist;
	QString   current_server;
	QString   mailcmd;
	QString   submitaddress;
	QStrList  cddbsubmitlist;
	QString   browsercmd;
	bool      cddb_remote_enabled;
	SMTPConfig::SMTPConfigData smtpConfigData;
};

struct cddb_data
{
	unsigned long magicID;
	QString	      data;
	QStrList      titlelist;
	QStrList       extlist;
	QString       category;
	QStrList      discidlist;
	int 	      revision;
	QStrList      playlist;
	unsigned int  tracks;
	QStrList      querylist;
	unsigned long absframe;
	offset_frame  o_frame;
        QString       mp3_categ;
        QString       mp3_year;
};

class KrabWidget : public QWidget
{
	Q_OBJECT

public:
	KrabWidget(QWidget *parent = 0, const char *name = 0, TodoList *todolist_p=NULL);
  	virtual ~KrabWidget();

//	QListBox *QListBox_todo;
	KConfig *kconfig;
	KTabListBox *KTabListBox_todo;
	QFrame *QFrame_controll;
	bool working;

signals:
	void new_status_info(QString);
	void new_size_info(unsigned int);
	void enable_widgets( bool );
	void ask_4_new_cd();
	void play_emited();
	void play_finished_emited();

public slots:

// ifdef KDE2 did not work here for the qt moc....:
// enable this for KDE/QT 2
//	void dropEvent( QDropEvent * );
// enable this for KDE/QT 1.x
	void DropEvent(KDNDDropZone *);

	void todolist_update();
	void todolist_update(bool b);
	void todolist_update(int i);
	void todolist_update_cdr();
	void test_enviroment( bool, bool );
	void delete_job( unsigned int );
	void select_files();
	void select_file( QString, unsigned int tracknr=0 );
	void select_dir( QString, unsigned int tracknr=1 );
	void configure(unsigned int);
	void update_tags();
	void update_filenames();
	void show_info();
	void play_job( unsigned int );
	void play_finished();

private slots:
	void smtpMessageSent(void);
	void smtpError(int);
	void updateCurrentCDDBServer();
	void cddb_done();
	void mycddb_inexact_read();
	void cddb_timed_out();
	void cddb_failed();
	void cddb_no_info();
	void cddb_ready();
	void put_cddb_local();
	void put_cddb_remote( unsigned int );
	bool put_cddb_info( QString, bool );
	bool get_cddb_info_local();
	void get_cddb_info_remote( unsigned int );
	void get_pathlist( QStrList & );
//	void save_cddb_entry(QString& path, bool upload);
	void getCDDBserversDone();
	void getCDDBserversFailed();
	void getCDDBservers();

	void get_cddb( bool force_remote=0 );
        void get_cddb_from_server(bool);
        void send_cddb_to_server(bool);

	void cdrecord_out(KProcess *, char * ,int );
	void load_config();
	void save_config();
	void sorry();
        void cddalister();
	void test_env_stdout(KProcess *, char * ,int ); 
	void krabwidget_setEnabled( bool );
	void resizeEvent( QResizeEvent * );
	QString character_replace( QString input, QString from, QString to );
	void identify( QString, char*, unsigned int );
	void direct_convert_toggled( bool state );
	void xing_vbr_toggled(bool);
	void KTabListBox_selected(int,int);
	void KTabListBox_midClick(int,int);
	void KTabListBox_highlighted(int,int);
	void KTabListBox_popupMenu(int,int);
        bool checkProgram(int& mode, int index, int type,
			  const char* execName,
                          const char* flags, const char* expected,
                          int start = 0);

	void cdr_check();	
	void cdr_check_end();
	void cdr_blank_start();	
	void cdr_job_end();
	void cdr_fix( bool force_fix=0 );	
	void cdr_fix_();
	void doit();
	void doit_control();
	void kill_doit();
	void help_prog_df_stdout(KProcess *, char *, int);
	void doit_read_prog( bool forc_fifo_out=0 );
	void read_prog_dead();
	void progress_update();
	void doit_write_prog(unsigned int);
	void write_prog_dead();
	void write_mp3_header( QString, unsigned int );
	void send_signal_process( unsigned int );
	void renice_process( int );
	
	void CddaListDestroyed();
	void NameWidgetDestroyed();
	void define_outputpath();
	void define_names();
	void QComboBox_bitrate_clicked(int z);
	void QLineEdit_bitrate_other_changed(const char*);
	void QComboBox_encoder_clicked(int n);
	void KSlider_smp_moved(int);
	void KSlider_nice_moved(int);
	void cd_changed();
	void cd_change();
	void pop_up_4_new_cd();
	unsigned long current_discid(unsigned int);

private:
	SMTPConfig  *smtpconfig;
	CDDBSetup        *cddb_setup;
	SMTP		 *smtpMailer;
	cddbconfigstruct cddb_config;
	configstruct     krabber_config;
	CDDB             cddb;
	cddb_data        _cddb_data;
	bool		 connected,
			 cddb_inexact_sentinel;

	bool Init,
	     cdparanoia,
	     cdrecord,
	     sox, 
	     mpg123, 
	     hz8_mp3,
	     lame,
	     encode, 
	     l3enc, 
	     bladeenc,
	     mp3enc,
	     xing,
	     wait_4_cd_change,
	     doit_control_run,
	     progress_update_run,
	     read_prog_run,
	     play_prog_run,
	     check_cdr_run,
	     use_sox,
	     use_fifo,
	     write_prog_run[MAX_ENCODERS];
	QVector<int> temp_file;
	QVector<QString> tempfilename;
	TodoList   *todolist;
	namewidget *NameWidget;
	cddalist   *CddaList;
        QString cdrecord_device[20],
                cdrecord_device_id[20];
	unsigned int jobnumber,
                     write_jobnumber,
		     write_job_track[MAX_ENCODERS],
		     jobs_finished,
		     encoder_max,
		     devices_count,
		     cdrecord_devices_count,
		     modes[30],
		     cdparanoia_version,
		     bitrate,
		     dest_bitrate,
		     dest_bits,
		     dest_channels,
		     cddb_remote_jobnr,
		     cdrecord_progress;
	unsigned long total_final_size,
                      total_finished,
	              read_prog_final_size,
		      write_prog_final_size[MAX_ENCODERS];
	double needed_encd_space,
 	       space_avaible;

	long cdr_pre_loaded;
	
	int nice_level,
	    pipe_[2],
	    dummy[2];
	pid_t sox_pid;
	QString help_prog_stdout_temp,
		blade_prog_stdout_temp,
		FIFO,
	        devices[16],
		outputfilename[MAX_ENCODERS],
		received_data_stdout,
		empty;
	status_window *Status;
	
	KProcess *help_prog,
		 *convert_prog,
		 *read_prog,
		 *write_prog[MAX_ENCODERS];
	KIconLoader *iloader;
	KSlider *KSlider_nice;
	KSlider *KSlider_smp;
	
	QLabel  *QLabel_nice,
		*QLabel_smp,
		*cd_change_Label_1,
		*cd_change_Label_2,
		*cd_change_Label_3;

	QTimer *QTimer_progress,
	       *QTimer_process,
	       *QTimer_cddb;

	QLineEdit *QLineEdit_bitrate_other;

	QGroupBox *QGroupBox_encoder;

	KComboExt *QComboBox_encoder;
	QComboBox *QComboBox_bitrate;
	QComboBox *QComboBox_device;
	QComboBox *QComboBox_8hz_br;
	QComboBox *QComboBox_blade_br;
	QComboBox *QComboBox_lame_br;
	QComboBox *QComboBox_lame_ql;
	QComboBox *QComboBox_encode_br;
	QComboBox *QComboBox_encode_ql;
	QComboBox *QComboBox_l3_br;
	QComboBox *QComboBox_mp3_br;
	QComboBox *QComboBox_mp3_ql;
	QComboBox *QComboBox_xing_ql;
	QComboBox *QComboBox_xing_br;
	QComboBox *QComboBox_cdr_device;
	QComboBox *QComboBox_cdr_speed;

	QPushButton  *QPushButton_cdr_fix;
	QPushButton  *QPushButton_cdr_blank;
	QPushButton  *QPushButton_cdr_size;

	QGroupBox *QGroupBox_opt2;
	QGroupBox *QGroupBox_options;
        QGroupBox *QGroupBox_cddb;
	QGroupBox *QGroupBox_nice;
	QGroupBox *QGroupBox_cache;

	QCheckBox *QCheckBox_tempfiles;
	QCheckBox *QCheckBox_clear_list;
	QCheckBox *QCheckBox_q_resample;
	QCheckBox *QCheckBox_stereo;
	QCheckBox *QCheckBox_16bit;
	QCheckBox *QCheckBox_jitter;
	QCheckBox *QCheckBox_8hz_copyright;
	QCheckBox *QCheckBox_8hz_original;
	QCheckBox *QCheckBox_blade_copyright;
	QCheckBox *QCheckBox_blade_private;
	QCheckBox *QCheckBox_blade_crc;
        QCheckBox *QCheckBox_lame_original;
        QCheckBox *QCheckBox_lame_copyright;
        QCheckBox *QCheckBox_lame_fast;
        QCheckBox *QCheckBox_lame_gui;
	QCheckBox *QCheckBox_encode_original;
	QCheckBox *QCheckBox_encode_copyright;
	QCheckBox *QCheckBox_encode_crc;
	QCheckBox *QCheckBox_bitrate;
	QCheckBox *QCheckBox_l3_hq;
	QCheckBox *QCheckBox_l3_crc;
	QCheckBox *QCheckBox_l3_speech;
	QCheckBox *QCheckBox_mp3_header;
        QCheckBox *QCheckBox_mp3_crc;
        QCheckBox *QCheckBox_cdr_dummy;
        QCheckBox *QCheckBox_cdr_eject;
        QCheckBox *QCheckBox_cdr_fix;
        QCheckBox *QCheckBox_xing_vbr;
        QCheckBox *QCheckBox_xing_copyright;
        QCheckBox *QCheckBox_xing_original;
        QCheckBox *QCheckBox_xing_nohigh;

   	QPushButton *QPushButton_cdda_list;
	QPushButton *QPushButton_outputpath;
	QPushButton *QPushButton_cd;
	QPushButton *QPushButton_names;
	QPushButton *QPushButton_doit;
	QPushButton *cd_change_PushButton;
	QPushButton *QPushButton_cddb_get;
	QPushButton *QPushButton_cddb_send;

	QWidget *cd_change_Widget;
	QTabDialog *tabdialog;

	QPixmap *pix_wav,
		*pix_mp2,
		*pix_mp3,
		*pix_cdda,
		*pix_dat,
		*pix_cdr,
		*pix_raw,
		*pix_svx,
		*pix_aiff,
		*pix_au,
		*pix_maud,
		*pix_sf,
		*pix_smp,
		*pix_voc,
		*pix_cvs,
		*pix_vms,
		*pix_sndr,
		*pix_sndt,
		*pix_txw,
		*pix_wve,
		*pix_hcom,
		*pix_push_1_off,
		*pix_push_2_off,
		*pix_push_3_off,
		*pix_push_4_off,
		*pix_push_5_off,
		*pix_push_6_off,
		*pix_push_1,
		*pix_push_2,
		*pix_push_3,
		*pix_push_4,
		*pix_push_5,
		*pix_push_6;

	QFile *read_prog_file,
	      *write_prog_file[MAX_ENCODERS];

	unsigned long calc_total_size();

  	void initLayout(void);
	void initLayout_wav();
	void initLayout_8hz();
	void initLayout_blade();
	void initLayout_lame();
	void initLayout_encode();
	void initLayout_xing();
	void initLayout_cdrecord();
	void initLayout_l3();
	void initLayout_mp3();
	void initLayout_dat();
	void initLayout_cdr();
	void initLayout_raw();
	void initLayout_svx();
	void initLayout_aiff();
	void initLayout_au();
	void initLayout_maud();
	void initLayout_sf();
	void initLayout_smp();
	void initLayout_voc();
	void initLayout_cvs();
	void initLayout_vms();
	void initLayout_sndr();
	void initLayout_sndt();
	void initLayout_txw();
	void initLayout_wve();
	void initLayout_hcom();
	void initLayout_wav_del();
	void initLayout_lame_del();
	void initLayout_encode_del();
	void initLayout_xing_del();
	void initLayout_cdrecord_del();
	void initLayout_8hz_del();
	void initLayout_blade_del();
	void initLayout_l3_del();
	void initLayout_mp3_del();
        void initLayout_dat_del();
        void initLayout_cdr_del();
        void initLayout_raw_del();
        void initLayout_svx_del();
        void initLayout_aiff_del();
        void initLayout_au_del();
        void initLayout_maud_del();
        void initLayout_sf_del();
        void initLayout_smp_del();
        void initLayout_voc_del();
        void initLayout_cvs_del();
        void initLayout_vms_del();
        void initLayout_sndr_del();
        void initLayout_sndt_del();
        void initLayout_txw_del();
        void initLayout_wve_del();
        void initLayout_hcom_del();

	static const unsigned int bitrate_8hz[], bitrate_lame[], bitrate_encode[], bitrate_blade[], bitrate_l3[], bitrate_mp3[], bitrate_xing[], bitrate_mpeg_layer2[], bitrate_mpeg_layer3[];
	static const unsigned int bitrate_8hz_max, bitrate_encode_max, bitrate_lame_max, bitrate_blade_max, bitrate_l3_max, bitrate_mp3_max, bitrate_xing_max;
	static const char *format_in[], *format_out[];
};

#endif // KRABWIDGET_H 




