
// original written by Adrian Schroeter.

// completly rewritten by Pedro Morais
// thanks a lot
// and some minor changes again by adrian...

#include <qdir.h>
#include <qmessagebox.h>

#include <krabwidget.h>
#include <kapp.h>

//#include <strstream.h>

#define GET_STDOUT 0
#define GET_STDERR 1
#define GET_BOTH 2

// checkProgram checks if a program exists.
bool KrabWidget::checkProgram(int& mode, int index, int type,
			      const char* execName,
			      const char* flags, const char* expected,
			      int start) 
{
	if (!expected) return FALSE;
	int size = strlen(expected);

	help_prog_stdout_temp = "";
	KProcess process;
	process.clearArguments();
	process.setExecutable(execName);
	if (type == GET_STDOUT || type == GET_BOTH) {
		connect(&process, SIGNAL(receivedStdout(KProcess *, char *, int )),
			this, SLOT(test_env_stdout(KProcess *, char *, int  )));
	}
	if (type == GET_STDERR || type == GET_BOTH) {
		connect(&process, SIGNAL(receivedStderr(KProcess *, char *, int )),
			this, SLOT(test_env_stdout(KProcess *, char *, int  )));
	}
	if (flags) process << flags;
	process.start(KProcess::Block, KProcess::AllOutput);

	if ( help_prog_stdout_temp.mid(start, size) == expected ) {
		if (index > -1) {
			modes[mode++] = index;
			QComboBox_encoder->insertItem(execName);
		}
		return TRUE;
	} else {
#ifdef DEBUG
		cerr << "Got: "<< help_prog_stdout_temp.mid(start, size)
			 << " Expected: " << expected << endl;
#endif
		return FALSE;
	}
}



// for audio device have a look at config.h(.bot) or krab.cpp

void KrabWidget::test_enviroment( bool show, bool test_devices )
{

	emit( new_status_info(i18n("Check your System...")));

	int mode = 0;
	QString msg, temp, temp2;

#ifdef HAVE_LINUX
	QString possible_devices[] = {
		"/dev/sg0", "/dev/sg1", "/dev/sg2", "/dev/sg3","/dev/sg4",
		"/dev/sg5", "/dev/sg6", "/dev/sg7", "/dev/sg8","/dev/sg9",
		"/dev/sga", "/dev/sgb", "/dev/sgc", "/dev/sgd","/dev/sge",
		"/dev/sgf", "/dev/sgg", "/dev/sgh", "/dev/sgj","/dev/sgk",
		"/dev/sr0", "/dev/sr1", "/dev/sr2", "/dev/sr3",
		"/dev/hda", "/dev/hdb", "/dev/hdc", "/dev/hdd",
		"/dev/sonycd", "/dev/cdrom", 0, 0 };
#else
	#ifdef HAVE_BSD
	QString possible_devices[] = {
		"/dev/scd0a", 0, 0 };
		#warning "cdparanoia do not support FreeBSD yet, but it should soon ..."
	#else
		#warning "This OS is not supported by Krabber, compile anyway...."
	#endif
#endif

	// this piece of code guarantees that you won't get the same
	// device checked twice if it is on the list and is the the user choice
	int k = 0;
	bool alreadyPresent = FALSE;
	while (possible_devices[k]) {
		if (!alreadyPresent && krabber_config.scan_device == possible_devices[k]) {
			alreadyPresent = TRUE;
		}
		k++;      
	}
	if (!alreadyPresent) {
		possible_devices[k] = krabber_config.scan_device;
	}

	// begin the test
	msg = i18n("I have tested your system for required applications:\n\n");
	QComboBox_encoder->clear();

#define CDPARANOIA 0
#define MPG123 1
#define CDRECORD 2
#define SOX 3

#define HZ8MP3 4
#define LAME 5
#define ENCODE 6
#define BLADEENC 7
#define L3ENC 8
#define MP3ENC 9
#define XING 10

#define LAST_ITEM 10

  const char *apps[] = { "cdparanoia",
			 "mpg123",
			 "cdrecord",
			 "sox "SOX_VERSION,
			 "8hz-mp3",
			 "lame",
			 "encode",
			 "bladeenc",
			 "l3enc",
			 "mp3enc",
			 "xingmp3enc"
		        };

  bool check[] = {
    checkProgram(mode, -1, GET_BOTH,
		 "cdparanoia", "-h","cdparanoia III"),
    checkProgram(mode, -1, GET_BOTH,
		 "mpg123", 0, "High Performance MPEG"),
    checkProgram(mode, 15, GET_BOTH,
		 "cdrecord", 0, "cdrecord"),
    checkProgram(mode, -1, GET_STDERR,
		 "sox", "-h", "sox: Version "SOX_VERSION),

    checkProgram(mode, 2, GET_STDERR,
		 "8hz-mp3", "-h", "8hz-mp3"),
    ( checkProgram(mode, 1, GET_STDERR,
		   "lame", 0, "LAMER", 8) ||
      checkProgram(mode, 1, GET_STDERR,
		   "lame", 0, "LAME") ),
    checkProgram(mode, 6, GET_STDERR,
		 "encode", 0, "usage: encode"),
    checkProgram(mode, 5, GET_STDOUT,
		 "bladeenc", 0, "BladeEnc 0.8", 1),
    checkProgram(mode, 3, GET_STDERR,
		 "l3enc", "-h", "****  L3ENC V", 2),
    checkProgram(mode, 4, GET_STDOUT,
		 "mp3enc", "-h", "-v"),
    checkProgram(mode, 7, GET_STDOUT,
		 "xingmp3enc", "", "XingMP3 Encoder")
  };

  if (msg)
    for ( int i=0; i <= LAST_ITEM ; i++ ){
       msg += apps[i];
       if ( check[i] )
	  msg += i18n(" is present\n");
       else
          msg += i18n(" is missing\n");
    };

  // pretty dirty :-)
  // don't worry, I'm thinking about one better, more global way
  // but that would make me change other files...

#define II(n) modes[mode++] = n; QComboBox_encoder->insertItem
  II(0) (i18n("Windows RIFF .wav"));
  if (check[SOX]) {
    II(17) (i18n("CD-R .cdr"));
    II(18) (i18n("RAW .raw"));
    II(16) (i18n("Plain Text .dat"));
    II(19) (i18n("Amiga .8svx"));
    II(20) (i18n("Apple/SGI .aiff"));
    II(21) (i18n("Sun .au"));
    II(22) (i18n("Amiga .maud"));
    II(23) (i18n("Ircam .sf"));
    II(24) (i18n("Turtle Beach .smp"));
    II(25) (i18n("SoundBlaster .voc"));
    //II(26) (i18n(".cvs"));
    //II(27) (i18n(".vms"));
    II(28) (i18n("DOS .sndr"));
    II(29) (i18n("DOS .sndt"));
    II(30) (i18n(".txw"));
    II(31) (i18n("Psion .wve"));
    II(32) (i18n("Apple .hcom"));
  }
#undef II
  

  // find read devices
	if ( check[CDPARANOIA] && test_devices ) {
		QComboBox_device->clear();
		devices_count = 0;
		int i = 0;
		while ( possible_devices[i] ) {
			help_prog_stdout_temp = "";
			help_prog = new KProcess;
			help_prog->clearArguments();
			help_prog->setExecutable( "cdparanoia" );
			connect(help_prog, SIGNAL(receivedStdout(KProcess *, char *, int )),
				this, SLOT(test_env_stdout(KProcess *, char *, int  )));
			connect(help_prog, SIGNAL(receivedStderr(KProcess *, char *, int )),
				this, SLOT(test_env_stdout(KProcess *, char *, int  )));
			*help_prog << "-Q" << "-d" << possible_devices[i];
			help_prog->start(KProcess::Block, KProcess::AllOutput );
			delete help_prog;
			help_prog = 0;
			if (help_prog_stdout_temp.contains("Unable to read", TRUE) ||
		    	help_prog_stdout_temp.contains("Table of contents", TRUE) ||
			    help_prog_stdout_temp.contains("Could not read any data", TRUE) ) {
					cdparanoia_version = 9;
					devices_count++;
					devices[devices_count] = possible_devices[i];
					QComboBox_device->insertItem( possible_devices[i] );
			}
			i++;
		}
		if ( devices_count == 0 ) {
			msg += i18n("No CD-ROM/WORM devices found.\n");
			QComboBox_device->insertItem(i18n( "No devices found"));
		}
	};

  // find write devices
  if ( check[CDRECORD] && test_devices ){
    help_prog_stdout_temp = "";
    help_prog = new KProcess;
    help_prog->clearArguments();
    help_prog->setExecutable( "cdrecord" );
    connect(help_prog, SIGNAL(receivedStdout(KProcess *, char *, int )),
	    this, SLOT(test_env_stdout(KProcess *, char *, int  )));
    connect(help_prog, SIGNAL(receivedStderr(KProcess *, char *, int )),
	    this, SLOT(test_env_stdout(KProcess *, char *, int  )));
    *help_prog << "-scanbus";
    help_prog->start(KProcess::Block, KProcess::AllOutput );
    delete help_prog;
    help_prog = 0;

    cdrecord_devices_count = 0;
    int position = 0;
    int pos_max = 0;
    int busnumber = 0;
    int pos_2 = 0;

    while ( (int)(help_prog_stdout_temp.length()-60)>(position=help_prog_stdout_temp.find("scsibus",position,0)) && position > 0 ){
		position++;
		pos_max = help_prog_stdout_temp.find( "scsibus", position, 0);
		if ( pos_max <= 0 )
			pos_max = help_prog_stdout_temp.length()-60 ;
		position=help_prog_stdout_temp.find("\n",position, 0);
		while( pos_max > position ){
			pos_2 = help_prog_stdout_temp.find("Removable CD-ROM", position, 0 );
			if ( !(0+position < pos_2 < pos_max) )
				pos_2 = help_prog_stdout_temp.find("Removable WORM", position, 0 );

			if (0+position < pos_2 < pos_max){
				cdrecord_device[++cdrecord_devices_count] =
					help_prog_stdout_temp.mid( pos_2-36, 8 ) + " " +
					help_prog_stdout_temp.mid( pos_2-25, 14 );
				temp = help_prog_stdout_temp.mid( pos_2-41, 2 );
				temp2.setNum( busnumber );
				cdrecord_device_id[cdrecord_devices_count] = temp2;
				//TO FIX: lun is always 0. is it fine 4 everybody  ???
				cdrecord_device_id[cdrecord_devices_count] +=
					"," + temp.stripWhiteSpace() + ",0";
				position = pos_2;
			}else
				position++;
		position=help_prog_stdout_temp.find("\n",position, 0);
		};
		if ( pos_max > pos_2 )
			position = pos_max;
		else
			position = pos_2;

	      busnumber++;
    	};
	    if ( cdrecord_devices_count == 0 ) {
			msg += i18n("No CD-Writers devices found.\n");
		}
	};

  QComboBox_encoder_clicked( 0 );

  msg += "\n\n";

  // final message
  bool encoders = ( check[HZ8MP3] || check[BLADEENC] || check[L3ENC] &&
		    check[MP3ENC] || check[LAME] || check[ENCODE] );

  if ( encoders && check[CDPARANOIA] && check[CDRECORD] &&
       check[SOX] && check[MPG123] && devices_count != 0 ) {
    msg += i18n("All applications has been found :-)\n");

  } else {

    if (!check[CDPARANOIA] || devices_count == 0) {
      msg += i18n("You can NOT grab audio cds :-(\n");
    }

    if (!check[SOX]) {
      msg += i18n("You can NOT handle cdr, raw, dat, 8svx, aiff,\n"
		  "au, maud, IRCAM, Turtle Beach or voc files.\n");
    }

    if (!check[MPG123]) {
      msg += i18n("You can NOT decode mp2 or mp3 files to wav\n"
		  "or mp3(with a different bitrate) again :-(\n");
    }

    if (!encoders) {
      msg += i18n("You can NOT produce mp3 files (only WAV output)\n"
		  "Go and get the FREE 8hz-mp3 encoder\n"
		  "(look in the help for the WEB Site)\n");
    }

    if (!check[CDPARANOIA] && !check[MPG123] && !encoders && !check[SOX] ) {
      msg += i18n("!!! THIS APPLICATION CAN NOT DO ANYTHING !!!\n"
		  "!!! GO AND GET SOME REQUIRED APPS !!!\n");
    }

    if (!check[CDRECORD] || !check[SOX]) {
      msg += i18n("Unable to burn audio CD's\n");
    }

  };
	


  //  bring up the message box, and update parameters
  if (show ||
      check[CDPARANOIA]!=cdparanoia || check[CDRECORD]!=cdrecord ||
      check[SOX]!=sox || check[MPG123]!=mpg123 || check[HZ8MP3]!=hz8_mp3 ||
      check[BLADEENC]!=bladeenc || check[L3ENC]!=l3enc || check[LAME]!=lame ||
      check[MP3ENC]!=mp3enc||check[XING]!=xing ) {

//#ifdef KDE2
    QMessageBox::information( this, i18n ("System Check"), msg);	
//#else
//    KMsgBox::message( this, i18n ("System Check"), msg);	
//#endif

  };
  cdparanoia = check[CDPARANOIA];
  cdrecord = check[CDRECORD];
  sox = check[SOX];
  mpg123 = check[MPG123];
  hz8_mp3 = check[HZ8MP3];
  lame = check[LAME];
  encode = check[ENCODE];
  bladeenc = check[BLADEENC];
  l3enc = check[L3ENC];
  mp3enc = check[MP3ENC];
  xing = check[XING];

  // we're finished
  emit( new_status_info(i18n("System check finished...")));
}
