/*
	xcdroast.h
*/

#include <time.h>

typedef struct setup_data {
	gint writer_devnr;
	gchar writer_vendor[9];
	gchar writer_model[17];
	gint writer_mode;
	gint writer_speed;
	gint writer_fifo;

	gint readdev1_devnr;
	gchar readdev1_vendor[9];
	gchar readdev1_model[17];
	gint readdev2_devnr;
	gchar readdev2_vendor[9];
	gchar readdev2_model[17];
	gint audioread_interface;
	gint audioread_speed;
	gint audioread_overlap;
	gint audioread_sectorburst;

	GList *image_dirs;

	gchar *dsp_device;
	gchar *mix_device;
	gint notify_via;
	gint notify_at;

	gchar *cddb_host;
	gint cddb_port;

	gchar *logfile;
	gint loglevel;

	gint language;	

	gboolean option_tooltips;
	gboolean option_autoraise;
	gboolean option_savepos;
	gboolean option_personimage;
	gboolean option_overwritewarn;
	gboolean option_autodelete;
	gboolean option_titleprogress;
	gboolean option_displaycdtext;
	gboolean option_selectionmode;
} setup_data_t;

typedef struct scsi_devices {
	gint devnr;
	gint bus;
	gint id;
	gchar vendor[9];
	gchar model[17];
	gchar rev[5];
	gint removable;
	gint type;
} scsi_devices_t;

typedef struct writer_driver {
	gchar *driver;
	gchar *desc;
} writer_driver_t;

typedef struct current_set {
	gint writer_devnr;
	gint reader_devnr;
	gint writer_speed;
	gint audioread_speed;
	gint image_index;  /* index in image_dir-glist, -1 = automatic */
	gint proc_view;
	gchar *file_prefix;
	gchar *tocfile;
	gint cdrtype;	   /* in minutes */
	gint writemode;	   /* 0: DAO, 1: TAO */
	gboolean writesimul;
	gboolean writeeject;
	gboolean writepad;
	gboolean writeswap;
	gboolean nofixate;
	gboolean multisession;
	gint blankmode;
	gboolean blank_force;
	gboolean blank_eject;
	gboolean noaudioverify;
	gboolean verifyfailabort;
	gboolean indexscan;
} current_set_t;

typedef struct cd_info {
	gint nr_tracks;  /* -1 = no cd loaded */
	gint total_size; /* all sizes in frames */
	gchar cddb_discid[9];
	gint have_cdtext;
	gint have_cdextra;
	gchar *title;
	gchar *artist;
	gchar *cddb_dtitle;
	gint leadout;
} cd_info_t;

typedef struct track_info {
	gint track_nr;
	gint start_sec;
	gint size;     /* in frames */
	gint type;     /* 0: data,        1: audio */
	gint rec_type; /* 0: incremental, 1: uninterrupted */
	gint preemp;   /* 0: linear,      1: preemp */
	gint copyperm; /* 0: denied,      1: allowed */
	gint stereo;   /* 0: quadro,      1: stereo */
	gchar *volname; 
	gchar *title;
	gchar *artist;
	gchar *cddb_ttitle;
} track_info_t;

typedef struct cddb_match {
	gint exact;  /* 0: not exact match, 1: exact */
	gchar *categ;
	gchar *discid;
	gchar *dtitle;
} cddb_match_t;

typedef struct image_files {
	gchar *path;
	glong size;    /* in bytes */
	time_t mtime;  /* modification time */
	gint type;     /* 0: iso9660 data, 1: valid audio, 2: invalid audio, 
			  3: unknown data, 4: toc-file */
	gint readable; /* 0: no, 1: yes */
	gint from_track;
	gchar *volname; 
	gchar *title;
	gchar *artist;
	gchar *cddb_ttitle;
	gchar *cd_discid;
	gint isosize;  /* in sectors */
} image_files_t;

typedef struct track_read_param {
	gint trackinfo_index;  
	gint starttrack;
	gint endtrack;      /* set to 0 when not used */
	gint tracktype;     /* 0: data,        1: audio */
	gint startoffset;   /* for data tracks: startsector */
	gint endoffset;     /* for data tracks: endsector */
	gfloat percent;	    /* how many percent does this track take on cd */
	gint kbyte; 	   
	gint frames;
	gchar *trackfile;
} track_read_param_t;

typedef struct track_read_set {
	gint nrtracks;
	gint cdsize;	/* in frames */
	gchar *tocfile;
	gchar *cdtitle;
	GList *trackparams;
} track_read_set_t;

typedef struct image_dir_free {
	gchar *path;
	gint free;
} image_dir_free_t;

typedef struct write_track_param {
	gint nrtracks;
	gint simulation;
	gint *tracktype;
	gint *frames;
	gfloat *pct_so_far_arr;
	gfloat *pct_this_track_arr;
} write_track_param_t;

typedef struct language_list {
	gchar *langstring;
	GList *codes;
} language_list_t;

typedef struct mstr_redirect {
	gchar *mstr_path;
	gchar *redir_path;
} mstr_redirect_t;

typedef struct master_param {
	GList *mstr_redir;
	GList *exclude_paths;
	gint image_type;
	gboolean show_only_dirs;
	gboolean opt[16];
	gint bootable;
	gchar *boot_image;
	gchar *boot_catalog;
	gint boot_type;
	gchar *sparc_boot;
	gchar *volid;
	gchar *publisher;
	gchar *preparer;
	gchar *application;
	gchar *abstract;
	gchar *biblio;
	gchar *copyright;
	gchar *image_filename;
	gint old_session_size;	/* in 2048 byte blocks */
	gint session_size;	/* in 2048 byte blocks */
} master_param_t;

/*---------------------------------------------------------*/

#define XCDROAST_VERSION "0.98alpha6"
#define MAXLINE 1024
#define CDDAFRAME 2352
#define DATASECTORSIZE 2048
#define XCDRLOGO "icons/xcdrlogo.gif"
#define CDDBLOGO "icons/cddblogo.gif"
#define BURNPROOFLOGO "icons/burnproof.gif"
#define IMG_EXTENSIONS { ".iso", ".raw", ".img", ".wav", ".toc", NULL }
#define CDR_TYPES_MIN {20,21,63,-74,80,89,99,0} /* negative value is default */
#define XCDROAST_INFO_EXT ".xinf"
#define CDDA2WAV_INFO_EXT ".inf"

/* icons for dialogs (borrowed from gnome) */
#define ICO_ERROR "icons/dialog_error.png"
#define ICO_INFO "icons/dialog_information.png"
#define ICO_WARN "icons/dialog_warning.png"
#define ICO_QUEST "icons/dialog_question.png"
#define ICO_DEF "icons/dialog_default.png"

/* external called programms */
#define DF "/bin/df -k" 
#define UNAME "/bin/uname -a"
#define WAVPLAY "bin/wavplay"
#define RMTOOL "bin/rmtool"
#define VRFYTOOL "bin/vrfytool"
#define CDDBTOOL "bin/cddbtool"

/* this paths can be specified relative to lib-dir or absolute */
#define CDRECORD "bin/cdrecord" 
#define CDDA2WAV "bin/cdda2wav"
#define READCD "bin/readcd"
#define MKISOFS "bin/mkisofs"

#define CDRECORD_VERSION "1.8.1"
#define MKISOFS_VERSION "1.12.1"
#define CDDA2WAV_VERSION "1.8.1"
#define READCD_VERSION "1.8.1"

/* demo sound */
#define DSPTESTSOUND "sound/test.wav"

/* max scanned scsi devices */
#define MAXDEVICES 128

/* max support for cdrecord-drivers (as in cdrecord driver=help) */ 
#define MAXDRIVERS 64 

/* max support for blank-modes (as in cdrecord blank=help) */
#define MAXBLANKMODES 16

/* max tracks a CD can have */
#define MAXTRACKS 100

/* max CDDB matches */
#define MAXCDDB 100

/* max args for pipes */
#define MAXPIPEARGS 1024 

/* timeout in seconds for CDDB queries over network */
#define NETIOTIMEOUT 10;

/* default image-file-prefix */
#define IMGFILEPREFIX "track"

/* default filenames for process-output-savefiles */
#define SAVEOUTCDRECORD "cdrecord.out"
#define SAVEREADTRACKS "readtracks.out"
#define SAVEVERIFYTRACKS "verifytracks.out"
#define SAVEMASTERTRACK "mkisofs.out"

/* default paths/configfile */
#define CONFIGDIR "~/.xcdroast"
#define LIBDIR "/usr/local/lib/xcdroast-0.98"
#define CONFFILE "xcdr098.cfg"
#define LOGFILE "xcdr098.log"
#define ISOOPTFILE "isoopt.cfg"
#define ISOHEADERFILE "isoheader.cfg"

/* language stuff*/
#define MAXENTRIES 512 
#define MAXLANG 32 

/* font definitions */
/* - old definitions, not working with fontsets 
#define NORMFONT "-*-helvetica-medium-r-normal-*-*-120-*-*-p-*-*"
#define BOLDFONT "-*-helvetica-bold-r-normal-*-*-120-*-*-p-*-*"
#define BIGFONT "-*-helvetica-bold-r-normal-*-*-240-*-*-p-*-*"
#define SLANTFONT "-*-helvetica-medium-o-normal-*-*-100-*-*-*-*-*-*"
*/ 

#define NORMFONT "-*-helvetica-medium-r-normal-*-*-120-*-*-p-*-*,-default-*-medium-r-*--14-*-*-*-*-*-*-*,*-medium-r-*,*"
#define BOLDFONT "-*-helvetica-bold-r-normal-*-*-120-*-*-p-*-*,-default-*-bold-r-*--16-*-*-*-*-*-*-*,*-bold-r-*,*"
#define BIGFONT "-*-helvetica-bold-r-normal-*-*-240-*-*-p-*-*,-default-*-bold-r-*--24-*-*-*-*-*-*-*,*-bold-r-*,*"
#define SLANTFONT "-*-helvetica-medium-o-normal-*-*-100-*-*-*-*-*-*,-default-*-medium-i-*--12-*-*-*-*-*-*-*,*-medium-i-*,*"

/* default window sizes - usage autodetected */
/* 75dpi fonts */
#define XCDR_TOPLEVEL_X0 790
#define XCDR_TOPLEVEL_Y0 540
/* 100dpi fonts */
#define XCDR_TOPLEVEL_X1 980
#define XCDR_TOPLEVEL_Y1 670

/* color definitions
   - may not be used at all because they interfere with themes */
#define TOOLTIPCOL "light yellow"
#define GTKLISTCOL "gray95"
#define SELECTEDCLISTCOL "gray70"

/* text shortcuts */
#define T_YES text(8)
#define T_NO text(9)
#define T_CANCEL text(10)
#define T_OK text(11)
#define T_CLOSE text(195)
#define T_ANYWAY text(199)

/* predefined master iso-opitions */
/* there must be the same number of TYPES as TEXTS (minus the 0 element) */
#define MASTER_TEXTS { 384, 385, 386, 0 }
#define MASTER_TYPES { 	{ 1,1,0,0,1,0,1,1,0,0,0,0,0,0,0 }, \
			{ 1,0,0,0,1,0,0,1,0,0,0,0,0,0,0 }, \
			{ 0,0,0,0,0,1,0,0,0,0,0,0,0,0,0 } }
	
#define DEF_BOOT_CATALOG "/boot/boot.catalog"
 
